@@ -49,16 +49,16 @@ sub appendCommonVars {
     my $session = $self->session;
     my $var     = shift;
     my $inbox   = shift || WebGUI::Inbox->new($session);
-    my $user    = $session->user;
+    my $user    = $self->getUser;
     my $method  = $self->method;
 
     $self->SUPER::appendCommonVars($var);
 
-    $var->{'view_inbox_url'          } = $self->getUrl("module=inbox;do=view");
+    $var->{'view_inbox_url'          } = $self->getUrl("module=inbox;do=view", 'useUid');
     $var->{'view_invitations_url'    } = $self->getUrl("module=inbox;do=manageInvitations");
-    $var->{'unread_message_count'    } = $inbox->getUnreadMessageCount;
+    $var->{'unread_message_count'    } = $inbox->getUnreadMessageCount($user->userId);
     $var->{'invitation_count'        } = $self->getInvitationCount;
-    $var->{'invitations_enabled'     } = $session->user->profileField('ableToBeFriend');
+    $var->{'invitations_enabled'     } = $user->profileField('ableToBeFriend');
     $var->{'user_invitations_enabled'} = $session->setting->get("inboxInviteUserEnabled");
     $var->{'invite_friend_url'       } = $self->getUrl("module=inbox;do=inviteUser");
 
@@ -77,7 +77,9 @@ Returns whether or not the user can view the inbox tab
 
 sub canView {
     my $self    = shift;
-    return ($self->uid eq ""); 
+    my $session = $self->session;
+    return $self->uid eq ""
+        || $self->uid ne "" && $session->user->isInGroup($session->setting->get('groupIdAdminUser'));
 }
 
 #-------------------------------------------------------------------
@@ -218,12 +220,44 @@ sub editSettingsForm {
         hoverHelp => $i18n->get("invite user confirm template hoverHelp")
 	);
     $f->selectRichEditor(
-        name        => "inboxRichEditorId",
+        name        => "inboxRichEditId",
         value       => $self->getRichEditorId,
         label       => $i18n->get("inbox rich editor label"),
         hoverHelp   => $i18n->get("inbox rich editor description"),
     );
-    
+    $f->yesNo(
+        name      => "inboxCopySender",
+        value     => $setting->get("inboxCopySender"),
+        label     => $i18n->get("inbox copy sender label"),
+        hoverHelp => $i18n->get("inbox copy sender hoverHelp")
+    );
+    $f->yesNo(
+        name         => 'sendInboxNotificationsOnly',
+        label        => $i18n->get('send inbox notifications only'),
+        hoverHelp    => $i18n->get('send inbox notifications only help'),
+        defaultValue => $setting->get('sendInboxNotificationsOnly'),
+    );
+    $f->text(
+        name         => 'inboxNotificationsSubject',
+        label        => $i18n->get('inbox notifications subject'),
+        hoverHelp    => $i18n->get('inbox notifications subject help'),
+        defaultValue => $setting->get('inboxNotificationsSubject'),
+    );
+    $f->template(
+        name         => 'inboxNotificationTemplateId',
+        label        => $i18n->get('inbox notification template'),
+        hoverHelp    => $i18n->get('inbox notification template help'),
+        defaultValue => $self->getInboxNotificationTemplateId,
+        namespace    => 'Account/Inbox/Notification',
+    );
+    $f->template(
+        name         => 'inboxSmsNotificationTemplateId',
+        label        => $i18n->get('inbox sms notification template'),
+        hoverHelp    => $i18n->get('inbox sms notification template help'),
+        defaultValue => $self->getInboxSmsNotificationTemplateId,
+        namespace    => 'Account/Inbox/Notification',
+    );
+
     return $f->printRowsOnly;
 }
 
@@ -242,28 +276,35 @@ sub editSettingsFormSave {
     my $form    = $session->form;
 
     #Messages Settings
-    $setting->set("inboxStyleTemplateId", $form->process("inboxStyleTemplateId","template"));
-    $setting->set("inboxLayoutTemplateId", $form->process("inboxLayoutTemplateId","template"));
-    $setting->set("inboxViewTemplateId", $form->process("inboxViewTemplateId","template"));
-    $setting->set("inboxViewMessageTemplateId",$form->process("inboxViewMessageTemplateId","template"));
-    $setting->set("inboxSendMessageTemplateId",$form->process("inboxSendMessageTemplateId","template"));
+    $setting->set("inboxStyleTemplateId",              $form->process("inboxStyleTemplateId",              "template"));
+    $setting->set("inboxLayoutTemplateId",             $form->process("inboxLayoutTemplateId",             "template"));
+    $setting->set("inboxViewTemplateId",               $form->process("inboxViewTemplateId",               "template"));
+    $setting->set("inboxViewMessageTemplateId",        $form->process("inboxViewMessageTemplateId",        "template"));
+    $setting->set("inboxSendMessageTemplateId",        $form->process("inboxSendMessageTemplateId",        "template"));
     $setting->set("inboxMessageConfirmationTemplateId",$form->process("inboxMessageConfirmationTemplateId","template"));
-    $setting->set("inboxErrorTemplateId",$form->process("inboxErrorTemplateId","template"));
+    $setting->set("inboxErrorTemplateId",              $form->process("inboxErrorTemplateId",              "template"));
     #Friends Invitations Settings
-    $setting->set("inboxManageInvitationsTemplateId",$form->process("inboxManageInvitationsTemplateId","template"));
-    $setting->set("inboxViewInvitationTemplateId",$form->process("inboxViewInvitationTemplateId","template"));
-    $setting->set("inboxInvitationConfirmTemplateId",$form->process("inboxInvitationConfirmTemplateId","template"));
+    $setting->set("inboxManageInvitationsTemplateId",  $form->process("inboxManageInvitationsTemplateId",  "template"));
+    $setting->set("inboxViewInvitationTemplateId",     $form->process("inboxViewInvitationTemplateId",     "template"));
+    $setting->set("inboxInvitationConfirmTemplateId",  $form->process("inboxInvitationConfirmTemplateId",  "template"));
     #User Invitation Settings
-    $setting->set("inboxInviteUserEnabled",$form->process("inboxInviteUserEnabled","yesNo"));
-    $setting->set("inboxInviteUserRestrictSubject",$form->process("inboxInviteUserRestrictSubject","yesNo"));
-    $setting->set("inboxInviteUserSubject",$form->process("inboxInviteUserSubject","text"));
-    $setting->set("inboxInviteUserRestrictMessage",$form->process("inboxInviteUserRestrictMessage","yesNo"));
-    $setting->set("inboxInviteUserMessage",$form->process("inboxInviteUserMessage","HTMLArea"));    
-    $setting->set("inboxInviteUserMessageTemplateId",$form->process("inboxInviteUserMessageTemplateId","template"));
-    $setting->set("inboxInviteUserTemplateId",$form->process("inboxInviteUserTemplateId","template"));
-    $setting->set("inboxInviteUserConfirmTemplateId",$form->process("inboxInviteUserConfirmTemplateId","template"));
+    $setting->set("inboxInviteUserEnabled",            $form->process("inboxInviteUserEnabled",            "yesNo"));
+    $setting->set("inboxInviteUserRestrictSubject",    $form->process("inboxInviteUserRestrictSubject",    "yesNo"));
+    $setting->set("inboxInviteUserSubject",            $form->process("inboxInviteUserSubject",            "text"));
+    $setting->set("inboxInviteUserRestrictMessage",    $form->process("inboxInviteUserRestrictMessage",    "yesNo"));
+    $setting->set("inboxInviteUserMessage",            $form->process("inboxInviteUserMessage",            "HTMLArea"));    
+    $setting->set("inboxInviteUserMessageTemplateId",  $form->process("inboxInviteUserMessageTemplateId",  "template"));
+    $setting->set("inboxInviteUserTemplateId",         $form->process("inboxInviteUserTemplateId",         "template"));
+    $setting->set("inboxInviteUserConfirmTemplateId",  $form->process("inboxInviteUserConfirmTemplateId",  "template"));
+    #General Inbox Settings
+    $setting->set("inboxRichEditId",                   $form->process("inboxRichEditId",                 "selectRichEditor") );
+    $setting->set("inboxCopySender",                   $form->process("inboxCopySender",                   "yesNo"));
 
-    $setting->set("inboxRichEditorId", $form->process("inboxRichEditorId", "selectRichEditor") );
+    #Inbox Notification Settings
+    $setting->set("sendInboxNotificationsOnly",        $form->process("sendInboxNotificationsOnly", "yesNo"));
+    $setting->set("inboxNotificationsSubject",         $form->process("inboxNotificationsSubject", "text"));
+    $setting->set("inboxNotificationTemplateId",       $form->process("inboxNotificationTemplateId","template"));
+    $setting->set("inboxSmsNotificationTemplateId",    $form->process("inboxSmsNotificationTemplateId","template"));
 }
 
 
@@ -282,6 +323,32 @@ sub getInboxErrorTemplateId {
 
 #-------------------------------------------------------------------
 
+=head2 getInboxNotificationTemplateId ( )
+
+This method returns the template ID for inbox notifications.
+
+=cut
+
+sub getInboxNotificationTemplateId {
+    my $self = shift;
+    return $self->session->setting->get("inboxNotificationTemplateId") || "b1316COmd9xRv4fCI3LLGA";
+}
+
+#-------------------------------------------------------------------
+
+=head2 getInboxSmsNotificationTemplateId ( )
+
+This method returns the template ID for inbox SMS notifications.
+
+=cut
+
+sub getInboxSmsNotificationTemplateId {
+    my $self = shift;
+    return $self->session->setting->get("inboxSmsNotificationTemplateId") || "i9-G00ALhJOr0gMh-vHbKA";
+}
+
+#-------------------------------------------------------------------
+
 =head2 getInvitationCount ( )
 
 This method returns the total number of invitations in the invitation box.
@@ -399,7 +466,7 @@ This method returns the rich editor ID users compose messages with.
 
 sub getRichEditorId {
     my $self    = shift;
-    return $self->session->setting->get("inboxRichEditorId") || "PBrichedit000000000001";
+    return $self->session->setting->get("inboxRichEditId") || "PBrichedit000000000001";
 }
 
 #-------------------------------------------------------------------
@@ -575,7 +642,7 @@ sub www_deleteMessage {
     my $message   = $inbox->getMessage($messageId);
 
     $self->store->{tab} = "inbox";
-    
+
     if (!(defined $message) || !$inbox->canRead($message)) {
         #View will handle displaying these errors
         return $self->www_viewMessage;
@@ -593,7 +660,7 @@ sub www_deleteMessage {
         }
     }
     $message->delete;
-    
+
     return $self->www_viewMessage($displayMessage->getId);
 }
 
@@ -623,6 +690,79 @@ sub www_deleteMessages {
 
 #-------------------------------------------------------------------
 
+=head2 www_actOnMessages ( )
+
+Acts on a list of messages selected for the current user
+
+=cut
+
+sub www_actOnMessages {
+    my $self    = shift;
+    my $session = $self->session;
+    my $action  = $session->form->process( 'action' );
+    my $i18n    = WebGUI::International->new( $session, 'Account_Inbox' );
+
+    my %handler = (
+        $i18n->get( 'delete label' )         => \&www_deleteMessages,
+        $i18n->get( 'mark as read label' )   => \&www_markAsReadMessages,
+        $i18n->get( 'mark as unread label' ) => \&www_markAsUnreadMessages,
+    );
+    if ( defined $action && length $action && defined $handler{$action} ) {
+        return $handler{$action}->( $self, @_ );
+    }
+    return $self->www_view();
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_markAsReadMessages ( )
+
+Marks a list of messages selected for the current user as read
+
+=cut
+
+sub www_markAsReadMessages {
+    my $self    = shift;
+    my $session = $self->session;
+
+    $self->store->{tab} = 'inbox';
+
+    my @messages = $session->form->process( 'message', 'checkList' );
+
+    foreach my $messageId ( @messages ) {
+        my $message = WebGUI::Inbox::Message->new( $session, $messageId );
+        $message->setRead;
+    }
+
+    return $self->www_view();
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_markAsUnreadMessages ( )
+
+Marks a list of messages selected for the current user as unread
+
+=cut
+
+sub www_markAsUnreadMessages {
+    my $self    = shift;
+    my $session = $self->session;
+
+    $self->store->{tab} = 'inbox';
+
+    my @messages = $session->form->process( 'message', 'checkList' );
+
+    foreach my $messageId ( @messages ) {
+        my $message = WebGUI::Inbox::Message->new( $session, $messageId );
+        $message->setUnread;
+    }
+
+    return $self->www_view();
+}
+
+#-------------------------------------------------------------------
+
 =head2 www_inviteUser ( )
 
 Form for inviting a user to join the site.
@@ -635,7 +775,7 @@ sub www_inviteUser {
     my $form         = $session->form;
     my $setting      = $session->setting;
     my $user         = $session->user;
-    
+
     my $displayError = shift;
     my $var          = {};
 
@@ -644,11 +784,11 @@ sub www_inviteUser {
 
     #Add any error passed in to be displayed if the form reloads
     $var->{'message_display_error'}  = $displayError;
-   
+
     #Message From
     $var->{'message_from'     }  = $user->getWholeName;
     $var->{'message_from_id'  }  = $user->userId;
-    
+
     #Message To
     $var->{'form_to'          } = WebGUI::Form::email($session, {
         name  => "to",
@@ -699,7 +839,7 @@ sub www_inviteUser {
         action => $self->getUrl("module=inbox;do=inviteUserSave"),
         extras => q{name="inviteForm"}
     });
-    
+
     $var->{'submit_button'    }  = WebGUI::Form::submit($session,{});
     $var->{'form_footer'      }  = WebGUI::Form::formFooter($session, {});
     $var->{'back_url'         }  = $session->env->get("HTTP_REFERER") || $var->{'view_inbox_url'};
@@ -843,11 +983,11 @@ sub www_manageInvitations {
     my $i18n    = WebGUI::International->new($session,'Account_Inbox');
 
     $self->store->{tab} = "invitations";
-    
+
     #Deal with rows per page
     my $rpp          = $session->form->get("rpp") || 25;
     my $rpp_url      = ";rpp=$rpp";
-    
+
     #Cache the base url
     my $inboxUrl     =  $self->getUrl("op=account;module=inbox;do=manageInvitations");
 
@@ -859,15 +999,15 @@ sub www_manageInvitations {
         $rpp
     );
     $p->setDataByQuery($sql,undef,undef,[$user->userId]);
-    
+
     #Export page to template
     my @msg    = ();
     foreach my $row ( @{$p->getPageData} ) {
         my $inviter   = WebGUI::User->new($session,$row->{inviterId});
         next if($inviter->isVisitor); # Inviter account got deleted
-        
+
         my $epoch = WebGUI::DateTime->new(mysql => $row->{dateSent} )->epoch;
-        
+
         my $hash                       = {};
         $hash->{'invite_id'          } = $row->{inviteId};
         $hash->{'message_url'        } = $self->getUrl("module=inbox;do=viewInvitation;inviteId=".$row->{inviteId});
@@ -883,7 +1023,7 @@ sub www_manageInvitations {
 	  	push(@msg,$hash);
    	}
     my $msgCount  = $p->getRowCount;
-         
+
    	$var->{'message_loop'  } = \@msg;
     $var->{'has_messages'  } = $msgCount > 0;
     $var->{'message_total' } = $msgCount;
@@ -942,10 +1082,11 @@ sub www_sendMessage {
     #Add any error passed in to be displayed if the form reloads
     $var->{'message_display_error'}  = $displayError;
 
-    #Add common template variable for displaying the inbox
     my $inbox     = WebGUI::Inbox->new($session); 
+
+    #Add common template variable for displaying the inbox
     $self->appendCommonVars($var,$inbox);
-    
+
     my $messageId = $form->get("messageId");
     my $userId    = $form->get("userId");
     my $pageUrl   = $session->url->page;
@@ -955,7 +1096,7 @@ sub www_sendMessage {
     if($messageId) {
         #This is a reply to a message - automate who the user is
         my $message = $inbox->getMessage($messageId);
-        
+
         #Handle Errors
         if (!(defined $message)) {
             #Message doesn't exist
@@ -1002,14 +1143,14 @@ sub www_sendMessage {
             $var->{'isInbox'} = "true";
             return $self->showError($var,$errorMsg,$backUrl,$self->getInboxErrorTemplateId);
         }
-        
+
         $var->{'isPrivateMessage'} = "true";
         $var->{'message_to'      } = $toUser->getWholeName;
     }
     else {
         #This is a new message
         $var->{'isNew'     } = "true";
-        
+
         my $friends           = $fromUser->friends->getUserList;
         my @checkedFriends    = ();
         my @friendsChecked    = $form->process("friend","checkList");
@@ -1025,16 +1166,12 @@ sub www_sendMessage {
                 $disabled  = "";
                 $activeFriendCount++;
             }
-            my $fname      = $friend->profileField("firstName");
-            my $lname      = $friend->profileField("lastName");
-            my $wholeName  = "";
-            $wholeName     = $fname." ".$lname if($fname && $lname);
 
             my $isChecked  = WebGUI::Utility::isIn($friendId,@friendsChecked);            
             my $friendHash = {
                 'friend_id'        => $friendId,
                 'friend_name'      => $friends->{$friendId},
-                'friend_wholeName' => $wholeName,
+                'friend_wholeName' => $friend->getWholeName,
             };
 
             push(@checkedFriends,$friendHash) if($isChecked);
@@ -1048,7 +1185,7 @@ sub www_sendMessage {
 
             push (@friendsLoop, $friendHash);
         }
-        
+
         #You can't send new messages if you don't have any friends to send to
         unless(@friendIds) {
             my $i18n  = WebGUI::International->new($session,'Account_Inbox');
@@ -1068,9 +1205,9 @@ sub www_sendMessage {
         $var->{'friends_loop'       } = \@friendsLoop;
         $var->{'checked_fiends_loop'} = \@checkedFriends;
     }
- 
+
     $var->{'message_from'         }  = $fromUser->getWholeName;
-    
+
     my $subject = $form->get("subject");
     if($subject eq "" && $messageId) {
         $subject = "Re: ".$var->{'message_subject'};
@@ -1083,7 +1220,7 @@ sub www_sendMessage {
     });
 
     $var->{'message_body'     } = $form->get('message');
-    
+
     $var->{'form_message_text'}  = WebGUI::Form::textarea($session, {
         name   =>"message",
         value  =>$var->{'message_body'} || "",
@@ -1097,12 +1234,12 @@ sub www_sendMessage {
         width => "600",
         richEditId => $self->getRichEditorId,
     });
-    
+
     $var->{'form_header'      }  = WebGUI::Form::formHeader($session,{
         action => $self->getUrl("module=inbox;do=sendMessageSave;messageId=$messageId;userId=$userId"),
         extras => q{name="messageForm"}
     });
-    
+
     $var->{'submit_button'    }  = WebGUI::Form::submit($session,{});
     $var->{'form_footer'      }  = WebGUI::Form::formFooter($session, {});
     $var->{'back_url'         }  = $backUrl;
@@ -1131,7 +1268,7 @@ sub www_sendMessageSave {
 
     #Add common template variable for displaying the inbox
     my $inbox     = WebGUI::Inbox->new($session); 
-    
+
     my $messageId = $form->get("messageId");
     my $userId    = $form->get("userId");
     my @friends   = $form->get("friend","checkList");    
@@ -1169,6 +1306,10 @@ sub www_sendMessageSave {
         }
     }
 
+    if($session->setting->get('inboxCopySender')) {
+        push @toUsers, $session->user->userId;
+    }
+
     #Check for client errors
     if($subject eq "") {
         my $i18n  = WebGUI::International->new($session,'Account_Inbox');
@@ -1190,13 +1331,86 @@ sub www_sendMessageSave {
     return $self->www_sendMessage($errorMsg) if $hasError;
 
     foreach my $uid (@toUsers) {
-        $inbox->addMessage({
+        my $messageProperties = {
             message => $message,
             subject => $subject,
-            userId  => $uid,
             status  => 'unread',
             sentBy  => $fromUser->userId
-        });
+        };
+        my $messageOptions = {};
+        
+        # Handle Email/SMS Notifications
+        my $user = WebGUI::User->new($session, $uid);
+        
+        # Sender only gets CCd on inbox message (not real email)
+        my $isSender = $uid eq $session->user->userId;
+        $messageOptions->{no_email} = 1 if $isSender;
+        
+        # Optionally set SMS notification details (excluding sender)
+        my $smsAddress = $user->getInboxSmsNotificationAddress;
+        if ( $smsAddress && !$isSender ) {
+            my $smsNotificationTemplate
+                = WebGUI::Asset::Template->new($session, $self->getInboxSmsNotificationTemplateId);
+            if ($smsNotificationTemplate) {
+                ##Create template variables
+                my $var = {
+                    fromUsername => $fromUser->username,
+                    subject      => $messageProperties->{subject},
+                    message      => $messageProperties->{message},
+                    inboxLink    => $session->url->append($session->url->getSiteURL, 'op=account;module=inbox'),
+                };
+                ##Fill in template
+                my $output = $smsNotificationTemplate->process($var);
+                ##Evaluate macros by hand
+                WebGUI::Macro::process($session, \$output);
+                ##Assign template output to $messageProperties->{emailMessage}
+                $messageProperties->{smsMessage} = $output;
+                $messageProperties->{smsAddress} = $smsAddress;
+                $messageProperties->{smsSubject} = $self->session->setting->get('smsGatewaySubject');
+            }
+            else {
+                $session->log->warn(sprintf "Unable to instanciate notification template: ". $self->getInboxSmsNotificationTemplateId);
+            }
+        }
+        
+        # Optionally set email notification details (excluding sender)
+        if ($session->setting->get('sendInboxNotificationsOnly') && !$isSender) {
+            my $notificationAddresses = $user->getInboxNotificationAddresses;
+            
+            # If user has turned off email notifications and admin has turned on sendInboxNotificationsOnly,
+            # user gets no email at all - because email and email notification are mutually exclusive.
+            # Note that they can still possibly get SMS notification above
+            if (!$notificationAddresses) {
+                $messageOptions->{no_email} = 1;
+            } else {
+                my $template = WebGUI::Asset::Template->new($session, $self->getInboxNotificationTemplateId);
+                if ($template) {
+                    ##Create template variables
+                    my $var = {
+                        fromUsername => $fromUser->username,
+                        subject      => $messageProperties->{subject},
+                        message      => $messageProperties->{message},
+                        inboxLink    => $session->url->append($session->url->getSiteURL, 'op=account;module=inbox'),
+                    };
+                    ##Fill in template
+                    my $output = $template->process($var);
+                    ##Evaluate macros by hand
+                    WebGUI::Macro::process($session, \$output);
+                    ##Assign template output to $messageProperties->{emailMessage}
+                    $messageProperties->{emailMessage} = $output;
+                    $messageProperties->{emailSubject} = $session->setting->get('inboxNotificationsSubject');
+                }
+                else {
+                    $session->log->warn(sprintf "Unable to instanciate notification template: ". $self->getInboxNotificationTemplateId);
+                }
+            }
+        }
+        
+        $messageProperties->{userId} = $uid;
+        my $thisMessage = $inbox->addMessage($messageProperties, $messageOptions);
+        if ($uid eq $session->user->userId) {
+            $thisMessage->setRead;
+        }
     }
 
     $self->appendCommonVars($var,$inbox);
@@ -1216,15 +1430,16 @@ The main view page for editing the user's profile.
 sub www_view {
     my $self    = shift;
     my $session = $self->session;
-    my $user    = $session->user;
+    my $user    = $self->getUser;
+
     my $var     = {};
 
     $self->store->{tab} = "inbox";
-   
+
     #Deal with sort order
     my $sortBy       = $session->form->get("sortBy") || undef;
     my $sort_url     = ($sortBy)?";sortBy=$sortBy":"";
-    
+
     #Deal with sort direction
     my $sortDir      = $session->form->get("sortDir") || "desc";
     my $sortDir_url  = ";sortDir=".(($sortDir eq "desc")?"asc":"desc");
@@ -1232,26 +1447,37 @@ sub www_view {
     #Deal with rows per page
     my $rpp          = $session->form->get("rpp") || 25;
     my $rpp_url      = ";rpp=$rpp";
-    
+
+    #Deal with user filtering
+    my $userFilter      = $session->form->get("userFilter") || 'all';
+    my $userFilter_url  = ";userFilter=$userFilter";
+
     #Cache the base url
-    my $inboxUrl     =  $self->getUrl;
+    my $inboxUrl     =  $self->getUrl('', 'useUid');
+
+    my $urlFrag = $sortDir_url . $rpp_url . $userFilter_url;
 
     #Create sortBy headers
-    $var->{'subject_url'   } = $inboxUrl.";sortBy=subject".$sortDir_url.$rpp_url;
-   	$var->{'status_url'    } = $inboxUrl.";sortBy=status".$sortDir_url.$rpp_url;
-    $var->{'from_url'      } = $inboxUrl.";sortBy=sentBy".$sortDir_url.$rpp_url;
-    $var->{'dateStamp_url' } = $inboxUrl.";sortBy=dateStamp".$sortDir_url.$rpp_url;
-    $var->{'rpp_url'       } = $inboxUrl.$sort_url.";sortDir=".$sortDir;
-    
+    $var->{'subject_url'   } = $inboxUrl.";sortBy=subject"   . $urlFrag;
+   	$var->{'status_url'    } = $inboxUrl.";sortBy=status"    . $urlFrag;
+    $var->{'from_url'      } = $inboxUrl.";sortBy=sentBy"    . $urlFrag;
+    $var->{'dateStamp_url' } = $inboxUrl.";sortBy=dateStamp" . $urlFrag;
+
+    $var->{'rpp_url'       } = $inboxUrl.$sort_url.$sortDir_url.$userFilter_url;
+
     #Create the paginator
     my $inbox     = WebGUI::Inbox->new($session);
-    my $p         = $inbox->getMessagesPaginator($session->user,{
+    my $messageOptions = {
         sortBy        => $sortBy,
         sortDir       => $sortDir,
-        baseUrl       => $inboxUrl.$sort_url.";sortDir=".$sortDir.$rpp_url,
-        paginateAfter => $rpp
-    });
-    
+        baseUrl       => $inboxUrl.$sort_url.$sortDir_url.$rpp_url.$userFilter_url,
+        paginateAfter => $rpp,
+    };
+    if ($userFilter ne 'all') {
+        $messageOptions->{whereClause} = sprintf 'ibox.sentBy=%s', $session->db->quote($session->form->get('userFilter'));
+    }
+    my $p         = $inbox->getMessagesPaginator($user, $messageOptions);
+
     #Export page to template
     my @msg       = ();
     foreach my $row ( @{$p->getPageData} ) {
@@ -1260,7 +1486,7 @@ sub www_view {
 
         my $hash                       = {};
         $hash->{'message_id'         } = $message->getId;
-        $hash->{'message_url'        } = $self->getUrl("module=inbox;do=viewMessage;messageId=".$message->getId);
+        $hash->{'message_url'        } = $self->getUrl("module=inbox;do=viewMessage;messageId=".$message->getId,'useUid');
         $hash->{'subject'            } = $message->get("subject");
         $hash->{'status_class'       } = $message->get("status");
         $hash->{'status'             } = $message->getStatus;
@@ -1280,7 +1506,7 @@ sub www_view {
 	  	push(@msg,$hash);
    	}
     my $msgCount  = $p->getRowCount;
-         
+
    	$var->{'message_loop'        } = \@msg;
     $var->{'has_messages'        } = $msgCount > 0;
     $var->{'message_total'       } = $msgCount;
@@ -1296,8 +1522,22 @@ sub www_view {
         extras  => q{onchange="location.href='}.$var->{'rpp_url'}.q{;rpp='+this.options[this.selectedIndex].value"}
     });
 
+    my $userSql = $inbox->getMessageSql(undef, { 'select' => <<EOSQL, });
+ibox.sentBy,
+(IF(userProfileData.firstName != '' and userProfileData.firstName is not null and userProfileData.lastName !='' and userProfileData.lastName is not null, concat(userProfileData.firstName,' ',userProfileData.lastName),users.username)) as fullName
+EOSQL
+    tie my %userHash, 'Tie::IxHash';
+    my $i18n = WebGUI::International->new($session, 'Account_Inbox');
+    %userHash = ( 'all' => $i18n->get('All users'), $session->db->buildHash($userSql) );
+    $var->{'userFilter'} = WebGUI::Form::selectBox($session,{
+        name    => 'userFilter',
+        options => \%userHash,
+        value   => $session->form->get('userFilter') || 'all',
+        extras  => q{onchange="location.href='}.$inboxUrl.q{;userFilter='+this.options[this.selectedIndex].value"}
+    });
+
     $var->{'form_header'} = WebGUI::Form::formHeader($session,{
-        action => $self->getUrl("module=inbox;do=deleteMessages")
+        action => $self->getUrl("module=inbox;do=actOnMessages")
     });
     $var->{'form_footer'} = WebGUI::Form::formFooter($session);
 
@@ -1351,9 +1591,9 @@ sub www_viewInvitation {
         $var->{'isInvitation'} = "true";
         return $self->showError($var,$errorMsg,$backUrl,$self->getInboxErrorTemplateId);
     }
-        
+
     my $epoch = WebGUI::DateTime->new(mysql => $invitation->{dateSent} )->epoch;
-        
+
     $var->{'invite_id'              } = $inviteId;
     $var->{'message_from_id'        } = $inviter->userId; 
     $var->{'message_from'           } = $inviter->getWholeName;
@@ -1370,7 +1610,7 @@ sub www_viewInvitation {
                 || $var->{'message_body'} =~ /\<p/ig) {
         $var->{'message_body'} =~ s/\n/\<br \/\>\n/g;
     }
-    
+
     #Build the action URLs
     my $nextInvitation = $friends->getPreviousInvitation($invitation);  #Messages sorted descending so next is actually previous
     if( $nextInvitation->{inviteId} ) {
@@ -1413,7 +1653,7 @@ The page on which users view their messages
 sub www_viewMessage {
     my $self      = shift;
     my $session   = $self->session;
-    my $user      = $session->user;
+    my $user      = $self->getUser;
 
     my $var       = {};
     my $messageId = shift || $session->form->get("messageId");
@@ -1426,7 +1666,7 @@ sub www_viewMessage {
 
     #Add common template variable for displaying the inbox
     $self->appendCommonVars($var,$inbox);
-    
+
     #Handler Errors
     if (!(defined $message)) {
         my $i18n  = WebGUI::International->new($session,'Account_Inbox');
@@ -1442,7 +1682,7 @@ sub www_viewMessage {
         $var->{'isInvitation'} = "true";
         return $self->showError($var,$errorMsg,$backUrl,$self->getInboxErrorTemplateId);
     }
-    
+
     $message->setStatus("read") unless ($message->isRead);
 
     $var->{'message_id'             } = $messageId;
