@@ -18,6 +18,7 @@ use Carp qw( croak confess );
 use Scalar::Util qw( blessed );
 use Clone qw(clone);
 use JSON;
+use HTML::Packer;
 
 use WebGUI::AssetBranch;
 use WebGUI::AssetClipboard;
@@ -35,6 +36,7 @@ use WebGUI::Form;
 use WebGUI::HTML;
 use WebGUI::HTMLForm;
 use WebGUI::Keyword;
+use WebGUI::ProgressBar;
 use WebGUI::Search::Index;
 use WebGUI::TabForm;
 use WebGUI::Utility;
@@ -294,7 +296,13 @@ sub canView {
 
 =head2 checkView ( )
 
-Returns error messages if a user can't view due to publishing problems, otherwise it sets the cookie and returns undef. This is sort of a hack until we find something better.
+Returns error messages if a user can't view due to publishing problems,
+otherwise it sets the cookie and returns undef. This is sort of a hack
+until we find something better.
+
+If SSL in enabled in the config file, and the asset has encryptPage set, and
+HTTPS is set and SSLPROXY is not set in the ENV, then this page is redirected
+to SSL.
 
 =cut
 
@@ -302,19 +310,27 @@ sub checkView {
 	my $self = shift;
 	return $self->session->privilege->noAccess() unless $self->canView;
 	my ($conf, $env, $var, $http) = $self->session->quick(qw(config env var http));
-    if ($conf->get("sslEnabled") && $self->get("encryptPage") && $env->get("HTTPS") ne "on" && !$env->get("SSLPROXY")) {
+    if ($conf->get("sslEnabled") && $self->get("encryptPage") && ! $env->sslRequest) {
         # getUrl already changes url to https if 'encryptPage'
         $http->setRedirect($self->getUrl);
         $http->sendHeader;
         return "chunked";
 	}
     elsif ($var->isAdminOn && $self->get("state") =~ /^trash/) { # show em trash
-		$http->setRedirect($self->getUrl("func=manageTrash"));
+        my $queryFrag = "func=manageTrash";
+        if ($self->session->form->process('revision')) {
+            $queryFrag .= ";revision=".$self->session->form->process('revision');
+        }
+		$http->setRedirect($self->getUrl($queryFrag));
         $http->sendHeader;
 		return "chunked";
 	} 
     elsif ($var->isAdminOn && $self->get("state") =~ /^clipboard/) { # show em clipboard
-		$http->setRedirect($self->getUrl("func=manageClipboard"));
+        my $queryFrag = "func=manageClipboard";
+        if ($self->session->form->process('revision')) {
+            $queryFrag .= ";revision=".$self->session->form->process('revision');
+        }
+		$http->setRedirect($self->getUrl($queryFrag));
         $http->sendHeader;
 		return "chunked";
 	} 
@@ -330,6 +346,26 @@ sub checkView {
 
 #-------------------------------------------------------------------
 
+=head2 cloneFromDb ( )
+
+Fetches a new fresh clone of this object from the database.  Often used after
+calling commit on version tags.
+
+Returns the new Asset object.
+
+=cut
+
+sub cloneFromDb {
+	my $self = shift;
+    return WebGUI::Asset->new($self->session,
+        $self->getId,
+        $self->get('className'),
+        $self->get('revisionDate')
+    );
+}
+
+#-------------------------------------------------------------------
+
 =head2 definition ( session, [ definition ] )
 
 Basic definition of an Asset. Properties, default values. Returns an array reference containing tableName,className,properties
@@ -446,6 +482,20 @@ sub definition {
                         fieldType=>'codearea',
                         defaultValue=>undef,
                         customDrawMethod => 'drawExtraHeadTags',
+                        filter  => 'packExtraHeadTags',
+                    },
+                    extraHeadTagsPacked => {
+                        fieldType       => 'hidden',
+                        defaultValue    => undef,
+                        noFormPost      => 1,
+                    },
+                    usePackedHeadTags => {
+                        tab             => "meta",
+                        label           => $i18n->get('usePackedHeadTags label'),
+                        hoverHelp       => $i18n->get('usePackedHeadTags description'),
+                        uiLevel         => 7,
+                        fieldType       => 'yesNo',
+                        defaultValue    => 0,
                     },
 				    isPackage=>{
 					    label=>$i18n->get("make package"),
@@ -537,11 +587,12 @@ Completely remove an asset from existence.
 
 sub DESTROY {
 	my $self = shift;
-	# something bad happens when the following is enabled, not sure why
-	# must check this out later
-	#$self->{_parent}->DESTROY if (exists $self->{_parent});
-	$self->{_firstChild}->DESTROY if (exists $self->{_firstChild});
-	$self->{_lastChild}->DESTROY if (exists $self->{_lastChild});
+
+	# Let the parent be garbage collected if no one else is referencing
+	# him.  firstChild and lastChild are weak references, so no need to
+	# worry about them here.
+	delete $self->{_parent};
+
 	$self = undef;
 }
 
@@ -630,12 +681,12 @@ sub fixUrl {
 		$url =~ s/(.*)\..*/$1/;
 		$url .= '/'.$self->getValue("menuTitle");
 	}
-	$url = $self->session->url->urlize($url);
 
     # if we're inheriting the URL from our parent, set that appropriately
     if($self->get('inheritUrlFromParent')) {
        $url = $self->fixUrlFromParent($url); 
     }
+	$url = $self->session->url->urlize($url);
 
 	# fix urls used by uploads and extras
 	# and those beginning with http
@@ -670,7 +721,6 @@ sub fixUrl {
 	# add automatic extension if we're supposed to
 	if ($self->session->setting->get("urlExtension") ne "" #don't add an extension if one isn't set
 	&&  !($url =~ /\./)                           # don't add an extension of the url already contains a dot
-    &&  !$self->get("url")                        # Only add it if this is a new asset.
     &&  $url ne lc($self->getId)                  # but don't assign it the original time
 	) {
 		$url .= ".".$self->session->setting->get("urlExtension");
@@ -718,15 +768,10 @@ sub fixUrlFromParent {
     my @parts = split(m{/}, $url);
 
     # don't do anything unless we need to
-    if("/$url" ne $self->getParent->getUrl . '/' . $parts[-1]) {
-        $url = $self->getParent->getUrl . '/' . $parts[-1];
+    if($url ne $self->getParent->get('url') . '/' . $parts[-1]) {
+        $url = $self->getParent->get('url') . '/' . $parts[-1];
     }
 
-    ##Note we do not need to call fixUrl on the url argument.  Here's the reasoning why.
-    ##If a URL has not been set to updated at the same time that inheritUrlFromParent is
-    ##called, then it has already been "fixed".
-    ##On the other hand, if it has, the sideEffect nature of this method guarantees that
-    ##the URL was "fixed" before it was called.
     return $url;
 }
 
@@ -781,6 +826,26 @@ sub getAdminConsole {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 getCache ( )
+
+Returns a cache object specific to this asset, and whether or not the request is in SSL mode.
+
+=cut
+
+sub getCache {
+	my $self     = shift;
+    my $session  = $self->session;
+    my $cacheKey = "view_".$self->getId;
+    if ($session->env->sslRequest) {
+        $cacheKey .= '_ssl';
+    }
+    my $cache = WebGUI::Cache->new($session, $cacheKey);
+    return $cache;
+}
+
+
 #-------------------------------------------------------------------
 
 =head2 getContainer ( )
@@ -877,11 +942,17 @@ sub getEditForm {
 			$ac->addSubmenuItem($self->getUrl("func=edit;revision=".$version), $interval." ".$units." ".$ago);
 		}
 	}
-	if ($session->form->process("proceed")) {
+	if (my $proceed = $session->form->process("proceed")) {
 		$tabform->hidden({
 			name=>"proceed",
-			value=>$self->session->form->process("proceed")
-			});
+			value=>$proceed,
+        });
+        if (my $returnUrl = $session->form->process('returnUrl')) {
+            $tabform->hidden({
+                name=>"returnUrl",
+                value=>$returnUrl,
+            });
+        }
 	}
 	
 	# create tabs
@@ -932,7 +1003,7 @@ sub getEditForm {
 			label       => $i18n->get('keywords'),
 			hoverHelp   => $i18n->get('keywords help'),
 			value       => $self->get('keywords'),
-			fieldType	=> 'text',
+			fieldType	=> 'keywords',
 			tab			=> 'meta',
 		}
 	);
@@ -1025,7 +1096,7 @@ sub getEditForm {
 		}
 	}
 
-	# send back the rendered form
+	# send back the object
 	return $tabform;
 }
 
@@ -1087,7 +1158,10 @@ Returns the extraHeadTags stored in the asset.  Called in $self->session->style-
 
 sub getExtraHeadTags {
 	my $self = shift;
-	return $self->get("extraHeadTags");
+	return $self->get('usePackedHeadTags') 
+            ? $self->get('extraHeadTagsPacked')
+            : $self->get("extraHeadTags")
+            ;
 }
 
 
@@ -1148,18 +1222,50 @@ sub getImportNode {
 
 #-------------------------------------------------------------------
 
-=head2 getIsa ( $session )
+=head2 getIsa ( $session, [ $offset ] )
 
-A class method to return an iterator for getting all Assets by class (and all sub-classes)
-as Asset objects, one at a time.  When the end of the assets is reached, then the iterator
-will close the database handle that it uses and return undef.
+A class method to return an iterator for getting all committed Assets by
+class (and all sub-classes) as Asset objects, one at a time.  When the end
+of the assets is reached, then the iterator will close the database handle
+that it uses and return undef.
+
+Assets are processed in order by revisionDate.  If the iterator cannot
+instanciate an asset, it will not return undef.  Instead, it will throw
+an exception.  This allows the error condition to be distinguished from the
+end of the set of assets.
 
 It should be used like this:
 
-my $productIterator = WebGUI::Asset::Product->getIsa($session);
-while (my $product = $productIterator->()) {
-  ##Do something useful with $product
-}
+    my $productIterator = WebGUI::Asset::Product->getIsa($session);
+    ASSET: while (1) {
+        my $product = eval { $productIterator->() };
+        if (my $e = Exception::Class->caught()) {
+            $session->log->error($@);
+            next ASSET;
+        }
+        last ASSET unless $product;
+        ##Do something useful with $product
+    }
+
+In upgrade scripts, the eval and exception handling are best left off, because it is a good time
+to make the user aware that they have broken assets in their database.
+
+=head3 $session
+
+A reference to a WebGUI::Session object.
+
+=head3 $offset
+
+An offset, from the beginning of the results returned from the query, to really begin
+returning results.  This allows very large sets of results to be handled in chunks.
+
+=head3 $options
+
+A hashref of options to change how getIsa works.
+
+=head4 returnAll
+
+If set to true, then all assets will be returned, regardless of status and state.
 
 =cut
 
@@ -1167,24 +1273,54 @@ sub getIsa {
     my $class    = shift;
     my $session  = shift;
     my $offset   = shift;
+    my $options  = shift;
     my $def = $class->definition($session);
     my $tableName = $def->[0]->{tableName};
-    my $sql = "select distinct(assetId) from $tableName";
-    if (defined $offset) {
-        $sql .= ' LIMIT '. $offset . ',1234567890';
+    #Strategy, generate the correct set of assetIds
+    my $sql = "select assetId from assetData as ad ";
+    if ($tableName ne 'assetData') {
+        $sql .= "join `$tableName` using (assetId, revisionDate) ";
     }
-    my $sth = $session->db->read($sql);
+    $sql .= 'WHERE ';
+    if (! $options->{returnAll}) {
+        $sql .= q{(status='approved' OR status='archived') AND };
+    }
+    $sql .= q{revisionDate = (SELECT MAX(revisionDate) FROM assetData AS a WHERE a.assetId = ad.assetId) order by revisionDate };
+    if (defined $offset) {
+        $sql .= 'LIMIT '. $offset . ',1234567890 ';
+    }
+    my $sth    = $session->db->read($sql);
     return sub {
-        my ($assetId) = $sth->array;
+        my ($assetId, $revisionDate) = $sth->array;
         if (!$assetId) {
             $sth->finish;
             return undef;
         }
-        return WebGUI::Asset->newPending($session, $assetId);
+        my $asset = eval { WebGUI::Asset->newPending($session, $assetId); };
+        if (!$asset) {
+            warn "got an error\n";
+            WebGUI::Error::ObjectNotFound->throw(id => $assetId);
+        }
+        return $asset;
     };
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 getManagerUrl ( )
+
+Returns the URL for the asset manager.
+
+=cut
+
+sub getManagerUrl {
+	my $self = shift;
+	return $self->getUrl( 'op=assetManager' );
+}
+
+
+
 #-------------------------------------------------------------------
 
 =head2 getMedia ( session )
@@ -1255,6 +1391,32 @@ sub getNotFound {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 getPrototypeList ( )
+
+Returns an array of all assets that the user can view and edit that are prototypes.
+
+=cut
+
+sub getPrototypeList {
+    my $self    = shift;
+    my $session = $self->session;
+    my $db      = $session->db;
+    my @prototypeIds = $db->buildArray("select distinct assetId from assetData where isPrototype=1");
+    my $userUiLevel = $session->user->profileField('uiLevel');
+    my @assets;
+    ID: foreach my $id (@prototypeIds) {
+        my $asset = WebGUI::Asset->newByDynamicClass($session, $id);
+        next ID unless defined $asset;
+        next ID unless $asset->get('isPrototype');
+        next ID unless ($asset->get('status') eq 'approved' || $asset->get('tagId') eq $session->scratch->get("versionTag"));
+        push @assets, $asset;
+    }
+    return \@assets;
+
+}
+
 #-------------------------------------------------------------------
 
 =head2 getRoot ( session )
@@ -1274,6 +1436,21 @@ sub getRoot {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 getSearchUrl ( )
+
+Returns the URL for the search screen of the asset manager.
+
+=cut
+
+sub getSearchUrl {
+	my $self = shift;
+	return $self->getUrl( 'op=assetManager;method=search' );
+}
+
+
+
 #-------------------------------------------------------------------
 
 =head2 getSeparator
@@ -1433,7 +1610,7 @@ sub getToolbar {
     if ($userUiLevel >= $uiLevels->{"manage"}) {
         $output
             .= '<li class="yuimenuitem"><a class="yuimenuitemlabel" href="'
-            . $self->getUrl("op=assetManager") . '">' . $i18n->get("manage") . '</a></li>';
+            . $self->getManagerUrl . '">' . $i18n->get("manage") . '</a></li>';
     }
     $output .= '</ul></div></div>' . $toolbar . '</div>';
     $self->{_toolbar} = $output;
@@ -1588,10 +1765,10 @@ Loads an asset module if it's not already in memory. This is a class method. Ret
 
 sub loadModule {
     my ($class, $session, $className) = @_;
-    if ($className !~ /^WebGUI::Asset(?:$|::)/ ) {
+    if ($className !~ /^WebGUI::Asset(?:::\w+)*$/ ) {
         return undef;
     }
-    (my $module = $className . '.pm') =~ s{::|'}{/}g;
+    (my $module = $className . '.pm') =~ s{::}{/}g;
     if (eval { require $module; 1 }) {
         return $className;
     }
@@ -1690,6 +1867,13 @@ sub new {
     if (defined $properties) {
         my $object = { _session=>$session, _properties => $properties };
         bless $object, $class;
+        foreach my $definition (@{ $object->definition($session) }) {
+            foreach my $property (keys %{ $definition->{properties} }) {
+                if ($definition->{properties}->{$property}->{serialize} && $object->{_properties}->{$property} ne '') {
+                    $object->{_properties}->{$property} = JSON->new->canonical->decode($object->{_properties}->{$property});
+                }
+            }
+        }
         return $object;
     }	
     $session->errorHandler->error("Something went wrong trying to instanciate a '$className' with assetId '$assetId', but I don't know what!");
@@ -2000,6 +2184,29 @@ OUTPUT
 
 #-------------------------------------------------------------------
 
+=head2 packExtraHeadTags ( unpacked )
+
+Pack the extra head tags. Return the unpacked head tags (as per
+filter guidelines).
+
+=cut
+
+sub packExtraHeadTags {
+    my ( $self, $unpacked ) = @_;
+    return $unpacked if !$unpacked;
+    my $packed  = $unpacked;
+    HTML::Packer::minify( \$packed, {
+        remove_comments     => 1,
+        remove_newlines     => 1,
+        do_javascript       => "shrink",
+        do_stylesheet       => "minify",
+    } );
+    $self->update({ extraHeadTagsPacked => $packed });
+    return $unpacked;
+}
+
+#-------------------------------------------------------------------
+
 =head2 prepareView ( )
 
 Executes what is necessary to make the view() method work with content chunking.
@@ -2127,17 +2334,18 @@ sub processTemplate {
     my $var = shift;
     my $templateId = shift;
     my $template = shift;
+    my $session  = $self->session;
 
     # Sanity checks
     if (ref $var ne "HASH") {
-        $self->session->errorHandler->error("First argument to processTemplate() should be a hash reference.");
+        $session->errorHandler->error("First argument to processTemplate() should be a hash reference.");
         return "Error: Can't process template for asset ".$self->getId." of type ".$self->get("className");
     }
-
-    $template = WebGUI::Asset->new($self->session, $templateId,"WebGUI::Asset::Template") unless (defined $template);
+    $template = WebGUI::Asset->new($session, $templateId,"WebGUI::Asset::Template") unless (defined $template);
     if (defined $template) {
         $var = { %{ $var }, %{ $self->getMetaDataAsTemplateVariables } };
-        $var->{'controls'} = $self->getToolbar if $self->session->var->isAdminOn;
+        $var->{'controls'}   = $self->getToolbar if $session->var->isAdminOn;
+        $var->{'assetIdHex'} = $session->id->toHex($self->getId);
         my %vars = (
             %{$self->{_properties}},
             'title'     => $self->getTitle,
@@ -2147,7 +2355,7 @@ sub processTemplate {
         return $template->process(\%vars);
     }
     else {
-        $self->session->errorHandler->error("Can't instantiate template $templateId for asset ".$self->getId);
+        $session->errorHandler->error("Can't instantiate template $templateId for asset ".$self->getId);
         my $i18n = WebGUI::International->new($self->session, 'Asset');
         return $i18n->get('Error: Cannot instantiate template').' '.$templateId;
     }
@@ -2359,9 +2567,9 @@ sub update {
 #			next unless (exists $properties->{$property} || exists $definition->{properties}{$property}{defaultValue});
             # skip a property unless it was specified to be set by the properties field
 			next unless (exists $properties->{$property});
-
+            my $propertyDefinition = $definition->{properties}{$property};
             # skip a property if it has the display only flag set
-            next if ($definition->{properties}{$property}{displayOnly});
+            next if ($propertyDefinition->{displayOnly});
 
             # skip properties that aren't yet in the table
             if (!exists $tableFields{$property}) {
@@ -2377,22 +2585,29 @@ sub update {
             }
 
             # apply filter logic on a property to validate or fix it's value
-			if (exists $definition->{properties}{$property}{filter}) {
-				my $filter = $definition->{properties}{$property}{filter};
-				$value = $self->$filter($value, $property);
-			}
+            if (exists $propertyDefinition->{filter}) {
+                my $filter = $propertyDefinition->{filter};
+                $value = $self->$filter($value, $property);
+            }
 
-            # use the default value because default and update were both undef
-            if ($value eq "" && exists $definition->{properties}{$property}{defaultValue}) {
-                $value = $definition->{properties}{$property}{defaultValue};
+            # if the value is undefined, use the default if possible
+            # unless allowEmpty has been set, do this for empty strings as well
+            if ( ( !defined $value || ( $value eq q{} && ! $propertyDefinition->{allowEmpty} ) )
+                 && exists $propertyDefinition->{defaultValue} ) {
+                $value = $propertyDefinition->{defaultValue};
                 if (ref($value) eq 'ARRAY') {
                     $value = $value->[0];
                 }
             }
 
             # set the property
+            if ($propertyDefinition->{serialize}) {
+                $setPairs{$property} = JSON->new->canonical->encode($value);
+            }
+            else {
+                $setPairs{$property} = $value;
+            }
 			$self->{_properties}{$property} = $value;
-			$setPairs{$property} = $value;
 		}
 
         # if there's anything to update, then do so
@@ -2590,7 +2805,7 @@ sub www_changeUrlConfirm {
 	}
 
 	if ($self->session->form->param("proceed") eq "manageAssets") {
-		$self->session->http->setRedirect($self->getUrl('op=assetManager'));
+		$self->session->http->setRedirect($self->getManagerUrl);
 	} else {
 		$self->session->http->setRedirect($self->getUrl());
 	}
@@ -2617,18 +2832,20 @@ sub www_edit {
 
 =head2 www_editSave ( )
 
-Saves and updates history. If canEdit, returns www_manageAssets() if a new Asset is created, otherwise returns www_view().  Will return an insufficient Privilege if canEdit returns False.
+Saves and updates history. If canEdit, returns www_manageAssets() if a new Asset is created, otherwise returns www_view().  Will return an insufficient Privilege if canEdit returns False, or if the submitted form does not pass the C<$session->form->validToken> check.
 
 NOTE: Don't try to override or overload this method. It won't work. What you are looking for is processPropertiesFromFormPost().
 
 =cut
 
 sub www_editSave {
-    my $self = shift;
+    my $self    = shift;
+    my $session = $self->session;
+
     ##If this is a new asset (www_add), the parent may be locked.  We should still be able to add a new asset.
-    my $isNewAsset = $self->session->form->process("assetId") eq "new" ? 1 : 0;
-    return $self->session->privilege->locked() if (!$self->canEditIfLocked and !$isNewAsset);
-    return $self->session->privilege->insufficient() unless $self->canEdit;
+    my $isNewAsset = $session->form->process("assetId") eq "new" ? 1 : 0;
+    return $session->privilege->locked() if (!$self->canEditIfLocked and !$isNewAsset);
+    return $session->privilege->insufficient() unless $self->canEdit && $session->form->validToken;
     if ($self->session->config("maximumAssets")) {
         my ($count) = $self->session->db->quickArray("select count(*) from asset");
         my $i18n = WebGUI::International->new($self->session, "Asset");
@@ -2636,7 +2853,7 @@ sub www_editSave {
     }
     my $object;
     if ($isNewAsset) {
-        $object = $self->addChild({className=>$self->session->form->process("class","className")});	
+        $object = $self->addChild({className=>$session->form->process("class","className")});	
         return $self->www_view unless defined $object;
         $object->{_parent} = $self;
         $object->{_properties}{url} = undef;
@@ -2646,15 +2863,15 @@ sub www_editSave {
             $object = $self->addRevision;
         } 
         else {
-            return $self->session->asset($self->getContainer)->www_view;
+            return $session->asset($self->getContainer)->www_view;
         }
     }
 
     # Process properties from form post
     my $errors = $object->processPropertiesFromFormPost;
     if (ref $errors eq 'ARRAY') {
-        $self->session->stow->set('editFormErrors', $errors);
-        if ($self->session->form->process('assetId') eq 'new') {
+        $session->stow->set('editFormErrors', $errors);
+        if ($session->form->process('assetId') eq 'new') {
             $object->purge;
             return $self->www_add();
         } else {
@@ -2662,44 +2879,59 @@ sub www_editSave {
             return $self->www_edit();
         }
     }
-    
+
     $object->updateHistory("edited");
 
     # we handle auto commit assets here in case they didn't handle it themselves
-    if ($object->getAutoCommitWorkflowId && $self->hasBeenCommitted) {
+    if ($object->getAutoCommitWorkflowId) {
         $object->requestAutoCommit;
+        #Since the version tag makes new objects, fetch a fresh one here.
+        $object = $object->cloneFromDb;
     }
     # else, try to to auto commit
-    elsif(WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session, {
-        override        => scalar $self->session->form->process('saveAndCommit'),
-        allowComments   => 1,
-        returnUrl       => $self->getUrl,
-    }) eq 'redirect') {
-        return undef;
+    else {
+        my $commitStatus = WebGUI::VersionTag->autoCommitWorkingIfEnabled($session, {
+            override        => scalar $session->form->process('saveAndCommit'),
+            allowComments   => 1,
+            returnUrl       => $self->getUrl,
+        });
+        if ($commitStatus eq 'redirect') {
+            ##Redirect set by tag.  Return nothing to send the user over to the redirect.
+            return undef;
+        }
+        elsif ($commitStatus eq 'commit') {
+            ##Commit was successful.  Update the local object cache so that it will no longer
+            ##register as locked.
+            $object = $object->cloneFromDb;
+        }
     }
-
     # Handle "saveAndReturn" button
-    if ( $self->session->form->process( "saveAndReturn" ) ne "" ) {
+    if ( $session->form->process( "saveAndReturn" ) ne "" ) {
         return $object->www_edit;
     }
 
     # Handle "proceed" form parameter
-    if ($self->session->form->process("proceed") eq "manageAssets") {
-        $self->session->asset($object->getParent);
-        return $self->session->asset->www_manageAssets;
+    my $proceed = $session->form->process('proceed');
+    if ($proceed eq "manageAssets") {
+        $session->asset($object->getParent);
+        return $session->asset->www_manageAssets;
     }
-    elsif ($self->session->form->process("proceed") eq "viewParent") {
-        $self->session->asset($object->getParent);
-        return $self->session->asset->www_view;
+    elsif ($proceed eq "viewParent") {
+        $session->asset($object->getParent);
+        return $session->asset->www_view;
     }
-    elsif ($self->session->form->process("proceed") ne "") {
-        my $method = "www_".$self->session->form->process("proceed");
-        $self->session->asset($object);
-        return $self->session->asset->$method();
+    elsif ($proceed eq "goBackToPage" && $session->form->process('returnUrl')) {
+        $session->http->setRedirect($session->form->process("returnUrl"));
+        return undef;
+    }
+    elsif ($proceed ne "") {
+        my $method = "www_".$session->form->process("proceed");
+        $session->asset($object);
+        return $session->asset->$method();
     }
 
-    $self->session->asset($object->getContainer);
-    return $self->session->asset->www_view;
+    $session->asset($object->getContainer);
+    return $session->asset->www_view;
 }
 
 
@@ -2707,14 +2939,28 @@ sub www_editSave {
 
 =head2 www_manageAssets ( )
 
-Redirect to the asset manager content handler (for backwards 
-compatibility)
+Redirect to the asset manager content handler (for backwards compatibility)
 
 =cut
 
 sub www_manageAssets {
     my $self = shift;
-    $self->session->http->setRedirect( $self->getUrl( 'op=assetManager' ) );
+    $self->session->http->setRedirect( $self->getManagerUrl );
+    return "redirect";
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_searchAssets ( )
+
+Redirect to the asset manager content handler (for backwards 
+compatibility)
+
+=cut
+
+sub www_searchAssets {
+    my $self = shift;
+    $self->session->http->setRedirect( $self->getSearchUrl );
     return "redirect";
 }
 
