@@ -245,11 +245,11 @@ By specifying this method, you activate this feature.
 
 sub getAutoCommitWorkflowId {
     my $self = shift;
-
+    
     if($self->session->form->process("assetId") eq "new"){
         return $self->getParent->get("submissionApprovalWorkflowId");
     }
-    return undef
+    return undef;
 }
 
 #-------------------------------------------------------------------
@@ -313,7 +313,8 @@ sub getEditForm {
             label       =>$i18n->get('maintainer label'),
             hoverHelp   =>$i18n->get('maintainer description'),
             );
-    }else{
+    }
+    else{
         my $userId;
         if ($func eq "add"){
             $userId = $session->user->userId;
@@ -364,7 +365,7 @@ sub getEditForm {
             $attribute->{subtext}   = $attribute->{description};
             $attribute->{name}      = 'attribute_'.$attribute->{attributeId}; 
             unless($session->form->process('func') eq 'add'){
-                $attribute->{value} = $db->quickScalar("select value from MatrixListing_attribute
+                $attribute->{value} = $db->quickScalar("select value from MatrixListing_attribute 
                     where attributeId = ? and matrixId = ? and matrixListingId = ?",
                     [$attribute->{attributeId},$matrixId,$self->getId]);
             }
@@ -521,7 +522,7 @@ sub processPropertiesFromFormPost {
                     my $newHeight = $height * ($maxWidth / $width);
                     if ($newHeight > $maxHeight){
                         # Heigth requires more resizing so use maxHeight
-                        $storage->resize($file, 0, $maxHeight);                    
+                        $storage->resize($file, 0, $maxHeight);
                     }
                     else{
                         $storage->resize($file, $maxWidth);
@@ -616,7 +617,7 @@ sub setRatings {
 
 #-------------------------------------------------------------------
 
-=head2 view ( hasRated )
+=head2 updateScore ( )
 
 Updates the score of a MatrixListing. 
 
@@ -643,7 +644,7 @@ A boolean indicating if the user has rated this listing.
 
 =head3 emailSent 
 
-A boolean indicating if an email to the listing maintianer was sent.
+A boolean indicating if an email to the listing maintainer was sent.
 
 =cut
 
@@ -709,8 +710,6 @@ sub view {
     {type =>'text/javascript'});
     $self->session->style->setScript($self->session->url->extras('yui/build/button/button-min.js'),
     {type =>'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('wobject/Matrix/matrixListing.js'), {type =>
-    'text/javascript'});
     $self->session->style->setLink($self->session->url->extras('yui/build/datatable/assets/skins/sam/datatable.css'),
         {type =>'text/css', rel=>'stylesheet'});
 
