@@ -23,22 +23,46 @@ use WebGUI::Utility;
 our @ISA = qw(WebGUI::Asset::Post);
 
 #-------------------------------------------------------------------
+
+=head2 addRevision 
+
+Extend the base method to handle creating a subscription group for this Thread.
+
+=cut
+
 sub addRevision {
-        my $self = shift;
-        my $newSelf = $self->SUPER::addRevision(@_);
-	$newSelf->createSubscriptionGroup;
-        return $newSelf;
+    my $self = shift;
+    my $newSelf = $self->SUPER::addRevision(@_);
+    $newSelf->createSubscriptionGroup;
+    return $newSelf;
 }
 
 #-------------------------------------------------------------------
+
+=head2 archive 
+
+Archives all posts under this thread.  Update the thread count in the parent CS.
+
+=cut
+
 sub archive {
 	my $self = shift;
 	foreach my $post (@{$self->getPosts}) {
 		$post->setStatusArchived;
 	}
+    my $cs = $self->getParent;
+    $cs->incrementThreads($cs->get("lastPostId"), $cs->get("lastPostDate"));
 }
 
 #-------------------------------------------------------------------
+
+=head2 canAdd ($session)
+
+A class method.  Returns true if the current user in in the group canStartThreadGroupId
+in the default asset, which better be a Collaboration System.
+
+=cut
+
 sub canAdd {
     my $class   = shift;
     my $session = shift;
@@ -46,6 +70,18 @@ sub canAdd {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canReply ( [$userId ] )
+
+Determines if a user can reply to a post.  True if the thread is not locked, the parent CS allows replies, and the user canPost according to the parent CS.
+
+=head3 $userId
+
+The ID of the user to check for permissions.  If left out, then the current session
+user is checked instead.
+
+=cut
+
 sub canReply {
     my $self        = shift;
     my $userId      = shift || $self->session->user->userId;
@@ -56,6 +92,19 @@ sub canReply {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canSubscribe  ( [$userId ] )
+
+Determines if a user can subscribe to this Thread.  Returns true if the user is
+not visitor, and the user can view this Thread.
+
+=head3 $userId
+
+The ID of the user to check for permissions.  If left out, then the current session
+user is checked instead.
+
+=cut
+
 sub canSubscribe {
     my $self    = shift;
     my $userId  = shift || $self->session->user->userId;
@@ -64,6 +113,12 @@ sub canSubscribe {
 
 #-------------------------------------------------------------------
 
+=head2 commit 
+
+Extends the base method to increment the number of threads in the parent CS.
+
+=cut
+
 sub commit {
 	my $self = shift;
 	$self->SUPER::commit;
@@ -76,6 +131,13 @@ sub commit {
 # Override duplicateBranch here so that new posts get their threadId set correctly.
 # Buggo: should this be part of the addRevision override instead?
 
+=head2 duplicateBranch 
+
+Extends the base method to handle updating the threadId of the newly copied posts, and
+to update the Thread with the lastPost information.
+
+=cut
+
 sub duplicateBranch {
 	my $self = shift;
 	my $newAsset = $self->SUPER::duplicateBranch(@_);
@@ -89,6 +151,14 @@ sub duplicateBranch {
 }
 
 #-------------------------------------------------------------------
+
+=head2 createSubscriptionGroup 
+
+Creates a group to hold subscribers to this thread, if the Thread does not
+already have one.
+
+=cut
+
 sub createSubscriptionGroup {
 	my $self = shift;
 	return undef if ($self->get("subscriptionGroupId"));
@@ -170,6 +240,13 @@ sub definition {
 
 #-------------------------------------------------------------------
 
+=head2 DESTROY 
+
+Extends the base DESTROY method to handle deleting the cached previous
+and next threads, and to delete the parent CS.
+
+=cut
+
 sub DESTROY {
 	my $self = shift;
 	return undef unless defined $self;
@@ -249,12 +326,46 @@ sub getArchiveUrl {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getAutoCommitWorkflowId 
+
+Override the base method to fetch the threadApprovalWorkflow from the parent CS.
+
+=cut
+
 sub getAutoCommitWorkflowId {
 	my $self = shift;
 	return $self->getThread->getParent->get("threadApprovalWorkflow");
 }
 
 #-------------------------------------------------------------------
+
+=head2 getDirectLinkUrl ( )
+
+Extend the base method from Post to remove the pagination query fragment
+
+=cut
+
+sub getDirectLinkUrl {
+	my $self = shift;
+    my $url = $self->SUPER::getDirectLinkUrl();
+    $url =~ s/\?pn=\d+//;
+    if ($url =~ m{;revision=\d+}) {
+        $url =~ s/;revision/?revision/;
+    }
+
+    return $url;
+}
+
+
+#-------------------------------------------------------------------
+
+=head2 getLastPost 
+
+Fetches the last post in this thread, otherwise, returns itself.
+
+=cut
+
 sub getLastPost {
 	my $self = shift;
 	my $lastPostId = $self->get("lastPostId");
@@ -383,6 +494,13 @@ sub getSubscribeUrl {
 
 
 #-------------------------------------------------------------------
+
+=head2 getThread 
+
+Overrides the base method to return itself, since it's a thread.
+
+=cut
+
 sub getThread {
 	return shift;
 }
@@ -578,6 +696,13 @@ sub prepareView {
 
 
 #-------------------------------------------------------------------
+
+=head2 postProcess 
+
+Extend the base method from Post to process the karmaScale.
+
+=cut
+
 sub postProcess {
 	my $self = shift;
 	if ($self->getParent->canEdit) {
@@ -589,6 +714,13 @@ sub postProcess {
 }
 
 #-------------------------------------------------------------------
+
+=head2 processPropertiesFromFormPost 
+
+Extend the base method to do captcha processing.
+
+=cut
+
 sub processPropertiesFromFormPost {
     my $self = shift;
 
@@ -675,6 +807,13 @@ sub setLastPost {
 }
 
 #-------------------------------------------------------------------
+
+=head2 normalizeLastPost 
+
+Looks up the last postId and date from the db and sets it.
+
+=cut
+
 sub normalizeLastPost {
 	my $self = shift;
 	# Hmm.  Is this right?
@@ -857,15 +996,17 @@ sub updateThreadRating {
     my $self        = shift;
     my $session     = $self->session;
 
-    my $calcRating  = 0; 
     my $postIds     = $self->getLineage(["descendants","self"], {
         includeOnlyClasses  => ["WebGUI::Asset::Post","WebGUI::Asset::Post::Thread"],
         includeArchived     => 1,
     });
 
-    $calcRating += $session->db->quickScalar(
-        "SELECT SUM(rating) FROM Post_rating WHERE assetId IN (".$session->db->quoteAndJoin($postIds).")"
-    );     
+    my $calcRating = 0;
+    if (scalar @{ $postIds }) {
+        $calcRating += $session->db->quickScalar(
+            "SELECT SUM(rating) FROM Post_rating WHERE assetId IN (".$session->db->quoteAndJoin($postIds).")"
+        );     
+    }
 
     $self->update({
         threadRating    => $calcRating
@@ -874,7 +1015,8 @@ sub updateThreadRating {
     my $parent = $self->getParent;
     if (defined $parent) {
         $parent->recalculateRating;
-    } else {
+    }
+    else {
         $self->session->errorHandler->error("Couldn't get parent for thread ".$self->getId);
     }    
 }
@@ -897,6 +1039,13 @@ sub validParent {
 }
 
 #-------------------------------------------------------------------
+
+=head2 view 
+
+Render this thread and its posts.
+
+=cut
+
 sub view {
     my $self = shift;
 	my $currentPost = shift || $self;
@@ -1032,7 +1181,7 @@ sub www_archive {
 
 #-------------------------------------------------------------------
 
-=head2 www_lock (  )
+=head2 www_lockThread (  )
 
 The web method to lock a thread.
 
@@ -1152,7 +1301,7 @@ sub www_unarchive {
 
 #-------------------------------------------------------------------
 
-=head2 www_unlock ( )
+=head2 www_unlockThread ( )
 
 The web method to unlock a thread.
 
@@ -1180,7 +1329,7 @@ sub www_unstick {
 
 #-------------------------------------------------------------------
 
-=head2 www_threadUnsubscribe ( )
+=head2 www_unsubscribe ( )
 
 The web method to unsubscribe from a thread.
 
