@@ -58,19 +58,30 @@ specified by the session variable.
 
 sub apply {
 	my $self    = shift;
-	my $userId  = shift || $self->session->user->userId;
+    my $session = $self->session;
+	my $userId  = shift || $session->user->userId;
 	my $groupId = $self->get('subscriptionGroup');
 
 	# Make user part of the right group and adjust the expiration date
-	my $group   = WebGUI::Group->new($self->session,$groupId);
-	$group->addUsers( [$userId], $self->getExpirationOffset );
+	my $group   = WebGUI::Group->new($session, $groupId);
+    my $user    = WebGUI::User->new($session, $userId);
+    if ($user->isInGroup($group->getId) && ! $self->isRecurring) {
+        my $expireDate = $group->userGroupExpireDate($userId);
+        $expireDate   += $self->getExpirationOffset;
+        $group->userGroupExpireDate($userId, $expireDate);
+    }
+    else {
+        $group->addUsers( [$userId], $self->getExpirationOffset );
+    }
 
 	# Add karma to the user's account
-	WebGUI::User->new($self->session,$userId)->karma($self->get('karma'), 'Subscription', 'Added for purchasing subscription '.$self->get('title'));
+    if ($session->setting->get('userKarma')) {
+        WebGUI::User->new($session,$userId)->karma($self->get('karma'), 'Subscription', 'Added for purchasing subscription '.$self->get('title'));
+    }
 
 	# Process the executeOnPurchase field
 	my $command = $self->get('executeOnSubscription');
-	WebGUI::Macro::process($self->session,\$command);
+	WebGUI::Macro::process($session,\$command);
 	system($command) if ($self->get('executeOnSubscription') ne "");
 }
 
@@ -79,8 +90,6 @@ sub apply {
 
 =head2 definition
 
-Adds templateId, thankYouMessage, and defaultPrice fields.
-
 =cut
 
 sub definition {
@@ -269,6 +278,27 @@ sub generateSubscriptionCodeBatch {
 
 #-------------------------------------------------------------------
 
+=head2 getAddToCartForm ( )
+
+Returns a form to add this Sku to the cart.  Used when this Sku is part of
+a shelf.  Override master class to add different form.
+
+=cut
+
+sub getAddToCartForm {
+    my $self    = shift;
+    my $session = $self->session;
+    my $i18n = WebGUI::International->new($session, 'Asset_Subscription');
+    return
+        WebGUI::Form::formHeader($session, {action => $self->getUrl})
+      . WebGUI::Form::hidden(    $session, {name => 'func', value => 'purchaseSubscription'})
+      . WebGUI::Form::submit(    $session, {value => $i18n->get('purchase button')})
+      . WebGUI::Form::formFooter($session)
+      ;
+}
+
+#-------------------------------------------------------------------
+
 =head2 getAdminConsoleWithSubmenu ( )
 
 Returns an admin console with management links added to the submenu.
@@ -505,7 +535,7 @@ sub redeemCode {
 
 =head2 view
 
-Displays the donation form.
+Displays the subscription form.
 
 =cut
 
@@ -933,7 +963,7 @@ sub www_listSubscriptionCodes {
 
 #-------------------------------------------------------------------
 
-=head2 wwww_purchaseSubscription
+=head2 www_purchaseSubscription
 
 Add this subscription to the cart.
 
