@@ -20,9 +20,9 @@ use WebGUI::Paginator;
 use WebGUI::Utility;
 use WebGUI::Asset::Wobject;
 use WebGUI::Workflow::Cron;
-use WebGUI::Asset::RSSCapable;
-use base 'WebGUI::Asset::RSSCapable';
-use base 'WebGUI::Asset::Wobject';
+use Class::C3;
+use base qw(WebGUI::AssetAspect::RssFeed WebGUI::Asset::Wobject);
+
 
 #-------------------------------------------------------------------
 sub _computePostCount {
@@ -63,39 +63,53 @@ sub _visitorCacheOk {
 }
 
 #-------------------------------------------------------------------
-# encode a string to include in xml (for RSS export)
-sub _xml_encode {
-	my $text = shift;
-        $text =~ s/&/&amp;/g;
-        $text =~ s/</&lt;/g;
-        $text =~ s/\]\]>/\]\]&gt;/g;
-        return $text;
-}
 
-#-------------------------------------------------------------------
+=head2 addChild 
+
+Extend the base method to allow only Threads as children.
+
+=cut
+
 sub addChild {
 	my $self = shift;
 	my $properties = shift;
 	my @other = @_;
-	if ($properties->{className} ne "WebGUI::Asset::Post::Thread"
-	    and $properties->{className} ne 'WebGUI::Asset::RSSFromParent') {
+	if ($properties->{className} ne "WebGUI::Asset::Post::Thread") {
 		$self->session->errorHandler->security("add a ".$properties->{className}." to a ".$self->get("className"));
 		return undef;
 	}
-	return $self->SUPER::addChild($properties, @other);
+	return $self->next::method($properties, @other);
 }
 
 
 #-------------------------------------------------------------------
+
+=head2 appendPostListTemplateVars ($var, $p)
+
+Takes a WebGUI::Paginator object that should be full of Posts, and appends template
+variables to the hash reference.
+
+=head3 $var
+
+A hash reference.  Template variables will be added to it.
+
+=head3 $p
+
+A reference to a WebGUI::Paginator object.
+
+=cut
+
 sub appendPostListTemplateVars {
-	my $self = shift;
-	my $var = shift;
-	my $p = shift;
-	my $page = $p->getPageData;
-	my $i = 0;
-    my ($icon, $datetime) = $self->session->quick(qw(icon datetime));
+	my $self    = shift;
+    my $session = $self->session;
+	my $var     = shift;
+	my $p       = shift;
+	my $page    = $p->getPageData;
+	my $i       = 0;
+    my ($icon, $datetime) = $session->quick(qw(icon datetime));
+    my $isVisitor         = $session->user->isVisitor;
 	foreach my $row (@$page) {
-		my $post = WebGUI::Asset->new($self->session,$row->{assetId}, $row->{className}, $row->{revisionDate});
+		my $post = WebGUI::Asset->new($session,$row->{assetId}, $row->{className}, $row->{revisionDate});
 		$post->{_parent} = $self; # caching parent for efficiency 
 		my $controls = $icon->delete('func=delete',$post->get("url"),"Delete") . $icon->edit('func=edit',$post->get("url"));
 		if ($self->get("sortBy") eq "lineage") {
@@ -116,10 +130,11 @@ sub appendPostListTemplateVars {
 			if ($self->get("displayLastReply")) {
 				my $lastPost = $post->getLastPost();
 				%lastReply = (
-					"lastReply.url"                 => $lastPost->getUrl.'#'.$lastPost->getId,
+					"lastReply.url"                 => $lastPost->getDirectLinkUrl,
                     "lastReply.title"               => $lastPost->get("title"),
                     "lastReply.user.isVisitor"      => $lastPost->get("ownerUserId") eq "1",
                     "lastReply.username"            => $lastPost->get("username"),
+                    "lastReply.hideProfileUrl"      => $lastPost->get("ownerUserId") eq "1" || $isVisitor,
                     "lastReply.userProfile.url"     => $lastPost->getPosterProfileUrl(),
                     "lastReply.dateSubmitted.human" => $datetime->epochToHuman($lastPost->get("creationDate"),"%z"),
                     "lastReply.timeSubmitted.human" => $datetime->epochToHuman($lastPost->get("creationDate"),"%Z"),
@@ -127,16 +142,10 @@ sub appendPostListTemplateVars {
 			}
 			$hasRead = $post->isMarkedRead;
 		}
-		my $url;
-		if ($post->get("status") eq "pending") {
-			$url = $post->getUrl("revision=".$post->get("revisionDate"))."#".$post->getId;
-		} else {
-			$url = $post->getUrl."#".$post->getId;
-		}
 		my %postVars = (
 			%{$post->get},
             "id"                    => $post->getId,
-            "url"                   => $url,
+            "url"                   => $post->getDirectLinkUrl,
 			rating_loop             => \@rating_loop,
 			"content"               => $post->formatContent,
             "status"                => $post->getStatus,
@@ -146,14 +155,15 @@ sub appendPostListTemplateVars {
             "dateUpdated.human"     => $datetime->epochToHuman($post->get("revisionDate"),"%z"),
             "timeSubmitted.human"   => $datetime->epochToHuman($post->get("creationDate"),"%Z"),
             "timeUpdated.human"     => $datetime->epochToHuman($post->get("revisionDate"),"%Z"),
+            "hideProfileUrl"        => $post->get('ownerUserId') eq '1' || $isVisitor,
             "userProfile.url"       => $post->getPosterProfileUrl,
             "user.isVisitor"        => $post->get("ownerUserId") eq "1",
         	"edit.url"              => $post->getEditUrl,
 			'controls'              => $controls,
-            "isSecond"              => (($i+1)%2==0),
-            "isThird"               => (($i+1)%3==0),
-            "isFourth"              => (($i+1)%4==0),
-            "isFifth"               => (($i+1)%5==0),
+            "isSecond"              => (($i+1) == 2),
+            "isThird"               => (($i+1) == 3),
+            "isFourth"              => (($i+1) == 4),
+            "isFifth"               => (($i+1) == 5),
 			"user.hasRead"          => $hasRead,
             "user.isPoster"         => $post->isPoster,
             "avatar.url"            => $post->getAvatarUrl,
@@ -163,13 +173,24 @@ sub appendPostListTemplateVars {
 		if ($row->{className} =~ m/^WebGUI::Asset::Post::Thread/) {
 			$postVars{'rating'} = $post->get('threadRating');
 		}
-                push(@{$var->{post_loop}}, \%postVars );
+        push(@{$var->{post_loop}}, \%postVars );
 		$i++;
 	}
 	$p->appendTemplateVars($var);
 }
 
 #-------------------------------------------------------------------
+
+=head2 appendTemplateLabels ($var)
+
+Appends a whole mess of internationalized labels for use in a template.
+
+=head3 $var
+
+A hash reference.  Template labels will be appended to it.
+
+=cut
+
 sub appendTemplateLabels {
 	my $self = shift;
 	my $var = shift;
@@ -248,6 +269,18 @@ sub appendTemplateLabels {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canEdit ( [ $userId ] )
+
+Extends the base method to include adding Threads to this CS.
+
+=head3 $userId
+
+A userId to check for edit permissions. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canEdit {
         my $self    = shift;
         my $userId  = shift     || $self->session->user->userId;
@@ -263,18 +296,43 @@ sub canEdit {
 			) && 
 			$self->canStartThread( $userId )
 		) || # account for new threads
-		$self->SUPER::canEdit( $userId )
+		$self->next::method( $userId )
 	);
 }
 
 #-------------------------------------------------------------------
+
+=head2 canModerate  ( [ $userId ] )
+
+Returns true if the user can edit this Collaboration System.
+
+=head3 $userId
+
+A userId to check for permission. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canModerate {
     my $self    = shift;
     my $userId  = shift     || $self->session->user->userId;
-    return $self->SUPER::canEdit( $userId );
+    return $self->WebGUI::Asset::canEdit( $userId );
 }
 
 #-------------------------------------------------------------------
+
+=head2 canPost ( [ $userId ] )
+
+Returns true if the user can post to the CS.  Checks that the CS is committed,
+that the user is in the group to post, or that the user can edit this CS.
+
+=head3 $userId
+
+A userId to check for edit permissions. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canPost {
     my $self    = shift;
     my $userId  = shift;
@@ -294,12 +352,25 @@ sub canPost {
     }
     # Users who can edit the collab can post
     else {
-        return $self->SUPER::canEdit( $userId );
+        return $self->WebGUI::Asset::canEdit( $userId );
     }
 }
 
 
 #-------------------------------------------------------------------
+
+=head2 canSubscribe  ( [ $userId ] )
+
+Returns true if the user can subscribe to the CS.  Checks that the user is registered
+and that they canView the Post.
+
+=head3 $userId
+
+A userId to check for edit permissions. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canSubscribe {
     my $self    = shift;
     my $userId  = shift;
@@ -312,6 +383,19 @@ sub canSubscribe {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canStartThread   ( [ $userId ] )
+
+Returns true if the user can start a thread in the CS.  Checks that the user is in the
+canStartThreadGroup or that they canEdit the CS.
+
+=head3 $userId
+
+A userId to check for edit permissions. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canStartThread {
     my $self    = shift;
     my $userId  = shift;
@@ -322,22 +406,43 @@ sub canStartThread {
                 ;
     return (
         $user->isInGroup($self->get("canStartThreadGroupId")) 
-        || $self->SUPER::canEdit( $userId )
+        || $self->WebGUI::Asset::canEdit( $userId )
     );
 }
 
 
 #-------------------------------------------------------------------
+
+=head2 canView ( [ $userId ] )
+
+Extends the base method to also allow users who canPost to the CS.
+
+=head3 $userId
+
+A userId to check for edit permissions. If $userId is false, then it checks
+the current session user.
+
+=cut
+
 sub canView {
 	my $self = shift;
         my $userId  = shift     || $self->session->user->userId;
-	return $self->SUPER::canView( $userId ) || $self->canPost( $userId );
+	return $self->next::method( $userId ) || $self->canPost( $userId );
 }
 
 #-------------------------------------------------------------------
+
+=head2 commit 
+
+Extend the base method to handle making a cron job for fetching mail for the CS.  The
+cron job is created even if the CS does not have email enabled.  The cron is disabled
+in that case.
+
+=cut
+
 sub commit {
     my $self = shift;
-    $self->SUPER::commit;
+    $self->next::method;
     my $cron = undef;
     if ($self->get("getMailCronId")) {
         $cron = WebGUI::Workflow::Cron->new($self->session, $self->get("getMailCronId"));
@@ -362,6 +467,13 @@ sub commit {
 }
 
 #-------------------------------------------------------------------
+
+=head2 createSubscriptionGroup 
+
+Creates a group to hold users who want to receive posts to this CS by email.
+
+=cut
+
 sub createSubscriptionGroup {
 	my $self = shift;
 	my $group = WebGUI::Group->new($self->session, "new");
@@ -604,6 +716,13 @@ sub definition {
 			label=>$i18n->get('filter code'),
 			hoverHelp=>$i18n->get('filter code description'),
 			},
+		replyFilterCode =>{
+			fieldType=>"filterContent",
+			defaultValue=>'most',
+			tab=>'security',
+			label=>$i18n->get('reply filter code'),
+			hoverHelp=>$i18n->get('reply filter code description'),
+			},
 		richEditor =>{
 			fieldType=>"selectRichEditor",
 			defaultValue=>"PBrichedit000000000002",
@@ -611,6 +730,13 @@ sub definition {
 			label=>$i18n->get('rich editor'),
 			hoverHelp=>$i18n->get('rich editor description'),
 			},
+		replyRichEditor =>{
+			fieldType=>"selectRichEditor",
+			defaultValue=>"PBrichedit000000000002",
+			tab=>'display',
+			label=>$i18n->get('reply rich editor'),
+			hoverHelp=>$i18n->get('reply rich editor description'),
+			},
 		attachmentsPerPost =>{
 			fieldType=>"integer",
 			defaultValue=>0,
@@ -799,14 +925,31 @@ sub definition {
                 className=>'WebGUI::Asset::Wobject::Collaboration',
                 properties=>\%properties,
 		});
-        return $class->SUPER::definition($session, $definition);
+        return $class->next::method($session, $definition);
 }
 
 #-------------------------------------------------------------------
+
+=head2 duplicate 
+
+Extend the base method to handle making a subscription group for the new CS.
+
+=cut
+
 sub duplicate {
 	my $self = shift;
-	my $newAsset = $self->SUPER::duplicate(@_);
+	my $newAsset = $self->next::method(@_);
 	$newAsset->createSubscriptionGroup;
+    my $i18n = WebGUI::International->new($self->session, "Asset_Collaboration");
+    my $newCron = WebGUI::Workflow::Cron->create($self->session, {
+            title=>$self->getTitle." ".$i18n->get("mail"),
+            minuteOfHour=>"*/".($self->get("getMailInterval")/60),
+            className=>(ref $self),
+            methodName=>"new",
+            parameters=>$self->getId,
+            workflowId=>"csworkflow000000000001"
+    });
+    $self->update({getMailCronId=>$newCron->getId});
 	return $newAsset;
 }
 
@@ -821,7 +964,7 @@ Add a tab for the mail interface.
 sub getEditTabs {
 	my $self = shift;
 	my $i18n = WebGUI::International->new($self->session,"Asset_Collaboration");
-	return ($self->SUPER::getEditTabs(), ['mail', $i18n->get('mail'), 9]);
+	return ($self->next::method, ['mail', $i18n->get('mail'), 9]);
 }
 
 #-------------------------------------------------------------------
@@ -838,7 +981,14 @@ sub getNewThreadUrl {
 }
 
 #-------------------------------------------------------------------
-sub getRssItems {
+
+=head2 getRssFeedItems 
+
+Returns an array ref of Posts for use in making the feeds for this CS.
+
+=cut
+
+sub getRssFeedItems {
 	my $self = shift;
 
 	# XXX copied and reformatted this query from www_viewRSS, but why is it constructed like this?
@@ -861,6 +1011,7 @@ SQL
 	my $datetime = $self->session->datetime;
 
     my @posts;
+    my $rssLimit = $self->get('itemsPerFeed');
     for my $postId (@postIds) {
 		my $post = WebGUI::Asset->new($self->session, $postId, 'WebGUI::Asset::Post::Thread');
 		my $postUrl = $siteUrl . $post->getUrl;
@@ -881,24 +1032,26 @@ SQL
             }
         }
         
-        push @posts, { 
+        push @posts, {
             author          => $post->get('username'),
 		    title           => $post->get('title'),
-		    'link'          => $postUrl, 
+		    'link'          => $postUrl,
             guid            => $postUrl,
 		    description     => $post->get('synopsis'),
             epochDate       => $post->get('creationDate'),
 		    pubDate         => $datetime->epochToMail($post->get('creationDate')),
-		    attachmentLoop  => $attachmentLoop, 
+		    attachmentLoop  => $attachmentLoop,
 			userDefined1 => $post->get("userDefined1"),
 			userDefined2 => $post->get("userDefined2"),
 			userDefined3 => $post->get("userDefined3"),
 			userDefined4 => $post->get("userDefined4"),
 			userDefined5 => $post->get("userDefined5"),
 		 };
+
+         last if $rssLimit <= scalar(@posts);
 	}
 
-    return @posts;
+    return \@posts;
 }
 
 #-------------------------------------------------------------------
@@ -944,6 +1097,10 @@ sub getSortBy {
     my $self = shift;
     my $scratchSortBy = $self->getId."_sortBy";
     my $sortBy = $self->session->scratch->get($scratchSortBy) || $self->getValue("sortBy");
+    # XXX: This should be fixed in an upgrade and in the definition, NOT HERE
+    if ( $sortBy eq "rating" ) {
+        $sortBy = "threadRating";
+    }
     return $sortBy;
 }
 
@@ -993,6 +1150,7 @@ sub getThreadsPaginator {
     my $sortOrder = $self->session->scratch->get($scratchSortOrder) || $self->get("sortOrder");
 	if ($sortBy ne $self->session->scratch->get($scratchSortBy) && $self->session->form->process("func") ne "editSave") {
 		$self->session->scratch->set($scratchSortBy,$self->session->form->process("sortBy"));
+        $self->session->scratch->set($scratchSortOrder, $sortOrder);
 	} elsif ($self->session->form->process("sortBy") && $self->session->form->process("func") ne "editSave") {
                 if ($sortOrder eq "asc") {
                         $sortOrder = "desc";
@@ -1008,6 +1166,7 @@ sub getThreadsPaginator {
         $sortBy = 'threadRating';
     } 
     $sortBy = join('.', map { $self->session->db->dbh->quote_identifier($_) } split(/\./, $sortBy));
+
 	my $sql = "
 		select 
 			asset.assetId,
@@ -1058,15 +1217,23 @@ sub getUnsubscribeUrl {
 
 
 #-------------------------------------------------------------------
+
+=head2 getViewTemplateVars 
+
+Returns a hash reference full of template variables that are used in
+several CS templates.
+
+=cut
+
 sub getViewTemplateVars {
 	my $self = shift;
 	my %var;
 	$var{'user.canPost'} = $self->canPost;
 	$var{'user.canStartThread'} = $self->canStartThread;
-        $var{"add.url"} = $self->getNewThreadUrl;
-        $var{"rss.url"} = $self->getRssUrl;
-        $var{'user.isModerator'} = $self->canModerate;
-        $var{'user.isVisitor'} = ($self->session->user->isVisitor);
+    $var{"add.url"} = $self->getNewThreadUrl;
+    $var{"rss.url"} = $self->getRssFeedUrl;
+    $var{'user.isModerator'} = $self->canModerate;
+    $var{'user.isVisitor'} = ($self->session->user->isVisitor);
 	$var{'user.isSubscribed'} = $self->isSubscribed;
 	$var{'sortby.title.url'} = $self->getSortByUrl("title");
 	$var{'sortby.username.url'} = $self->getSortByUrl("username");
@@ -1169,12 +1336,9 @@ See WebGUI::Asset::prepareView() for details.
 =cut
 
 sub prepareView {
-	my $self = shift;
-	$self->SUPER::prepareView();
-	my $template = WebGUI::Asset::Template->new($self->session, $self->get("collaborationTemplateId")) or die "no good: ".$self->get("collaborationTemplateId");
-    if ($self->get('rssCapableRssEnabled')) {
-        $self->session->style->setLink($self->getRssUrl,{ rel=>'alternate', type=>'application/rss+xml', title=>$self->get('title') . ' RSS' });
-    }
+    my $self = shift;
+    $self->next::method;
+    my $template = WebGUI::Asset::Template->new($self->session, $self->get("collaborationTemplateId"));
     if (!$template) {
         WebGUI::Error::ObjectNotFound::Template->throw(
             error      => qq{Template not found},
@@ -1182,16 +1346,25 @@ sub prepareView {
             assetId    => $self->getId,
         );
     }
-	$template->prepare($self->getMetaDataAsTemplateVariables);
-	$self->{_viewTemplate} = $template;
+    $template->prepare($self->getMetaDataAsTemplateVariables);
+    $self->{_viewTemplate} = $template;
 }
 
 
 #-------------------------------------------------------------------
+
+=head2 processPropertiesFromFormPost 
+
+Extend the base method to handle creating subscription groups, propagating
+group privileges to all descendants and clearing scratch variables for sort key
+and direction.
+
+=cut
+
 sub processPropertiesFromFormPost {
 	my $self = shift;
         my $updatePrivs = ($self->session->form->process("groupIdView") ne $self->get("groupIdView") || $self->session->form->process("groupIdEdit") ne $self->get("groupIdEdit"));
-	$self->SUPER::processPropertiesFromFormPost;
+	$self->next::method;
 	if ($self->get("subscriptionGroupId") eq "") {
 		$self->createSubscriptionGroup;
 	}
@@ -1209,6 +1382,13 @@ sub processPropertiesFromFormPost {
 
 
 #-------------------------------------------------------------------
+
+=head2 purge 
+
+Extend the base method to delete the subscription group and cron job for emails.
+
+=cut
+
 sub purge {
 	my $self = shift;
 	my $group = WebGUI::Group->new($self->session, $self->get("subscriptionGroupId"));
@@ -1219,14 +1399,14 @@ sub purge {
 		my $cron = WebGUI::Workflow::Cron->new($self->session, $self->get("getMailCronId"));
 		$cron->delete if defined $cron;
 	}
-	$self->SUPER::purge;
+	$self->next::method;
 }
 
 #-------------------------------------------------------------------
 
 =head2 purgeCache ( )
 
-See WebGUI::Asset::purgeCache() for details.
+Extend the base method to delete view and visitor caches.
 
 =cut
 
@@ -1234,7 +1414,7 @@ sub purgeCache {
 	my $self = shift;
 	WebGUI::Cache->new($self->session,"view_".$self->getId)->delete;
 	WebGUI::Cache->new($self->session,$self->_visitorCacheKey)->delete;
-	$self->SUPER::purgeCache;
+	$self->next::method;
 }
 
 #-------------------------------------------------------------------
@@ -1368,6 +1548,13 @@ sub unsubscribe {
 
 
 #-------------------------------------------------------------------
+
+=head2 view 
+
+Render the CS, and handle local caching.
+
+=cut
+
 sub view {
 	my $self = shift;
 	if ($self->_visitorCacheOk) {
@@ -1463,11 +1650,31 @@ sub www_unsubscribe {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_view 
+
+Extend the base method to handle the visitor cache timeout.
+
+=cut
+
 sub www_view {
 	my $self = shift;
 	my $disableCache = ($self->session->form->process("sortBy") ne "");
 	$self->session->http->setCacheControl($self->get("visitorCacheTimeout")) if ($self->session->user->isVisitor && !$disableCache);
-	return $self->SUPER::www_view(@_);
+	return $self->next::method(@_);
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_viewRSS ( )
+
+Deprecated. Use www_viewRss() instead.
+
+=cut
+
+sub www_viewRSS {
+	my $self = shift;
+	return $self->www_viewRss;
 }
 
 1;
