@@ -27,8 +27,11 @@ use WebGUI::Asset::Wobject;
 use WebGUI::Pluggable;
 use WebGUI::DateTime;
 use WebGUI::User;
+use WebGUI::Utility;
 use WebGUI::Group;
 use WebGUI::AssetCollateral::DataForm::Entry;
+use WebGUI::Form::SelectRichEditor;
+use WebGUI::Paginator;
 use JSON ();
 
 our @ISA = qw(WebGUI::Asset::Wobject);
@@ -50,29 +53,34 @@ These methods are available from this class:
 
 #-------------------------------------------------------------------
 sub _createForm {
-    my $self = shift;
-    my $data = shift;
-    my $value = shift;
+    my $self    = shift;
+    my $data    = shift;
+    my $value   = shift;
+    my $session = $self->session;
     # copy select entries
-    my %param = map { $_ => $data->{$_} } qw(name width extras vertical defaultValue options);
-    $param{value} = $value;
-    $param{size} = $param{width};
+    my %param      = map { $_ => $data->{$_} } qw(name width extras vertical defaultValue options);
+    $param{value}  = $value;
+    $param{size}   = $param{width};
     $param{height} = $data->{rows};
 
-    WebGUI::Macro::process($self->session, \( $param{defaultValue} ));
+    WebGUI::Macro::process($session, \( $param{defaultValue} ));
 
-    my $type = "\u$data->{type}";
+    my $type = ucfirst $data->{type};
     my $class = "WebGUI::Form::$type";
-    eval {
-        WebGUI::Pluggable::load("WebGUI::Form::$type");
-    } || return undef;
+    if (!  eval { WebGUI::Pluggable::load("WebGUI::Form::$type"); } ) {
+        $session->log->error( "Unable to load form control - $type" );
+        return undef;
+    }
     if ($type eq "Checkbox") {
         $param{defaultValue} = ($param{defaultValue} =~ /checked/i);
     }
     elsif ( $class->isa('WebGUI::Form::List') ) {
         delete $param{size};
     }
-    return $class->new($self->session, \%param);
+    elsif ( $type eq 'HTMLArea' && $data->{htmlAreaRichEditor} ne '**Use_Default_Editor**') {
+        $param{richEditId} = $data->{htmlAreaRichEditor}  ;
+    }
+    return $class->new($session, \%param);
 }
 
 #-------------------------------------------------------------------
@@ -111,22 +119,58 @@ sub _createTabInit {
 
 #-------------------------------------------------------------------
 
+=head2 defaultViewForm 
+
+Returns true if defaultView is set to 0.
+
+=cut
+
 sub defaultViewForm {
     my $self = shift;
     return ($self->get("defaultView") == 0);
 }
 
+#-------------------------------------------------------------------
+
+=head2 defaultView 
+
+Returns the kind of default view.  If defaultView == 0, it returns 'form'.  Otherwise,
+it returns 'list'.
+
+=cut
+
 sub defaultView {
     my $self = shift;
     return ($self->get("defaultView") == 0 ? 'form' : 'list');
 }
 
+#-------------------------------------------------------------------
+
+=head2 currentView 
+
+By priority, returns that the current view is.  First, it checks in internally
+cached mode, then it checks for a C<mode> form parameter, then it resorts to defaultView.
+
+=cut
+
 sub currentView {
     my $self = shift;
     my $view = $self->{_mode} || $self->session->form->param('mode') || $self->defaultView;
     return $view;
 }
 
+#-------------------------------------------------------------------
+
+=head2 deleteField ($fieldName)
+
+Removes a field from the DataForm.
+
+=head3 $fieldName
+
+The name of a field to delete.
+
+=cut
+
 sub deleteField {
     my $self = shift;
     my $fieldName = shift;
@@ -142,6 +186,18 @@ sub deleteField {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
+=head2 deleteTab ( $tabId )
+
+Deletes a tab from the tabs in this DataForm.
+
+=head3 $tabId
+
+The GUID of a tab to delete.
+
+=cut
+
 sub deleteTab {
     my $self = shift;
     my $tabId = shift;
@@ -160,14 +216,40 @@ sub deleteTab {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
+=head2 getContentLastModified 
+
+Extends the base method to modify caching.  If the currentView is in list mode, or
+an entry is being viewed, bypass caching altogether.
+
+=cut
+
 sub getContentLastModified {
     my $self = shift;
-    if ($self->currentView eq 'list' || $self->session->form->process('entryId')) {
+    my $hasCaptcha = isIn('Captcha', map { $_->{type} } map { $self->getFieldConfig($_) } @{ $self->getFieldOrder });
+    if ($self->currentView eq 'list' || $self->session->form->process('entryId') || $hasCaptcha) {
         return time;
     }
     return $self->SUPER::getContentLastModified;
 }
 
+#-------------------------------------------------------------------
+
+=head2 renameField ($oldName, $newName)
+
+Renames a field by name
+
+=head3 $oldName
+
+The old name of the field.
+
+=head3 $newName
+
+The new name of the field.
+
+=cut
+
 sub renameField {
     my $self = shift;
     my $oldName = shift;
@@ -184,6 +266,8 @@ sub renameField {
     return $self->getFieldConfig->{$newName}{name} = $newName;
 }
 
+#-------------------------------------------------------------------
+
 sub _saveFieldConfig {
     my $self = shift;
     my @config = map {
@@ -193,6 +277,8 @@ sub _saveFieldConfig {
     $self->update({fieldConfiguration => $data});
 }
 
+#-------------------------------------------------------------------
+
 sub _saveTabConfig {
     my $self = shift;
     my @config = map {
@@ -220,6 +306,12 @@ sub definition {
     my $definition = shift;
     my $i18n = WebGUI::International->new($session,"Asset_DataForm");
     my %properties;
+    
+    # populate hash of Rich Editors and add an entry to the list to use the default
+    my $selectRichEditor = WebGUI::Form::SelectRichEditor->new($session,{}) ;
+    my $richEditorOptions  = $selectRichEditor->getOptions() ;
+    $richEditorOptions->{'**Use_Default_Editor**'} = $i18n->get("Use Default Rich Editor");
+    
     tie %properties, 'Tie::IxHash';
     %properties = (
         templateId => {
@@ -231,6 +323,14 @@ sub definition {
             hoverHelp       => $i18n->get('82 description'),
             afterEdit       => 'func=edit',
         },
+        htmlAreaRichEditor =>{
+            fieldType=>"selectBox",
+            defaultValue=>0,
+            options=>$richEditorOptions,
+            tab=>'display',
+            label=>$i18n->get('htmlAreaRichEditor'),
+            hoverHelp=>$i18n->get('htmlAreaRichEditor description'),
+	},
         emailTemplateId => {
             fieldType       => "template",
             defaultValue    => 'PBtmpl0000000000000085',
@@ -384,6 +484,8 @@ sub definition {
     return $class->SUPER::definition($session, $definition);
 }
 
+#-------------------------------------------------------------------
+
 sub _cacheFieldConfig {
     my $self = shift;
     if (!$self->{_fieldConfig}) {
@@ -407,6 +509,8 @@ sub _cacheFieldConfig {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
 sub _cacheTabConfig {
     my $self = shift;
     if (!$self->{_tabConfig}) {
@@ -429,6 +533,19 @@ sub _cacheTabConfig {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
+=head2 getFieldConfig ($field)
+
+Returns the configuration for 1 field.
+
+=head3 $field
+
+The GUID of the field to return.  If left blank, it will return configurations for
+all fields in this DataForm.
+
+=cut
+
 sub getFieldConfig {
     my $self = shift;
     my $field = shift;
@@ -441,12 +558,33 @@ sub getFieldConfig {
     }
 }
 
+#-------------------------------------------------------------------
+
+=head2 getFieldOrder 
+
+Returns the internally cached field order, an array reference.
+
+=cut
+
 sub getFieldOrder {
     my $self = shift;
     $self->_cacheFieldConfig;
     return $self->{_fieldOrder};
 }
 
+#-------------------------------------------------------------------
+
+=head2 getTabConfig ( $tabId )
+
+Returns the configuration for 1 tab.
+
+=head3 $tabId
+
+The GUID of the tab to return a configuration for.  If no tabId is passed, then
+it returns the configurations for all of them.
+
+=cut
+
 sub getTabConfig {
     my $self = shift;
     my $tabId = shift;
@@ -459,6 +597,14 @@ sub getTabConfig {
     }
 }
 
+#-------------------------------------------------------------------
+
+=head2 getTabOrder 
+
+Returns the order of the tabs, an array reference.
+
+=cut
+
 sub getTabOrder {
     my $self = shift;
     $self->_cacheTabConfig;
@@ -467,6 +613,13 @@ sub getTabOrder {
 
 
 #-------------------------------------------------------------------
+
+=head2 deleteAttachedFiles 
+
+Deletes all files attached to this DataForm, or to any fields or entries in the DataForm.
+
+=cut
+
 sub deleteAttachedFiles {
     my $self = shift;
     my %params = @_;
@@ -505,6 +658,16 @@ sub deleteAttachedFiles {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getAttachedFiles ( $entryData )
+
+Return an array reference to every file in every storage location in every field for
+one set of entryData.
+
+=head3 $entryData
+
+=cut
+
 sub getAttachedFiles {
     my $self = shift;
     my $entryData = shift;
@@ -523,10 +686,22 @@ sub getAttachedFiles {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getListTemplateVars ( $var )
+
+Appends template variables for list mode.
+
+=head3 $var
+
+A hash reference.  New template variables will be appended to it.
+
+=cut
+
 sub getListTemplateVars {
-	my $self = shift;
-	my $var = shift;
-	my $i18n = WebGUI::International->new($self->session,"Asset_DataForm");
+	my $self    = shift;
+    my $session = $self->session;
+	my $var     = shift;
+	my $i18n    = WebGUI::International->new($session,"Asset_DataForm");
 	$var->{"back.url"} = $self->getFormUrl;
 	$var->{"back.label"} = $i18n->get('go to form');
     my $fieldConfig = $self->getFieldConfig;
@@ -540,7 +715,9 @@ sub getListTemplateVars {
     } @{ $self->getFieldOrder };
     $var->{field_loop} = \@fieldLoop;
     my @recordLoop;
-    my $entryIter = $self->entryClass->iterateAll($self);
+    my $p = WebGUI::Paginator->new($session);
+    $p->setDataByCallback(sub { return $self->entryClass->iterateAll($self, { offset => $_[0], limit => $_[1], }); });
+    my $entryIter = $p->getPageIterator();
     while ( my $entry = $entryIter->() ) {
         my $entryData = $entry->fields;
         my @dataLoop;
@@ -558,9 +735,9 @@ sub getListTemplateVars {
         push @recordLoop, {
             "record.ipAddress"              => $entry->ipAddress,
             "record.edit.url"               => $self->getFormUrl("func=view;entryId=".$entry->getId),
-            "record.edit.icon"              => $self->session->icon->edit("func=view;entryId=".$entry->getId, $self->get('url')),
+            "record.edit.icon"              => $session->icon->edit("func=view;entryId=".$entry->getId, $self->get('url')),
             "record.delete.url"             => $self->getUrl("func=deleteEntry;entryId=".$entry->getId),
-            "record.delete.icon"            => $self->session->icon->delete("func=deleteEntry;entryId=".$entry->getId, $self->get('url'), $i18n->get('Delete entry confirmation')),
+            "record.delete.icon"            => $session->icon->delete("func=deleteEntry;entryId=".$entry->getId, $self->get('url'), $i18n->get('Delete entry confirmation')),
             "record.username"               => $entry->username,
             "record.userId"                 => $entry->userId,
             "record.submissionDate.epoch"   => $entry->submissionDate->epoch,
@@ -570,11 +747,22 @@ sub getListTemplateVars {
         };
     }
     $var->{record_loop} = \@recordLoop;
+    $p->appendTemplateVars($var);
     return $var;
 }
 
 #-------------------------------------------------------------------
 
+=head2 getFormUrl ($params)
+
+Returns a URL to this DataForm in form mode.
+
+=head3 $params
+
+URL parameters to append to the form URL.
+
+=cut
+
 sub getFormUrl {
     my $self = shift;
     my $params = shift;
@@ -616,7 +804,20 @@ sub getListUrl {
 }
 
 #-------------------------------------------------------------------
-# Template variables for normal form view and email message
+
+=head2 getRecordTemplateVars ($var, $entry)
+
+Template variables for normal form view and email message
+
+=head3 $var
+
+A hash reference.  Template variables will be appended to it.
+
+=head3 $entry
+
+The data entered by the user, as a WebGUI::AssetCollateral::DataForm::Entry object.
+
+=cut
 
 sub getRecordTemplateVars {
     my $self = shift;
@@ -686,6 +887,11 @@ sub getRecordTemplateVars {
         my $hidden
             = ($field->{status} eq 'hidden' && !$session->var->isAdminOn)
             || ($field->{isMailField} && !$self->get('mailData'));
+	
+	# populate Rich Editor field if the field is an HTMLArea
+	if ($field->{type} eq "HTMLArea") { 
+		$field->{htmlAreaRichEditor} = $self->get("htmlAreaRichEditor") ;
+	}
         my $form = $self->_createForm($field, $value);
         $value = $form->getValueAsHtml;
         my %fieldProperties = (
@@ -733,11 +939,15 @@ sub getRecordTemplateVars {
 
 #----------------------------------------------------------------------------
 
-=head2 getTemplateVars ( )
+=head2 getTemplateVars ( $var )
 
 Gets the default template vars for the asset. Includes the asset properties
 as well as shared template vars.
 
+=head3 $var
+
+A hash reference.  Template variables are appended to it.
+
 =cut
 
 sub getTemplateVars {
@@ -785,7 +995,7 @@ sub hasEntries {
 
 =head2 prepareView ( )
 
-See WebGUI::Asset::prepareView() for details.
+Extends the base class to handle form and list mode.
 
 =cut
 
@@ -802,6 +1012,14 @@ sub prepareView {
 }
 
 #-------------------------------------------------------------------
+
+=head2 purge 
+
+Extends the base method to handle deleting attached files and purging the
+entry collateral.
+
+=cut
+
 sub purge {
     my $self = shift;
     $self->deleteAttachedFiles;
@@ -810,6 +1028,22 @@ sub purge {
 }
 
 #-------------------------------------------------------------------
+
+=head2 sendEmail ($var, $entry)
+
+Sends an email with information about the data entered by the user.  The email
+is templated, and macros in the template will be expanded.
+
+=head3 $var
+
+A hash reference of template variables
+
+=head3 $entry
+
+The data entered by the user, as a WebGUI::AssetCollateral::DataForm::Entry object.
+
+=cut
+
 sub sendEmail {
     my $self = shift;
     my $var = shift;
@@ -898,6 +1132,13 @@ sub useCaptcha {
 }
 
 #-------------------------------------------------------------------
+
+=head2 view 
+
+Based on the view mode, renders either the form or the list.
+
+=cut
+
 sub view {
     my $self = shift;
     my $view = $self->currentView;
@@ -910,6 +1151,14 @@ sub view {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canView 
+
+Extends the base method to include users who can edit the DataForm in list mode,
+or are part of the groupToViewEntries.
+
+=cut
+
 sub canView {
     my $self = shift;
     return 0
@@ -924,6 +1173,14 @@ sub canView {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
+=head2 prepareViewList 
+
+Like prepareView, but for the list view of the template.
+
+=cut
+
 sub prepareViewList {
     my $self = shift;
     my $templateId = $self->get('listTemplateId');
@@ -939,12 +1196,28 @@ sub prepareViewList {
     $self->{_viewListTemplate} = $template;
 }
 
+#-------------------------------------------------------------------
+
+=head2 viewList 
+
+Renders the list view of the DataForm.
+
+=cut
+
 sub viewList {
     my $self    = shift;
     my $var     = $self->getTemplateVars;
     return $self->processTemplate($self->getListTemplateVars($var), undef, $self->{_viewListTemplate});
 }
 
+#-------------------------------------------------------------------
+
+=head2 prepareViewForm 
+
+Prepare the template for the form mode of the template.
+
+=cut
+
 sub prepareViewForm {
     my $self = shift;
     $self->session->style->setLink($self->session->url->extras('tabs/tabs.css'), {"type"=>"text/css"});
@@ -964,6 +1237,23 @@ sub prepareViewForm {
 
 #-------------------------------------------------------------------
 
+=head2 viewForm ($passedVars, $entry)
+
+Render the template for viewing the form of the DataForm.
+
+=head3 $passedVars
+
+A hash ref of template variables.  If passed in, the default record template
+variables will not be fetched.
+
+=head3 $entry
+
+A DataForm::Entry collateral object.  If not passed in, then it will try to look up
+an entry via the form variable C<entryId>.  If the user cannot edit the DataForm,
+then they cannot edit data that has already been entered.
+
+=cut
+
 sub viewForm {
     my $self        = shift;
     my $passedVars  = shift;
@@ -977,11 +1267,26 @@ sub viewForm {
     return $self->processTemplate($var, undef, $self->{_viewFormTemplate});
 }
 
+#-------------------------------------------------------------------
+
+=head2 entryClass 
+
+Returns a string, the classname for entry collateral.
+
+=cut
+
 sub entryClass {
     return 'WebGUI::AssetCollateral::DataForm::Entry';
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_deleteAllEntriesConfirm 
+
+Web facing method for deleting all data entered into the DataForm.
+
+=cut
+
 sub www_deleteAllEntriesConfirm {
     my $self = shift;
     return $self->session->privilege->insufficient
@@ -1002,6 +1307,15 @@ sub www_deleteAllEntriesConfirm {
 #}
 
 #-------------------------------------------------------------------
+
+=head2 www_deleteEntry 
+
+Web facing method for deleting one entry from the DataForm, identified
+by the form variable C<entryId>.  Returns insufficient unless the
+current user canEdit the DataForm.  Returns the user to www_view.
+
+=cut
+
 sub www_deleteEntry {
     my $self = shift;
     return $self->session->privilege->insufficient
@@ -1014,34 +1328,75 @@ sub www_deleteEntry {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_deleteFieldConfirm 
+
+Web facing method for deleting one field from the DataForm, identified
+by the form variable C<fieldName>.  This operation is revisioned.
+
+Returns insufficient unless the current user canEdit the DataForm.
+
+Returns the user to www_view.
+
+=cut
+
 sub www_deleteFieldConfirm {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     $newSelf->deleteField($self->session->form->process("fieldName"));
     $newSelf->{_mode} = 'form';
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_deleteTabConfirm
+
+Web facing method for deleting one tab from the DataForm, identified
+by the form variable C<tabId>.  This operation is revisioned.
+
+Returns insufficient unless the current user canEdit the DataForm.
+
+Returns the user to www_view.
+
+=cut
+
 sub www_deleteTabConfirm {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     $newSelf->deleteTab($self->session->form->process("tabId"));
     $newSelf->{_mode} = 'form';
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editField 
+
+Renders a form to edit a field, identified by the form variable C<fieldName>.
+
+Returns insufficient unless the current user canEdit the DataForm.
+
+=cut
+
 sub www_editField {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $i18n = WebGUI::International->new($self->session,"Asset_DataForm");
     my $fieldName = shift || $self->session->form->process("fieldName");
     my $field;
@@ -1178,11 +1533,21 @@ sub www_editField {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editFieldSave 
+
+Process the editField form.  Returns insufficient unless the current
+user canEdit the DataForm.
+
+=cut
+
 sub www_editFieldSave {
     my $self = shift;
     my $session = $self->session;
     return $session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $form = $session->form;
     my $fieldName = $form->process('fieldName');
     my $newName = $session->url->urlize($form->process('newName') || $form->process('label'));
@@ -1236,15 +1601,33 @@ sub www_editFieldSave {
         $newSelf->createField($newName, \%field);
     }
 
-
+    WebGUI::VersionTag->autoCommitWorkingIfEnabled($session);
+    my $freshSelf = $newSelf->cloneFromDb();
     if ($form->process("proceed") eq "editField") {
-        return $newSelf->www_editField('new');
+        return $freshSelf->www_editField('new');
     }
-    $newSelf->{_mode} = 'form';
-    WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    $freshSelf->{_mode} = 'form';
+    return $freshSelf->www_view;
 }
 
+#-------------------------------------------------------------------
+
+=head2 createField ($fieldName, $field)
+
+Create a new field in the DataForm.  Returns 1 if the creation
+was successful
+
+=head3 $fieldName
+
+The name of the field to create.  Returns 0 if a field with the
+same name already exists.
+
+=head3 $field
+
+A hash ref of field data, added to the field configuration for this field.
+
+=cut
+
 sub createField {
     my $self = shift;
     my $fieldName = shift;
@@ -1261,6 +1644,23 @@ sub createField {
     return 1;
 }
 
+#-------------------------------------------------------------------
+
+=head2 setField ($fieldName, $field)
+
+Updates a field in the DataForm.  Returns 1 if the update was successful
+
+=head3 $fieldName
+
+The name of the field to update.  Returns 0 if a field with the
+same name does not exist.
+
+=head3 $field
+
+A hash ref of field data, added to the field configuration for this field.
+
+=cut
+
 sub setField {
     my $self = shift;
     my $fieldName = shift;
@@ -1278,10 +1678,21 @@ sub setField {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editTab 
+
+Renders a web form for adding new tabs, or editing existing tabs.  Returns
+insufficient unless the current user canEdit this DataForm.  The GUID of
+the tab is looked for in the form variable C<tabId>.
+
+=cut
+
 sub www_editTab {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $i18n = WebGUI::International->new($self->session,"Asset_DataForm");
     my $tabId = shift || $self->session->form->process("tabId") || "new";
     my $tab;
@@ -1325,10 +1736,20 @@ sub www_editTab {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editTabSave 
+
+Process the editTab form.  Returns insufficient unless the current user canEdit
+this DataForm.
+
+=cut
+
 sub www_editTabSave {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $name = $self->session->form->process("name") || $self->session->form->process("label");
     $name = $self->session->url->urlize($name);
     my $tabId = $self->session->form->process('tabId');
@@ -1356,6 +1777,14 @@ sub www_editTabSave {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_exportTab 
+
+Exports all the data entered into the DataForm in CSV format.  Returns
+insufficient unless the current user canEdit this DataForm.
+
+=cut
+
 sub www_exportTab {
     my $self = shift;
     my $session = $self->session;
@@ -1399,17 +1828,43 @@ sub www_exportTab {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_moveFieldDown 
+
+Web facing method to move a field one position down.  The field is identified
+by the form variable C<fieldName>.  Returns insufficient unless the current user canEdit
+this DataForm.
+
+This operation is revisioned.
+
+=cut
+
 sub www_moveFieldDown {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     my $fieldName = $self->session->form->process('fieldName');
     $newSelf->moveFieldDown($fieldName);
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
+#-------------------------------------------------------------------
+
+=head2 moveFieldDown ($fieldName)
+
+Shifts the field one position down in the field order.
+
+=head3 $fieldName
+
+The name of the field to move.
+
+=cut
+
 sub moveFieldDown {
     my $self = shift;
     my $fieldName = shift;
@@ -1433,17 +1888,43 @@ sub moveFieldDown {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_moveFieldUp 
+
+Web facing method to move a field one position up.  The field is identified
+by the form variable C<fieldName>.  Returns insufficient unless the current user canEdit
+this DataForm.
+
+This operation is revisioned.
+
+=cut
+
 sub www_moveFieldUp {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     my $fieldName = $self->session->form->process('fieldName');
     $newSelf->moveFieldUp($fieldName);
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
+#-------------------------------------------------------------------
+
+=head2 moveFieldUp ($fieldName)
+
+Shifts the field one position up in the field order.
+
+=head3 $fieldName
+
+The name of the field to move.
+
+=cut
+
 sub moveFieldUp {
     my $self = shift;
     my $fieldName = shift;
@@ -1468,18 +1949,44 @@ sub moveFieldUp {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_moveTabRight 
+
+Web facing method to move a tab one position to the right.  The tab is identified
+by the form variable C<tabId>.  Returns insufficient unless the current user canEdit
+this DataForm.
+
+This operation is revisioned.
+
+=cut
+
 sub www_moveTabRight {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     my $tabId = $self->session->form->process('tabId');
     $newSelf->moveTabRight($tabId);
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 moveTabRight ($tabId)
+
+Shifts the tab one position to the right in the tab order.
+
+=head3 $tabId
+
+The GUID of the tab to move.
+
+=cut
+
 sub moveTabRight {
     my $self = shift;
     my $tabId = shift;
@@ -1498,17 +2005,42 @@ sub moveTabRight {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_moveTabLeft 
+
+Web facing method to move a tab one position to the left.  The tab is identified
+by the form variable C<tabId>.  Returns insufficient unless the current user canEdit
+this DataForm.
+
+This operation is revisioned.
+
+=cut
+
 sub www_moveTabLeft {
     my $self = shift;
     return $self->session->privilege->insufficient
         unless $self->canEdit;
+    return $self->session->privilege->locked
+        unless $self->canEditIfLocked;
     my $newSelf = $self->addRevision;
     my $tabId = $self->session->form->process('tabId');
     $newSelf->moveTabLeft($tabId);
     WebGUI::VersionTag->autoCommitWorkingIfEnabled($self->session);
-    return $newSelf->www_view;
+    my $freshSelf = $newSelf->cloneFromDb();
+    return $freshSelf->www_view;
 }
 
+#-------------------------------------------------------------------
+
+=head2 moveTabLeft ($tabId)
+
+Shifts the tab one position to the left in the tab order.
+
+=head3 $tabId
+
+The GUID of the tab to move.
+
+=cut
 
 sub moveTabLeft {
     my $self = shift;
@@ -1529,6 +2061,14 @@ sub moveTabLeft {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_process 
+
+Process the form for a user entering new data.  Returns insufficient unless
+the current user canView the DataForm.
+
+=cut
+
 sub www_process {
     my $self = shift;
     return $self->session->privilege->insufficient
@@ -1546,11 +2086,26 @@ sub www_process {
         my $default = $field->{defaultValue};
         WebGUI::Macro::process($self->session, \$default);
         my $value = $entry->field( $field->{name} ) || $default;
+
+        # WebGUI::Form::Integer::getValue() returns 0 even if no number is passed in.
+        # Not really a suitable default if we want to trigger the error message
+
         if ($field->{status} eq "required" || $field->{status} eq "editable") {
+
+            # get the raw value (by sending field type as blank)
+            my $rawValue = $session->form->process($field->{name}, '');
+
             $value = $session->form->process($field->{name}, $field->{type}, undef, {
                 defaultValue    => $default,
                 value           => $value,
             });
+
+            # this is a hack, but it's better than changing the default getValue() of Integer, which
+            # could have massive effects downstream in other uses.
+            if(($field->{type} =~ /integer/i) && defined($rawValue) && ($rawValue eq '') && ($value eq "0")) {
+                $value = $rawValue;
+            }
+
             WebGUI::Macro::filter(\$value);
         }
         if ($field->{status} eq "required" && (! defined($value) || $value =~ /^\s*$/)) {
