@@ -32,7 +32,8 @@ use WebGUI::HTMLForm;
 use WebGUI::International;
 use WebGUI::Utility;
 use WebGUI::Workflow::Instance;
-
+use Tie::IxHash;
+use Data::Dumper;
 
 
 #-------------------------------------------------------------------
@@ -53,12 +54,27 @@ sub definition {
 		},
 		templateId => {
 			fieldType 		=> 'template',
-			defaultValue 	=> '2rC4ErZ3c77OJzJm7O5s3w',
+			defaultValue 	        => '2rC4ErZ3c77OJzJm7O5s3w',
 			tab				=> 'display',
 			label			=> $i18n->get('main template'),
 			hoverHelp		=> $i18n->get('main template help'),
 			namespace		=> 'EMS',
 		},
+		scheduleTemplateId => {
+			fieldType 		=> 'template',
+			defaultValue 	        => 'S2_LsvVa95OSqc66ITAoig',
+			tab			=> 'display',
+			label			=> $i18n->get('schedule template'),
+			hoverHelp		=> $i18n->get('schedule template help'),
+			namespace		=> 'EMS/Schedule',
+		},
+		scheduleColumnsPerPage => {
+			fieldType 		=> 'Integer',
+			defaultValue 		=> '5',
+			tab			=> 'display',
+			label			=> $i18n->get('schedule number of columns'),
+			hoverHelp		=> $i18n->get('schedule number of columns help'),
+		},
 		badgeBuilderTemplateId => {
 			fieldType 		=> 'template',
 			defaultValue 	=> 'BMybD3cEnmXVk2wQ_qEsRQ',
@@ -257,6 +273,30 @@ sub getEventFieldsForImport {
 	return \@fields;
 }
 
+#-------------------------------------------------------------------
+
+=head2 getLocations ()
+
+Returns an array of all locations & dates for this EMS
+may be SQL optimized for quick access
+
+=cut
+
+sub getLocations {
+    my $self = shift;
+    my $dateRef = shift || [ ];
+
+    my %hash;
+    my %hashDate;
+    my %h;
+    my $tickets = $self->getTickets;
+# this is a really compact 'uniq' operation
+    my @locations = map { $h{$_}++ == 0 ? $_ : () } ( map { $_->get('location') } ( @$tickets ) );
+# the dates have the time data removed with a pattern substitution
+    push @$dateRef, map { s/\s*\d+:\d+(:\d+)?//; $h{$_}++ == 0 ? $_ : () } ( map { $_->get('startDate') } ( @$tickets ) );
+
+    return @locations;
+}
 
 #-------------------------------------------------------------------
 
@@ -366,6 +406,14 @@ sub prepareView {
 
 #------------------------------------------------------------------
 
+=head2 purge ( )
+
+See WebGUI::Asset::purge() for details.  Extend SUPERclass
+to handle deleting tickets, tokens, ribbons, registrants, badge groups
+and event meta data.
+
+=cut
+
 sub purge {
     my $self = shift;
     my $db = $self->session->db;
@@ -409,6 +457,7 @@ sub view {
 	my %var = (
 		addBadgeUrl			=> $self->getUrl('func=add;class=WebGUI::Asset::Sku::EMSBadge'),
 		buildBadgeUrl		=> $self->getUrl('func=buildBadge'),
+		viewScheduleUrl		=> $self->getUrl('func=viewSchedule'),
 		manageBadgeGroupsUrl=> $self->getUrl('func=manageBadgeGroups'),
 		getBadgesUrl		=> $self->getUrl('func=getBadgesAsJson'),
 		canEdit				=> $self->canEdit,
@@ -1084,6 +1133,120 @@ sub www_getRibbonsAsJson {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 www_getScheduleDataJSON ()
+
+returns the JSON data for a page of the schedule table
+
+=cut
+
+
+sub www_getScheduleDataJSON {
+    my $self = shift;
+    my $session = $self->session;
+    my $emptyRecord = JSON->new->encode( {
+        records         => [ ],   totalRecords        => 0,     recordsReturned    => 0,
+        startIndex      => 0,     currentLocationPage => 0,     totalLocationPages => 0,
+        currentDatePage => 0,     totalDatePages      => 0,     dateRecords        => [ ],
+        sort            => undef, dir                 => 'asc', pageSize           => 0,
+    });
+    return $emptyRecord unless $self->canView;
+    # the following two are expected to be configurable...
+    my $locationsPerPage   = $self->get('scheduleColumnsPerPage');
+
+    my ($db, $form)        = $session->quick(qw(db form));
+    my $locationPageNumber = $form->get('locationPage') || 1;
+    my $datePageNumber     = $form->get('datePage')     || 1;
+    my @dateRecords;
+    my @ticketLocations = $self->getLocations( \@dateRecords );
+    # the total number of pages is the number of locations divided by the number of locations per page
+    my $numberOfLocationPages = int( .9 + scalar(@ticketLocations) / $locationsPerPage );
+        # skip everything else if there are no locations/pages
+    return $emptyRecord if $numberOfLocationPages == 0;
+    # now we pick out the locations to be displayed on this page
+    my $indexFirstLocation = ($locationPageNumber-1)*$locationsPerPage;
+    my $indexLastLocation  = $locationPageNumber*$locationsPerPage - 1;
+    my $currentDate        = $dateRecords[$datePageNumber-1];
+    @ticketLocations       = @ticketLocations[$indexFirstLocation..$indexLastLocation];
+	my $tickets = $db->read( q{
+             select assetData.assetId, sku.description, assetData.title, EMSTicket.startDate, EMSTicket.location
+               from EMSTicket
+               join sku using (assetId,revisionDate)
+               join assetData using (assetId,revisionDate)
+               join asset using (assetId)
+              where asset.parentId = ? 
+                 and DATE_FORMAT( EMSTicket.startDate, '%Y-%m-%d' ) = ?
+                 and EMSTicket.location in (  } . 
+		         join( ',', (map { $db->quote($_) } (@ticketLocations))) .
+			 q{ )
+                 and assetData.revisionDate = (
+                           select max(revisionDate)
+                             from assetData
+                            where assetData.assetId=asset.assetId
+                              and ( assetData.status = 'approved'
+                                  or assetData.tagId = ? )
+	      )
+              order by EMSTicket.startDate, eventNumber asc
+                     },[  $self->getId,  $currentDate,
+                           $session->scratch->get("versionTag")
+                      ]);
+    my %hash;
+    tie %hash, 'Tie::IxHash';
+    while( my $row = $tickets->hashRef ) {
+	$row->{type} = 'ticket';
+        $row->{location} = '&nbsp;' if $row->{location} eq '';
+        push @{$hash{$row->{startDate}}{$row->{location}}}, $row;
+    }
+    grep { $_ = '&nbsp;' if defined $_ && $_ eq '' } @ticketLocations;
+    my %results = ();
+    $results{records} = [];  ##Initialize to an empty array
+    my $ctr = 0;
+    my %locationMap = map { 'col' . ++$ctr , $_ } @ticketLocations;
+         # fill out the columns in the table
+    while( $ctr < $locationsPerPage ) { $locationMap{ 'col' . ++$ctr } = '' };
+    push @{$results{records}}, { colDate => '' , map { $_ , { type => 'label', title => $locationMap{$_} || '' } } ( keys %locationMap ) };
+    my $redo = 0;
+    for my $startDate ( keys %hash ) {
+        $redo = 0;
+        my $row = { colDate => $startDate };
+	my $empty = 1;
+	for my $col ( keys %locationMap ) {
+	    my $location = $locationMap{$col};
+	    if( exists $hash{$startDate}{$location} ) {
+	        $row->{$col} = pop @{$hash{$startDate}{$location}};
+		$empty = 0;
+                $redo = 1 if scalar(@{$hash{$startDate}{$location}}) > 0;
+                delete $hash{$startDate}{$location} if scalar(@{$hash{$startDate}{$location}}) == 0;
+	    } else {
+	        $row->{$col} = { type => 'empty' };
+	    }
+	}
+	next if $empty;
+	push @{$results{records}}, $row;
+        redo if $redo;
+    }
+
+    my $rowCount = scalar(@{$results{records}});
+    $results{totalRecords} = $rowCount;
+    $results{recordsReturned} = $rowCount;
+    $results{rowsPerPage} = $rowCount;
+    $results{startIndex} = 0; 
+    $results{sort}       = undef;
+    $results{dir}        = "asc";
+    $results{pageSize}   = 10;
+             # these next two are used to configure the paginator
+    $results{totalLocationPages} = $numberOfLocationPages;
+    $results{currentLocationPage} = $locationPageNumber;
+    $results{totalDatePages} = scalar(@dateRecords);
+    $results{currentDatePage} = $datePageNumber;
+    $results{dateRecords} = \@dateRecords;
+    $session->http->setMimeType('application/json');
+    return JSON->new->encode(\%results);
+}
+
+
+
 #-------------------------------------------------------------------
 
 =head2 www_getTicketsAsJson ()
@@ -1096,11 +1259,13 @@ sub www_getTicketsAsJson {
     my ($self) = @_;
 	my $session = $self->session;
     return $session->privilege->insufficient() unless $self->canView;
-    my ($db, $form) = $session->quick(qw(db form));
-    my $startIndex = $form->get('startIndex') || 0;
-    my $numberOfResults = $form->get('results') || 25;
+    my ($db, $form)     = $session->quick(qw(db form));
+    my $startIndex      = $form->get('startIndex') || 0;
+    my $numberOfResults = $form->get('results')    || 25;
+    my $sortDir         = $form->get('sortDir')    || 'ASC';
+    my $sortKey         = $form->get('sortKey')    || 'eventNumber';
     my %results = ();
-	my @ids = ();
+	my @ids     = ();
 	my $keywords = $form->get('keywords');
 	
 	# looking for specific events
@@ -1122,7 +1287,7 @@ sub www_getTicketsAsJson {
 	# just get all tickets
 	else {
 		@ids = $db->buildArray("select assetId from asset left join EMSTicket using (assetId) where parentId=? and
-className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate=(select max(revisionDate) from EMSTicket where assetId=asset.assetId) order by eventNumber", [$self->getId]);
+className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate=(select max(revisionDate) from EMSTicket where assetId=asset.assetId) order by $sortKey $sortDir", [$self->getId]);
 	}
 	
 	# get badge's badge groups
@@ -1136,16 +1301,13 @@ className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate
 	
 	# get a list of tickets already associated with the badge
 	my @existingTickets = $db->buildArray("select ticketAssetId from EMSRegistrantTicket where badgeId=?",[$badgeId]);
-	
+
 	# get assets
 	my $counter = 0;
 	my $totalTickets = scalar(@ids);
 	my @records = ();
 	foreach my $id (@ids) {
 
-		# gotta get to the page we're working with
-		next unless ($counter >= $startIndex);
-
 		# skip tickets we already have
 		if (isIn($id, @existingTickets)) {
 			$totalTickets--;
@@ -1186,6 +1348,10 @@ className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate
 				next;
 			}
 		}
+
+        # gotta get to the page we're working with
+        $counter++;
+        next unless ($counter >= ($startIndex * $numberOfResults));
 		
 		# publish the data for this ticket
         my $description = $ticket->get('description');
@@ -1216,7 +1382,6 @@ className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate
 			duration			=> $ticket->get('duration'),
 			});
 		last unless (scalar(@records) < $numberOfResults);
-		$counter++;
 	}
 	
 	# humor
@@ -1239,6 +1404,7 @@ className='WebGUI::Asset::Sku::EMSTicket' and state='published' and revisionDate
 }
 
 
+
 #-------------------------------------------------------------------
 
 =head2 www_getTokensAsJson ()
@@ -1349,8 +1515,8 @@ Handle the uploading of a CSV event data file, along with other options.
 sub www_importEventsSave {
 	my $self = shift;
 	my $session = $self->session;
-
 	return $session->privilege->insufficient unless $self->canEdit;
+$|=1;
 
 	# set up
 	$session->http->setMimeType("text/plain");
@@ -1364,11 +1530,13 @@ sub www_importEventsSave {
 	my $validate = WebGUI::FormValidator->new($session);
 	
 	# find fields to import
+    $out->print("Finding fields to import...\n",1);
 	my @import = $form->get("fieldsToImport");
 	my $i = 0;
 	my $assetIdIndex = undef;
 	foreach my $field (@import) {
 		if ($field eq "assetId") {
+            $out->print("\t$i\n",1);
 			$assetIdIndex = $i;
 			last;
 		}
@@ -1384,7 +1552,7 @@ sub www_importEventsSave {
 	my $first = 1;
 	if (open my $file, "<", $storage->getPath($filename)) {
 		$out->print("Processing file...\n",1);
-		while (my $line = <$file>) {
+		ROW: while (my $line = <$file>) {
 			if ($first) {
 				$first = 0;
 				if ($ignoreFirst) {
@@ -1393,6 +1561,8 @@ sub www_importEventsSave {
 			}
 			if ($csv->parse($line)) {
 				my @row = $csv->fields;
+                my $start = [Time::HiRes::gettimeofday];
+        		$out->print("Processing ".join(",", @row)."\n",1);
 				my $event = undef;
 				if (defined $assetIdIndex) {
 					$event = WebGUI::Asset::Sku::EMSTicket->new($session, $row[$assetIdIndex]);
@@ -1405,26 +1575,43 @@ sub www_importEventsSave {
 					$out->print("Adding new asset ".$event->getId."\n",1)
 				}
 				my %properties = ();
-				my %metadata = $event->getEventMetaData;
+				my $metadata = $event->getEventMetaData;
 				my $i = 0;
 				foreach my $field (@{$fields}) {
 					next unless isIn($field->{name}, @import);
+            		$out->print("\tAdding field ".$field->{label}."\n",1);
 					my $type = $field->{type};
+                    ##Force the use of Form::DateTime and MySQL Format
+                    if ($field->{name} eq 'startDate') {
+                        $type = 'dateTime';
+                        $field->{defaultValue} = '1999-05-24 17:30:00';
+                    }
 					my $value = $validate->$type({
 							name			=> $field->{name},
 							defaultValue	=> $field->{defaultValue},
 							options			=> $field->{options},
 							},$row[$i]);
+                    if ($field->{name} eq 'startDate' && !$value) {
+                        $out->print('Skipping event on line '.$line.' due to bad date format');
+                        next ROW;
+                    }
 					if ($field->{isMeta}) {
-						$metadata{$field->{name}} = $value;
+						$metadata->{$field->{label}} = $value;
 					}
 					else {
 						$properties{$field->{name}} = $value;
 					}
 					$i++;
 				}
+                $out->print("\tUpdating properties\n",1);
+                $properties{menuTitle} = $properties{title};
+                $properties{url} = $self->get("url")."/".$properties{title};
 				$event->update(\%properties);
-				$event->setEventMetaData(\%metadata);
+                $out->print("\tUpdating meta data\n",1);
+				$event->setEventMetaData($metadata);
+                $out->print("\tCommitting asset\n",1);
+                WebGUI::VersionTag->getWorking($session)->commit;
+                $out->print("\tAdding event took ".Time::HiRes::tv_interval($start)." seconds to run.\n",1);
 			}
 			else {
 				$out->print($csv->error_input() . ": ". $line."\n",1);
@@ -1784,7 +1971,8 @@ sub www_printTicket {
 	my $registrant = $self->getRegistrant($form->get('badgeId'));
 	my $ticket = WebGUI::Asset::Sku::EMSTicket->new($session, $form->get('ticketAssetId'));
 	$registrant->{ticketTitle} = $ticket->getTitle;
-	$registrant->{ticketStart} = $ticket->get('startDate');
+        my $startTime = WebGUI::DateTime->new($ticket->get('startDate'))->set_time_zone($self->get('timezone'));
+	$registrant->{ticketStart} = $startTime->strftime('%F %R');
 	$registrant->{ticketDuration} = $ticket->get('duration');
 	$registrant->{ticketLocation} = $ticket->get('location');
 	$registrant->{ticketEventNumber} = $ticket->get('eventNumber');
@@ -1859,4 +2047,37 @@ sub www_toggleRegistrantCheckedIn {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 www_viewSchedule ()
+
+View the schedule table.
+
+=cut
+
+sub www_viewSchedule {
+	my $self             = shift;
+    return $self->session->privilege->insufficient() unless $self->canView;
+	my $db               = $self->session->db;
+    my $rowsPerPage      = 25;
+    my $locationsPerPage = $self->get('scheduleColumnsPerPage');
+
+    my @columnNames = map { "'col" . $_ . "'" } ( 1..$locationsPerPage );
+    my $fieldList   = join ',', @columnNames;
+    my $dataColumns = join ",\n",  map {
+	    '{key:' . $_ . ',sortable:false,label:"",formatter:formatViewScheduleItem}'
+                     }  @columnNames;
+
+	return $self->processStyle(
+               $self->processTemplate({
+                      backUrl => $self->getUrl,
+                      rowsPerPage => $rowsPerPage,
+                      dataColumns => $dataColumns,
+                      fieldList => $fieldList,
+                      dataSourceUrl => $self->getUrl('func=getScheduleDataJSON'),
+                  },$self->get('scheduleTemplateId')));
+
+}
+
 1;
+
