@@ -11,7 +11,8 @@ package WebGUI::Asset::Wobject::GalleryAlbum;
 #-------------------------------------------------------------------
 
 use strict;
-use base 'WebGUI::Asset::Wobject';
+use Class::C3;
+use base qw(WebGUI::AssetAspect::RssFeed WebGUI::Asset::Wobject);
 use Carp qw( croak );
 use File::Find;
 use File::Spec;
@@ -19,6 +20,8 @@ use File::Temp qw{ tempdir };
 use Tie::IxHash;
 use WebGUI::International;
 use WebGUI::Utility;
+use WebGUI::HTML;
+use WebGUI::ProgressBar;
 
 use Archive::Any;
 
@@ -77,12 +80,12 @@ sub definition {
         properties          => \%properties,
     };
 
-    return $class->SUPER::definition($session, $definition);
+    return $class->next::method($session, $definition);
 }
 
 #----------------------------------------------------------------------------
 
-=head2 addArchive ( filename, properties )
+=head2 addArchive ( filename, properties, [$outputSub] )
 
 Add an archive of Files to this Album. C<filename> is the full path of the 
 archive. C<properties> is a hash reference of properties to assign to the
@@ -93,12 +96,27 @@ a directory outside of the storage location.
 
 Will only handle file types handled by the parent Gallery.
 
+=head3 filename
+
+The name of the file archive to import.
+
+=head3 properties
+
+A base set of properties to add to each file in the archive.
+
+=head3 $outputSub
+
+A callback to use for outputting data, most likely to a progress bar.  It expects the
+callback to accept an i18n key for use in sprintf, and then any extra fields to stuff
+into the translated key.
+
 =cut
 
 sub addArchive {
     my $self        = shift;
     my $filename    = shift;
     my $properties  = shift;
+    my $outputSub   = shift || sub {};
     my $gallery     = $self->getParent;
     
     my $archive     = Archive::Any->new( $filename );
@@ -107,11 +125,12 @@ sub addArchive {
         if $archive->is_naughty;
 
     my $tempdirName = tempdir( "WebGUI-Gallery-XXXXXXXX", TMPDIR => 1, CLEANUP => 1);
+    $outputSub->('Extracting archive');
     $archive->extract( $tempdirName );
 
     # Get all the files in the archive
     my @files;
-    my $wanted      = sub { push @files, $File::Find::name };
+    my $wanted      = sub { push @files, $File::Find::name; $outputSub->('Found file: %s', $File::Find::name); };
     find( {
         wanted      => $wanted,
     }, $tempdirName );
@@ -125,7 +144,8 @@ sub addArchive {
         next unless $class; # class is undef for those files the Gallery can't handle
 
         $self->session->errorHandler->info( "Adding $filename to album!" );
-        # Remove the file extention
+        $outputSub->('Adding %s to album', $filename);
+        # Remove the file extension
         $filename   =~ s{\.[^.]+}{};
 
         $properties->{ className        } = $class;
@@ -141,6 +161,7 @@ sub addArchive {
     $versionTag->set({ 
         "workflowId" => $self->getParent->get("workflowIdCommit"),
     });
+    $outputSub->('Requesting commit for version tag');
     $versionTag->requestCommit;
 
     return undef;
@@ -174,7 +195,7 @@ sub addChild {
         return undef;
     }
 
-    return $self->SUPER::addChild( $properties, @_ );
+    return $self->next::method( $properties, @_ );
 }
 
 #----------------------------------------------------------------------------
@@ -285,16 +306,12 @@ sub canEdit {
     my $form        = $self->session->form;
 
     # Handle adding a photo
-    if ( $form->get("func") eq "add" ) {
-        return $self->canAddFile;
-    }
-    elsif ( $form->get("func") eq "editSave" && $form->get("className") eq __PACKAGE__ ) {
+    if ( $form->get("func") eq "add" || $form->get("func") eq "editSave" ) {
         return $self->canAddFile;
     }
     else {
         return 1 if $userId eq $self->get("ownerUserId");
-            
-        return $gallery->canEdit($userId);
+        return $gallery && $gallery->canEdit($userId);
     }
 }
 
@@ -400,7 +417,7 @@ sub getCurrentRevisionDate {
         return $revisionDate;
     }
     else {
-        return $class->SUPER::getCurrentRevisionDate( $session, $assetId );
+        return $class->next::method( $session, $assetId );
     }
 }
 
@@ -497,6 +514,41 @@ sub getPreviousAlbum {
     return $self->{_previousAlbum};
 }
 
+#-------------------------------------------------------------------
+
+=head2 getRssFeedItems ()
+
+Returns an array reference of hash references. Each hash reference has a title,
+description, link, and date field. The date field can be either an epoch date, an RFC 1123
+date, or a ISO date in the format of YYYY-MM-DD HH:MM::SS. Optionally specify an
+author, and a guid field.
+
+=cut
+
+sub getRssFeedItems {
+    my $self        = shift;
+
+    my $p
+        = $self->getFilePaginator( { 
+            perpage     => $self->get('itemsPerFeed'),
+        } );
+    
+    my $var = [];
+    for my $assetId ( @{ $p->getPageData } ) {
+        my $asset       = WebGUI::Asset::Wobject::GalleryAlbum->newPending( $self->session, $assetId );
+        push @{ $var }, {
+            'link'          => $asset->getUrl,
+            'guid'          => $asset->{_properties}->{ 'assetId' },
+            'title'         => $asset->getTitle,
+            'description'   => $asset->{_properties}->{ 'description' },
+            'date'          => $asset->{_properties}->{ 'creationDate' },
+            'author'        => WebGUI::User->new($self->session, $asset->{_properties}->{ 'ownerUserId' })->username
+        };
+    }
+    
+    return $var;
+}
+
 #----------------------------------------------------------------------------
 
 =head2 getTemplateVars ( )
@@ -639,7 +691,7 @@ See WebGUI::Asset::prepareView() for details.
 
 sub prepareView {
     my $self = shift;
-    $self->SUPER::prepareView();
+    $self->next::method();
 
     my $templateId  = $self->getParent->get("templateIdViewAlbum");
 
@@ -726,7 +778,7 @@ approval workflow.
 sub processPropertiesFromFormPost {
     my $self        = shift;
     my $form        = $self->session->form;
-    my $errors      = $self->SUPER::processPropertiesFromFormPost || [];
+    my $errors      = $self->next::method || [];
 
     # Return if error
     return $errors  if @$errors;
@@ -771,7 +823,7 @@ Override update to force isHidden=1 on all albums.
 sub update {
     my $self        = shift;
     my $properties  = shift;
-    return $self->SUPER::update({ %{ $properties }, isHidden=>1 });
+    return $self->next::method({ %{ $properties }, isHidden=>1 });
 }
 
 #----------------------------------------------------------------------------
@@ -881,7 +933,7 @@ sub www_addArchive {
 
     my $i18n = WebGUI::International->new($session);
 
-    $var->{ error           } = $params->{ error };
+    $var->{ error           } = $params->{ error } || $form->get('error');
 
     $var->{ form_start      } 
         = WebGUI::Form::formHeader( $session, {
@@ -936,32 +988,27 @@ sub www_addArchiveSave {
     my $session     = $self->session;
     my $form        = $self->session->form;
     my $i18n        = WebGUI::International->new( $session, 'Asset_GalleryAlbum' );
+    my $pb          = WebGUI::ProgressBar->new($session);
     my $properties  = {
         keywords        => $form->get("keywords"),
         friendsOnly     => $form->get("friendsOnly"),
     };
     
+    $pb->start($i18n->get('Uploading archive'), $session->url->extras('adminConsole/assets.gif'));
     my $storageId   = $form->get("archive", "File");
     my $storage     = WebGUI::Storage->get( $session, $storageId );
     if (!$storage) {
-        return $self->www_addArchive({
-            error       => sprintf $i18n->get('addArchive error too big'),
-        });
+        return $pb->finish($self->getUrl('func=addArchive;error='.$i18n->get('addArchive error too big')));
     }
     my $filename    = $storage->getPath( $storage->getFiles->[0] );
 
-    eval { $self->addArchive( $filename, $properties ) };
+    eval { $self->addArchive( $filename, $properties, sub{ $pb->update(sprintf $i18n->get(shift), @_); }); };
+    $storage->delete;
     if ( my $error = $@ ) {
-        return $self->www_addArchive({
-            error       => sprintf( $i18n->get('addArchive error generic'), $error ),
-        });
+        return $pb->finish($self->getUrl('func=addArchive;error='.sprintf $i18n->get('addArchive error generic'), $error ));
     }
 
-    $storage->delete;
-
-    return $self->processStyle(
-        sprintf $i18n->get('addArchive message'), $self->getUrl,
-    );
+    return $pb->finish($self->getUrl);
 }
 
 #----------------------------------------------------------------------------
@@ -1033,7 +1080,7 @@ sub www_addFileService {
 #    my $filePath = $storage->getPath( $storage->getFiles->[0] );
  #   $self->setFile( $filePath );
   #  $storage->delete;
-    $session->log->warn('XX:'. $filename);
+    #$session->log->warn('XX:'. $filename);
     
     $file->requestAutoCommit;
     
@@ -1303,8 +1350,8 @@ sub www_showConfirmation {
     my $output      = '<p>' . sprintf( $i18n->get('save message'), $self->getUrl ) . '</p>'
                     . '<p>' . $i18n->get('what next') . '</p>'
                     . '<ul>'
-                    . sprintf( $i18n->get('add photo'), $self->getUrl('func=add;class=WebGUI::Asset::File::GalleryFile::Photo') )
-                    . sprintf( $i18n->get('return to album'), $self->getUrl )
+                    . sprintf( '<li><a href="%s">%s</a></li>', $self->getUrl('func=add;class=WebGUI::Asset::File::GalleryFile::Photo'), $i18n->get('add photo')  )
+                    . sprintf( '<li><a href="%s">%s</a></li>', $self->getUrl, $i18n->get('return to album') )
                     . '</ul>'
                     ;
 
@@ -1377,18 +1424,27 @@ sub www_viewRss {
 
     my $var         = $self->getTemplateVars;
     $self->appendTemplateVarsFileLoop( $var, $self->getFileIds );
-    
+
     # Fix URLs to be full URLs
     for my $key ( qw( url url_viewRss ) ) {
         $var->{ $key } = $self->session->url->getSiteURL . $var->{ $key };
     }
 
+    # Encode XML entities
+    for my $key ( qw( title description synopsis gallery_title gallery_menuTitle ) ) {
+        $var->{ $key } = WebGUI::HTML::filter($var->{$key}, 'xml');
+    }
+
     # Process the file loop to add additional params
     for my $file ( @{ $var->{file_loop} } ) {
         # Fix URLs to be full URLs
         for my $key ( qw( url ) ) { 
             $file->{ $key }  = $self->session->url->getSiteURL . $file->{$key}; 
         }
+        # Encode XML entities
+        for my $key ( qw( title description synopsis ) ) {
+            $file->{ $key } = WebGUI::HTML::filter($file->{$key}, 'xml');
+        }
 
         $file->{ rssDate } 
             = $self->session->datetime->epochToMail( $file->{creationDate} );
