@@ -86,16 +86,8 @@ sub definition {
 	my $definition = shift;
 	my $i18n = WebGUI::International->new($session, 'Asset_Matrix');
 
-    my @groups = $session->db->buildArray("select groupId from groups order by groupId");
-    my @names = $session->db->buildArray("select groupName from groups order by groupId");
-    
-    tie my %maxGroup, 'Tie::IxHash';
-    for (my $i = 0; $i < scalar(@groups); ++$i) {
-        $maxGroup{$groups[$i]} = $names[$i];
-    }
-
 	my %properties;
-    tie %properties, 'Tie::IxHash';
+	tie %properties, 'Tie::IxHash';
 	%properties = (
 	    templateId =>{
 		    fieldType       =>"template",  
@@ -239,9 +231,8 @@ sub definition {
             label           =>$i18n->get('max comparisons privileged label'),
         },
         maxComparisonsGroup=>{
-            fieldType       =>"selectBox",
+            fieldType       =>"group",
             tab             =>"properties",
-            options         =>\%maxGroup,
             hoverHelp       =>$i18n->get('maxgroup description'),
             label           =>$i18n->get('maxgroup label'),
         },
@@ -253,7 +244,7 @@ sub definition {
             label           =>$i18n->get('maxgroup per label'),
         },
         groupToAdd=>{
-            fieldType       =>"selectBox",
+            fieldType       =>"group",
             tab             =>"security",
             defaultValue    =>2,
             hoverHelp       =>$i18n->get('group to add description'),
@@ -282,6 +273,14 @@ sub definition {
             label           => $i18n->get("statistics cache timeout label"),
             hoverHelp       => $i18n->get("statistics cache timeout description")
         },
+        listingsCacheTimeout => {
+            tab             => "display",
+            fieldType       => "interval",
+            defaultValue    => 3600,
+            uiLevel         => 8,
+            label           => $i18n->get("listings cache timeout label"),
+            hoverHelp       => $i18n->get("listings cache timeout description")
+        },
 	);
 	push(@{$definition}, {
 		assetName=>$i18n->get('assetName'),
@@ -468,13 +467,13 @@ sub getCompareForm {
     if($self->session->user->isVisitor){
         $maxComparisons = $self->get('maxComparisons');
     }
-    elsif($self->session->user->isInGroup( $self->get("maxComparisonsGroup") )) {
+    elsif($self->session->user->isInGroup( $self->get("maxComparisonsGroup") )) { 
         $maxComparisons = $self->get('maxComparisonsGroupInt');
     }
     else{
         $maxComparisons = $self->get('maxComparisonsPrivileged');
-    }        
-    # warn(sprintf("maxComparisons: $maxComparisons: %d: %d: %d: %d", $self->session->user->isVisitor, $self->getId, $self->get('maxComparisonsGroupInt'), $self->get('maxComparisonsPrivileged')));
+    }
+    $maxComparisons += 0;
     $form .=  "\n<script type='text/javascript'>\n".
         'var maxComparisons = '.$maxComparisons.";\n".
         "var matrixUrl = '".$self->getUrl."';\n".
@@ -484,6 +483,78 @@ sub getCompareForm {
 
 #-------------------------------------------------------------------
 
+=head2 getListings  (  )
+
+Returns the listings as an arrayRef of hashRefs.
+
+=head3 sort 
+
+The criterium by which the listings should be sorted.
+
+=cut
+
+sub getListings {
+
+    my $self        = shift;
+    my $session     = $self->session;
+    my $sort        = shift || $session->scratch->get('matrixSort') || $self->get('defaultSort');
+    my $versionTag  = WebGUI::VersionTag->getWorking($session, 1);
+    my ($listings, $listingsEncoded);
+
+    my $noCache =
+        $session->var->isAdminOn
+        || $self->get("listingsCacheTimeout") <= 10
+        || ($versionTag && $versionTag->getId eq $self->get("tagId"));
+    unless ($noCache) {
+        $listingsEncoded = WebGUI::Cache->new($session,"matrixListings_".$self->getId)->get;
+    }
+
+    if ($listingsEncoded){
+        $listings = JSON->new->decode($listingsEncoded);
+    }
+    else{
+        my $sortDirection   = ' desc';
+        if ($sort eq 'title'){
+            $sortDirection = ' asc';
+        }
+
+        my $sql = "
+        select
+            assetData.title,
+            assetData.url,
+            listing.assetId,
+            listing.views,
+            listing.compares,
+            listing.clicks,
+            listing.lastUpdated
+        from asset
+            left join assetData using(assetId)
+            left join MatrixListing as listing on listing.assetId = assetData.assetId and listing.revisionDate =
+assetData.revisionDate
+        where
+            asset.parentId=?
+            and asset.state='published'
+            and asset.className='WebGUI::Asset::MatrixListing'
+            and assetData.revisionDate = (SELECT max(revisionDate) from assetData where assetId=asset.assetId and status='approved')
+            and status='approved'
+        order by ".$sort.$sortDirection;
+    
+        $listings = $session->db->buildArrayRefOfHashRefs($sql,[$self->getId]);
+
+        foreach my $listing (@{$listings}) {
+            $listing->{url}      = $session->url->gateway($listing->{url});
+        }
+
+        $listingsEncoded = JSON->new->encode($listings);
+            WebGUI::Cache->new($session,"matrixListings_".$self->getId)->set(
+                $listingsEncoded,$self->get("listingsCacheTimeout")
+            );
+    }
+    return $listings;
+}
+
+#-------------------------------------------------------------------
+
 =head2 getEditForm ( )
 
 returns the tabform object that will be used in generating the edit page for Matrix.
@@ -552,37 +623,40 @@ sub view {
 	my $self    = shift;
 	my $session = $self->session;	
     my $db      = $session->db; 
+    my $i18n    = WebGUI::International->new($session, 'Asset_Matrix');
 
     # javascript and css files for compare form datatable
-    $self->session->style->setLink($self->session->url->extras('yui/build/datatable/assets/skins/sam/datatable.css'), 
-        {type =>'text/css', rel=>'stylesheet'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/json/json-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/connection/connection-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/get/get-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/element/element-beta-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/datasource/datasource-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/datatable/datatable-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('yui/build/button/button-min.js'), {type =>
-    'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('wobject/Matrix/matrix.js'), {type =>
-    'text/javascript'});
+    $session->style->setLink(  $session->url->extras('yui/build/datatable/assets/skins/sam/datatable.css'), {type =>'text/css', rel=>'stylesheet'});
+    $session->style->setScript($session->url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/json/json-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/connection/connection-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/get/get-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/element/element-beta-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/datasource/datasource-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/datatable/datatable-min.js'), {type => 'text/javascript'});
+    $session->style->setScript($session->url->extras('yui/build/button/button-min.js'), {type => 'text/javascript'});
     
     my ($varStatistics,$varStatisticsEncoded);
 	my $var = $self->get;
-    $var->{isLoggedIn}              = ($self->session->user->userId ne "1");
+    $var->{isLoggedIn}              = ($session->user->userId ne "1");
     $var->{addMatrixListing_url}    = $self->getUrl('func=add;class=WebGUI::Asset::MatrixListing'); 
-    $var->{compareForm}             = $self->getCompareForm;
     $var->{exportAttributes_url}    = $self->getUrl('func=exportAttributes');
     $var->{listAttributes_url}      = $self->getUrl('func=listAttributes');
     $var->{search_url}              = $self->getUrl('func=search');
+    $var->{matrix_url}              = $self->getUrl();
+
+    my $maxComparisons;
+    if($session->user->isVisitor){
+        $maxComparisons = $self->get('maxComparisons');
+    }
+    elsif($session->user->isInGroup( $self->get("maxComparisonsGroup") )) {
+        $maxComparisons = $self->get('maxComparisonsGroupInt');
+    }
+    else{
+        $maxComparisons = $self->get('maxComparisonsPrivileged');
+    }
+    $maxComparisons += 0;
+    $var->{maxComparisons} = $maxComparisons;
    
     if ($self->canEdit){
         # Get all the MatrixListings that are still pending.
@@ -614,7 +688,9 @@ sub view {
         $varStatistics = JSON->new->decode($varStatisticsEncoded);
     }
     else{
-        $varStatistics->{alphanumeric_sortButton}   = "<span id='sortByName'><button type='button'>Sort by name</button></span><br />";
+        $varStatistics->{alphanumeric_sortButton}   = "<span id='sortByName'><button type='button'>"
+                                                    . $i18n->get('Sort by name')
+                                                    . "</button></span><br />";
 
         # Get the MatrixListing with the most views as an object using getLineage.
         my ($bestViews_listing) = @{ $self->getLineage(['descendants'], {
@@ -628,7 +704,9 @@ sub view {
             $varStatistics->{bestViews_url}           = $bestViews_listing->getUrl;
             $varStatistics->{bestViews_count}         = $bestViews_listing->get('views');
             $varStatistics->{bestViews_name}          = $bestViews_listing->get('title');
-            $varStatistics->{bestViews_sortButton}    = "<span id='sortByViews'><button type='button'>Sort by views</button></span><br />";
+            $varStatistics->{bestViews_sortButton}    = "<span id='sortByViews'><button type='button'>"
+                                                      . $i18n->get('Sort by views')
+                                                      . "</button></span><br />";
         }
 
         # Get the MatrixListing with the most compares as an object using getLineage.
@@ -644,7 +722,9 @@ sub view {
             $varStatistics->{bestCompares_url}        = $bestCompares_listing->getUrl;
             $varStatistics->{bestCompares_count}      = $bestCompares_listing->get('compares');
             $varStatistics->{bestCompares_name}       = $bestCompares_listing->get('title');
-            $varStatistics->{bestCompares_sortButton} = "<span id='sortByCompares'><button type='button'>Sort by compares</button></span><br />";
+            $varStatistics->{bestCompares_sortButton} = "<span id='sortByCompares'><button type='button'>"
+                                                      . $i18n->get('Sort by compares')
+                                                      . "</button></span><br />";
         }
 
         # Get the MatrixListing with the most clicks as an object using getLineage.
@@ -659,7 +739,9 @@ sub view {
             $varStatistics->{bestClicks_url}          = $bestClicks_listing->getUrl;
             $varStatistics->{bestClicks_count}        = $bestClicks_listing->get('clicks');
             $varStatistics->{bestClicks_name}         = $bestClicks_listing->get('title');
-            $varStatistics->{bestClicks_sortButton}   = "<span id='sortByClicks'><button type='button'>Sort by clicks</button></span><br />";
+            $varStatistics->{bestClicks_sortButton}   = "<span id='sortByClicks'><button type='button'>"
+                                                      . $i18n->get('Sort by clicks')
+                                                      . "</button></span><br />";
         }
 
         # Get the 5 MatrixListings that were last updated as objects using getLineage.
@@ -675,10 +757,12 @@ sub view {
             push (@{ $varStatistics->{last_updated_loop} }, {
                         url         => $lastUpdatedListing->getUrl,
                         name        => $lastUpdatedListing->get('title'),
-                        lastUpdated => $self->session->datetime->epochToHuman($lastUpdatedListing->get('lastUpdated'),"%z")
+                        lastUpdated => $session->datetime->epochToHuman($lastUpdatedListing->get('lastUpdated'),"%z")
                     });
         }
-        $varStatistics->{lastUpdated_sortButton}  = "<span id='sortByUpdated'><button type='button'>Sort by updated</button></span><br />";
+        $varStatistics->{lastUpdated_sortButton}  = "<span id='sortByUpdated'><button type='button'>"
+                                                  . $i18n->get('Sort by updated')
+                                                  . "</button></span><br />";
 
         # For each category, get the MatrixListings with the best ratings.
 
@@ -689,9 +773,9 @@ sub view {
             assetData.title as productName,
             assetData.url,
             rating.listingId, 
-            rating.meanValue, 
+            rating.meanValue,
             rating.medianValue,
-            rating.countValue,
+            rating.countValue, 
             asset.parentId 
         from 
             MatrixListing_ratingSummary as rating 
@@ -701,6 +785,7 @@ sub view {
             rating.category =? 
             and asset.parentId=? 
             and asset.state='published' 
+            and rating.countValue >= 10
             and assetData.revisionDate=(
                 select
                     max(revisionDate)
@@ -716,21 +801,21 @@ sub view {
         
         $data = $db->quickHashRef($sql." desc limit 1",[$category,$self->getId]);
         push(@{ $varStatistics->{best_rating_loop} },{
-            url=>'/'.$data->{url},
-            category=>$category,
-            name=>$data->{productName},
-            mean=>$data->{meanValue},
-            median=>$data->{medianValue},
-            count=>$data->{countValue}
+            url      => $session->url->gateway($data->{url}),
+            category => $category,
+            name     => $data->{productName},
+            mean     => 0+$data->{meanValue},
+            median   => 0+$data->{medianValue},
+            count    => 0+$data->{countValue}
             });
         $data = $db->quickHashRef($sql." asc limit 1",[$category,$self->getId]);
         push(@{ $varStatistics->{worst_rating_loop} },{
-            url=>'/'.$data->{url},
-            category=>$category,
-            name=>$data->{productName},
-            mean=>$data->{meanValue},
-            median=>$data->{medianValue},
-            count=>$data->{countValue}
+            url      => $session->url->gateway($data->{url}),
+            category => $category,
+            name     => $data->{productName},
+            mean     => 0+$data->{meanValue},
+            median   => 0+$data->{medianValue},
+            count    => 0+$data->{countValue}
             });
         }
 
@@ -803,8 +888,6 @@ sub www_compare {
     {type =>'text/javascript'});
     $self->session->style->setScript($self->session->url->extras('yui/build/container/container-min.js'),
     {type =>'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('wobject/Matrix/matrixCompareList.js'), {type =>
-    'text/javascript'});
     $self->session->style->setLink($self->session->url->extras('yui/build/datatable/assets/skins/sam/datatable.css'),
         {type =>'text/css', rel=>'stylesheet'});
     $self->session->style->setScript($self->session->url->extras('hoverhelp.js'), {type =>
@@ -816,26 +899,25 @@ sub www_compare {
     if($self->session->user->isVisitor){
         $maxComparisons = $self->get('maxComparisons');
     }
-    elsif($self->session->user->isInGroup( $self->get("maxComparisonsGroup") )) {
+    elsif($self->session->user->isInGroup( $self->get("maxComparisonsGroup") )) { 
         $maxComparisons = $self->get('maxComparisonsGroupInt');
     }
     else{
         $maxComparisons = $self->get('maxComparisonsPrivileged');
     }
-    # warn(sprintf("maxComparisons: $maxComparisons: %d", $self->session->user->isVisitor));
+    $maxComparisons += 0;
 
     foreach my $listingId (@listingIds){
         my $listingId_safe = $listingId;
         $listingId_safe =~ s/-/_____/g;
         push(@responseFields, $listingId_safe, $listingId_safe."_compareColor");
     }
-    
-    $var->{javascript} = "<script type='text/javascript'>\n".
-        'var listingIds = new Array('.join(", ",map {'"'.$_.'"'} @listingIds).");\n".
-        'var responseFields = new Array("attributeId", "name", "description","fieldType", "checked", '.join(", ",map {'"'.$_.'"'} @responseFields).");\n".
-        "var maxComparisons = ".$maxComparisons.";\n".
-        "var matrixUrl = '".$self->getUrl."';\n".
-        "</script>";
+
+    $var->{maxComparisons}  = $maxComparisons;
+    $var->{matrixUrl}       = $self->getUrl;
+    $var->{listingIds}      = join(", ",map {'"'.$_.'"'} @listingIds);
+    $var->{responseFields}  = '"attributeId", "name", "description","fieldType", "checked", '
+                              .join(", ",map{'"'.$_.'"'} @responseFields);
 
     return $self->processStyle($self->processTemplate($var,$self->get("compareTemplateId")));
 }
@@ -1072,6 +1154,7 @@ sub www_getCompareFormData {
     my @listingIds = $session->form->checkList("listingId");
     
     $session->http->setMimeType("application/json");
+    my $db = $session->db;
 
     my (@searchParams,@searchParams_sorted,@searchParamList,$searchParamList);
     if($form->process("search")){
@@ -1084,76 +1167,76 @@ sub www_getCompareFormData {
                 $attributeId =~ s/^search_//;
                 $attributeId =~ s/_____/-/g;
                 $parameter->{attributeId} = $attributeId;
-                push(@searchParamList,'"'.$parameter->{attributeId}.'"');
-                push(@searchParams,$parameter);
+                push(@searchParamList,  $db->quote($parameter->{attributeId}) );
+                push(@searchParams,     $parameter);
+            }
+        }
+        if (! scalar @searchParamList) {
+            ##Use defaults for all form values
+            foreach my $category (keys %{$self->getCategories}) {
+                my $attributes = $db->read("select * from Matrix_attribute where category =? and assetId = ?",
+                    [$category,$self->getId]);
+                while (my $attribute = $attributes->hashRef) {
+                    push @searchParamList, $db->quote($attribute->{attributeId});
+                    push @searchParams, {
+                        name        => $attribute->{name},
+                        value       => $attribute->{defaultValue},
+                        attributeId => $attribute->{attributeId},
+                    };
+                }
             }
         }
         $searchParamList        = join(',',@searchParamList);
         @searchParams_sorted    = sort { $b->{value} <=> $a->{value} } @searchParams;
     }
 
-    my $sql = "
-        select
-            assetData.title,
-            assetData.url,
-            listing.assetId,
-            listing.views,
-            listing.compares,
-            listing.clicks,
-            listing.lastUpdated
-        from asset
-            left join assetData using(assetId)
-            left join MatrixListing as listing on listing.assetId = assetData.assetId and listing.revisionDate =
-assetData.revisionDate
-        where
-            asset.parentId=?
-            and asset.state='published'
-            and asset.className='WebGUI::Asset::MatrixListing'
-            and assetData.revisionDate = (SELECT max(revisionDate) from assetData where assetId=asset.assetId and status='approved')
-            and status='approved'
-        order by ".$sort.$sortDirection;
-    
-    my $sth = $session->db->read($sql,[$self->getId]);
     my @results;
-
-    if($form->process("search")){
-        while (my $result = $sth->hashRef) {
+    if($form->process("search")) {
+        if ($searchParamList) {
+            RESULT: foreach my $result (@{$self->getListings}) {
+                my $checked = '';
                 my $matrixListing_attributes = $session->db->buildHashRefOfHashRefs("
                             select value, fieldType, attributeId from Matrix_attribute
                             left join MatrixListing_attribute as listing using(attributeId)
                             where listing.matrixListingId = ? 
                             and attributeId IN(".$searchParamList.")",
                             [$result->{assetId}],'attributeId');
+                ##Searching is AND based.
                 PARAM: foreach my $param (@searchParams_sorted) {
                         my $fieldType       = $matrixListing_attributes->{$param->{attributeId}}->{fieldType};
                         my $listingValue    = $matrixListing_attributes->{$param->{attributeId}}->{value};
                         if(($fieldType eq 'MatrixCompare') && ($listingValue < $param->{value})){
-                            $result->{checked} = '';
+                            $checked = '';
                             last PARAM;
                         }
                         elsif(($fieldType ne 'MatrixCompare' && $fieldType ne '') && ($param->{value} ne $listingValue)){
-                            $result->{checked} = '';
+                            $checked = '';
                             last PARAM;
                         }
                         else{
-                            $result->{checked} = 'checked';
+                            $checked = 'checked';
                         }
                 }
-            $result->{assetId}  =~ s/-/_____/g;
-            $result->{url}      = $session->url->gateway($result->{url});
-            push @results, $result;
+                $result->{assetId}  =~ s/-/_____/g;
+                push @results, $result if $checked eq 'checked';
+            }
         }
-    }else{
-        while (my $result = $sth->hashRef) {
+        else {   
+            foreach my $result (@{$self->getListings}) {
+                $result->{assetId}  =~ s/-/_____/g;
+                push @results, $result;
+            }
+        }
+    }
+    else {
+        foreach my $result (@{$self->getListings}) {
             $result->{assetId}  =~ s/-/_____/g;
             if(WebGUI::Utility::isIn($result->{assetId},@listingIds)){
                 $result->{checked} = 'checked';
             }
-            $result->{url}      = $session->url->gateway($result->{url});
             push @results, $result;
         }
     }
-    $sth->finish;
 
     my $jsonOutput;
     $jsonOutput->{ResultSet} = {Result=>\@results};
@@ -1285,26 +1368,31 @@ sub www_listAttributes {
     
     return $session->privilege->insufficient() unless($self->canEdit);
     
-    my $i18n = WebGUI::International->new($session,'Asset_Matrix');
-    my $output = "<br /><a href='".$self->getUrl("func=editAttribute;attributeId=new")."'>"
-                .$i18n->get('add attribute label')."</a><br /><br />";
-    
+    my $i18n       = WebGUI::International->new($session,'Asset_Matrix');
+    my $console    = $self->getAdminConsole();
     my $attributes = $session->db->read("select attributeId, name from Matrix_attribute where assetId=? order by name"
         ,[$self->getId]);
+    my $output = '';
     while (my $attribute = $attributes->hashRef) {
-        $output .= $session->icon->delete("func=deleteAttribute;attributeId=".$attribute->{attributeId}
-            , $self->getUrl,$i18n->get("delete attribute confirm message"))
-            .$session->icon->edit("func=editAttribute;attributeId=".$attribute->{attributeId})
-            .' '.$attribute->{name}."<br />\n";
+        $output .= $session->icon->delete(
+                       "func=deleteAttribute;attributeId=".$attribute->{attributeId},
+                       $self->getUrl,$i18n->get("delete attribute confirm message")
+                   )
+                . $session->icon->edit("func=editAttribute;attributeId=".$attribute->{attributeId})
+                . ' '
+                . $attribute->{name}
+                ."<br />\n";
     }
-    return $self->getAdminConsole->render($output, $i18n->get('list attributes title'));
+    $console->addSubmenuItem($self->getUrl("func=editAttribute;attributeId=new"), $i18n->get('add attribute label'));
+    $console->addSubmenuItem($self->getUrl, $i18n->get('Return to Matrix'));
+    return $console->render($output, $i18n->get('list attributes title'));
 }
 
 #-------------------------------------------------------------------
 
 =head2 www_search  (  )
 
-Returns the search screen.
+Returns the search screen.  Uses www_getCompareFormData with search=1 for doing AJAX requests.
 
 =cut
 
@@ -1335,8 +1423,6 @@ sub www_search {
     {type =>'text/javascript'});
     $self->session->style->setScript($self->session->url->extras('yui/build/button/button-min.js'),
     {type =>'text/javascript'});
-    $self->session->style->setScript($self->session->url->extras('wobject/Matrix/matrixSearch.js'), {type =>
-    'text/javascript'});
     $self->session->style->setLink($self->session->url->extras('yui/build/datatable/assets/skins/sam/datatable.css'),
         {type =>'text/css', rel=>'stylesheet'});
 
