@@ -90,6 +90,13 @@ sub definition {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getEditForm ( )
+
+Manually build the edit form due to javascript elements.
+
+=cut
+
 sub getEditForm {
 	my $self = shift;
 	my $tabform = $self->SUPER::getEditForm;
@@ -274,20 +281,6 @@ sub getEditForm {
 		".($ancestorsChecked ? "" : "toggleAncestorEndPoint();")."
 		//]]>
 		</script>");
-	my $previewButton;# = qq{
-                          # <INPUT TYPE="button" VALUE="Preview" NAME="preview"
-                          #  OnClick="
-                          #      window.open('', 'navPreview', 'toolbar=no,status=no,location=no,scrollbars=yes,resizable=yes');
-                          #      this.form.func.value='preview';
-                          #      this.form.target = 'navPreview';
-                          #      this.form.submit()">};
-	my $saveButton = ' <input type="button" class="forwardButton" value="'.$i18n->get(62,'WebGUI').'" onclick="
-		this.value=\''.$i18n->get(452,'WebGUI').'\';
-		this.form.func.value=\'editSave\';
-		this.form.target=\'_self\';
-		this.form.submit();
-		" />';
-	$tabform->{_submit} = $previewButton." ".$saveButton;
 	return $tabform;
 }
 
@@ -346,7 +339,7 @@ sub getToolbar {
 
 =head2 prepareView ( )
 
-See WebGUI::Asset::prepareView() for details.
+Extend the superclass to add metadata and to preprocess the template.
 
 =cut
 
@@ -367,6 +360,13 @@ sub prepareView {
 
 
 #-------------------------------------------------------------------
+
+=head2 view ( )
+
+See WebGUI::Asset::view() for details.
+
+=cut
+
 sub view {
 	my $self = shift;
 	# we've got to determine what our start point is based upon user conditions
@@ -525,6 +525,13 @@ sub view {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_goBackToPage ( )
+
+Do a redirect to the form parameter returnUrl if it exists.
+
+=cut
+
 sub www_goBackToPage {
 	my $self = shift;
 	$self->session->http->setRedirect($self->session->form->process("returnUrl")) if ($self->session->form->process("returnUrl"));
