@@ -202,9 +202,9 @@ sub definition {
             label=>$i18n->get("download mimetype"),
         },
         downloadUserGroup=>{
-            fieldType=>"group",
-            defaultValue=>"text/html",
-            label=>$i18n->get("download usergroup"),
+            fieldType    => "group",
+            defaultValue => "7",
+            label        => $i18n->get("download usergroup"),
         },
     );
     push(@{$definition}, {
@@ -270,9 +270,9 @@ sub getEditForm {
 	# Download Type
 	my %downloadTypes;
 	tie %downloadTypes, 'Tie::IxHash', 
-		"none" 		=> "No Download",
-		"csv"		=> "CSV",
-		"template"	=> "Template",
+		"none" 		=> $i18n->get("No Download"),
+		"csv"		=> $i18n->get("CSV"),
+		"template"	=> $i18n->get("Template"),
 		;
 		
 	$tabform->getTab("properties")->radioList(
@@ -474,7 +474,9 @@ sub download {
 	} 
     elsif ($self->getValue("downloadType") eq "template") { 
         my $data	= $self->_processQuery(1,0);
-		return $self->processTemplate($data,$self->get("downloadTemplateId"));
+        my $output  = $self->processTemplate($data,$self->get("downloadTemplateId"));
+        WebGUI::Macro::process($self->session, \$output);
+		return $output;
 	} 
     else {
 		# I don't know what to do
@@ -523,10 +525,19 @@ sub purgeCache {
 }
 
 #-------------------------------------------------------------------
+
+=head2 view ( )
+
+See WebGUI::Asset::view() for details.  This method also performs content caching
+if the user is not in Admin Mode.
+
+=cut
+
 sub view {
 	my $self = shift;
 	if (!$self->session->var->isAdminOn && $self->get("cacheTimeout") > 10) {
-		my $out = WebGUI::Cache->new($self->session,"view_".$self->getId)->get;
+        my $cache = $self->getCache;
+        my $out   = $cache->get if defined $cache;
 		return $out if $out;
 	}
         # Initiate an empty debug loop
@@ -789,12 +800,10 @@ sub www_download {
 		unless $self->session->user->isInGroup($self->getValue("downloadUserGroup"));
 	
 	# Set filename and mimetype
-	if ($self->getValue("downloadType") eq "csv")
-	{
+	if ($self->getValue("downloadType") eq "csv") {
 		$self->session->http->setFilename($self->getValue("downloadFilename"),"application/octet-stream");
 	}
-	else
-	{
+	else {
 		$self->session->http->setFilename($self->getValue("downloadFilename"),$self->getValue("downloadMimeType"));
 	}
 	
