@@ -168,7 +168,8 @@ sub view {
 	my %var;
 	
     $var{'form_header'  } = WebGUI::Form::formHeader($session, {
-        action=>$self->getUrl
+        action => $self->getUrl,
+        method => "GET"
 		})
     .WebGUI::Form::hidden($self->session,{name=>"doit", value=>"1"});
 	$var{'form_footer'  } = WebGUI::Form::formFooter($session);
@@ -194,7 +195,7 @@ sub view {
 		$search->search(\%rules);
 		
         #Instantiate the highlighter
-        my @words     = split(/\s+/,$keywords);
+        my @words     = grep { $_ ne '' } map { tr/+?*//d; $_; } split(/\s+/,$keywords);
         my @wildcards = map { "%" } @words;
         my $hl = HTML::Highlight->new(
             words     => \@words,
