@@ -10,7 +10,8 @@ package WebGUI::Asset::Wobject::WikiMaster;
 # http://www.plainblack.com                     info@plainblack.com
 #-------------------------------------------------------------------
 
-use base 'WebGUI::Asset::Wobject';
+use Class::C3;
+use base qw(WebGUI::AssetAspect::RssFeed WebGUI::Asset::Wobject);
 use strict;
 use Tie::IxHash;
 use WebGUI::International;
@@ -19,6 +20,20 @@ use HTML::Parser;
 use URI::Escape;
 
 #-------------------------------------------------------------------
+
+=head2 appendMostPopular ($var, [ $limit ])
+
+=head3 $var
+
+A hash reference of template variables.  An array reference containing the most popular wiki pages
+in order of popularity.
+
+=head3 $limit
+
+If passed in, this will override the mostPopularCount set in the object.
+
+=cut
+
 sub appendMostPopular {
 	my $self = shift;
 	my $var = shift;
@@ -36,6 +51,20 @@ sub appendMostPopular {
 }
 
 #-------------------------------------------------------------------
+
+=head2 appendRecentChanges ( $var [, $limit ] )
+
+=head3 $var
+
+A hash reference of template variables.  Template variables will be appended
+to the hash ref.
+
+=head3 $limit
+
+If passed in, this will override the mostChangesCount set in the object.
+
+=cut
+
 sub appendRecentChanges {
 	my $self = shift;
 	my $var = shift;
@@ -70,11 +99,27 @@ sub appendRecentChanges {
 			username=>$user->username,
 			date=>$self->session->datetime->epochToHuman($asset->get("revisionDate")),
 			isAvailable=>$isAvailable,
+            assetId=>$id,
 			});
 	}
 }
 
 #-------------------------------------------------------------------
+
+=head2 appendSearchBoxVars  ($var, $queryText)
+
+Appends template variables for creating a search box to search the wiki.
+
+=head3 $var
+
+A hash reference of template variables.  The search box variables will be appended to this.
+
+=head3 $queryText
+
+Default value for the search box.
+
+=cut
+
 sub appendSearchBoxVars {
 	my $self = shift;
 	my $var = shift;
@@ -91,23 +136,46 @@ sub appendSearchBoxVars {
 }
 
 #-------------------------------------------------------------------
+
+=head2 autolinkHtml ($html, [options])
+
+Scan HTML for words and phrases that match wiki titles, and automatically
+link them to those wiki pages.  Returns the modified HTML.
+
+=head3 $html
+
+The HTML to scan.
+
+=head3 options
+
+Either a hashref, or a hash of options.
+
+=head4 skipTitles
+
+An array reference of titles that should not be autolinked.
+
+=cut
+
 sub autolinkHtml {
 	my $self = shift;
 	my $html = shift;
     # opts is always the last parameter, and a hash ref
     my %opts = ref $_[-1] eq 'HASH' ? %{pop @_} : ();
-    my $skipTitles = $opts{skipTitles} || [];
+    $opts{skipTitles} ||= [];
+
+    # LC all the skip titles once, for efficiency
+    my @skipTitles = map { lc $_ } @{ $opts{skipTitles} };
     # TODO: ignore caching for now, but maybe do it later.
-	my %mapping = $self->session->db->buildHash("SELECT LOWER(d.title), d.url FROM asset AS i INNER JOIN assetData AS d ON i.assetId = d.assetId WHERE i.parentId = ? and className='WebGUI::Asset::WikiPage' and i.state='published' and d.status='approved'", [$self->getId]);
-	foreach my $key (keys %mapping) {
-        if (grep {lc $_ eq $key} @$skipTitles) {
-            delete $mapping{$key};
-            next;
-        }
-        $key =~ s{\(}{\\\(}gxms; # escape parens
-        $key =~ s{\)}{\\\)}gxms; # escape parens
-		$mapping{$key} = $self->session->url->gateway($mapping{$key});
-	}
+    # This query returns multiple entries for each asset, so we order by revisionDate and count on the hash to only have the
+    # latest version.
+    my %mapping = $self->session->db->buildHash("SELECT LOWER(d.title), d.url FROM asset AS i INNER JOIN assetData AS d ON i.assetId = d.assetId WHERE i.parentId = ? and className='WebGUI::Asset::WikiPage' and i.state='published' and d.status='approved' order by d.revisionDate ASC", [$self->getId]);
+    TITLE: foreach my $title (keys %mapping) {
+        my $url = delete $mapping{$title};
+        ##isIn short circuits and is faster than grep and/or first
+        next TITLE if isIn($title, @skipTitles);
+        $mapping{$title} = $self->session->url->gateway($url);
+    }   
+
 	return $html unless %mapping;
     # sort by length so it prefers matching longer titles 
 	my $matchString = join('|', map{quotemeta} sort {length($b) <=> length($a)} keys %mapping);
@@ -137,9 +205,17 @@ sub autolinkHtml {
 }
 
 #-------------------------------------------------------------------
+
+=head2 canAdminister 
+
+Returns true if the current user is in the groupToAdminister group, or the user can edit
+this WikiMaster.
+
+=cut
+
 sub canAdminister {
 	my $self = shift;
-	return $self->session->user->isInGroup($self->get('groupToAdminister')) || $self->SUPER::canEdit;
+	return $self->session->user->isInGroup($self->get('groupToAdminister')) || $self->WebGUI::Asset::Wobject::canEdit;
 }
 
 #-------------------------------------------------------------------
@@ -164,11 +240,19 @@ sub canEdit {
                         ) &&
                         $self->canEditPages
                 ) || # account for new posts
-                $self->SUPER::canEdit()
+                $self->next::method()
         );
 }
 
 #-------------------------------------------------------------------
+
+=head2 canEditPages 
+
+Returns true is the current user is in the group that can edit page, or if
+they can administer the wiki (canAdminister).
+
+=cut
+
 sub canEditPages {
 	my $self = shift;
 	return $self->session->user->isInGroup($self->get("groupToEditPages")) || $self->canAdminister;
@@ -337,13 +421,51 @@ sub definition {
 	      properties => \%properties,
 	     };
 
-        return $class->SUPER::definition($session, $definition);
+        return $class->next::method($session, $definition);
 }
 
 #-------------------------------------------------------------------
+
+=head2 getRssFeedItems ()
+
+Returns an array reference of hash references. Each hash reference has a title,
+description, link, and date field. The date field can be either an epoch date, an RFC 1123
+date, or a ISO date in the format of YYYY-MM-DD HH:MM::SS. Optionally specify an
+author, and a guid field.
+
+=cut
+
+sub getRssFeedItems {
+    my $self        = shift;
+    my $vars = {};
+    $self->appendRecentChanges( $vars, $self->get('itemsPerFeed') );
+    my $var = [];
+    foreach my $item ( @{ $vars->{recentChanges} } ) {
+        my $asset       = WebGUI::Asset->newByDynamicClass( $self->session, $item->{assetId} );
+        push @{ $var }, {
+            'link'          => $asset->getUrl,
+            'guid'          => $item->{ 'assetId' } . $asset->get( 'revisionDate' ),
+            'title'         => $asset->getTitle,
+            'description'   => $item->{ 'actionTaken' },
+            'date'          => $item->{ 'date' },
+            'author'        => $item->{ 'username' },
+        };
+    }
+    
+    return $var;
+}
+
+#-------------------------------------------------------------------
+
+=head2 prepareView 
+
+Prepare the front page template.
+
+=cut
+
 sub prepareView {
 	my $self = shift;
-	$self->SUPER::prepareView;
+	$self->next::method;
 	$self->{_frontPageTemplate} =
 	    WebGUI::Asset::Template->new($self->session, $self->get('frontPageTemplateId'));
     if (!$self->{_frontPageTemplate}) {
@@ -357,12 +479,19 @@ sub prepareView {
 }
 
 #-------------------------------------------------------------------
+
+=head2 processPropertiesFromFormPost 
+
+Extend the master method to propagate view and edit permissions down to the wiki pages.
+
+=cut
+
 sub processPropertiesFromFormPost {
 	my $self = shift;
 	my $groupsChanged =
 	    (($self->session->form->process('groupIdView') ne $self->get('groupIdView'))
 	     or ($self->session->form->process('groupIdEdit') ne $self->get('groupIdEdit')));
-	my $ret = $self->SUPER::processPropertiesFromFormPost(@_);
+	my $ret = $self->next::method(@_);
 	if ($groupsChanged) {
 		foreach my $child (@{$self->getLineage(['children'], {returnObjects => 1})}) {
 			$child->update({ groupIdView => $self->get('groupIdView'),
@@ -373,6 +502,13 @@ sub processPropertiesFromFormPost {
 }
 
 #-------------------------------------------------------------------
+
+=head2 view 
+
+Render the front page of the wiki.
+
+=cut
+
 sub view {
 	my $self = shift;
 	my $i18n = WebGUI::International->new($self->session, "Asset_WikiMaster");
@@ -401,6 +537,13 @@ sub view {
 
 
 #-------------------------------------------------------------------
+
+=head2 www_byKeyword 
+
+Return search results that match the keyword from the form variable C<keyword>.
+
+=cut
+
 sub www_byKeyword {
     my $self = shift;
     my $keyword = $self->session->form->process("keyword");
@@ -410,7 +553,7 @@ sub www_byKeyword {
         keyword         => $keyword,   
         usePaginator    => 1,
         });
-    $p->setBaseUrl($self->getUrl("func=byKeyword"));
+    $p->setBaseUrl($self->getUrl("func=byKeyword;keyword=".$keyword));
     foreach my $assetData (@{$p->getPageData}) {
         my $asset = WebGUI::Asset->newByDynamicClass($self->session, $assetData->{assetId});
         next unless defined $asset;
@@ -430,6 +573,13 @@ sub www_byKeyword {
 
 
 #-------------------------------------------------------------------
+
+=head2 www_mostPopular 
+
+Render a templated page that lists the most popular wiki pages.
+
+=cut
+
 sub www_mostPopular {
 	my $self = shift;
 	my $i18n = WebGUI::International->new($self->session, "Asset_WikiMaster");
@@ -447,6 +597,13 @@ sub www_mostPopular {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_recentChanges 
+
+Render a templated page that shows the most recently changed wiki pages.
+
+=cut
+
 sub www_recentChanges {
 	my $self = shift;
 	my $i18n = WebGUI::International->new($self->session, "Asset_WikiMaster");
@@ -466,6 +623,13 @@ sub www_recentChanges {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_search 
+
+Render a search form and process the contents, returning the results.
+
+=cut
+
 sub www_search {
 	my $self = shift;
 	my $i18n = WebGUI::International->new($self->session, "Asset_WikiMaster");
