@@ -16,6 +16,7 @@ package WebGUI::Asset;
 
 use strict;
 use Carp qw( croak );
+use Scalar::Util qw( weaken );
 
 =head1 NAME
 
@@ -55,11 +56,9 @@ An epoch representing the time this asset was created.
 
 =head3 options
 
-A hash reference that allows passed in options to change how this method behaves.
-
-=head4 skipAutoCommitWorkflows
-
-If this is set to 1 assets that normally autocommit their workflows (like CS Posts) won't do that.
+A hash reference that allows passed in options to change how this method behaves.  Currently,
+these options are passed down to L<addRevision>, and are not actually used by C<addChild>.
+Please see the POD for L<addRevision> for a list of options.
 
 =cut
 
@@ -84,13 +83,43 @@ sub addChild {
 	$properties->{assetId} = $id;
 	$properties->{parentId} = $self->getId;
 	my $temp = WebGUI::Asset->newByPropertyHashRef($self->session,$properties) || croak "Couldn't create a new $properties->{className} asset!";
-	$temp->{_parent} = $self;
+        # Do not set the parent here, since it could be stale and poison the child
+	#$temp->{_parent} = $self;
 	my $newAsset = $temp->addRevision($properties, $now, $options); 
 	$self->updateHistory("added child ".$id);
 	$self->session->http->setStatus(201,"Asset Creation Successful");
 	return $newAsset;
 }
 
+#-------------------------------------------------------------------
+
+=head2 cacheChild ( [first|last], asset? )
+
+A cache is kept of the first and last child assets in several cases.  In order
+to avoid memory leaks, these references must be weak, and the child assets
+must have a _parent reference to avoid early collection.  cacheChild maintains
+this delicate state, and so should be called instead of setting this cache
+directly.
+
+If called without an asset argument, the cached child is simply returned.
+
+=cut
+
+sub cacheChild {
+	my ($self, $which, $child) = @_;
+	my $slot = "_${which}Child";
+
+	if ($child) {
+		$self->{$slot} = $child;
+		$child->{_parent} = $self;
+		weaken($self->{$slot});
+	}
+	else {
+		$child = $self->{$slot};
+	}
+
+	return $child;
+}
 
 #-------------------------------------------------------------------
 
@@ -138,14 +167,20 @@ Swaps lineage with sister below. Returns 1 if there is a sister to swap. Otherwi
 
 This will update the lineage of $self, but not the sister.
 
+=head3 outputSub
+
+A reference to a subroutine that output messages should be sent to.  Typically this would
+go to ProgressBar.
+
 =cut
 
 sub demote {
-	my $self = shift;
+	my $self      = shift;
+    my $outputSub = shift || sub {};
 	my ($sisterLineage) = $self->session->db->quickArray("select min(lineage) from asset 
 		where parentId=? and state='published' and lineage>?",[$self->get('parentId'), $self->get('lineage')]);
 	if (defined $sisterLineage) {
-		$self->swapRank($sisterLineage);
+		$self->swapRank($sisterLineage, undef, $outputSub);
 		$self->{_properties}{lineage} = $sisterLineage;
 		return 1;
 	}
@@ -223,7 +258,8 @@ Returns the highest rank, top of the highest rank Asset under current Asset.
 
 sub getFirstChild {
 	my $self = shift;
-	unless (exists $self->{_firstChild}) {
+	my $child = $self->cacheChild('first');
+	unless ($child) {
 		my $assetLineage = $self->session->stow->get("assetLineage");
 		my $lineage = $assetLineage->{firstChild}{$self->getId};
 		unless ($lineage) {
@@ -233,9 +269,10 @@ sub getFirstChild {
 				$self->session->stow->set("assetLineage", $assetLineage);
 			}
 		}
-		$self->{_firstChild} = WebGUI::Asset->newByLineage($self->session,$lineage);
+		$child = WebGUI::Asset->newByLineage($self->session,$lineage);
+		$self->cacheChild(first => $child);
 	}
-	return $self->{_firstChild};
+	return $child;
 }
 
 
@@ -249,7 +286,8 @@ Returns the lowest rank, bottom of the lowest rank Asset under current Asset.
 
 sub getLastChild {
 	my $self = shift;
-	unless (exists $self->{_lastChild}) {
+	my $child = $self->cacheChild('last');
+	unless ($child) {
 		my $assetLineage = $self->session->stow->get("assetLineage");
 		my $lineage = $assetLineage->{lastChild}{$self->getId};
 		unless ($lineage) {
@@ -257,9 +295,10 @@ sub getLastChild {
 			$assetLineage->{lastChild}{$self->getId} = $lineage;
 			$self->session->stow->set("assetLineage", $assetLineage);
 		}
-		$self->{_lastChild} = WebGUI::Asset->newByLineage($self->session,$lineage);
+		$child = WebGUI::Asset->newByLineage($self->session,$lineage);
+		$self->cacheChild(last => $child);
 	}
-	return $self->{_lastChild};
+	return $child;
 }
 
 #-------------------------------------------------------------------
@@ -377,11 +416,14 @@ sub getLineage {
 		}
 		# since we have the relatives info now, why not cache it
 		if ($rules->{returnObjects}) {
-			my $parent = $relativeCache{$parentId};
 			$relativeCache{$id} = $asset;
-			$asset->{_parent} = $parent if exists $relativeCache{$parentId};
-			$parent->{_firstChild} = $asset unless(exists $parent->{_firstChild});
-			$parent->{_lastChild} = $asset;
+			if (my $parent = $relativeCache{$parentId}) {
+				$asset->{_parent} = $parent; 
+				unless ($parent->cacheChild('first')) {
+					$parent->cacheChild(first => $asset);
+				}
+				$parent->cacheChild(last => $asset);
+			}
 		}
 		push(@lineage,$asset);
 	}
@@ -742,9 +784,9 @@ Returns 1 or the count of Assets with the same parentId as current Asset's asset
 sub hasChildren {
 	my $self = shift;
 	unless (exists $self->{_hasChildren}) {
-		if (exists $self->{_firstChild}) {
+		if ($self->cacheChild('first')) {
 			$self->{_hasChildren} = 1;
-		} elsif (exists $self->{_lastChild}) {
+		} elsif ($self->cacheChild('last')) {
 			$self->{_hasChildren} = 1;
 		} else {
 			$self->{_hasChildren} = $self->getChildCount;
@@ -779,6 +821,11 @@ sub newByLineage {
 	$class = $assetLineage->{$lineage}{class};
     unless ($id && $class) {
         ($id,$class) = $session->db->quickArray("select assetId, className from asset where lineage=?",[$lineage]);
+        if (!$id || !$class) {
+            $session->errorHandler->error("Couldn't instantiate asset from lineage: ".$lineage. ": class name or assetId missing");
+            return undef;
+        }
+        return undef if !$id || !$class;
         $assetLineage->{$lineage}{id} = $id;
         $assetLineage->{$lineage}{class} = $class;
         $session->stow->set("assetLineage",$assetLineage);
@@ -789,20 +836,26 @@ sub newByLineage {
 
 #-------------------------------------------------------------------
 
-=head2 promote ( )
+=head2 promote ( [ $outputSub ] )
 
 Keeps the same rank of lineage, swaps with sister above. Returns 1 if there is a sister to swap. Otherwise returns 0.
 
 This will update the lineage of $self, but not the sister.
 
+=head3 outputSub
+
+A reference to a subroutine that output messages should be sent to.  Typically this would
+go to ProgressBar.
+
 =cut
 
 sub promote {
-	my $self = shift;
+	my $self      = shift;
+    my $outputSub = shift || sub {};
 	my ($sisterLineage) = $self->session->db->quickArray("select max(lineage) from asset 
 		where parentId=? and state='published' and lineage<?",[$self->get("parentId"), $self->get("lineage")]);
 	if (defined $sisterLineage) {
-		$self->swapRank($sisterLineage);
+		$self->swapRank($sisterLineage, undef, $outputSub);
 		$self->{_properties}{lineage} = $sisterLineage;
 		return 1;
 	}
@@ -829,7 +882,6 @@ sub setParent {
 	return 0 if ($newParent->getId eq $self->get("parentId")); # don't move it to where it already is
 	return 0 if ($newParent->getId eq $self->getId); # don't move it to itself
     my $oldLineage = $self->get("lineage");
-    return 0 unless $newParent->canEdit;
     my $lineage = $newParent->get("lineage").$newParent->getNextChildRank; 
     return 0 if ($lineage =~ m/^$oldLineage/); # can't move it to its own child
     $self->session->db->beginTransaction;
@@ -847,7 +899,7 @@ sub setParent {
 
 #-------------------------------------------------------------------
 
-=head2 setRank ( newRank )
+=head2 setRank ( newRank, [ $outputSub ] )
 
 Returns 1. Changes rank of Asset.
 
@@ -855,11 +907,17 @@ Returns 1. Changes rank of Asset.
 
 Value of new Rank.
 
+=head3 outputSub
+
+A reference to a subroutine that output messages should be sent to.  Typically this would
+go to ProgressBar, and it must handle doing sprintf'ed i18n calls.
+
 =cut
 
 sub setRank {
-	my $self = shift;
-	my $newRank = shift;
+	my $self      = shift;
+	my $newRank   = shift;
+    my $outputSub = shift || sub {};
 	my $currentRank = $self->getRank;
 	return 1 if ($newRank == $currentRank); # do nothing if we're moving to ourself
 	my $parentLineage = $self->getParentLineage;
@@ -870,13 +928,16 @@ sub setRank {
 	my $temp = substr($self->session->id->generate(),0,6);
 	my $previous = $self->get("lineage");
 	$self->session->db->beginTransaction;
+    $outputSub->('moving %s aside', $self->getTitle);
 	$self->cascadeLineage($temp);
 	foreach my $sibling (@{$siblings}) {
 		if (isBetween($sibling->getRank, $newRank, $currentRank)) {
+            $outputSub->('moving %s', $sibling->getTitle);
 			$sibling->cascadeLineage($previous);
 			$previous = $sibling->get("lineage");
 		}
 	}
+    $outputSub->('moving %s back', $self->getTitle);
 	$self->cascadeLineage($previous,$temp);
 	$self->{_properties}{lineage} = $previous;
 	$self->session->db->commit;
@@ -899,13 +960,16 @@ no in the objects.
 =cut
 
 sub swapRank {
-	my $self = shift;
-	my $second = shift;
-	my $first = shift || $self->get("lineage");
+	my $self      = shift;
+	my $second    = shift;
+	my $first     = shift || $self->get("lineage");
+    my $outputSub = shift || sub {};
 	my $temp = substr($self->session->id->generate(),0,6); # need a temp in order to do the swap
 	$self->session->db->beginTransaction;
+    $outputSub->('swap first');  ##Note, i18n call passed in from caller-1
 	$self->cascadeLineage($temp,$first);
 	$self->cascadeLineage($first,$second);
+    $outputSub->('swap second');
 	$self->cascadeLineage($second,$temp);
 	$self->session->db->commit;
 	$self->updateHistory("swapped lineage between ".$first." and ".$second);
@@ -922,10 +986,15 @@ Demotes self and returns www_view method of getContainer of self if canEdit, oth
 =cut
 
 sub www_demote {
-	my $self = shift;
-	return $self->session->privilege->insufficient() unless $self->canEdit;
-	$self->demote;
-	return $self->session->asset($self->getContainer)->www_view; 
+    my $self    = shift;
+    my $session = $self->session;
+    return $self->session->privilege->insufficient() unless $self->canEdit;
+    my $i18n    = WebGUI::International->new($session, 'Asset');
+    my $pb      = WebGUI::ProgressBar->new($session);
+    $pb->start($i18n->get('demote'), $session->url->extras('adminConsole/assets.gif'));
+    $pb->update(sprintf $i18n->get('demote %s'), $self->getTitle);
+    $self->demote(sub{ $pb->update($i18n->get(shift))});
+    $pb->finish($self->getContainer->getUrl);
 }
 
 
@@ -938,10 +1007,15 @@ Returns www_view method of getContainer of self. Promotes self. If canEdit is Fa
 =cut
 
 sub www_promote {
-	my $self = shift;
-	return $self->session->privilege->insufficient() unless $self->canEdit;
-	$self->promote;
-	return $self->session->asset($self->getContainer)->www_view;
+    my $self    = shift;
+    my $session = $self->session;
+    return $self->session->privilege->insufficient() unless $self->canEdit;
+    my $i18n    = WebGUI::International->new($session, 'Asset');
+    my $pb      = WebGUI::ProgressBar->new($session);
+    $pb->start($i18n->get('promote'), $session->url->extras('adminConsole/assets.gif'));
+    $pb->update(sprintf $i18n->get('promote %s'), $self->getTitle);
+    $self->promote(sub{ $pb->update($i18n->get(shift))});
+    $pb->finish($self->getContainer->getUrl);
 }
 
 
@@ -982,6 +1056,45 @@ sub www_setRank {
 	return $self->getParent->www_manageAssets();
 }
 
+#-------------------------------------------------------------------
+
+=head2 www_setRanks ( )
+
+Utility method for the AssetManager.  Reorders 1 pagefull of assets via rank.
+AssetIds are passed in via the C<assetId> form variable.
+
+If the current user cannot edit the current asset, or if a valid CSRF token
+is not submitted with the form, it returns the insufficient privileges page.
+
+Returns the user to the manage assets screen.
+
+=cut
+
+sub www_setRanks {
+    my $self    = shift;
+    my $session = $self->session;
+    return $session->privilege->insufficient() unless $session->asset->canEdit && $session->form->validToken;
+    my $i18n    = WebGUI::International->new($session, 'Asset');
+    my $pb      = WebGUI::ProgressBar->new($session);
+    my $form    = $session->form;
+
+    $pb->start($i18n->get('Set Rank'), $session->url->extras('adminConsole/assets.gif'));
+    my @assetIds    = $form->get( 'assetId' );
+    ASSET: for my $assetId ( @assetIds ) {
+        my $asset  = WebGUI::Asset->newByDynamicClass( $session, $assetId );
+        next ASSET unless $asset;
+        my $rank   = $form->get( $assetId . '_rank' );
+        next ASSET unless $rank; # There's no such thing as zero
+
+        $asset->setRank( $rank, sub { $pb->update(sprintf $i18n->get(shift), shift); } );
+    }
+
+    $pb->finish($session->asset->getManagerUrl);
+    return "redirect";
+    #return $www_manageAssets();
+}
+
+
 
 1;
 
