@@ -92,7 +92,7 @@ sub addRevision {
 
     my $autoCommitId     = $self->getAutoCommitWorkflowId() unless ($options->{skipAutoCommitWorkflows});
 
-    my $workingTag;
+    my ($workingTag, $oldWorking);
     if ( $autoCommitId ) {
         $workingTag  
             = WebGUI::VersionTag->create( $self->session, { 
@@ -101,7 +101,18 @@ sub addRevision {
             } ); 
     }
     else {
-        $workingTag = WebGUI::VersionTag->getWorking($self->session);
+        my $parentAsset;
+        if ( not defined( $parentAsset = $self->getParent ) ) {
+            $parentAsset = WebGUI::Asset->newPending( $self->session, $self->get('parentId') );
+        }
+        if ( $parentAsset->hasBeenCommitted ) {
+            $workingTag = WebGUI::VersionTag->getWorking( $self->session );
+        }
+        else {
+            my $oldWorking = WebGUI::VersionTag->getWorking($self->session, 'noCreate');
+            $workingTag = WebGUI::VersionTag->new( $self->session, $parentAsset->get('tagId') );
+            $workingTag->setWorking();
+        }
     }
     
     #Create a dummy revision to be updated with real data later
@@ -126,7 +137,7 @@ sub addRevision {
 		# get the default values of each property
 		foreach my $property (keys %{$definition->{properties}}) {
 			$defaults{$property} = $definition->{properties}{$property}{defaultValue};
-            if (ref($defaults{$property}) eq 'ARRAY') {
+            if (ref($defaults{$property}) eq 'ARRAY' && !$definition->{properties}{$property}{serialize}) {
                 $defaults{$property} = $defaults{$property}->[0];
             }
 		}
@@ -144,6 +155,9 @@ sub addRevision {
 	# merge the defaults, current values, and the user set properties
 	my %mergedProperties = (%defaults, %{$self->get}, %{$properties}, (status => 'pending'));
     
+    # Force the packed head block to be regenerated
+    delete $mergedProperties{extraHeadTagsPacked};
+
     #Instantiate new revision and fill with real data
     my $newVersion = WebGUI::Asset->new($self->session,$self->getId, $self->get("className"), $now);
     $newVersion->setSkipNotification if ($options->{skipNotification});
@@ -151,6 +165,7 @@ sub addRevision {
     $newVersion->setVersionLock;
     $newVersion->update(\%mergedProperties);
     $newVersion->setAutoCommitTag($workingTag) if (defined $autoCommitId);
+    $oldWorking->setWorking if $oldWorking;
     
     return $newVersion;
 }
@@ -370,6 +385,41 @@ sub purgeRevision {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 moveAssetToVersionTag ( tag )
+
+=head3 moveToTag
+
+Migrate the current asset to the designated version tag
+
+=cut
+
+sub moveAssetToVersionTag {
+    my ( $self, $moveToTag ) = @_;
+
+    # Determine if we were passed a version tagId or a VersionTag Class and act appropriately
+    #
+    my $moveToTagId = $moveToTag;
+    if ( ref($moveToTag) eq "WebGUI::VersionTag" ) {
+        $moveToTagId = $moveToTag->get('tagId');
+    }
+    else {
+        $moveToTag = WebGUI::VersionTag->new( $self->session, $moveToTagId );
+    }
+
+    my $tag = WebGUI::VersionTag->new( $self->session, $self->get('tagId') );
+
+    $self->setVersionTag($moveToTagId);
+
+    my $versionTag = $self->session->db->quickScalar("SELECT tagId FROM assetData WHERE assetId=? AND revisionDate=?",[$self->getId,$self->get('revisionDate')]);
+    
+    # If no revisions remain, delete the version tag
+    if ( $tag->getRevisionCount <= 0 ) {
+        $tag->rollback;
+    }
+} ## end sub moveAssetToVersionTag
+
 #-------------------------------------------------------------------
 
 =head2 requestAutoCommit ( )
@@ -379,13 +429,29 @@ Requests an autocommit tag be commited. See also getAutoCommitWorkflowId() and s
 =cut
 
 sub requestAutoCommit {
-	my $self = shift;
-	my $tag = $self->{_autoCommitTag};
-	if (defined $tag) {
-		$tag->requestCommit;
-		delete $self->{_autoCommitTag};
-	}
-}
+    my $self = shift;
+
+    my $parentAsset;
+    if ( not defined( $parentAsset = $self->getParent ) ) {
+        $parentAsset = WebGUI::Asset->newPending( $self->session, $self->get('parentId') );
+    }
+    unless ( $parentAsset->hasBeenCommitted ) {
+        my $tagId = $parentAsset->get('tagId');
+
+        if ($tagId) {
+            if ( $tagId ne $self->get('tagId') ) {
+                $self->moveAssetToVersionTag($tagId);
+                return;
+            }
+        }
+    }
+
+    my $tag = $self->{_autoCommitTag};
+    if ( defined $tag ) {
+        $tag->requestCommit;
+        delete $self->{_autoCommitTag};
+    }
+} ## end sub requestAutoCommit
 
 
 #-------------------------------------------------------------------
@@ -428,10 +494,11 @@ Sets the versioning lock to "on" so that this piece of content may not be edited
 =cut
 
 sub setVersionLock {
-	my $self = shift;
-	$self->session->db->write("update asset set isLockedBy=? where assetId=?", [$self->session->user->userId, $self->getId]);
-	$self->updateHistory("locked");
-	$self->purgeCache;
+    my $self = shift;
+    $self->session->db->write("update asset set isLockedBy=? where assetId=?", [$self->session->user->userId, $self->getId]);
+    $self->{_properties}{isLockedBy} = $self->session->user->userId;
+    $self->updateHistory("locked");
+    $self->purgeCache;
 }
 
 #-------------------------------------------------------------------
@@ -450,6 +517,7 @@ sub setVersionTag {
 	my $self = shift;
     my $tagId = shift;
     $self->session->db->write("update assetData set tagId=? where assetId=? and tagId = ?", [$tagId, $self->getId,$self->get('tagId')]);
+        $self->{_properties}{tagId} = $tagId;
 	$self->updateHistory("changed version tag to $tagId");
 	$self->purgeCache;
 }
@@ -479,10 +547,11 @@ Sets the versioning lock to "off" so that this piece of content may be edited on
 =cut
 
 sub unsetVersionLock {
-	my $self = shift;
-	$self->session->db->write("update asset set isLockedBy=NULL where assetId=".$self->session->db->quote($self->getId));
-	$self->updateHistory("unlocked");
-	$self->purgeCache;
+    my $self = shift;
+    $self->session->db->write("update asset set isLockedBy=NULL where assetId=?",[$self->getId]);
+    $self->{_properties}{isLockedBy} = undef;
+    $self->updateHistory("unlocked");
+    $self->purgeCache;
 }
 
 
