@@ -242,7 +242,7 @@ sub createAccount {
             $formField   = $field->formField($properties, undef, undef, undef, $defaultValue);
         }
         else {
-            $formField   = $field->formField($properties);
+            $formField   = $field->formField($properties, undef, undef, undef, undef, undef, 'useFormDefault');
         }
        
 
@@ -315,11 +315,14 @@ sub createAccountSave {
     $self->saveParams($userId,$self->authMethod,$properties);
 
 	if ($self->getSetting("sendWelcomeMessage")){
-		my $authInfo = "\n\n".$i18n->get(50).": ".$username;
-		$authInfo .= "\n".$i18n->get(51).": ".$password if($password);
-		$authInfo .= "\n\n";
-		WebGUI::Inbox->new($self->session)->addMessage({
-			message	=> $self->getSetting("welcomeMessage").$authInfo,
+        my $var;
+        $var->{welcomeMessage}      = $self->getSetting("welcomeMessage");
+        $var->{newUser_username}    = $username;
+        $var->{newUser_password}    = $password;
+        my $message = WebGUI::Asset::Template->new($self->session,$self->getSetting('welcomeMessageTemplate'))->process($var);
+        WebGUI::Macro::process($self->session,\$message);
+        WebGUI::Inbox->new($self->session)->addMessage({
+            message => $message,
 			subject	=> $i18n->get(870),
 			userId	=> $self->userId,
             status  => 'completed',
@@ -698,8 +701,8 @@ Returns a hash reference with the user's authentication information.  This metho
 =cut
 
 sub getParams {
-	my $self = shift;
-	my $userId = $_[0] || $self->userId;
+	my $self       = shift;
+	my $userId     = $_[0] || $self->userId;
 	my $authMethod = $_[1] || $self->authMethod;
 	return $self->session->db->buildHashRef("select fieldName, fieldData from authentication where userId=".$self->session->db->quote($userId)." and authMethod=".$self->session->db->quote($authMethod));
 }
@@ -833,6 +836,9 @@ sub login {
         ) {
             return $self->showMessageOnLogin;
         }
+        elsif ( $self->session->setting->get("redirectAfterLoginUrl") ) {
+        $self->session->http->setRedirect($self->session->setting->get("redirectAfterLoginUrl"));
+        }
         elsif ( $self->session->form->get('returnUrl') ) {
 		$self->session->http->setRedirect( $self->session->form->get('returnUrl') );
 	  	$self->session->scratch->delete("redirectAfterLogin");
@@ -1011,6 +1017,7 @@ sub showMessageOnLogin {
 
     # Add the link to continue
     my $redirectUrl =  $self->session->form->get( 'returnUrl' )
+                    || $self->session->setting->get("redirectAfterLoginUrl")
                     || $self->session->scratch->get( 'redirectAfterLogin' )
                     || $self->session->url->getSiteURL . $self->session->url->gateway()
                     ;
