@@ -40,9 +40,11 @@ i.e., it does not validate their username or ensure their account is active.
 =cut
 
 sub _isValidLDAPUser {
-    my $self = shift;
+    my $self    = shift;
+    my $session = $self->session;
+    my $form    = $session->form;
     my ($error, $ldap, $search, $auth, $connectDN);
-    my $i18n = WebGUI::International->new($self->session);
+    my $i18n = WebGUI::International->new($session);
 
     my $connection = $self->getLDAPConnection;
     return 0 unless $connection;
@@ -53,8 +55,8 @@ sub _isValidLDAPUser {
         $self->error('<li>'.$i18n->get(2,'AuthLDAP').'</li>');
         return 0;
     }
-    my $username = $self->session->form->get("authLDAP_ldapId") || $self->session->form->get("username");
-    my $password = $self->session->form->get("authLDAP_identifier") || $self->session->form->get("identifier");
+    my $username = $form->get("authLDAP_ldapId")     || $form->get("username");
+    my $password = $form->get("authLDAP_identifier") || $form->get("identifier");
 
     my $uri = URI->new($connection->{ldapUrl}) or $error = '<li>'.$i18n->get(2,'AuthLDAP').'</li>';
 
@@ -64,7 +66,7 @@ sub _isValidLDAPUser {
     }
 
     # Create an LDAP object
-    if ($ldap = Net::LDAP->new($uri->host, (port=>$uri->port))) {
+    if ($ldap = Net::LDAP->new($uri->host, (port=>$uri->port,scheme=>$uri->scheme))) {
 
         # Bind as a proxy user to search for the user trying to login
         if($connection->{connectDn}) {
@@ -102,27 +104,27 @@ sub _isValidLDAPUser {
                 # Invalid login credentials, directory did not authenticate the user
                 if ($auth->code == 48 || $auth->code == 49) {
                     $error .= '<li>'.$i18n->get(68).'</li>';
-                    $self->session->errorHandler->warn("Invalid LDAP information for registration of LDAP ID: ".$self->session->form->process('authLDAP_ldapId'));
+                    $session->log->warn("Invalid LDAP information for registration of LDAP ID: ".$self->session->form->process('authLDAP_ldapId'));
                 }
                 elsif ($auth->code > 0) {  # Some other LDAP error occured
                     $error .= '<li>LDAP error "'.$self->ldapStatusCode($auth->code).'" occured. '.$i18n->get(69).'</li>';
-                    $self->session->errorHandler->error("LDAP error: ".$self->ldapStatusCode($auth->code));
+                    $session->log->error("LDAP error: ".$self->ldapStatusCode($auth->code));
                 }
                 $ldap->unbind;
             }
             else { # Could not find the user in the directory to build a DN
                 $error .= '<li>'.$i18n->get(68).'</li>';
-                $self->session->errorHandler->warn("Invalid LDAP information for registration of LDAP ID: ".$self->session->form->process("authLDAP_ldapId"));
+                $session->log->warn("Invalid LDAP information for registration of LDAP ID: ".$self->session->form->process("authLDAP_ldapId"));
             }
         }
         else { # Unable to bind with proxy user credentials or anonymously for our search
             $error = '<li>'.$i18n->get(2,'AuthLDAP').'</li>';
-            $self->session->errorHandler->error("Couldn't bind to LDAP server: ".$connection->{ldapUrl});
+            $session->log->error("Couldn't bind to LDAP server: ".$connection->{ldapUrl});
         }
    }
    else { # Could not create our LDAP object
       $error = '<li>'.$i18n->get(2,'AuthLDAP').'</li>';
-      $self->session->errorHandler->error("Couldn't create LDAP object: ".$connection->{ldapUrl});
+      $session->log->error("Couldn't create LDAP object: ".$connection->{ldapUrl});
    }
 
    $self->error($error);
@@ -138,8 +140,7 @@ sub _isValidLDAPUser {
 This method checks a given username and password for the following:
 
 1) Is there a WebGUI user account for the user and is it active
-2) Does the user account have the properties set necessary to authenticate using
-   LDAP.
+2) Does the user account have the properties set necessary to authenticate using LDAP.
 3) Can we bind to the LDAP server using their account information
 
 Returns 1 on success.
@@ -177,21 +178,32 @@ sub authenticate {
 
         # Try to bind using the users dn and password
         $auth = $ldap->bind(dn=>$userData->{connectDN}, password=>$identifier);
+
+        # Failure to bind could have resulted from change to in DN on LDAP server.
+        # Test for new DN and update user account as needed
+        if ($auth->code > 0 && $self->_isValidLDAPUser()) {
+            # Update user profile and log change
+            # _isValidLDAPUser will set _connectDN to new correct value
+            $auth = $ldap->bind(dn=>$self->{_connectDN}, password=>$identifier);
+            my $message = "DN has been changed for user ".$_[0]." from \"".$userData->{connectDN}."\" to \"".$self->{_connectDN}."\"";
+            $self->saveParams($self->user->userId, $self->authMethod, { connectDN => $self->{_connectDN} });
+            $self->session->errorHandler->warn($message);
+        }
         
         # Authentication failed
-        if ($auth->code == 48 || $auth->code == 49){
+        if ($auth->code == 48 || $auth->code == 49 || $auth->code == 32){
             $error .= $self->SUPER::authenticationError;
         }
         elsif ($auth->code > 0) { # Some other LDAP error happened
             $error .= '<li>LDAP error "'.$self->ldapStatusCode($auth->code).'" occured.'.$i18n->get(69).'</li>';
-            $self->session->errorHandler->error("LDAP error: ".$self->ldapStatusCode($auth->code));
+            $self->session->log->error("LDAP error: ".$self->ldapStatusCode($auth->code));
         }
 
         $ldap->unbind;
     }
     else { 
         $error .= '<li>'.$i18n->get(13,'AuthLDAP').'</li>';
-        $self->session->errorHandler->error("Could not process this LDAP URL: ".$userData->{ldapUrl});
+        $self->session->log->error("Could not process this LDAP URL: ".$userData->{ldapUrl});
     }
 
     if($error ne ""){
@@ -203,6 +215,13 @@ sub authenticate {
 }
 
 #-------------------------------------------------------------------
+
+=head2 connectToLDAP ( ) 
+
+Create a new WebGUI::LDAPLink connection, if possible.
+
+=cut
+
 sub connectToLDAP {
 
     # This method needs to do some excpetion handling when we try to create an LDAPLink object
@@ -224,6 +243,13 @@ sub connectToLDAP {
 }
 
 #-------------------------------------------------------------------
+
+=head2 createAccount ( message, confirm ) 
+
+Show the form to create a new LDAP account relationship
+
+=cut
+
 sub createAccount {
     my $self = shift;
 	my $message = shift;
@@ -274,6 +300,13 @@ sub createAccount {
 }
 
 #-------------------------------------------------------------------
+
+=head2 createAccountSave (  )
+
+Process the form to create a new LDAP account relationship
+
+=cut
+
 sub createAccountSave {
     my $self = shift;
     my $username = $self->session->form->process('authLDAP_ldapId');
@@ -292,7 +325,7 @@ sub createAccountSave {
     }
     #Get connectDN from settings   
     my $uri = URI->new($connection->{ldapUrl});
-    my $ldap = Net::LDAP->new($uri->host, (port=>$uri->port));
+    my $ldap = Net::LDAP->new($uri->host, (port=>$uri->port,scheme=>$uri->scheme));
     my $auth;
     if($connection->{connectDn}) {
         $auth = $ldap->bind(dn=>$connection->{connectDn}, password=>$connection->{identifier});
@@ -334,6 +367,13 @@ sub createAccountSave {
 }
 
 #-------------------------------------------------------------------
+
+=head2 deactivateAccount ( )
+
+Show the confirmation form to deactivate the user's account
+
+=cut
+
 sub deactivateAccount {
     my $self = shift;
     return $self->displayLogin if($self->userId eq '1');
@@ -341,6 +381,13 @@ sub deactivateAccount {
 }
 
 #-------------------------------------------------------------------
+
+=head2 deactivateAccountConfirm ( )
+
+Confirm the user is deactivating their account.
+
+=cut
+
 sub deactivateAccountConfirm {
    my $self = shift;
    return $self->displayLogin unless ($self->session->setting->get("selfDeactivation"));
@@ -348,6 +395,13 @@ sub deactivateAccountConfirm {
 }
 
 #-------------------------------------------------------------------
+
+=head2 displayAccount ( message ) 
+
+Display the account details.
+
+=cut
+
 sub displayAccount {
     my $self = shift;
     my $vars;
@@ -375,6 +429,13 @@ sub displayAccount {
 }
 
 #-------------------------------------------------------------------
+
+=head2 displayLogin ( message )
+
+Web-facing method to display the login form.
+
+=cut
+
 sub displayLogin {
    my $self = shift;
    my $vars;
@@ -485,6 +546,13 @@ sub editUserSettingsForm {
 }
 
 #-------------------------------------------------------------------
+
+=head2 editUserSettingsFormSave ( )
+
+Process the editUserSettingsForm
+
+=cut
+
 sub editUserSettingsFormSave {
    my $self = shift;
    my $f = $self->session->form;
@@ -493,6 +561,13 @@ sub editUserSettingsFormSave {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getAccountTemplateId ( )
+
+Get the templateId of the template to show the user's account details
+
+=cut
+
 sub getAccountTemplateId {
     my $self = shift;
     my $ldapConnect = $self->getLDAPConnection;
@@ -501,12 +576,26 @@ sub getAccountTemplateId {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getConnectDN ( )
+
+Get the Distinguished Name used to connect to the LDAP server.
+
+=cut
+
 sub getConnectDN {
    my $self = shift;
    return $self->{_connectDN};
 }
 
 #-------------------------------------------------------------------
+
+=head2 getCreateAccountTemplateId ( )
+
+Get the templateId to create a new LDAP account relationship
+
+=cut
+
 sub getCreateAccountTemplateId {
     my $self = shift;
     my $ldapConnect = $self->getLDAPConnection;
@@ -515,6 +604,13 @@ sub getCreateAccountTemplateId {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getLDAPConnection ( )
+
+Get the properties of the LDAP connection we're using for Auth.
+
+=cut
+
 sub getLDAPConnection {
    my $self = shift;
 
@@ -523,6 +619,13 @@ sub getLDAPConnection {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getLDAPLink ( )
+
+Get the WebGUI::LDAPLink for this auth connection
+
+=cut
+
 sub getLDAPLink {
    my $self = shift;
    
@@ -530,6 +633,13 @@ sub getLDAPLink {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getLoginTemplateId ( )
+
+Get the template ID for the login form.
+
+=cut
+
 sub getLoginTemplateId {
     my $self = shift;
     my $ldapConnect = $self->getLDAPConnection;
@@ -538,11 +648,18 @@ sub getLoginTemplateId {
 }
 
 #-------------------------------------------------------------------
+
+=head2 login ( )
+
+Process the login form. Create a new account if auto registration is enabled.
+
+=cut
+
 sub login {
     my $self = shift;
     my $i18n = WebGUI::International->new($self->session);
-    my $username = $self->session->form->process("username");
-    my $identifier = $self->session->form->process("identifier");
+    my $username         = $self->session->form->process("username");
+    my $identifier       = $self->session->form->process("identifier");
     my $autoRegistration = $self->session->setting->get("automaticLDAPRegistration");
     my $hasAuthenticated = 0;
 
@@ -580,11 +697,19 @@ sub login {
    }
    return $self->SUPER::login() if $hasAuthenticated;  #Standard login routine for login
 
-   $self->session->errorHandler->security("login to account ".$self->session->form->process("username")." with invalid information.");
+   $self->session->log->security("login to account ".$self->session->form->process("username")." with invalid information.");
    return $self->displayLogin("<h1>".$i18n->get(70)."</h1>".$self->error);
 }
 
 #-------------------------------------------------------------------
+
+=head2 new ( session, authMethod, userId )
+
+Create a new Auth instance. C<authMethod> is the name of this auth method ("ldap").
+C<userId> is the ID of the user to be authenticated.
+
+=cut
+
 sub new {
    my $class = shift;
 	my $session = shift;
@@ -604,12 +729,26 @@ sub new {
 }
 
 #-------------------------------------------------------------------
+
+=head2 ldapStatusCode ( code )
+
+Get the friendly i18n message for the given LDAP status code.
+
+=cut
+
 sub ldapStatusCode {
    my ($self, $code) = @_;
    return $self->{_statusCode}->{$code};
 }
 
 #-------------------------------------------------------------------
+
+=head2 setConnectDN ( connectDn ) 
+
+Set a distinguished name to connect with
+
+=cut
+
 sub setConnectDN {
    my $self = shift;
    $self->{_connectDN} = $_[0];
