@@ -162,14 +162,16 @@ sub createAccount {
    		$vars->{'create.form.captcha.label'} = $i18n->get("captcha label","AuthWebGUI");
 	}
 
-    my $username = $form->process("authWebGUI.username");
-   $vars->{'create.form.username'} 
-        = WebGUI::Form::text($self->session, {
-            name   => "authWebGUI.username",
-            value  => $username,
-            extras => $self->getExtrasStyle($username)
-        });
-    $vars->{'create.form.username.label'} = $i18n->get(50);
+    unless($setting->get('webguiUseEmailAsUsername')){   
+        my $username = $form->process("authWebGUI.username");
+        $vars->{'create.form.username'} 
+            = WebGUI::Form::text($self->session, {
+                name   => "authWebGUI.username",
+                value  => $username,
+                extras => $self->getExtrasStyle($username)
+            });
+        $vars->{'create.form.username.label'} = $i18n->get(50);
+    }
     
     my $password = $form->process("authWebGUI.identifier");
     $vars->{'create.form.password'}
@@ -216,7 +218,13 @@ sub createAccountSave {
         $session->errorHandler->security($i18n->get("no registration hack", "AuthWebGUI"));
         return $self->displayLogin;
     }
-    my $username    = $form->process('authWebGUI.username');
+    my $username;
+    if($setting->get('webguiUseEmailAsUsername')){
+        $username    = $form->process('email');
+    }
+    else{
+        $username    = $form->process('authWebGUI.username');
+    }
     my $password    = $form->process('authWebGUI.identifier');
     my $passConfirm = $form->process('authWebGUI.identifierConfirm');
    
@@ -272,10 +280,13 @@ sub createAccountSave {
             to      => $profile->{email},
             subject => $i18n->get('email address validation email subject','AuthWebGUI')
         });
-        $mail->addText(
-            $i18n->get('email address validation email body','AuthWebGUI') . "\n\n"
-            . $session->url->page("op=auth;method=validateEmail;key=".$key, 'full') . "\n\n"
-        );
+        my $var;
+        $var->{newUser_username} = $username;
+        $var->{activationUrl} = $session->url->page("op=auth;method=validateEmail;key=".$key, 'full');
+        my $text =
+WebGUI::Asset::Template->new($self->session,$self->getSetting('accountActivationTemplate'))->process($var);
+        WebGUI::Macro::process($self->session,\$text);
+        $mail->addText($text);
         $mail->addFooter;
         $mail->send;
         $self->user->status("Deactivated");
@@ -493,12 +504,18 @@ sub editUserSettingsForm {
         -label     => $i18n->get(868,'WebGUI'),
         -hoverHelp => $i18n->get('868 help','WebGUI'),
     );
-    $f->textarea(
+    $f->HTMLArea(
         -name      => "webguiWelcomeMessage",
         -value     => $self->session->setting->get("webguiWelcomeMessage"),
         -label     => $i18n->get(869,'WebGUI'),
         -hoverHelp => $i18n->get('869 help','WebGUI'),
     );
+    $f->yesNo(
+        -name      => "webguiUseEmailAsUsername",
+        -value     => $self->session->setting->get("webguiUseEmailAsUsername"),
+        -label     => $i18n->get('use email as username label'),
+        -hoverHelp => $i18n->get('use email as username description'),
+    );
     $f->yesNo(
         -name      => "webguiChangeUsername",
         -value     => $self->session->setting->get("webguiChangeUsername"),
@@ -573,7 +590,21 @@ sub editUserSettingsForm {
 		-label     => $i18n->get("password recovery template"),
 		-hoverHelp => $i18n->get("password recovery template help")
     );
-   return $f->printRowsOnly;
+    $f->template(
+        -name      => "webguiWelcomeMessageTemplate",
+        -value     => $self->session->setting->get("webguiWelcomeMessageTemplate"),
+        -namespace => "Auth/WebGUI/Welcome",
+        -label     => $i18n->get("welcome message template"),
+        -hoverHelp => $i18n->get("welcome message template help")
+    );
+    $f->template(
+        -name      => "webguiAccountActivationTemplate",
+        -value     => $self->session->setting->get("webguiAccountActivationTemplate"),
+        -namespace => "Auth/WebGUI/Activation",
+        -label     => $i18n->get("account activation template"),
+        -hoverHelp => $i18n->get("account activation template help")
+    );
+    return $f->printRowsOnly;
 }
 
 #-------------------------------------------------------------------
@@ -593,6 +624,7 @@ sub editUserSettingsFormSave {
 	$s->set("webguiWelcomeMessage", $f->process("webguiWelcomeMessage","textarea"));
 	$s->set("webguiChangeUsername", $f->process("webguiChangeUsername","yesNo"));
 	$s->set("webguiChangePassword", $f->process("webguiChangePassword","yesNo"));
+    $s->set("webguiUseEmailAsUsername", $f->process("webguiUseEmailAsUsername"));
 
     # Make sure we have the ability to recover a password if we're trying to
     # enable password recovery
@@ -624,6 +656,8 @@ sub editUserSettingsFormSave {
 	$s->set("webguiExpiredPasswordTemplate", $f->process("webguiExpiredPasswordTemplate","template"));
 	$s->set("webguiLoginTemplate", $f->process("webguiLoginTemplate","template"));
 	$s->set("webguiPasswordRecoveryTemplate", $f->process("webguiPasswordRecoveryTemplate","template"));
+    $s->set("webguiWelcomeMessageTemplate", $f->process("webguiWelcomeMessageTemplate","template"));
+    $s->set("webguiAccountActivationTemplate", $f->process("webguiAccountActivationTemplate","template")); 
 
     if (@errors) {
         return \@errors;
