@@ -7,6 +7,7 @@ use URI;
 use WebGUI::Form;
 use WebGUI::Paginator;
 use WebGUI::Utility;
+use WebGUI::Macro::AdminBar;
 
 #----------------------------------------------------------------------------
 
@@ -18,17 +19,20 @@ Gets a select box to choose a class name.
 
 sub getClassSelectBox {
     my $session     = shift;
+    my $i18n        = WebGUI::International->new($session, 'Asset');
     
     tie my %classes, "Tie::IxHash", (
-        ""  => "Any Class", 
+        ""  => $i18n->get("Any Class"), 
         $session->db->buildHash("select distinct(className) from asset"),
     );
     delete $classes{"WebGUI::Asset"}; # don't want to search for the root asset
 
+    my $className = $session->scratch->get('assetManagerSearchClassName') || $session->form->process("class","className");
+    $session->scratch->set('assetManagerSearchClassName', $className);
     return WebGUI::Form::selectBox( $session, {
-        name            => "class", 
-        value           => $session->form->process("class","className"), 
-        defaultValue    => "", 
+        name            => "class",
+        value           => $className,
+        defaultValue    => "",
         options         => \%classes,
     });
 }
@@ -131,6 +135,7 @@ sub getSearchPaginator {
     
     my $s       = WebGUI::Search->new( $session, 0 );
     $s->search( {
+        assetIds        => $query->{ assetIds },
         keywords        => $query->{ keywords },
         classes         => $query->{ classes },
     } );
@@ -140,8 +145,10 @@ sub getSearchPaginator {
         $queryString    .= ';class=' . $class;
     }
 
-    my $p           = $s->getPaginatorResultSet( $session->url->page( $queryString ) );
+    my $pageNumber  = $session->form->get('pn') || $session->scratch->get('assetManagerSearchPageNumber');
+    my $p           = $s->getPaginatorResultSet( $session->url->page( $queryString ), undef, $pageNumber );
 
+    $session->scratch->set('assetManagerSearchPageNumber', $pageNumber);
     return $p;
 }
 
@@ -332,47 +339,6 @@ sub www_manage {
     ### Do Action
     my @assetIds    = $session->form->get( 'assetId' );
 
-    if ( $session->form->get( 'action_update' ) ) {
-        for my $assetId ( @assetIds ) {
-            my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-            next unless $asset;
-            my $rank        = $session->form->get( $assetId . '_rank' );
-            next unless $rank; # There's no such thing as zero
-
-            $asset->setRank( $rank );
-        }
-    }
-    elsif ( $session->form->get( 'action_delete' ) ) {
-        for my $assetId ( @assetIds ) {
-            my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-            next unless $asset;
-            $asset->trash;
-        }
-    }
-    elsif ( $session->form->get( 'action_cut' ) ) {
-        for my $assetId ( @assetIds ) {
-            my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-            next unless $asset;
-            $asset->cut;
-        }
-    }
-    elsif ( $session->form->get( 'action_copy' ) ) {
-        for my $assetId ( @assetIds ) {
-            # Copy == Duplicate + Cut
-            my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId);
-            my $newAsset    = $asset->duplicate( { skipAutoCommitWorkflows => 1 } );
-            $newAsset->update( { title => $newAsset->getTitle . ' (copy)' } );
-            $newAsset->cut;
-        }
-    }
-    elsif ( $session->form->get( 'action_duplicate' ) ) {
-        for my $assetId ( @assetIds ) {
-            my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-            next unless $asset;
-            $asset->duplicate( { skipAutoCommitWorkflows => 1 } );
-        }
-    }
-
     # Handle autocommit workflows
     if (WebGUI::VersionTag->autoCommitWorkingIfEnabled($session, {
         allowComments   => 1,
@@ -405,13 +371,11 @@ sub www_manage {
     $session->style->setScript( $session->url->extras( 'yui-webgui/build/assetManager/assetManager.js' ) );
     $session->style->setScript( $session->url->extras( 'yui-webgui/build/form/form.js' ) );
 
-    my $extras      = $session->url->extras;
     $session->style->setRawHeadTags( <<ENDHTML );
     <link type="text/css" rel="stylesheet" href="http://yui.yahooapis.com/2.6.0/build/logger/assets/skins/sam/logger.css">
     <script type="text/javascript" src="http://yui.yahooapis.com/2.6.0/build/logger/logger-min.js"></script> 
 
-<script type="text/javascript">
-        WebGUI.AssetManager.extrasUrl   = '$extras';
+    <script type="text/javascript">
         YAHOO.util.Event.onDOMReady( WebGUI.AssetManager.initManager );
     </script>
 ENDHTML
@@ -438,43 +402,51 @@ ENDHTML
     $output .= '</ol>';
     
     ### The page of assets
-    $output         .= q{<div>}
-                    . q{<form method="post" enctype="multipart/form-data">}
-                    . q{<input type="hidden" name="op" value="assetManager" />}
-                    . q{<input type="hidden" name="method" value="manage" />}
-                    . q{<div id="dataTableContainer">}
-                    . q{</div>} 
-                    . q{<p class="actions">} . $i18n->get( 'with selected' )
-                    . q{<input type="submit" name="action_update" value="} . $i18n->get( "update" ) . q{" />}
-                    . q{<input type="submit" name="action_delete" value="} . $i18n->get( "delete" ) . q{" onclick="return confirm('} . $i18n->get( 43 ) . q{')" />}
-                    . q{<input type="submit" name="action_cut" value="} . $i18n->get( 'cut' ) . q{" />}
-                    . q{<input type="submit" name="action_copy" value="} . $i18n->get( "Copy" ) . q{" />}
-                    . q{<input type="submit" name="action_duplicate" value="} . $i18n->get( "duplicate" ) . q{" />}
-                    . q{</p>}
-                    . q{</form>}
-                    . q{<div id="pagination"> } 
-                    . q{</div>}
-                    . q{</div>}
-                    ;
+    $output         .= sprintf <<EOHTML, $session->asset->getUrl, WebGUI::Form::CsrfToken->new($session)->toHtml, $i18n->get( 'with selected' ), $i18n->get( "update" ), $i18n->get( "delete" ), $i18n->get( '43' ), $i18n->get( 'cut' ), $i18n->get( "Copy" ), $i18n->get( "duplicate" );
+<div>
+<form method="post" enctype="multipart/form-data" action="%s">
+%s
+<input type="hidden" name="func"    value="manageAssets" />
+<input type="hidden" name="proceed" value="manageAssets" />
+<div id="dataTableContainer">
+</div>
+<p class="actions"> %s 
+<input type="submit" name="action_update" value="%s" onclick="this.form.func.value='setRanks'; this.form.submit();" />
+<input type="submit" name="action_delete" value="%s" onclick="if( confirm('%s')){ this.form.func.value='deleteList'; this.form.submit(); }{ return false; }" />
+<input type="submit" name="action_cut" value="%s" onclick="this.form.func.value='cutList'; this.form.submit();"/>
+<input type="submit" name="action_copy" value="%s"  onclick="this.form.func.value='copyList'; this.form.submit();"/>
+<input type="submit" name="action_duplicate" value="%s"  onclick="this.form.func.value='duplicateList'; this.form.submit();"/>
+</p>
+</form>
+<div id="pagination">
+</div>
+</div>
+EOHTML
     
     ### Clearing div
     $output         .= q{<div style="clear: both;">&nbsp;</div>};
 
     tie my %options, 'Tie::IxHash';
     my $hasClips = 0;
+    my $clipNum  = 0;
     foreach my $asset (@{$currentAsset->getAssetsInClipboard(1)}) {
             $options{$asset->getId} = '<img src="'.$asset->getIcon(1).'" alt="'.$asset->getName.'" style="border: 0px;" /> '.$asset->getTitle;
             $hasClips = 1;
+            $clipNum++;
     }
     if ($hasClips) {
             $output .= '<div class="functionPane"><fieldset><legend>'.$i18n->get(1082).'</legend>'
                     .WebGUI::Form::formHeader($session, {action=>$currentAsset->getUrl})
                     .WebGUI::Form::hidden($session,{name=>"func",value=>"pasteList"})
-                    .WebGUI::Form::checkbox($session,{extras=>'onclick="toggleClipboardSelectAll(this.form);"'})
-                    .' '.$i18n->get("select all").'<br />'
+                    .WebGUI::Form::hidden($session,{name=>"proceed",value=>"manageAssets"})
+                    .( $clipNum > 1
+                        ? WebGUI::Form::checkbox($session,{extras=>'onclick="toggleClipboardSelectAll(this.form);"'}).' '.$i18n->get("select all").'<br />'
+                        : ''
+                     )
+                    
                     .WebGUI::Form::checkList($session,{name=>"assetId",vertical=>1,options=>\%options})
                     .'<br />'
-                    .WebGUI::Form::submit($session,{value=>"Paste"})
+                    .WebGUI::Form::submit($session,{value=>$i18n->get('Paste')})
                     .WebGUI::Form::formFooter($session)
                     .' </fieldset></div> '
                     .'<script type="text/javascript">
@@ -566,53 +538,36 @@ sub www_search {
     $session->style->setScript( $session->url->extras( 'yui/build/yahoo-dom-event/yahoo-dom-event.js' ) );
     $session->style->setScript( $session->url->extras( 'yui-webgui/build/assetManager/assetManager.js' ) );
     $session->style->setScript( $session->url->extras( 'yui-webgui/build/form/form.js' ) );
+    my $keywords =  $session->form->get('keywords') || $session->scratch->get('assetManagerSearchKeywords');
 
     ### Show the form
     $output     .= q{<form method="post" enctype="multipart/form-data" action="} . $currentAsset->getUrl . q{"><p>}
                 . q{<input type="hidden" name="op" value="assetManager" />}
                 . q{<input type="hidden" name="method" value="search" />}
-                . q{<input type="text" size="45" name="keywords" value="} . $session->form->get('keywords') . q{" />}
+                . q{<input type="text" size="45" name="keywords" value="} . $keywords . q{" />}
                 . getClassSelectBox( $session )
                 . q{<input type="submit" name="action" value="}.$i18n->get( "search" ).q{" />}
                 . q{</p></form>}
                 ;
 
-    ### Actions
-    if ( my $action = lc $session->form->get( 'action' ) ) {
-        my @assetIds = $session->form->get( 'assetId' );
-
-        if ( $action eq "delete" ) { ##aka trash
-            for my $assetId ( @assetIds ) {
-                my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-                next unless $asset;
-                $asset->trash;
-            }
-        }
-        elsif ( $action eq "cut" ) {
-            for my $assetId ( @assetIds ) {
-                my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId );
-                next unless $asset;
-                $asset->cut;
-            }
-        }
-        elsif ( $action eq "copy" ) {
-            for my $assetId ( @assetIds ) {
-                # Copy == Duplicate + Cut
-		my $asset       = WebGUI::Asset->newByDynamicClass( $session, $assetId);
-                my $newAsset    = $asset->duplicate( { skipAutoCommitWorkflows => 1 } );
-                $newAsset->update( { title => $newAsset->getTitle . ' (copy)' } );
-                $newAsset->cut;
-            }
-        }
-    }
-
     ### Run the search
-    if ( $session->form->get( 'keywords' ) || $session->form->get( 'class' ) ) {
-        my $keywords        = $session->form->get( 'keywords' );
-        my @classes         = $session->form->get( 'class' );
+    if ( $keywords || $session->form->get( 'class' ) ) {
+        my @classes          = $session->form->get( 'class' );
+        my $keywordsScrubbed = $keywords;
 
-        my $p       = getSearchPaginator( $session, { 
-            keywords            => $keywords,
+        # Detect a helper word key
+        my @assetIds        = ($keywords =~ /assetid:\s*([^\s]+)/gi);
+
+        # purge helper word keys
+        if (@assetIds) {
+            $keywordsScrubbed =~ s/\bassetid:\s*[^\s]+//gi;
+        }
+        $keywordsScrubbed =~ s/^\s+//g;
+        $keywordsScrubbed =~ s/\s+$//g;
+
+        my $p       = getSearchPaginator( $session, {
+            assetIds            => \@assetIds,
+            keywords            => $keywordsScrubbed,
             classes             => \@classes,
             orderByColumn       => $session->form->get( 'orderByColumn' ),
             orderByDirection    => $session->form->get( 'orderByDirection' ),
@@ -623,9 +578,10 @@ sub www_search {
         }
         else {
             ### Display the search results 
-            $output         .= q{<form method="post" enctype="multipart/form-data">}
-                            . q{<input type="hidden" name="op" value="assetManager" />}
-                            . q{<input type="hidden" name="method" value="search" />}
+            $output         .= q{<form method="post" enctype="multipart/form-data" action="}.$currentAsset->getUrl.q{">}
+                            . q{<input type="hidden" name="func"    value="searchAssets" />}
+                            . q{<input type="hidden" name="proceed" value="searchAssets" />}
+                            . WebGUI::Form::CsrfToken->new($session)->toHtml
                             . q{<input type="hidden" name="pn" value="} . $session->form->get('pn') . q{" />}
                             . q{<input type="hidden" name="keywords" value="} . $keywords . q{" />}
                             ;
@@ -729,9 +685,9 @@ sub www_search {
             $output     .= q{</tbody>}
                         . q{</table>}
                         . q{<p class="actions">} . $i18n->get( 'with selected' )
-                        . q{<input type="submit" name="action" value="}.$i18n->get( 'delete' ) . q{" />}
-                        . q{<input type="submit" name="action" value="}.$i18n->get( "cut" )    . q{" />}
-                        . q{<input type="submit" name="action" value="}.$i18n->get( "copy" )    .q{" />}
+                        . q{<input type="submit" name="action" value="}.$i18n->get( 'delete' ) . q[" onclick="if(confirm('].$i18n->get('43').q[')){this.form.func.value='deleteList'; this.form.submit();}{ return false; }" />]
+                        . q{<input type="submit" name="action" value="}.$i18n->get( "cut" )    . q{" onclick="this.form.func.value='cutList'; this.form.submit();" />}
+                        . q{<input type="submit" name="action" value="}.$i18n->get( "Copy" )    .q{" onclick="this.form.func.value='copyList'; this.form.submit();" />}
                         . q{</p>}
                         . q{</form>}
                         ;
@@ -752,7 +708,9 @@ sub www_search {
 
     $output         .= '</div>';
 
+    $session->scratch->set('assetManagerSearchKeywords',  $keywords);
     return $ac->render( $output );
 }
 
+
 1;
