@@ -169,7 +169,10 @@ sub create {
 	# add defaults
 	my $properties = $class->crud_getProperties($session);
 	foreach my $property (keys %{$properties}) {
-		$data->{$property} ||= $properties->{$property}{defaultValue};
+	    # set a default value if it's empty or undef (as per L<update>)
+        if ($data->{$property} eq "") {
+            $data->{$property} = $properties->{$property}{defaultValue};
+        }
 	}
 
 	# determine sequence
@@ -192,6 +195,26 @@ sub create {
 
 #-------------------------------------------------------------------
 
+=head2 crud_createOrUpdateTable ( session )
+
+A detection class method used to affirm creation or update of the database table using the crud_definition(). Returns 1 on successful completion.
+
+=head3 session
+
+A reference to a WebGUI::Session.
+
+=cut
+
+sub crud_createOrUpdateTable {
+    my ( $class, $session ) = @_;
+    my $tableName   = $class->crud_getTableName($session);
+    my $tableExists = $session->db->dbh->do("show tables like '$tableName'");
+
+    return ( $tableExists ne '0E0' ? $class->crud_updateTable($session) : $class->crud_createTable($session) );
+}
+
+#-------------------------------------------------------------------
+
 =head2 crud_createTable ( session )
 
 A management class method used to create the database table using the crud_definition(). Returns 1 on successful completion. 
@@ -644,6 +667,10 @@ Here's an example of this structure:
 	{ "color=? or color=?" 	=> ['blue','black'] },
  ]
 
+would yield
+
+ ( price <= 44 ) AND ( color = 'blue' OR color = 'black' )
+
 =head4 join
 
 An array reference containing the tables you wish to join with this one, and the mechanisms to join them. Here's an example.
