@@ -19,6 +19,8 @@ use Tie::CPHash;
 use WebGUI::LDAPLink;
 use WebGUI::Macro;
 use WebGUI::Utility;
+use WebGUI::Pluggable;
+use WebGUI::International;
 
 
 =head1 NAME
@@ -256,17 +258,18 @@ sub dateCreated {
 
 =head2 delete ( )
 
-Deletes this group and all references to it.
+Deletes this group from the group related tables in the database and calls clearCaches.
 
 =cut
 
 sub delete {
-	my $self = shift;
-	$self->clearCaches;
-        $self->session->db->write("delete from groups where groupId=?", [$self->getId]);
-        $self->session->db->write("delete from groupings where groupId=?", [$self->getId]);
-        $self->session->db->write("delete from groupGroupings where inGroup=? or groupId=?", [$self->getId, $self->getId]);
-	undef $self;
+    my $self = shift;
+    $self->resetGroupFields;
+    $self->clearCaches;
+    $self->session->db->write("delete from groups where groupId=?", [$self->getId]);
+    $self->session->db->write("delete from groupings where groupId=?", [$self->getId]);
+    $self->session->db->write("delete from groupGroupings where inGroup=? or groupId=?", [$self->getId, $self->getId]);
+    undef $self;
 }
 
 #-------------------------------------------------------------------
@@ -543,7 +546,8 @@ sub getAllGroupsFor {
 =head2 getAllUsers ( [ withoutExpired ] )
 
 Returns an array reference containing a list of users that belong to this group
-and in any group that belongs to this group.
+and in any group that belongs to this group.  The list is unique, so that each
+userId is only in it one time.
 
 =head3 withoutExpired
 
@@ -803,11 +807,14 @@ sub getLDAPUsers {
 	foreach my $person (@{$people}) {
 	   $person =~ s/\s*,\s*/,/g;
 	   $person = lc($person);
-	   my ($userId) = $self->session->db->quickArray("select userId from authentication where authMethod='LDAP' and fieldName='connectDN' and lower(fieldData)=?",[$person]);
+	   my $personRegExp = "^uid=$person,";
+	   
+	   my ($userId) = $self->session->db->quickArray("select userId from authentication where authMethod='LDAP' and fieldName='connectDN' and lower(fieldData) = ? OR lower(fieldData) REGEXP ?",[$person,$personRegExp]);
+	   
 	   if($userId) {
 	      push(@ldapUsers,$userId);
 	   } else {
-	      $self->session->errorHandler->warn("Could not find matching userId for dn $person in WebGUI for group $gid");
+	      $self->session->errorHandler->warn("Could not find matching userId for dn/uid $person in WebGUI for group $gid");
 	   }
 	}
 	
@@ -857,7 +864,7 @@ EOQ
 
 =head2 getUserList ( [ withoutExpired ] )
 
-Returns a hash reference with key of userId and value of username for users in the group
+Returns a hash reference with key of userId and value of username for users in the group, sorted by username.
 
 =head3 withoutExpired
 
@@ -1135,6 +1142,113 @@ sub new {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 resetGroupFields (  )
+
+Looks through WebGUI and resets any group field that it can find, that uses this group,
+to the admin group, 3.  Called internally by delete.
+
+Currently handes these areas:
+
+=over 4
+
+=item *
+
+Anything in an Asset definition that is labeled as type group.  JSON data is not handled.
+
+=item *
+
+Everything in Operation/Settings, from its definition subroutine.
+
+=item *
+
+Settings fields hand picked from Shop/Admin and Account/FriendManager.
+
+=item *
+
+Any Workflow Activity data from the definition that is labeled as type group.
+
+=back
+
+=cut
+
+sub resetGroupFields {
+    my $self    = shift;
+    my $gid     = $self->getId;
+    my $session = $self->session;
+    my $db      = $session->db;
+    my $config  = $session->config;
+    my $assets  = $config->get('assets');
+    my $tableCache = {};
+
+    ##Note, I did assets in SQL instead of using the API because you would have to
+    ##instanciate every version of the asset that used the group.  This should be much quicker
+    ASSET: foreach my $asset (keys %{ $assets }) {
+        my $definition = WebGUI::Pluggable::instanciate($asset, 'definition', [$session]);
+        SUBDEF: foreach my $subdef (@{ $definition }) {
+            next SUBDEF if exists $tableCache->{$subdef->{tableName}}; 
+            PROP: while (my ($fieldName, $properties) = each %{ $subdef->{properties} }) {
+                next PROP unless $properties->{fieldType} eq 'group';
+                push @{ $tableCache->{$subdef->{tableName}} }, $fieldName;
+            }
+        }
+    }
+    ##VersionTags
+    $tableCache->{assetVersionTag} = ['groupToUse'];
+    $tableCache->{adSpace}         = ['groupToPurchase'];
+    foreach my $tableName (keys %{ $tableCache }) {
+        foreach my $fieldName (@{ $tableCache->{$tableName} }) {
+            my $sql = sprintf 'UPDATE %s SET %s=3 where %s=?',
+                $db->dbh->quote_identifier($tableName),
+                (($db->dbh->quote_identifier($fieldName)) x 2);
+            $db->write($sql, [ $gid ]);
+        }
+    }
+
+    SETTINGS: {
+        my $setting = $session->setting;
+        my $i18n = WebGUI::International->new($session);
+        my $definition = WebGUI::Pluggable::run('WebGUI::Operation::Settings', 'definition', [$session, $i18n]);
+        FIELD: foreach my $field (@{ $definition }) {
+            next FIELD unless $field->{fieldType} eq 'group'
+                          and $setting->get($field->{name}) eq $gid;
+            $setting->set($field->{name}, 3);
+        }
+    }
+    ##Settings in the settings table not from Operation/Settings.  These should all
+    ##be moved to definition style subroutines for future auto-probing.
+    AUX_SETTINGS: {
+        ##These are extra fields 
+        my $setting = $session->setting;
+        my @extraFields = qw/groupIdCashier groupIdAdminCommerce/;  ##Shop/Admin
+        push @extraFields, qw/groupIdAdminFriends groupsToManageFriends/; ##Account/FriendManager
+        FIELD: foreach my $field (@extraFields) {
+            next FIELD unless $setting->get($field) eq $gid;
+            $setting->set($field, 3);
+        }
+    }
+    ACTIVITY: {
+        my $workflowActivities = $config->get('workflowActivities');
+        my @activities;
+        foreach my $wfActivities (values %{ $workflowActivities} ) {
+            push @activities, @{ $wfActivities };
+        }
+        foreach my $activity (@activities) {
+            my $definition = WebGUI::Pluggable::instanciate($activity, 'definition', [$session]);
+            my $sth = $db->prepare('UPDATE WorkflowActivityData set value=3 where name=? and value=?');
+            SUBDEF: foreach my $subdef (@{ $definition }) {
+                PROP: while (my ($fieldName, $properties) = each %{ $subdef->{properties} }) {
+                    next PROP unless $properties->{fieldType} eq 'group';
+                    $sth->execute([$fieldName, $gid]);
+                }
+            }
+        }
+    }
+    ##Inbox messages, inbox table
+    return 1;
+}
+
 #-------------------------------------------------------------------
 
 =head2 scratchFilter ( [ value ] )
@@ -1447,4 +1561,24 @@ sub userGroupExpireDate {
 	}
 }	
 
+#-------------------------------------------------------------------
+
+=head2 vitalGroup ( $groupId )
+
+Class method to check to see if a group is a reserved WebGUI group.  Returns
+true or false.  Placed in here because I found two different lists in two
+different areas.
+
+=head3 $groupId
+
+A GUID of the group to check.
+
+=cut
+
+sub vitalGroup {
+	my $class   = shift;
+	my $groupId = shift;
+    return isIn ( $groupId, (1..17), qw/pbgroup000000000000015 pbgroup000000000000016 pbgroup000000000000017 / );
+}	
+
 1;
