@@ -97,16 +97,50 @@ Email message to use rather than inbox message contents.
 
 Email subject to use rather than inbox message subject.
 
+=head4 smsMessage
+
+SMS notification message to send to C<smsAddress>
+
+=head4 smsSubject
+
+SMS notification subject (typically used for SMS Gateway authentication)
+
+=head4 smsAddress
+
+Email address that SMS notification is sent to (typically the user's C<cellPhone> C<@> C<smsGateway>)
+
+=head3 options
+
+A hash reference containing options for handling the message. 
+
+=head4 no_email 
+
+If no_email is true, then no email will be made or sent.  Only
+the inbox message will be made.
+
+=head4 no_sms
+
+If no_sms is true, then no attempt to sms notifications will be sent.
+
+=head4 overridePerUserDelivery
+
+If true, then the C<isInbox> flag will not be passed to L<WebGUI::Mail::Send::Create>, and thus the
+per-user settings for email delivery will not be used. Useful if you want to force this message to
+be sent as an Email rather than allowing the user's C<receiveInboxEmailNotifications> setting to
+take effect.
+
 =cut
 
 sub create {
-	my $class = shift;
-	my $session = shift;
-	my $properties = shift;
+    my $class      = shift;
+    my $session    = shift;
+    my $properties = shift;
+    my $options    = shift || {};
 	my $self = {};
+    my $i18n = WebGUI::International->new($session);
 	$self->{_properties}{messageId} = "new";
 	$self->{_properties}{status}    = $properties->{status} || "pending";
-	$self->{_properties}{subject}   = $properties->{subject} || WebGUI::International->new($session)->get(523);
+	$self->{_properties}{subject}   = $properties->{subject} || $i18n->get(523,'WebGUI');
 	$self->{_properties}{message}   = $properties->{message};
 	$self->{_properties}{dateStamp} = time();
 	$self->{_properties}{userId}    = $properties->{userId};
@@ -116,6 +150,9 @@ sub create {
              $self->{_properties}{userId} = $session->user->userId;
          }
     my $status = $self->{_properties}{status};
+    my $smsMessage = $properties->{smsMessage};
+    my $smsSubject = $properties->{smsSubject};
+    my $smsAddress = $properties->{smsAddress};
     
 	if ($status eq "completed") {
 		$self->{_properties}{completedBy} = $session->user->userId;
@@ -147,32 +184,44 @@ sub create {
             );
         }
     }
-
-	my $subject = (defined $properties->{emailSubject}) ? $properties->{emailSubject} : $self->{_properties}{subject};
-	my $mail = WebGUI::Mail::Send->create($session, {
-		toUser=>$self->{_properties}{userId},
-		toGroup=>$self->{_properties}{groupId},
-		subject=>$subject,
-		});
-	if (defined $mail) {
+    unless ( $options->{ no_email } ) {
+        my $subject = (defined $properties->{emailSubject}) ? $properties->{emailSubject} : $self->{_properties}{subject};
+        my $mail = WebGUI::Mail::Send->create($session, {
+                   toUser=>$self->{_properties}{userId},
+                   toGroup=>$self->{_properties}{groupId},
+                   subject=>$subject,
+               }, 
+               $options->{overridePerUserDelivery} ? undef : 'isInbox',
+        );
         my $preface = "";
         my $fromUser = WebGUI::User->new($session, $properties->{sentBy});
         #Don't append prefaces to the visitor users or messages that don't specify a user (default case)
         unless ($fromUser->isVisitor || $fromUser->userId eq 3) {  #Can't use isAdmin because it will not send prefaces from normal users who in the admin group
-            my $i18n = WebGUI::International->new($session, 'Inbox_Message');
-            $preface = sprintf($i18n->get('from user preface'), $fromUser->username);
+            $preface = sprintf($i18n->get('from user preface', 'Inbox_Message'), $fromUser->username);
         }
         my $msg = (defined $properties->{emailMessage}) ? $properties->{emailMessage} : $self->{_properties}{message};
-		if ($msg =~ m/\<.*\>/) {
-            $msg = '<p>' . $preface . '</p><br />'.$msg if($preface ne "");
-			$mail->addHtml($msg);
-		} else {
-            $msg = $preface."\n\n".$msg if($preface ne "");
-			$mail->addText($msg);
-		}
-		$mail->addFooter;
+		$msg = '<p>' . $preface . '</p><br />'.$msg if($preface ne "");
+        $mail->addHtml($msg);
+        $mail->addFooter;
 		$mail->queue;
-	}
+    }
+    
+    unless ( $options->{ no_sms } ) {
+        # If smsAddress provided, send smsMessage too
+        if ( $smsAddress && $smsMessage) {
+            my $sms = WebGUI::Mail::Send->create(
+                $session,
+                {   to      => $smsAddress,
+                    subject => $smsSubject,
+                }
+            );
+            if ($sms) {
+                $sms->addText($smsMessage);
+                $sms->queue;
+            }
+        }
+    }
+	
 	$self->{_session} = $session;
 	bless $self, $class;
 }
