@@ -37,12 +37,13 @@ time is used instead.
 
 #-------------------------------------------------------------------
 sub process {
-	my $session = shift;
-        my (@param, $temp, $time);
-        @param = @_;
-	$time = $param[1] ||$session->datetime->time();
-	$temp =$session->datetime->epochToHuman($time,$param[0]);
-	return $temp;
+    my $session = shift;
+    my $time    = $_[1];
+    if (! defined $time) {
+        $time = time();
+    }
+    my $temp = $session->datetime->epochToHuman($time, $_[0]);
+    return $temp;
 }
 
 
