@@ -204,6 +204,14 @@ sub definition {
         hoverHelp=>$i18n->get("Enable Metadata description"),
         defaultValue=>$setting->get("metaDataEnabled")
 		});
+	push(@fields, {
+		tab=>"content",
+		fieldType=>"interval",
+        name=>"maxCacheTimeout",
+        label=>$i18n->get("Maximum cache timeout"),
+        hoverHelp=>$i18n->get("Maximum cache timeout description"),
+        defaultValue=>$setting->get("maxCacheTimeout")
+		});
 	# user interface settings
 	push(@fields, {
 		tab=>"ui",
@@ -262,6 +270,22 @@ sub definition {
         label           => $i18n->get('recaptcha private key'),
         defaultValue    => $setting->get('recaptchaPrivateKey'),
     });
+    push @fields, {
+        tab             => "ui",
+        fieldType       => "codearea",
+        name            => "globalHeadTags",
+        label           => $i18n->get('global head tags label'),
+        hoverHelp       => $i18n->get('global head tags description'),
+        defaultValue    => $setting->get('globalHeadTags'),
+    };
+    push @fields, {
+        tab             => 'ui',
+        fieldType       => 'yesNo',
+        name            => 'useMobileStyle',
+        label           => $i18n->get('mobile style label'),
+        hoverHelp       => $i18n->get('mobile style description'),
+        defaultValue    => $setting->get('useMobileStyle'),
+    };
 	# messaging settings
 	push(@fields, {
 		tab=>"messaging",
@@ -287,6 +311,22 @@ sub definition {
 		hoverHelp=>$i18n->get('mail return path help'),
 		defaultValue=>$setting->get("mailReturnPath")
 		});
+	push(@fields, {
+		tab          => 'messaging',
+		fieldType    => 'text',
+		name         => 'smsGateway',
+		label        => $i18n->get('sms gateway'),
+		hoverHelp    => $i18n->get('sms gateway help'),
+		defaultValue => $setting->get('smsGateway'),
+        });
+    push(@fields, {
+		tab          => 'messaging',
+		fieldType    => 'text',
+		name         => 'smsGatewaySubject',
+		label        => $i18n->get('sms gateway subject'),
+		hoverHelp    => $i18n->get('sms gateway subject help'),
+		defaultValue => $setting->get('smsGatewaySubject'),
+		});
 	# misc
 	push(@fields, {
 		tab=>"misc",
@@ -428,6 +468,14 @@ sub definition {
                 defaultValue=>$setting->get("passiveProfilingEnabled"),
                 extras=>'onchange="alert(\''.$i18n->get("Illegal Warning").'\')" '
 		});
+    push @fields, {
+        tab             => "user",
+        name            => "redirectAfterLoginUrl",
+        fieldType       => "url",
+        defaultValue    => $setting->get('redirectAfterLoginUrl'),
+        label           => $i18n->get( 'redirectAfterLoginUrl label' ),
+        hoverHelp       => $i18n->get( 'redirectAfterLoginUrl description' ),
+    };
     push @fields, {
         tab             => "user",
         name            => "showMessageOnLogin",
@@ -482,9 +530,11 @@ sub definition {
         groupIdAdminCache
         groupIdAdminCron
         groupIdAdminDatabaseLink
+        groupIdAdminFilePump
         groupIdAdminGraphics
         groupIdAdminGroup
         groupIdAdminGroupAdmin
+        groupIdAdminHistory
         groupIdAdminHelp
         groupIdAdminLDAPLink
         groupIdAdminLoginHistory
@@ -637,7 +687,7 @@ is in group Admin (3).  Returns the user to the Edit Settings screen, www_editSe
 
 sub www_saveSettings {
     my $session     = shift;
-    return $session->privilege->adminOnly() unless ($session->user->isAdmin);
+    return $session->privilege->adminOnly() unless ($session->user->isAdmin && $session->form->validToken);
     my $i18n        = WebGUI::International->new($session, "WebGUI");
     my $setting     = $session->setting;
     my $form        = $session->form;
