@@ -25,6 +25,8 @@ use WebGUI::SQL;
 use WebGUI::TabForm;
 use WebGUI::User;
 use WebGUI::Utility;
+use JSON;
+use XML::Simple;
 
 =head1 NAME
 
@@ -73,13 +75,19 @@ sub _submenu {
 		$ac->addSubmenuItem($session->url->page("op=editUser;uid=new"), $i18n->get(169));
 	}
 
+    $ac->setFormUrl($session->url->page('op=editUser;uid='.$userId));
+    my $formId = $ac->getSubmenuFormId;
 	if (canEdit($session)) {
 		unless ($session->form->process("op") eq "listUsers" 
 			|| $session->form->process("op") eq "deleteUser"
 			|| $userId eq "new") {
 			$ac->addSubmenuItem($session->url->page("op=editUser;uid=$userId"), $i18n->get(457));
-			$ac->addSubmenuItem($session->url->page("op=becomeUser;uid=$userId"), $i18n->get(751));
-			$ac->addConfirmedSubmenuItem($session->url->page("op=deleteUser;uid=$userId"), $i18n->get(750), $i18n->get(167));
+			$ac->addSubmenuItem($session->url->page('op=becomeUser;uid='.$userId), $i18n->get(751), qq|onclick="var thisForm=document.getElementById('$formId');thisForm.op.value='becomeUser';thisForm.submit(); return false;"|);
+            my $user = WebGUI::User->new($session, $userId);
+			$ac->addSubmenuItem($user->getProfileUrl(), $i18n->get('view profile'));
+            my $confirm = $i18n->get(167);
+            $confirm =~ s/([\\\'])/\\$1/g;
+			$ac->addSubmenuItem($session->url->page('op=deleteUser;uid='.$userId), $i18n->get(750), qq|onclick="var ack = confirm('$confirm'); if (ack) { var thisForm=document.getElementById('$formId');thisForm.op.value='deleteUser';thisForm.submit();} return false;"|);
 			if ($session->setting->get("useKarma")) {
 				$ac->addSubmenuItem($session->url->page("op=editUserKarma;uid=$userId"), $i18n->get(555));
 			}
@@ -123,6 +131,23 @@ sub canEdit {
 
 #----------------------------------------------------------------------------
 
+=head2 canUseService ( session )
+
+Returns true if the current session is allowed to use the web service, i.e.
+is in one of the configured CIDR subnets in the config file.
+
+=cut
+
+sub canUseService {
+    my ( $session ) = @_;
+    my $subnets = $session->config->get('serviceSubnets');
+    return 1 if !$subnets || !@{$subnets};
+    return 1 if WebGUI::Utility::isInSubnet( $session->env->getIp, $subnets );
+    return 0; # Don't go away mad, just go away
+}
+
+#----------------------------------------------------------------------------
+
 =head2 canView ( session [, user] )
 
 Returns true if the user is allowed to see this module. user defaults to the
@@ -138,6 +163,27 @@ sub canView {
 
 #-------------------------------------------------------------------
 
+=head2 createServiceResponse ( format, data ) 
+
+Create a string with the correct C<format> from the given C<data>.
+
+Possible formats are "json" and "xml".
+
+=cut
+
+sub createServiceResponse {
+    my ( $format, $data ) = @_;
+    
+    if ( lc $format eq "xml" ) {
+        return XML::Simple::XMLout($data, NoAttr => 1, RootName => "response" );
+    }
+    else {
+        return JSON->new->encode($data);
+    }
+}
+
+#-------------------------------------------------------------------
+
 =head2 doUserSearch ( session, op, returnPaginator, userFilter )
 
 Subroutine that actually performs the SQL search for users.
@@ -283,6 +329,246 @@ sub getUserSearchForm {
 	return $output;
 }
 
+#-------------------------------------------------------------------
+
+=head2 www_ajaxCreateUser ( )
+
+Create a user using a web service.
+
+=cut
+
+sub www_ajaxCreateUser {
+    my ( $session ) = @_;
+
+    ### Get desired output format first (for future error messages)
+    my $outputFormat    = "json";
+    my $mimeType        = "application/json";
+
+    # Allow XML
+    if ( lc $session->form->get('as') eq "xml" ) {
+        $outputFormat   = "xml";
+        $mimeType       = "application/xml";
+    }
+
+    $session->http->setMimeType( $mimeType ); 
+
+    # Verify access
+    if ( !canAdd($session) || !canUseService($session) ) {
+        # We need an automatic way to send a request for an http basic auth
+        $session->http->setStatus(401,'Unauthorized');
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::Unauthorized",
+            message     => "",
+        } );
+    }
+
+    ### Verify data
+    # User data is <PROPERTY_NAME> in form
+    my %userParam = (
+        map { $_ => $session->form->get($_) }
+        grep { !/^auth:/ && $_ ne "op" }
+        ( $session->form->param )
+    );
+
+    # Auth data is auth:<AUTH_METHOD>:<PROPERTY_NAME> in form
+    my %authParam    = ();
+    for my $formParam ( grep { /^auth:[^:]+:.+$/ } $session->form->get ) {
+        my ( $authMethod, $property ) = $formParam =~ /^auth:([^:]+):(.+)$/;
+        $authParam{$authMethod}{$property} = $session->form->get($formParam);
+    }
+
+    # User must have a username
+    if ( !$userParam{username} ) {
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::InvalidParam",
+            param       => "username",
+            message     => "",
+        } );
+    }
+    # User must not already exist
+    if ( $session->db->quickScalar( "SELECT * FROM users WHERE username=?", [$userParam{username}] ) ) {
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::InvalidParam",
+            param       => "username",
+            message     => "",
+        } );
+    }
+
+    ### Create user
+    my $user    = WebGUI::User->create( $session );
+    $user->update( \%userParam );
+    for my $authMethod ( keys %authParam ) {
+        my $auth = WebGUI::Operation::Auth::getInstance($session,$authMethod,$user->getId);
+
+        # XXX Special handling for WebGUI passwords. This should be removed when 
+        # Auth is fixed in WebGUI 8
+        if ( $authMethod eq 'WebGUI' && exists $authParam{$authMethod}{identifier} ) {
+            $authParam{$authMethod}{identifier}
+                = $auth->hashPassword( $authParam{$authMethod}{identifier} );
+        }
+
+        $auth->saveParams( $user->getId, $auth->authMethod, $authParam{$authMethod} );
+    }
+
+    ### Send new user's data
+    return createServiceResponse( $outputFormat, {
+        user        => $user->get,
+    } );
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_ajaxDeleteUser ( )
+
+Delete a user using a web service.
+
+=cut
+
+sub www_ajaxDeleteUser {
+    my ( $session ) = @_;
+    
+    ### Get desired output format first (for future error messages)
+    my $outputFormat    = "json";
+    my $mimeType        = "application/json";
+
+    # Allow XML
+    if ( lc $session->form->get('as') eq "xml" ) {
+        $outputFormat   = "xml";
+        $mimeType       = "application/xml";
+    }
+
+    $session->http->setMimeType( $mimeType ); 
+
+    # Verify access
+    if ( !canEdit($session) || !canUseService($session) ) {
+        # We need an automatic way to send a request for an http basic auth
+        $session->http->setStatus(401,'Unauthorized');
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::Unauthorized",
+            message     => "",
+        } );
+    }
+
+    # Verify data
+    my $userId  = $session->form->get('userId');
+    if ( !$userId ) {
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::InvalidParam",
+            param       => "userId",
+            message     => "",
+        } );
+    }
+    elsif ( $userId eq "1" || $userId eq "3" ) {
+        $session->http->setStatus(403,"Forbidden");
+        return createServiceResponse( $outputFormat, {
+            error       => 'WebGUI::Error::InvalidParam',
+            param       => 'userId',
+            message     => 'Cannot delete system user',
+        } );
+    }
+    elsif ( !WebGUI::User->validUserId( $session, $userId ) ) {
+        return createServiceResponse( $outputFormat, {
+            error       => 'WebGUI::Error::InvalidParam',
+            param       => 'userId',
+            message     => '',
+        } );
+    }
+
+    ### Delete user
+    my $user    = WebGUI::User->new( $session, $userId );
+    $user->delete;
+    
+    return createServiceResponse( $outputFormat, {
+        message         => 'User deleted',
+    } );
+}
+
+#-------------------------------------------------------------------
+
+=head2 www_ajaxUpdateUser ( )
+
+Update a user using a web service.
+
+=cut
+
+sub www_ajaxUpdateUser {
+    my ( $session ) = @_;
+    
+    ### Get desired output format first (for future error messages)
+    my $outputFormat    = "json";
+    my $mimeType        = "application/json";
+
+    # Allow XML
+    if ( lc $session->form->get('as') eq "xml" ) {
+        $outputFormat   = "xml";
+        $mimeType       = "application/xml";
+    }
+
+    $session->http->setMimeType( $mimeType ); 
+
+    # Verify access
+    if ( !canEdit($session) || !canUseService($session) ) {
+        # We need an automatic way to send a request for an http basic auth
+        $session->http->setStatus(401,'Unauthorized');
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::Unauthorized",
+            message     => "",
+        } );
+    }
+
+    ### Verify data
+    # User data is <PROPERTY_NAME> in form
+    my %userParam = (
+        map { $_ => $session->form->get($_) }
+        grep { !/^auth:/ && $_ ne "op" }
+        ( $session->form->param )
+    );
+
+    # Auth data is auth:<AUTH_METHOD>:<PROPERTY_NAME> in form
+    my %authParam    = ();
+    for my $formParam ( grep { /^auth:[^:]+:.+$/ } $session->form->param ) {
+        my ( $authMethod, $property ) = $formParam =~ /^auth:([^:]+):(.+)$/;
+        $authParam{$authMethod}{$property} = $session->form->get($formParam);
+    }
+
+    # User must have a userId
+    if ( !$userParam{userId} ) {
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::InvalidParam",
+            param       => "userId",
+            message     => "",
+        } );
+    }
+    # User must exist
+    if ( !WebGUI::User->validUserId( $session, $userParam{userId} ) ) {
+        return createServiceResponse( $outputFormat, {
+            error       => "WebGUI::Error::InvalidParam",
+            param       => "userId",
+            message     => "",
+        } );
+    }
+
+    ### Update user
+    my $user    = WebGUI::User->new( $session, delete $userParam{userId} );
+    $user->update( \%userParam );
+    for my $authMethod ( keys %authParam ) {
+        my $auth = WebGUI::Operation::Auth::getInstance($session,$authMethod,$user->getId);
+
+        # XXX Special handling for WebGUI passwords. This should be removed when 
+        # Auth is fixed in WebGUI 8
+        if ( $authMethod eq 'WebGUI' && exists $authParam{$authMethod}{identifier} ) {
+            $authParam{$authMethod}{identifier}
+                = $auth->hashPassword( $authParam{$authMethod}{identifier} );
+        }
+
+        $auth->saveParams( $user->getId, $auth->authMethod, $authParam{$authMethod} );
+    }
+
+    ### Send user's data
+    return createServiceResponse( $outputFormat, {
+        user        => $user->get,
+    } );
+}
 
 #-------------------------------------------------------------------
 
@@ -294,7 +580,7 @@ Allows an administrator to assume another user.
 
 sub www_becomeUser {
 	my $session = shift;
-	return $session->privilege->adminOnly() unless canEdit($session);
+	return $session->privilege->adminOnly() unless canEdit($session) && $session->form->validToken;
 	return undef unless WebGUI::User->validUserId($session, $session->form->process("uid"));
 	$session->var->end($session->var->get("sessionId"));
 	$session->user({userId=>$session->form->process("uid")});
@@ -313,18 +599,25 @@ after this.
 
 sub www_deleteUser {
 	my $session = shift;
-	return $session->privilege->adminOnly() unless canEdit($session);
-	my ($u);
-        if ($session->form->process("uid") eq '1' || $session->form->process("uid") eq '3') {
-	   return WebGUI::AdminConsole->new($session,"users")->render($session->privilege->vitalComponent());
-    } else {
-	   $u = WebGUI::User->new($session,$session->form->process("uid"));
-	   $u->delete;
-       return www_listUsers($session);
+	return $session->privilege->adminOnly() unless canEdit($session) && $session->form->validToken;
+    if ($session->form->process("uid") eq '1' || $session->form->process("uid") eq '3') {
+        return WebGUI::AdminConsole->new($session,"users")->render($session->privilege->vitalComponent());
+    }
+    else {
+        my $u = WebGUI::User->new($session,$session->form->process("uid"));
+        $u->delete;
+        return www_listUsers($session);
     }
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editUser ( )
+
+Provides a form for editing a user, or adding a new user.
+
+=cut
+
 sub www_editUser {
 	my $session = shift;
 	return $session->privilege->adminOnly() unless canAdd($session);
@@ -333,27 +626,27 @@ sub www_editUser {
 	my $i18n = WebGUI::International->new($session, "WebGUI");
 	my %tabs;
 	tie %tabs, 'Tie::IxHash';
-	%tabs = (
-		"account"=> { label=>$i18n->get("account")},
-		"profile"=> { label=>$i18n->get("profile")},
-		"groups"=> { label=>$i18n->get('89')},
-		);
+    %tabs = (
+        "account"=> { label=>$i18n->get("account")},
+        "profile"=> { label=>$i18n->get("profile")},
+        "groups"=> { label=>$i18n->get('89')},
+    );
 	my $tabform = WebGUI::TabForm->new($session,\%tabs);
 	$tabform->formHeader({extras=>'autocomplete="off"'});	
 	my $u = WebGUI::User->new($session,($uid eq 'new') ? '' : $uid); #Setting uid to '' when uid is 'new' so visitor defaults prefill field for new user
 	my $username = ($u->isVisitor && $uid ne "1") ? '' : $u->username;
-    	$tabform->hidden({name=>"op",value=>"editUserSave"});
-    	$tabform->hidden({name=>"uid",value=>$uid});
-    	$tabform->getTab("account")->raw('<tr><td width="170">&nbsp;</td><td>&nbsp;</td></tr>');
-	$tabform->getTab("account")->readOnly(value=>$uid,label=>$i18n->get(378));
-    	$tabform->getTab("account")->readOnly(value=>$u->karma,label=>$i18n->get(537)) if ($session->setting->get("useKarma"));
-    	$tabform->getTab("account")->readOnly(value=>$session->datetime->epochToHuman($u->dateCreated,"%z"),label=>$i18n->get(453));
-    	$tabform->getTab("account")->readOnly(value=>$session->datetime->epochToHuman($u->lastUpdated,"%z"),label=>$i18n->get(454));
-    	$tabform->getTab("account")->text(
-		-name=>"username",
-		-label=>$i18n->get(50),
-		-value=>$username
-		);
+    $tabform->hidden({name=>"op",value=>"editUserSave"});
+    $tabform->hidden({name=>"uid",value=>$uid});
+    $tabform->getTab("account")->raw('<tr><td width="170">&nbsp;</td><td>&nbsp;</td></tr>');
+    $tabform->getTab("account")->readOnly(value=>$uid,label=>$i18n->get(378));
+    $tabform->getTab("account")->readOnly(value=>$u->karma,label=>$i18n->get(537)) if ($session->setting->get("useKarma"));
+    $tabform->getTab("account")->readOnly(value=>$session->datetime->epochToHuman($u->dateCreated,"%z"),label=>$i18n->get(453));
+    $tabform->getTab("account")->readOnly(value=>$session->datetime->epochToHuman($u->lastUpdated,"%z"),label=>$i18n->get(454));
+    $tabform->getTab("account")->text(
+        -name=>"username",
+        -label=>$i18n->get(50),
+        -value=>$username
+    );
 	my %status;
 	tie %status, 'Tie::IxHash';
 	%status = (
@@ -366,7 +659,8 @@ sub www_editUser {
 			-name => "status",
 			-value => $u->status
 			);
-	} else {
+	}
+    else {
 		$tabform->getTab("account")->selectBox(
 			-name => "status",
 			-options => \%status,
@@ -383,7 +677,7 @@ sub www_editUser {
 		-options=>$options,
 		-label=>$i18n->get(164),
 		-value=>$u->authMethod,
-		);
+    );
 	foreach (@{$session->config->get("authMethods")}) {
 		my $authInstance = WebGUI::Operation::Auth::getInstance($session,$_,$u->userId);
         my $editUserForm = $authInstance->editUserForm;
@@ -397,9 +691,10 @@ sub www_editUser {
 		foreach my $field (@{$category->getFields}) {
 			next if $field->getId =~ /contentPositions/;
 			my $label = $field->getLabel . ($field->isRequired ? "*" : '');
-			if ($field->getId eq "alias" && $u->isVisitor) {
-				$tabform->getTab("profile")->raw($field->formField({label=>$label},1,undef,1));
-			} else {
+			if ($u->isVisitor) {
+				$tabform->getTab("profile")->raw($field->formField({label=>$label},1,undef,undef,undef,undef,'useFormDefault'));
+			}
+            else {
 				$tabform->getTab("profile")->raw($field->formField({label=>$label},1,$u));
 			}
 		}
@@ -453,6 +748,14 @@ sub www_editUser {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editUserSave ( )
+
+Process the editUser form data.  Returns adminOnly unless the user has privileges
+to add/edit users and the submitted form passes the validToken check.
+
+=cut
+
 sub www_editUserSave {
 	my $session = shift;
 	my $postedUserId = $session->form->process("uid"); #userId posted from www_editUser form
@@ -467,7 +770,7 @@ sub www_editUserSave {
 		$isSecondary = (canAdd($session) && $postedUserId eq "new");
 	}
 
-	return $session->privilege->adminOnly() unless ($isAdmin || $isSecondary);
+	return $session->privilege->adminOnly() unless ($isAdmin || $isSecondary) && $session->form->validToken;
 
 	# Check to see if 
 	# 1) the userId associated with the posted username matches the posted userId (we're editing an account)
@@ -476,7 +779,7 @@ sub www_editUserSave {
 	#
 	# Also verify that the posted username is not blank (we need a username)
 	#
-	
+
 	my $postedUsername = $session->form->process("username");
 	$postedUsername = WebGUI::HTML::filter($postedUsername, "all");
 
@@ -493,16 +796,15 @@ sub www_editUserSave {
 	   	$u->authMethod($session->form->process("authMethod"));
 	   	$u->status($session->form->process("status"));
 
-	   	# Loop through all of this users authentication methods
-	   	foreach (@{$session->config->get("authMethods")}) {
-
-	   		# Instantiate each auth object and call it's save method.  These methods are responsible for
-	   		# updating authentication information with values supplied by the www_editUser form.
-	      		my $authInstance = WebGUI::Operation::Auth::getInstance($session, $_, $actualUserId);
-	      		$authInstance->editUserFormSave();
-       		}
+        # Loop through all of this users authentication methods
+        foreach (@{$session->config->get("authMethods")}) {
+            # Instantiate each auth object and call it's save method.  These methods are responsible for
+            # updating authentication information with values supplied by the www_editUser form.
+                my $authInstance = WebGUI::Operation::Auth::getInstance($session, $_, $actualUserId);
+                $authInstance->editUserFormSave();
+        }
        		
-       		# Loop through all profile fields, and update them with new values.
+        # Loop through all profile fields, and update them with new values.
 		foreach my $field (@{WebGUI::ProfileField->getFields($session)}) {
 			next if $field->getId =~ /contentPositions/;
 			$u->profileField($field->getId,$field->formProcess($u));
@@ -551,6 +853,14 @@ sub www_editUserSave {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editUserKarma ( )
+
+Provides a form for directly editing the karma for a user.  Returns adminOnly
+unless the current user can manage users.
+
+=cut
+
 sub www_editUserKarma {
 	my $session = shift;
 	return $session->privilege->adminOnly() unless canEdit($session);
@@ -587,9 +897,18 @@ sub www_editUserKarma {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_editUserKarmaSave ( )
+
+Processes the form submitted  by www_editUserKarma.  Returns adminOnly
+unless the current user can manage users and the submitted from passes
+the validToken check.
+
+=cut
+
 sub www_editUserKarmaSave {
 	my $session = shift;
-	return $session->privilege->adminOnly() unless canEdit($session);
+	return $session->privilege->adminOnly() unless canEdit($session) && $session->form->validToken;
         my ($u);
         $u = WebGUI::User->new($session,$session->form->process("uid"));
         $u->karma($session->form->process("amount"),$session->user->username." (".$session->user->userId.")",$session->form->process("description"));
@@ -628,6 +947,15 @@ sub www_formUsers {
 
 
 #-------------------------------------------------------------------
+
+=head2 www_listUsers ( )
+
+Provides a paginated list of all users, and controls for adding a new user.  If the
+current user is only allowed to add users, then it sends them directly to www_editUser.
+If the current user is not allowed to edit or create users, it returns adminOnly.
+
+=cut
+
 sub www_listUsers {
 	my $session = shift;
 
