@@ -154,7 +154,7 @@ sub www_approveVersionTag {
     my $tag         = WebGUI::VersionTag->new( $session, $session->form->param("tagId") );
 
     return $session->privilege->insufficient 
-        unless canApproveVersionTag( $session, $tag );
+        unless canApproveVersionTag( $session, $tag ) && $session->form->validToken;
     
     my $instance    = $tag->getWorkflowInstance;
     my $activity    = $instance->getNextActivity;
@@ -271,7 +271,7 @@ A reference to the current session.
 
 sub www_editVersionTagSave {
 	my $session = shift;
-        return $session->session->privilege->insufficient() unless canView($session);
+    return $session->session->privilege->insufficient() unless canView($session) && $session->form->validToken;
 	if ($session->form->param("tagId") eq "new") {
 		my $tag = WebGUI::VersionTag->create($session, {
 			name=>$session->form->process("name","text", "Untitled"),
@@ -373,10 +373,9 @@ sub www_commitVersionTag {
             $session->url->page("op=commitVersionTag;tagId=".$tag->getId),
         );
     $p->setDataByQuery(q{
-        SELECT assetData.revisionDate, users.username, asset.assetId, asset.className 
+        SELECT assetData.revisionDate, assetData.revisedBy, asset.assetId, asset.className 
         FROM assetData 
         LEFT JOIN asset ON assetData.assetId = asset.assetId
-        LEFT JOIN users ON assetData.revisedBy = users.userId
         WHERE assetData.tagId=? },
         undef, 
         undef, 
@@ -384,8 +383,9 @@ sub www_commitVersionTag {
     );
 
     foreach my $row ( @{$p->getPageData} ) {
-        my ( $date, $by, $id, $class) = @{ $row }{ qw( revisionDate username assetId className ) };
+        my ( $date, $byUserId, $id, $class) = @{ $row }{ qw( revisionDate revisedBy assetId className ) };
         my $asset = WebGUI::Asset->new($session, $id, $class, $date);
+        my $byUser = WebGUI::User->new( $session, $byUserId );
         $output 
             .= '<tr><td>'
             .$session->icon->view("func=view;revision=".$date, $asset->get("url"))
@@ -393,7 +393,7 @@ sub www_commitVersionTag {
             <td>'.$asset->getTitle.'</td>
             <td><img src="'.$asset->getIcon(1).'" alt="'.$asset->getName.'" />'.$asset->getName.'</td>
             <td>'.$session->datetime->epochToHuman($date).'</td>
-            <td>'.$by.'</td></tr>';
+            <td>'.$byUser->get('username').'</td></tr>';
     }
     $output .= '</table>'.$p->getBarSimple;
     
@@ -419,7 +419,7 @@ sub www_commitVersionTagConfirm {
 	my $tagId = $session->form->param("tagId");
 	if ($tagId) {
 		my $tag = WebGUI::VersionTag->new($session, $tagId);
-		if (defined $tag && $session->user->isInGroup($tag->get("groupToUse"))) {
+		if (defined $tag && $session->user->isInGroup($tag->get("groupToUse")) && $session->form->validToken) {
 			my $i18n = WebGUI::International->new($session, "VersionTag");
 			
             my $startTime = WebGUI::DateTime->new($session,$session->form->process("startTime","dateTime"))->toDatabase;
@@ -451,9 +451,20 @@ sub www_commitVersionTagConfirm {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_leaveVersionTag ( session )
+
+Clears the current working version tag, and returns the user to www_manageVersions.
+
+=head3 session
+
+A reference to the current session.
+
+=cut
+
 sub www_leaveVersionTag {
     my $session = shift;
-    WebGUI::VersionTag->getWorking($session)->clearWorking;
+    WebGUI::VersionTag->getWorking($session)->leaveTag;
     return www_manageVersions($session);
 }
 
@@ -640,10 +651,12 @@ sub www_manageRevisionsInTag {
     $ac->addSubmenuItem($session->url->page('op=editVersionTag'), $i18n->get("add a version tag"));
     $ac->addSubmenuItem($session->url->page('op=manageCommittedVersions'), $i18n->get("manage committed versions")) if canView($session);
     $ac->addSubmenuItem($session->url->page('op=manageVersions'), $i18n->get("manage versions"));
+    $ac->addSubmenuItem($session->url->page('op=managePendingVersions'), $i18n->get("manage pending versions")) if canView($session);
 
     # Process any actions
-    my $action = lc $session->form->get('action');
-    if ( $action eq "purge" ) {
+    my $action     = lc $session->form->get('action');
+    my $validToken = $session->form->validToken;
+    if ( $action eq "purge" && $validToken) {
         # Purge these revisions
         my @assetInfo       = $session->form->get('assetInfo'); 
         for my $assetInfo ( @assetInfo ) {
@@ -654,11 +667,11 @@ sub www_manageRevisionsInTag {
 
         # If no revisions remain, delete the version tag
         if ( $tag->getRevisionCount <= 0 ) {
-            $tag->delete;
+            $tag->rollback;
             return www_manageVersions( $session );
         }
     }
-    elsif ( $action eq "move to:" ) {
+    elsif ( $action eq "move to:" && $validToken) {
         # Get the new version tag
         my $moveToTagId = $session->form->get('moveToTagId');
         my $moveToTag;
@@ -682,11 +695,11 @@ sub www_manageRevisionsInTag {
 
         # If no revisions remain, delete the version tag
         if ( $tag->getRevisionCount <= 0 ) {
-            $tag->delete;
+            $tag->rollback;
             return www_manageVersions( $session );
         }
     }
-    elsif ( $action eq "update version tag" ) {
+    elsif ( $action eq "update version tag" && $validToken) {
         my $startTime = WebGUI::DateTime->new($session,$session->form->process("startTime","dateTime"))->toDatabase;
         my $endTime   = WebGUI::DateTime->new($session,$session->form->process("endTime","dateTime"))->toDatabase;
         
@@ -796,11 +809,12 @@ sub www_manageRevisionsInTag {
         . '</tr> '
         ;
     my $p = WebGUI::Paginator->new($session,$session->url->page("op=manageRevisionsInTag;tagId=".$tag->getId));
-    $p->setDataByQuery("select assetData.revisionDate, users.username, asset.assetId, asset.className from assetData 
-            left join asset on assetData.assetId=asset.assetId left join users on assetData.revisedBy=users.userId
+    $p->setDataByQuery("select assetData.revisionDate, assetData.revisedBy, asset.assetId, asset.className from assetData 
+            left join asset on assetData.assetId=asset.assetId
             where assetData.tagId=?",undef, undef, [$tag->getId]);
     foreach my $row (@{$p->getPageData}) {
-            my ($date,$by,$id, $class) = ($row->{revisionDate}, $row->{username}, $row->{assetId}, $row->{className});
+            my ($date,$byUserId,$id, $class) = ($row->{revisionDate}, $row->{revisedBy}, $row->{assetId}, $row->{className});
+            my $byUser  = WebGUI::User->new( $session, $byUserId );
             my $asset = WebGUI::Asset->new($session,$id,$class,$date);
             # A checkbox for delete and move actions
             my $checkbox    = WebGUI::Form::checkbox( $session, {
@@ -814,7 +828,7 @@ sub www_manageRevisionsInTag {
                     <td>'.$asset->getTitle.'</td>
                     <td><img src="'.$asset->getIcon(1).'" alt="'.$asset->getName.'" />'.$asset->getName.'</td>
                     <td>'.$session->datetime->epochToHuman($date).'</td>
-                    <td>'.$by.'</td></tr>';
+                    <td>'.$byUser->username.'</td></tr>';
     }
     $output .= '</table>'.$p->getBarSimple.WebGUI::Form::formFooter( $session );
     $tag = $session->db->getRow("assetVersionTag","tagId",$tag->getId);
@@ -839,14 +853,16 @@ sub www_rollbackVersionTag {
 	return $session->privilege->adminOnly() unless canView($session);
 	my $tagId = $session->form->process("tagId");
 	return $session->privilege->vitalComponent() if ($tagId eq "pbversion0000000000001");
+    my $pb      = WebGUI::ProgressBar->new($session);
+    my $i18n    = WebGUI::International->new($session, 'VersionTag');
+    $pb->start($i18n->get('rollback version tag'), $session->url->extras('adminConsole/versionTags.gif'));
 	if ($tagId) {
 		my $tag = WebGUI::VersionTag->new($session, $tagId);
-		$tag->rollback if defined $tag;
+		$tag->rollback({ outputSub => sub { $pb->update(@_) }, }) if defined $tag;
 	}
-	if ($session->form->process("proceed") eq "manageCommittedVersions") {
-		return www_manageCommittedVersions($session);
-	}
-	return www_manageVersions($session);
+	my $method = $session->form->process("proceed");
+    $method    = $method eq "manageCommittedVersions" ? $method : 'manageVersions';
+    $pb->finish(WebGUI::Asset->getDefault($session)->getUrl('op='.$method));
 }
 
 
