@@ -22,7 +22,7 @@ use WebGUI::Operation::Shared;
 use WebGUI::HTML;
 use WebGUI::User;
 use WebGUI::Utility;
-
+use WebGUI::Pluggable;
 
 =head1 NAME
 
@@ -66,7 +66,50 @@ Return true iff fieldName is reserved and therefore not usable as a profile fiel
 sub isReservedFieldName {
     my $class = shift;
     my $fieldName = shift;
-    return isIn($fieldName, ('func', 'op', 'wg_privacySettings'));
+    return isIn($fieldName, qw/userId shop specialState func op wg_privacySettings username authMethod dateCreated lastUpdated karma status referringAffiliate friendsGroup/);
+}
+
+#-------------------------------------------------------------------
+
+=head2 fixDataColumnTypes ( session )
+
+Checks the column types of userProfileData against the form fields that they use.  If they
+differ then they are updated to match for the form field.  This is to account for bugs in
+this module, and changes in Form types.
+
+This is a class method.
+
+=head3 session
+
+A reference to the current session.
+
+=cut
+
+sub fixDataColumnTypes {
+    my $class       = shift;
+    my $session     = shift;
+    
+    my $dbh         = $session->db->dbh;
+
+    my $fields = WebGUI::ProfileField->getFields($session);
+    foreach my $field ( @{ $fields } ) {
+        my $columnInfo = $dbh->column_info(undef, undef, 'userProfileData', $field->getId)->fetchrow_hashref();
+        my $formField  = $field->formField(undef, undef, undef, undef, undef, 'returnObject');
+        my $columnType = $formField->getDatabaseFieldType();
+        $columnType =~ s/\s+\w+$//;
+        if ($columnType eq 'BOOLEAN') {
+            $columnType = 'TINYINT';  ##Alias for INT(1)
+        }
+        my $actualType = $columnInfo->{TYPE_NAME};
+        if ($columnType =~ m/\(\d+\)/) {
+            $actualType = sprintf('%s(%s)', $actualType, $columnInfo->{COLUMN_SIZE});
+        }
+        if ($actualType ne $columnType) {
+            $session->log->warn("Updating ".$field->getId." from $actualType to $columnType");
+            $session->db->write('ALTER TABLE userProfileData MODIFY COLUMN '.$dbh->quote_identifier($field->getId).' '.$columnType);
+        }
+    }
+
 }
 
 #-------------------------------------------------------------------
@@ -104,24 +147,24 @@ sub create {
 
     ### Check data
     # Check if the field already exists
-    my $fieldNameExists 
-        = $session->db->quickScalar(
-            "select count(*) from userProfileField where fieldName=?", 
-            [$fieldName]
-        );
-    return undef if $fieldNameExists;
+    $properties->{fieldType} ||= "ReadOnly";
+    return undef if $class->exists($session,$fieldName);
     return undef if $class->isReservedFieldName($fieldName);
 
     ### Data okay, create the field
     # Add the record
-    my $id 
-        = $session->db->setRow("userProfileField","fieldName",{fieldName=>"new"},$fieldName);
+    my $id = $session->db->setRow("userProfileField","fieldName",
+                {
+                    fieldName=>"new",
+                    fieldType => $properties->{fieldType},
+                },
+                $fieldName
+    );
     my $self = $class->new($session,$id);
     
     # Get the field's data type
-    $properties->{fieldType} ||= "ReadOnly";
     my $formClass   = $self->getFormControlClass;
-    eval "use $formClass;";
+    eval { WebGUI::Pluggable::load($formClass) };
     my $dbDataType = $formClass->getDatabaseFieldType;
 
     # Add the column to the userProfileData table
@@ -155,6 +198,24 @@ sub delete {
     $db->deleteRow("userProfileField","fieldName",$self->getId);
 }
 
+#-------------------------------------------------------------------
+
+=head2 exists ( session, fieldName )
+
+Class method that returns true if a field with the given name already 
+exists. The first argument is a WebGUI::Session object. C<fieldName> is 
+the field name to check
+
+=cut
+
+sub exists {
+    my ( $class, $session, $fieldName ) = @_;
+    
+    return 1 if $session->db->quickScalar(
+        "SELECT COUNT(*) FROM userProfileField WHERE fieldName=?",
+        [$fieldName]
+    );
+}
 
 #-------------------------------------------------------------------
 
@@ -201,7 +262,7 @@ sub _formProperties { my $self = shift; return $self->formProperties(@_); }
 
 #-------------------------------------------------------------------
 
-=head2 formField ( [ formProperties, withWrapper, userObject ] )
+=head2 formField ( [ formProperties, withWrapper, userObject, skipDefault, assignedValue ] )
 
 Returns an HTMLified form field element.
 
@@ -219,12 +280,22 @@ A WebGUI::User object reference to use instead of the currently logged in user.
 
 =head3 skipDefault
 
-If true, this causes the default value set up for the form field to be ignored.
+If true, this causes the default value set up for the form field to be ignored.  In choosing default,
+skipDefault has the highest priority.
 
 =head3 assignedValue
 
 If assignedValue is defined, it will be used to override the default value set up for the
-form.
+form.  assignedValue has the next highest priority.
+
+=head3 returnObject
+
+If true, it returns a WebGUI::Form object, instead of returning HTML.
+
+=head3 useFieldDefault
+
+If true, it uses the default setup for the ProfileField, instead of the user's default.  useFieldDefault
+has the lowest priority.
 
 =cut
 
@@ -233,13 +304,15 @@ form.
 # And refactor to not require all these arguments HERE but rather in the 
 # constructor or something...
 sub formField {
-    my $self          = shift;
-    my $session       = $self->session;
-    my $properties    = $self->formProperties(shift);
-    my $withWrapper   = shift;
-    my $u             = shift || $session->user;
-    my $skipDefault   = shift;
-    my $assignedValue = shift;
+    my $self             = shift;
+    my $session          = $self->session;
+    my $properties       = $self->formProperties(shift);
+    my $withWrapper      = shift;
+    my $u                = shift || $session->user;
+    my $skipDefault      = shift;
+    my $assignedValue    = shift;
+    my $returnObject     = shift;
+    my $useFieldDefault  = shift;
     
     if ($skipDefault) {
         $properties->{value} = undef;
@@ -247,6 +320,9 @@ sub formField {
     elsif (defined $assignedValue) {
         $properties->{value} = $assignedValue;
     }
+    elsif ($useFieldDefault) {
+        $properties->{value} = WebGUI::Operation::Shared::secureEval($session,$properties->{dataDefault});
+    }
     else {
         # start with specified (or current) user's data.  previous data needed by some form types as well (file).
         $properties->{value} = $u->profileField($self->getId);
@@ -259,12 +335,14 @@ sub formField {
             $properties->{value} = WebGUI::Operation::Shared::secureEval($session,$properties->{dataDefault});
         }
     }
+    my $form = WebGUI::Form::DynamicField->new($session,%{$properties});
+    return $form if $returnObject;
     if ($withWrapper == 1) {
-        return WebGUI::Form::DynamicField->new($session,%{$properties})->toHtmlWithWrapper;
+        return $form->toHtmlWithWrapper;
     } elsif ($withWrapper == 2) {
-        return WebGUI::Form::DynamicField->new($session,%{$properties})->getValueAsHtml;
+        return $form->getValueAsHtml;
     } else {
-        return WebGUI::Form::DynamicField->new($session,%{$properties})->toHtml;
+        return $form->toHtml;
     }
 }
 
@@ -418,7 +496,7 @@ sub getEditableFields {
 
 =head2 getFields ( session )
 
-Returns an array reference of WebGUI::ProfileField objects. This is a class method.
+Returns an array reference of all WebGUI::ProfileField objects. This is a class method.
 
 =cut
 
@@ -490,6 +568,8 @@ sub getRegistrationFields {
     return $class->_listFieldsWhere($session, "f.showAtRegistration = 1");
 }
 
+#-------------------------------------------------------------------
+
 =head2 getPasswordRecoveryFields ( session )
 
 Returns an array reference of profile field objects that are required
@@ -717,16 +797,11 @@ sub rename {
 
     ### Check data
     # Make sure the field doesn't exist
-    my $fieldNameExists
-    = $self->session->db->quickScalar(
-        "SELECT COUNT(*) FROM userProfileField WHERE fieldName=?",
-        [$newName]
-    );
-    return 0 if ($fieldNameExists);
+    return 0 if $self->exists($session, $newName);
 
     # Rename the userProfileData column
     my $fieldClass  = $self->getFormControlClass;
-    eval "use $fieldClass;";
+    eval { WebGUI::Pluggable::load($fieldClass) };
     my $dbDataType  = $fieldClass->getDatabaseFieldType;
 
     $self->session->db->write(
@@ -811,13 +886,15 @@ sub set {
     my $db          = $session->db;
 
     # Set the defaults
-    $properties->{visible} = 0 unless ($properties->{visible} == 1);
-    $properties->{editable} = 0 unless ($properties->{editable} == 1);
-    $properties->{protected} = 0 unless ($properties->{protected} == 1);
-    $properties->{required} = 0 unless ($properties->{required} == 1);
-    $properties->{label} = 'Undefined' if ($properties->{label} =~ /^[\"\']*$/);
-    $properties->{fieldType} = 'text' unless ($properties->{fieldType});
-    $properties->{extras} = '' unless ($properties->{extras});
+    $properties->{visible}   = 0           unless ($properties->{visible}   == 1);
+    $properties->{protected} = 0           unless ($properties->{protected} == 1);
+    $properties->{required}  = 0           unless ($properties->{required}  == 1);
+    $properties->{editable}  = $properties->{required} == 1 ? 1
+                             : $properties->{editable} == 1 ? 1
+                             : 0;
+    $properties->{label}     = 'Undefined' if     ($properties->{label} =~ /^[\"\']*$/);
+    $properties->{fieldType} = 'text'      unless ($properties->{fieldType});
+    $properties->{extras}    = ''          unless ($properties->{extras});
     if ($properties->{dataDefault} && $properties->{fieldType}=~/List$/) {
         unless ($properties->{dataDefault} =~ /^\[/) {
             $properties->{dataDefault} = "[".$properties->{dataDefault};
@@ -843,7 +920,7 @@ sub set {
     if ($properties->{fieldType} ne $originalFieldType) {
         # Create a copy of the new properties so we don't mess them up
         my $fieldClass  = $self->getFormControlClass;
-        eval "use $fieldClass;";
+        eval { WebGUI::Pluggable::load($fieldClass) };
         my $dbDataType 
         = $fieldClass->new($session, $self->formProperties($properties))->getDatabaseFieldType;
 
@@ -878,7 +955,7 @@ sub setCategory {
 
     return undef if ($categoryId eq $currentCategoryId);
 
-    my ($sequenceNumber) = $self->session->db->quickArray("select max(sequenceNumber) from userProfileField where profileCategoryId=".$self->session->db->quote($categoryId));
+    my ($sequenceNumber) = $self->session->db->quickArray("select max(sequenceNumber) from userProfileField where profileCategoryId=?",  [$categoryId]);
     $self->session->db->setRow("userProfileField","fieldName",{fieldName=>$self->getId, profileCategoryId=>$categoryId, sequenceNumber=>$sequenceNumber+1});
     $self->{_property}{profileCategoryId} = $categoryId;
     $self->{_property}{sequenceNumber} = $sequenceNumber+1;
