@@ -220,12 +220,13 @@ sub buildHashRef {
 
 =head2 buildArrayRefOfHashRefs ( sql )
 
-Builds an array reference of hash references of data 
-from a series of rows.  Useful for returning many rows at once.
+Builds an array reference of hash references of data from a series of rows.
+Useful for returning many rows at once.  Each element of the returned array
+reference is a hash of column names to column values.
 
 =head3 sql
 
-An SQL query. The query must select at least two columns of data, the first being the key for the hash, the second being the value. If the query selects more than two columns, then the last column will be the value and the remaining columns will be joined together by a colon ":" to form a complex key. If the query selects only one column, then the key and the value will be the same.
+An SQL query. 
 
 =head3 params
 
@@ -502,9 +503,11 @@ Disconnects from the database. And destroys the object.
 =cut
 
 sub disconnect {
-	my $self = shift;
-	$self->dbh->disconnect;
-	undef $self;
+    my $self = shift;
+    my $dbh = delete $self->{_dbh};
+    if ($dbh) {
+        $dbh->disconnect;
+    }
 }
 
 
