@@ -287,6 +287,7 @@ A string representing the output format for the date. Defaults to '%z %Z'. You c
  %P = An upper-case AM/PM.
  %s = A two digit second.
  %t = Time zone name.
+ %V = week number.
  %w = Day of the week. 
  %W = Day of the week abbreviated. 
  %y = A four digit year.
@@ -297,8 +298,11 @@ A string representing the output format for the date. Defaults to '%z %Z'. You c
 =cut
 
 sub epochToHuman {
-	my $self = shift;
-	my $epoch = shift || time();
+	my $self  = shift;
+	my $epoch = shift;
+    if (!defined $epoch || $epoch eq '') {
+        $epoch = time();
+    }
 	my $i18n = WebGUI::International->new($self->session);
 	my $language = $i18n->getLanguage($self->session->user->profileField('language'));
 	my $locale = $language->{languageAbbreviation} || 'en';
@@ -331,6 +335,7 @@ sub epochToHuman {
 		"p" => "P",
 		"P" => "p",
 		"s" => "S",
+		"V" => "V",
 		"w" => "A",
 		"W" => "a",
 		"y" => "Y",
@@ -810,7 +815,8 @@ sub new {
 
 =head2 secondsToInterval ( seconds )
 
-Returns an interval and internationalized units derived the number of seconds.
+Returns an interval and internationalized units derived the number
+of seconds, rounding to the closest unit smaller than the interval.
 
 =head3 seconds
 
@@ -856,6 +862,38 @@ sub secondsToInterval {
 
 #-------------------------------------------------------------------
 
+=head2 secondsToExactInterval ( seconds )
+
+Returns an interval and internationalized units derived the number of seconds.
+
+=head3 seconds
+
+The number of seconds in the interval. 
+
+=cut
+
+sub secondsToExactInterval {
+    my $self = shift;
+    my $seconds = shift;
+    my $i18n = WebGUI::International->new($self->session, 'WebGUI');
+    my %units = (
+        31536000    => "703", # years
+        2592000     => "702", # months
+        604800      => "701", # weeks
+        86400       => "700", # days
+        3600        => "706", # hours
+        60          => "705", # minutes
+    );
+    for my $unit (sort { $b <=> $a } keys %units) {
+        if ($seconds % $unit == 0) {
+            return ($seconds / $unit, $i18n->get($units{$unit}));
+        }
+    }
+    return ($seconds, $i18n->get("704")); # seconds
+}
+
+#-------------------------------------------------------------------
+
 =head2 secondsToTime ( seconds )
 
 Returns a time string of the format HH::MM::SS on a 24 hour clock. See also timeToSeconds().
