@@ -498,21 +498,31 @@ Returns whether all the required properties of the the cart are set.
 
 sub readyForCheckout {
     my $self    = shift;
-    
+
     # Check if the shipping address is set and correct
     my $address = eval{$self->getShippingAddress};
     return 0 if WebGUI::Error->caught;
 
-    # Check if the ship driver is chosen and existant
-    my $ship = eval {$self->getShipper};
-    return 0 if WebGUI::Error->caught;
-
     # Check if the cart has items
     return 0 unless scalar @{ $self->getItems };
     
     # fail if there are multiple recurring items or if
     return 0 if ($self->hasMixedItems);
 
+    # Check minimum cart checkout requirement
+    my $total = eval { $self->calculateTotal };
+    if (my $e = WebGUI::Error->caught) {
+        $error{id $self} = $e->error;
+        return 0;
+    }
+    my $requiredAmount = $self->session->setting->get( 'shopCartCheckoutMinimum' );
+    if ( $requiredAmount > 0 ) {
+        return 0 if $total < $requiredAmount;
+    }
+
+    ##Check for any other logged errors
+    return 0 if $error{ id $self };
+
     # All checks passed so return true
     return 1;
 }
@@ -593,7 +603,7 @@ sub updateFromForm {
             eval { $item->setQuantity($form->get("quantity-".$item->getId)) };
             if (WebGUI::Error->caught("WebGUI::Error::Shop::MaxOfItemInCartReached")) {
                 my $i18n = WebGUI::International->new($self->session, "Shop");
-                $error{id $self} = sprint($i18n->get("too many of this item"), $item->get("configuredTitle"));
+                $error{id $self} = sprintf($i18n->get("too many of this item"), $item->get("configuredTitle"));
             }
             elsif (my $e = WebGUI::Error->caught) {
                 $error{id $self} = "An unknown error has occured: ".$e->message;
@@ -733,6 +743,7 @@ sub www_view {
     my $url = $session->url;
     my $i18n = WebGUI::International->new($session, "Shop");
     my @items = ();
+    my $taxDriver = WebGUI::Shop::Tax->getDriver( $session );
 
     if($url->forceSecureConnection()){
             return "redirect";
@@ -783,12 +794,14 @@ sub www_view {
         unless (WebGUI::Error->caught) {
             $properties{shippingAddress} = $address->getHtmlFormatted;
         }
+
+        $taxDriver->appendCartItemVars( \%properties, $item );
+
         push(@items, \%properties);
     }
     my %var = (
         %{$self->get},
         items                   => \@items,
-        error                   => $error{id $self},
         formHeader              => WebGUI::Form::formHeader($session)
             . WebGUI::Form::hidden($session, {name=>"shop", value=>"cart"})
             . WebGUI::Form::hidden($session, {name=>"method", value=>"update"})
@@ -805,6 +818,10 @@ sub www_view {
         shipToButton    => WebGUI::Form::submit($session, {value=>$i18n->get("ship to button"), 
             extras=>q|onclick="setCallbackForAddressChooser(this.form);"|}),
         subtotalPrice           => $self->formatCurrency($self->calculateSubtotal()),
+        minimumCartAmount       => $session->setting->get( 'shopCartCheckoutMinimum' ) > 0
+                                 ? sprintf( '%.2f', $session->setting->get( 'shopCartCheckoutMinimum' ) )
+                                 : 0
+                                 ,
         );
 
     # get the shipping address    
@@ -814,7 +831,7 @@ sub www_view {
         $self->update({shippingAddressId=>''});
         
     }
-    
+   
     # if there is no shipping address we can't check out
     if (WebGUI::Error->caught) {
        $var{shippingPrice} = $var{tax} = $self->formatCurrency(0); 
@@ -824,7 +841,6 @@ sub www_view {
     else {
         $var{hasShippingAddress} = 1;
         $var{shippingAddress} = $address->getHtmlFormatted;
-        $var{tax} = $self->calculateTaxes;
         my $ship = WebGUI::Shop::Ship->new($self->session);
         my $options = $ship->getOptions($self);
         my %formOptions = ();
@@ -833,11 +849,25 @@ sub www_view {
             $defaultOption = $option;
             $formOptions{$option} = $options->{$option}{label}." (".$self->formatCurrency($options->{$option}{price}).")";
         }
-        $var{shippingOptions} = WebGUI::Form::selectBox($session, {name=>"shipperId", options=>\%formOptions, defaultValue=>$defaultOption, value=>$self->get("shipperId")});
-        $var{shippingPrice} = ($self->get("shipperId") ne "") ? $options->{$self->get("shipperId")}{price} : $options->{$defaultOption}{price};
-        $var{shippingPrice} = $self->formatCurrency($var{shippingPrice});
+        if ($defaultOption) {
+            $var{shippingOptions} = WebGUI::Form::selectBox($session, {name=>"shipperId", options=>\%formOptions, defaultValue=>$defaultOption, value=>$self->get("shipperId")});
+            if (! exists $options->{ $self->get('shipperId') }) {
+                $self->update({shipperId => ''});
+            }
+            $var{shippingPrice} = ($self->get("shipperId") ne "") ? $options->{$self->get("shipperId")}{price} : $options->{$defaultOption}{price};
+            $var{shippingPrice} = $self->formatCurrency($var{shippingPrice});
+        }
+        else {
+            $var{shippingOptions} = '';
+            $var{shippingPrice}   = 0;
+            $error{id $self}      = $i18n->get("No shipping plugins configured");
+        }
+
     }
-    
+  
+    # Tax variables
+    $var{tax} = $self->calculateTaxes;
+
     # POS variables
     $var{isCashier} = WebGUI::Shop::Admin->new($session)->isCashier;
     $var{posLookupForm} = WebGUI::Form::email($session, {name=>"posEmail"})
@@ -850,10 +880,12 @@ sub www_view {
     # calculate price adjusted for in-store credit
     $var{totalPrice} = $var{subtotalPrice} + $var{shippingPrice} + $var{tax};
     my $credit = WebGUI::Shop::Credit->new($session, $posUser->userId);
-    $var{inShopCreditAvailable} = $credit->getSum;
-    $var{inShopCreditDeduction} = $credit->calculateDeduction($var{totalPrice});
-    $var{totalPrice} = $self->formatCurrency($var{totalPrice} + $var{inShopCreditDeduction});
-    
+    $var{ inShopCreditAvailable } = $credit->getSum;
+    $var{ inShopCreditDeduction } = $credit->calculateDeduction($var{totalPrice});
+    $var{ totalPrice            } = $self->formatCurrency($var{totalPrice} + $var{inShopCreditDeduction});
+    $var{ readyForCheckout      } = $self->readyForCheckout;
+    $var{ error                 } = $error{id $self}; 
+
     # render the cart
     my $template = WebGUI::Asset::Template->new($session, $session->setting->get("shopCartTemplateId"));
     return $session->style->userStyle($template->process(\%var));
