@@ -1,5 +1,19 @@
 package WebGUI::Shop::PayDriver;
 
+=head1 LEGAL
+
+ -------------------------------------------------------------------
+  WebGUI is Copyright 2001-2009 Plain Black Corporation.
+ -------------------------------------------------------------------
+  Please read the legal notices (docs/legal.txt) and the license
+  (docs/license.txt) that came with this distribution before using
+  this software.
+ -------------------------------------------------------------------
+  http://www.plainblack.com                     info@plainblack.com
+ -------------------------------------------------------------------
+
+=cut
+
 use strict;
 
 use Class::InsideOut qw{ :std };
@@ -13,6 +27,7 @@ use WebGUI::Macro;
 use WebGUI::User;
 use WebGUI::Shop::Cart;
 use JSON;
+use Clone qw/clone/;
 use Scalar::Util qw/blessed/;
 
 =head1 NAME
@@ -21,13 +36,13 @@ Package WebGUI::Shop::PayDriver
 
 =head1 DESCRIPTION
 
-This package is the base class for all modules which implement a pyament driver.
+This package is the base class for all modules which implement a payment driver.
 
 =head1 SYNOPSIS
 
  use WebGUI::Shop::PayDriver;
 
- my $tax = WebGUI::Shop::PayDriver->new($session);
+ my $payDriver = WebGUI::Shop::PayDriver->new($session);
 
 =head1 METHODS
 
@@ -87,7 +102,9 @@ sub cancelRecurringPayment {
 
 =head2 canUse ( user )
 
-Checks to see if the user can use this Payment Driver.
+Checks to see if the user can use this Payment Driver.  Ability to use
+is based on whether or not this user has the correct privileges, and if
+the driver is enabled or not.
 
 =head3 user
 
@@ -107,6 +124,7 @@ A user object that will be used directly.
 
 sub canUse {
     my $self = shift;
+    return 0 unless $self->get('enabled');
     my $user = shift;
     my $userObject;
     if (!defined $user or ref($user) ne 'HASH') {
@@ -125,7 +143,8 @@ sub canUse {
     }
     return $userObject->isInGroup($self->get('groupToUse'));
 }
- #-------------------------------------------------------------------
+
+#-------------------------------------------------------------------
 
 =head2 className (  )
 
@@ -218,19 +237,6 @@ sub definition {
             hoverHelp       => $i18n->get('who can use help'),
             defaultValue    => 7,
         },
-        receiptEmailTemplateId => {
-            fieldType       => 'template',
-            namespace       => "Shop/EmailReceipt",
-            label           => $i18n->get("receipt email template"),
-            hoverHelp       => $i18n->get("receipt email template help"),
-            defaultValue    => 'bPz1yk6Y9uwMDMBcmMsSCg',
-        },
-        saleNotificationGroupId => {
-            fieldType       => 'group',
-            label           => $i18n->get("sale notification group"),
-            hoverHelp       => $i18n->get("sale notification group help"),
-            defaultValue    => '3',
-        },
     );
 
     my %properties = (
@@ -303,7 +309,7 @@ sub get {
         return $options->{ $param };
     }
     else {
-        return { %$options };
+        return { %{ $options } };
     }
 }
 
@@ -643,7 +649,7 @@ sub processTransaction {
     if ($success) {
         $transaction->completePurchase($transactionCode, $statusCode, $statusMessage);
         $cart->onCompletePurchase;
-        $self->sendNotifications($transaction);
+        $transaction->sendNotifications();
     }
     else {
         $transaction->denyPurchase($transactionCode, $statusCode, $statusMessage);
@@ -661,104 +667,6 @@ Accessor for the session object.  Returns the session object.
 
 =cut
 
-
-#-------------------------------------------------------------------
-
-=head2 sendNotifications ( transaction )
-
-Sends out a receipt and a sale notification to the buyer and the store owner respectively.
-
-=cut
-
-sub sendNotifications {
-    my ($self, $transaction) = @_;
-    my $session = $self->session;
-    my $i18n = WebGUI::International->new($session, 'PayDriver');
-    my ($style, $url) = $session->quick(qw(style url));
-    my %var = (
-        %{$transaction->get},
-        viewDetailUrl           => $url->page('shop=transaction;method=viewMy;transactionId='.$transaction->getId,1),
-        amount                  => sprintf("%.2f", $transaction->get('amount')),
-        inShopCreditDeduction   => sprintf("%.2f", $transaction->get('inShopCreditDeduction')),
-        taxes                   => sprintf("%.2f", $transaction->get('taxes')),
-        shippingPrice           => sprintf("%.2f", $transaction->get('shippingPrice')),
-        shippingAddress         => $transaction->formatAddress({
-                                        name        => $transaction->get('shippingAddressName'),
-                                        address1    => $transaction->get('shippingAddress1'),
-                                        address2    => $transaction->get('shippingAddress2'),
-                                        address3    => $transaction->get('shippingAddress3'),
-                                        city        => $transaction->get('shippingCity'),
-                                        state       => $transaction->get('shippingState'),
-                                        code        => $transaction->get('shippingCode'),
-                                        country     => $transaction->get('shippingCountry'),
-                                        phoneNumber => $transaction->get('shippingPhoneNumber'),
-                                        }),
-        paymentAddress          =>  $transaction->formatAddress({
-                                        name        => $transaction->get('paymentAddressName'),
-                                        address1    => $transaction->get('paymentAddress1'),
-                                        address2    => $transaction->get('paymentAddress2'),
-                                        address3    => $transaction->get('paymentAddress3'),
-                                        city        => $transaction->get('paymentCity'),
-                                        state       => $transaction->get('paymentState'),
-                                        code        => $transaction->get('paymentCode'),
-                                        country     => $transaction->get('paymentCountry'),
-                                        phoneNumber => $transaction->get('paymentPhoneNumber'),
-                                        }),
-        );
-    
-    # items
-    my @items = ();
-    foreach my $item (@{$transaction->getItems}) {
-        my $address = '';
-        if ($transaction->get('shippingAddressId') ne $item->get('shippingAddressId')) {
-            $address = $transaction->formatAddress({
-                            name        => $item->get('shippingAddressName'),
-                            address1    => $item->get('shippingAddress1'),
-                            address2    => $item->get('shippingAddress2'),
-                            address3    => $item->get('shippingAddress3'),
-                            city        => $item->get('shippingCity'),
-                            state       => $item->get('shippingState'),
-                            code        => $item->get('shippingCode'),
-                            country     => $item->get('shippingCountry'),
-                            phoneNumber => $item->get('shippingPhoneNumber'),
-                            });
-        }
-        push @items, {
-            %{$item->get},
-            viewItemUrl         => $url->page('shop=transaction;method=viewItem;transactionId='.$transaction->getId.';itemId='.$item->getId, 1),
-            price               => sprintf("%.2f", $item->get('price')),
-            itemShippingAddress => $address,
-            orderStatus         => $i18n->get($item->get('orderStatus'),'Shop'),
-        };
-    }
-    $var{items} = \@items;
-
-    # render
-    my $template = WebGUI::Asset::Template->new( $session, $self->get("receiptEmailTemplateId") );
-    my $inbox = WebGUI::Inbox->new($session);
-    my $receipt = $template->process(\%var);
-    WebGUI::Macro::process($session, \$receipt);
-
-    # purchase receipt
-    $inbox->addMessage( {
-        message     => $receipt,
-        subject     => $i18n->get('receipt subject').' '.$transaction->get('orderNumber'),
-        userId      => $transaction->get('userId'),
-        status      => 'completed',
-    } );
-    
-    # shop owner notification
-    $var{viewDetailUrl} = $url->page('shop=transaction;method=view;transactionId='.$transaction->getId,1);
-    my $notification = $template->process(\%var);
-    WebGUI::Macro::process($session, \$notification);
-    $inbox->addMessage( {
-        message     => $notification,
-        subject     => $i18n->get('a sale has been made').' '.$transaction->get('orderNumber'),
-        groupId     => $self->get('saleNotificationGroupId'),
-        status      => 'unread',
-    } );
-}
-
 #-------------------------------------------------------------------
 
 =head2 update ( $options )
@@ -783,6 +691,8 @@ sub update {
         $jsonOptions,
         $self->paymentGatewayId
     ]);
+    my $storedProperties = clone $properties;
+    $options{ id $self } = $storedProperties;
 
     return;
 }
