@@ -1,5 +1,19 @@
 package WebGUI::Shop::PayDriver::ITransact;
 
+=head1 LEGAL
+
+ -------------------------------------------------------------------
+  WebGUI is Copyright 2001-2009 Plain Black Corporation.
+ -------------------------------------------------------------------
+  Please read the legal notices (docs/legal.txt) and the license
+  (docs/license.txt) that came with this distribution before using
+  this software.
+ -------------------------------------------------------------------
+  http://www.plainblack.com                     info@plainblack.com
+ -------------------------------------------------------------------
+
+=cut
+
 use strict;
 use XML::Simple;
 use Data::Dumper;
@@ -276,6 +290,24 @@ sub cancelRecurringPayment {
 }
 
 #-------------------------------------------------------------------
+
+=head2 checkRecurringTransaction ( $xid, $expectedAmount )
+
+Make an XML request back to ITransact to verify a recurring transaction.  Returns 0 if
+the transaction cannot be verified or is incorrect.  Otherwise, it returns 1.
+
+NOTE: THIS CODE IS NOT CALLED ANYWHERE.
+
+=head3 $xid
+
+Transaction ID, from ITransact.
+
+=head3 $expectedAmount
+
+The amount we think should be charged in this transaction.
+
+=cut
+
 sub checkRecurringTransaction {
     my $self            = shift;
     my $xid             = shift;
@@ -290,7 +322,7 @@ sub checkRecurringTransaction {
                 HomePage        => ,
             },
             RecurDetails            => {
-                OperiationXID   => $xid,
+                OperiationXID   => $xid, ##BUGGO, typo?
             },
         }
     };
@@ -346,6 +378,8 @@ sub checkRecurringTransaction {
 sub definition {
     my $class       = shift;
     my $session     = shift;
+    WebGUI::Error::InvalidParam->throw(error => q{Must provide a session variable})
+        unless ref $session eq 'WebGUI::Session';
     my $definition  = shift;
 
     my $i18n = WebGUI::International->new($session, 'PayDriver_ITransact');
@@ -392,7 +426,7 @@ sub definition {
 
 #-------------------------------------------------------------------
 
-=head2 doXmlRequest ( xml [ isAdministrative ] )
+=head2 doXmlRequest ( xml, [ isGatewayInterface ] )
 
 Post an xml request to the ITransact backend. Returns a LWP::UserAgent response object.
 
@@ -425,7 +459,7 @@ sub doXmlRequest {
     # Create a request and stuff the xml in it
     my $request = HTTP::Request->new( POST => $xmlTransactionScript );
 	$request->content_type( 'text/xml' );
-	$request->content( $xml );
+	$request->add_content_utf8( $xml );
 
     # Do the request
     my $response = $userAgent->request($request);
@@ -434,6 +468,14 @@ sub doXmlRequest {
 }
 
 #-------------------------------------------------------------------
+
+=head2 getButton 
+
+Return a form to select this payment driver and to accept credentials from those
+who wish to use it.
+
+=cut
+
 sub getButton {
     my $self    = shift;
     my $session = $self->session;
@@ -459,6 +501,14 @@ sub handlesRecurring {
 }
 
 #-------------------------------------------------------------------
+
+=head2 processCredentials 
+
+Process the form where credentials (name, address, phone number and credit card information)
+are entered.
+
+=cut
+
 sub processCredentials {
     my $self    = shift;
     my $session = $self->session;
@@ -555,6 +605,17 @@ sub getBillingAddress {
 }
 
 #-------------------------------------------------------------------
+
+=head2 processPayment ($transaction)
+
+Contact ITransact and submit the payment data to them for processing.
+
+=head3 $transaction
+
+A WebGUI::Shop::Transaction object to pull information from.
+
+=cut
+
 sub processPayment {
     my $self        = shift;
     my $transaction = shift;
@@ -562,7 +623,6 @@ sub processPayment {
 
     # Get the payment definition XML
     my $xml = $self->_generatePaymentRequestXML( $transaction );
-    $session->errorHandler->debug("XML Request: $xml");
 
     # Send the xml to ITransact
     my $response = $self->doXmlRequest( $xml );
@@ -645,6 +705,17 @@ sub www_edit {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_getCredentials ( $errors )
+
+Build a templated form for asking the user for their credentials.
+
+=head3 $errors
+
+An array reference of errors to show the user.
+
+=cut
+
 sub www_getCredentials {
     my $self        = shift;
     my $errors      = shift;
@@ -712,7 +783,7 @@ sub www_getCredentials {
     });
     $var->{countryField} = WebGUI::Form::country($session, {
         name  => 'country',
-        value => ($form->process("country",'country') || $addressData->{ country } || $u->profileField("homeCountry") || 'United States'),
+        value => ($form->process("country",'country', '') || $addressData->{ country } || $u->profileField("homeCountry") || 'United States of A'),
     });
     $var->{phoneField} = WebGUI::Form::phone($session, {
         name  => 'phone',
@@ -754,6 +825,15 @@ sub www_getCredentials {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_pay 
+
+Makes sure that the user has all the requirements for checking out, including
+getting credentials, it processes the transaction and then displays a thank
+you screen.
+
+=cut
+
 sub www_pay {
     my $self        = shift;
     my $session     = $self->session;
@@ -782,6 +862,15 @@ sub www_pay {
 }
 
 #-------------------------------------------------------------------
+
+=head2 www_processRecurringTransactionPostback 
+
+Callback method for ITransact to dial up WebGUI and post the results of a
+recurring transaction.  This allows WebGUI to renew group memberships or
+do whatever other activity a Sku purchase would allow.
+
+=cut
+
 sub www_processRecurringTransactionPostback {
 	my $self    = shift;
     my $session = $self->session;
