@@ -23,6 +23,7 @@ use File::Find ();
 use File::Path ();
 use File::Spec;
 use Image::Magick;
+use Path::Class::Dir;
 use Storable ();
 use WebGUI::Utility qw(isIn);
 
@@ -105,6 +106,76 @@ sub _addError {
 	$self->session->errorHandler->error($errorMessage);
 }
 
+#-------------------------------------------------------------------
+
+=head2 _cdnAdd ( )
+
+Adds to CDN queue, for any of the add* methods.
+
+NOTE: This is a private method and should never be called except internally to this package.
+
+=cut
+
+sub _cdnAdd {
+    my $self   = shift;
+    my $cdnCfg = $self->session->config->get('cdn');
+    if ( $cdnCfg and $cdnCfg->{'enabled'} ) {
+        if ( $cdnCfg->{'queuePath'} ) {
+            my $cdnFile = $cdnCfg->{'queuePath'} . '/' . $self->getDirectoryId;
+            my $dest;
+            if ( open $dest, '>', $cdnFile ) {
+                close $dest;    # created empty file
+            }
+            else {
+                $self->_addError( "CDN: Couldn't open file $cdnFile for writing due to error: " . $! );
+            }
+        }
+        else {
+            $self->_addError('Invalid CDN configuration - missing queuePath');
+        }
+    }
+} ## end sub _cdnAdd
+
+#-------------------------------------------------------------------
+
+=head2 _cdnDel ( [delDotCdn] )
+
+Add file denoting deletion to the CDN queue, for the clear & delete methods.
+
+NOTE: This is a private method and should never be called except internally to this package.
+
+=head3 delDotCdn
+
+Delete the ".cdn" file - clear vs. delete.
+
+=cut
+
+sub _cdnDel {
+    my $self      = shift;
+    my $delDotCdn = shift;
+    my $cdnCfg    = $self->session->config->get('cdn');
+    if ( $cdnCfg and $cdnCfg->{'enabled'} ) {
+        my $cdnFile;    # path/name of flag and/or queue file
+        if ($delDotCdn) {
+            $cdnFile = $self->getPath . '/.cdn';
+            unlink $cdnFile;
+        }
+        if ( $cdnCfg->{'queuePath'} ) {
+            $cdnFile = $cdnCfg->{'queuePath'} . '/' . $self->getDirectoryId;
+            my $dest;
+            if ( open $dest, '>', $cdnFile ) {
+                print $dest "deleted\n";
+                close $dest;
+            }
+            else {
+                $self->_addError( "Couldn't open file $cdnFile for writing due to error: " . $! );
+            }
+        }
+        else {
+            $self->_addError('Invalid CDN configuration - missing queuePath');
+        }
+    } ## end if ( $cdnCfg and $cdnCfg...
+} ## end sub _cdnDel
 
 #-------------------------------------------------------------------
 
@@ -134,12 +205,16 @@ sub _makePath {
 
 #-------------------------------------------------------------------
 
-=head2 _changeOwner ( )
+=head2 _changeOwner ( $file )
 
-Changes the owner to be the same as that of the uploads directory
+Changes the permissions of $file to be the same as that of the uploads directory
 
 NOTE: This is a private method and should never be called except internally to this package.
 
+=head3 $file
+
+A file or directory.  It will have its permissions changed.
+
 =cut
 
 sub _changeOwner {
@@ -158,6 +233,8 @@ sub _changeOwner {
 
 Generates a captcha image (200x x 50px) and returns the filename and challenge string (6 random characters). For more information about captcha, consult the Wikipedia here: http://en.wikipedia.org/wiki/Captcha
 
+Note: captcha images will NOT be synchronized to a CDN, even if other files are.
+
 =cut 
 
 sub addFileFromCaptcha {
@@ -212,6 +289,7 @@ sub addFileFromCaptcha {
 =head2 addFileFromFilesystem( pathToFile )
 
 Grabs a file from the server's file system and saves it to a storage location and returns a URL compliant filename.  If there are errors encountered during the add, then it will return undef instead.
+If configured for CDN, add this storage location to CDN queue.
 
 =head3 pathToFile
 
@@ -256,6 +334,7 @@ sub addFileFromFilesystem {
         or $self->_addError("Couldn't copy $pathToFile to ".$self->getPath($filename).": $!");
     close $dest;
     close $source;
+    $self->_cdnAdd;
     return $filename;
 }
 
@@ -265,6 +344,7 @@ sub addFileFromFilesystem {
 =head2 addFileFromFormPost ( formVariableName, attachmentLimit )
 
 Grabs an attachment from a form POST and saves it to this storage location.
+If configured for CDN, add this storage location to CDN queue.
 
 =head3 formVariableName
 
@@ -288,9 +368,11 @@ sub addFileFromFormPost {
     my $filename;
     my $attachmentCount = 1;
     foreach my $upload ($session->request->upload($formVariableName)) {
-        $session->errorHandler->info("Trying to get " . $upload->filename);
-        return $filename
-            if $attachmentCount > $attachmentLimit;
+        $session->errorHandler->info("Trying to get " . $upload->filename." from ".$formVariableName);
+        if ( $attachmentCount > $attachmentLimit ) {
+            $self->_cdnAdd;
+            return $filename;
+        }
         my $clientFilename = $upload->filename;
         next
             unless $clientFilename;
@@ -299,6 +381,7 @@ sub addFileFromFormPost {
         next
             if ($upload->size > 1024 * $self->session->setting->get("maxAttachmentSize"));
         $clientFilename =~ s/.*[\/\\]//;
+        $clientFilename =~ s/^thumb-//;
         my $type = $self->getFileExtension($clientFilename);
         if (isIn($type, qw(pl perl sh cgi php asp html htm))) { # make us safe from malicious uploads
             $clientFilename =~ s/\./\_/g;
@@ -316,6 +399,7 @@ sub addFileFromFormPost {
             return undef;
         }
     }
+    $filename  and  $self->_cdnAdd;
     return $filename;
 }
 
@@ -325,6 +409,7 @@ sub addFileFromFormPost {
 =head2 addFileFromHashref ( filename, hashref )
 
 Stores a hash reference as a file and returns a URL compliant filename. Retrieve the data with getFileContentsAsHashref.
+If configured for CDN, add this storage location to CDN queue.
 
 =head3 filename
 
@@ -343,6 +428,7 @@ sub addFileFromHashref {
     Storable::nstore($hashref, $self->getPath($filename))
         or $self->_addError("Couldn't create file ".$self->getPath($filename)." because ".$!);
     $self->_changeOwner($self->getPath($filename));
+	$filename  and  $self->_cdnAdd;
 	return $filename;
 }
 
@@ -351,6 +437,7 @@ sub addFileFromHashref {
 =head2 addFileFromScalar ( filename, content )
 
 Adds a file to this storage location and returns a URL compliant filename.
+If configured for CDN, add this storage location to CDN queue.
 
 =head3 filename
 
@@ -373,6 +460,7 @@ sub addFileFromScalar {
 		print $FILE $content;
 		close($FILE);
         $self->_changeOwner($self->getPath($filename));
+        $self->_cdnAdd;
 	}
     else {
         $self->_addError("Couldn't create file ".$self->getPath($filename)." because ".$!);
@@ -414,16 +502,33 @@ sub adjustMaxImageSize {
 
 =head2 clear ( )
 
-Clears a storage locations of all files except the .wgaccess file
+Clears a storage location of all files.  If configured for CDN, add
+deletion of this location's files, to CDN queue.
 
 =cut
 
 sub clear {
 	my $self = shift;
-	my $filelist = $self->getFiles(1);
-	foreach my $file (@{$filelist}) {
-       $self->deleteFile($file);
+    my $dir  = $self->getPathClassDir;
+    return undef if !defined $dir;
+    my $errors;
+    CHILD: while (my $child = $dir->next()) {
+        my $rel = $child->relative($dir);
+        next CHILD if $rel->stringify eq '.'
+                   || $rel->stringify eq '..';
+        if ($child->is_dir) {
+            my $errors;
+            $child->rmtree({ error => \$errors});
+            foreach my $error (@{ $errors }){
+                $self->_addError($error);
+            }
+        }
+        else {
+            $child->remove;
+            next CHILD;
+        }
     }
+    $self->_cdnDel(1);
 }
 
 
@@ -431,7 +536,8 @@ sub clear {
 
 =head2 copy ( [ storage, filelist ] )
 
-Copies a storage location and it's contents. Returns a new storage location object. Note that this does not copy privileges or other special filesystem properties.
+Copies a storage location and its contents. Returns a new storage location object. Note that this does not copy privileges or other special filesystem properties.
+If configured for CDN, add the resulting new storage location to CDN queue.
 
 =head3 storage
 
@@ -446,15 +552,26 @@ Optionally pass in the list of filenames to copy from the specified storage loca
 sub copy {
     my $self = shift;
     my $newStorage = shift || WebGUI::Storage->create($self->session);
-    my $filelist = shift || $self->getFiles(1);
-    foreach my $file (@{$filelist}) {
-        open my $source, '<:raw', $self->getPath($file) or next;
-        open my $dest, '>:raw', $newStorage->getPath($file) or next;
-        File::Copy::copy($source, $dest) or $self->_addError("Couldn't copy file ".$self->getPath($file)." to ".$newStorage->getPath($file)." because ".$!);
-        close $dest;
-        close $source;
-        $newStorage->_changeOwner($newStorage->getPath($file));
+    my $filelist   = shift || $self->getFiles('all');
+    FILE: foreach my $file (@{$filelist}) {
+        next if isIn($file, '.cdn', '.');
+        my $origFile = $self->getPath($file);
+        my $copyFile = $newStorage->getPath($file);
+        if (-d $origFile) {
+            mkdir($copyFile) or
+                $self->_addError("Couldn't copy directory: $copyFile: $!");
+                next FILE;
+        }
+        else {
+            open my $source, '<:raw', $origFile or next FILE;
+            open my $dest,   '>:raw', $copyFile or next FILE;
+            File::Copy::copy($source, $dest) or $self->_addError("Couldn't copy file ".$origFile." to ".$copyFile." because ".$!);
+            close $dest;
+            close $source;
+        }
+        $newStorage->_changeOwner($copyFile);
     }
+    $newStorage->_cdnAdd;
     return $newStorage;
 }
 
@@ -464,6 +581,7 @@ sub copy {
 
 Copy a file in this storage location. C<filename> is the file to copy. 
 C<newFilename> is the new file to create.
+If configured for CDN, add this storage location to CDN queue.
 
 =cut
 
@@ -481,6 +599,7 @@ sub copyFile {
         || croak "Couldn't copy '$filename' to '$newFilename': $!";
     $self->_changeOwner($self->getPath($filename));
 
+    $self->_cdnAdd;
     return undef;
 }
 
@@ -528,7 +647,7 @@ sub createTemp {
 	my $path    = $session->id->toHex($id);
 
 	$path =~ m/^(.{2})/;
-	my $self = {_session=>$session, _id => $id, _pathParts => ['temp', $1, $path], _errors => []};
+	my $self = {_session=>$session, _id => $id, _hexId => $path, _pathParts => ['temp', $1, $path], _errors => []};
 	bless $self, ref($class)||$class;
 	$self->_makePath;
 	return $self;
@@ -539,6 +658,7 @@ sub createTemp {
 =head2 delete ( )
 
 Deletes this storage location and its contents (if any) from the filesystem.
+If configured for CDN, add deletion of this storage location to CDN queue.
 
 =cut
 
@@ -546,14 +666,15 @@ sub delete {
 	my $self = shift;
 
     my $path = $self->getPath || return undef;
-    File::Path::rmtree($path)
-        if (-d $path);
+    File::Path::rmtree($path) if (-d $path);
     foreach my $subDir (join('/', @{$self->{_pathParts}}[0,1]), $self->{_pathParts}[0]) {
         my $fullPath = $self->session->config->get('uploadsPath') . '/' . $subDir;
 
         # can only remove empty directories, will fail silently otherwise
         rmdir $fullPath;
     }
+    # Delete the content from the CDN - enqueue
+    $self->_cdnDel(0);
     $self->session->errorHandler->info("Deleted storage ".$self->getId);
     return undef;
 }
@@ -562,7 +683,8 @@ sub delete {
 
 =head2 deleteFile ( filename )
 
-Deletes a file from it's storage location.
+Deletes a file from its storage location, and any thumbnails that might have been made
+with it.
 
 =head3 filename
 
@@ -581,6 +703,40 @@ sub deleteFile {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 deleteFromCdn ( )
+
+Run config/cdn/deleteProgram to synchronize this location to Content Delivery Network.
+Replace %s with the path of the storage location.
+Also delete the related file in config/cdn/queuePath.
+
+=cut
+
+sub deleteFromCdn {
+    my $self   = shift;
+    my $cdnCfg = $self->session->config->get('cdn');
+    if (    $cdnCfg
+        and $cdnCfg->{'enabled'}
+        and $cdnCfg->{'syncProgram'} )
+    {
+        my $id = $self->getDirectoryId;
+        my $cmd = sprintf( $cdnCfg->{'deleteProgram'}, $id );
+        if ( $cmd =~ /$id/ ) {    # sanity check, no rm -rf /
+            system($cmd);
+            if ($?) {             # This may occur benign in the case delete after clear
+                $self->_addError("Error running CDN deleteProgram: $?");
+            }
+            if ( $cdnCfg->{'queuePath'} ) {
+                unlink $cdnCfg->{'queuePath'} . '/' . $id;
+            }
+        }
+        else {                    # Presume configuration error, missing %s
+            $self->_addError("CDN deleteProgram: storage ID missing from command: $cmd");
+        }
+    } ## end if ( $cdnCfg and $cdnCfg...
+} ## end sub deleteFromCdn
+
 #-------------------------------------------------------------------
 
 =head2 get ( session, id )
@@ -604,7 +760,7 @@ sub get {
     my $id      = shift;
     return undef
         unless $id;
-    my $self = bless {_session=>$session, _id => $id, _errors => []}, $class;
+    my $self = bless {_session=>$session, _id => $id, _hexId => $session->id->toHex($id), _errors => []}, $class;
 
     my $uploadsRoot = $session->config->get('uploadsPath');
     my @parts = ($id =~ m/^((.{2})(.{2}).+)/)[1,2,0];
@@ -613,8 +769,7 @@ sub get {
         return $self;
     }
     if (!-e join('/', $uploadsRoot, @parts)) {
-        my $hexId = $session->id->toHex($id);
-        @parts = ($hexId =~ m/^((.{2})(.{2}).+)/)[1,2,0];
+        @parts = ($self->getHexId =~ m/^((.{2})(.{2}).+)/)[1,2,0];
     }
     $self->{_pathParts} = \@parts;
     # create the folder in case it got deleted somehow
@@ -676,6 +831,80 @@ sub generateThumbnail {
 	return 1;
 }
 
+
+#-------------------------------------------------------------------
+
+=head2 getCdnFileIterator ( session )
+
+Class method to return an iterator method full of storage objects to
+be updated or deleted, based upon what's in config/cdn/queuePath.
+
+=head3 session
+
+A reference to the current session.
+
+=cut
+
+sub getCdnFileIterator {
+    my $class   = shift;
+    my $session = shift;
+    my $cdnCfg  = $session->config->get('cdn');
+    if ( $cdnCfg and $cdnCfg->{'enabled'} ) {
+        if ( $cdnCfg->{'queuePath'} ) {
+            if ( opendir my $DH, $cdnCfg->{'queuePath'} ) {
+                my @ids = grep { !/^\.+$/ } readdir($DH);
+                close $DH;
+                my $sub = sub {
+                    my $id = shift @ids;
+                    return if !$id;
+                    my $storageId;
+                    if (length($id) > 22) {
+                        # convert from hex
+                        $storageId = $session->id->fromHex($id);
+                    } else {
+                        $storageId = $id;
+                    }
+                    return $class->get( $session, $storageId );
+                };
+                return $sub;
+            }
+            else {
+                $session->errorHandler->warn("CDN: cannot read directory $cdnCfg->{'queuePath'}");
+            }
+        }
+        else {
+            $session->errorHandler->warn("CDN: enabled but no queuePath");
+        }
+    } ## end if ( $cdnCfg and $cdnCfg...
+} ## end sub getCdnFileIterator
+
+#-------------------------------------------------------------------
+
+=head2 getSize ( filename ) 
+
+Returns width and height of image.
+
+=head3 filename
+
+The file to generate a thumbnail for.
+
+=cut
+
+sub getSize {
+	my $self = shift;
+	my $filename = shift;
+        my $image = Image::Magick->new;
+        my $error = $image->Read($self->getPath($filename));
+	if ($error) {
+		$self->session->errorHandler->error("Couldn't read image for size reading: ".$error);
+		return 0;
+	}
+        my ($x, $y) = $image->Get('width','height');
+
+	return($x, $y);
+}
+
+
 #-------------------------------------------------------------------
 
 =head2 getErrorCount ( )
@@ -818,24 +1047,30 @@ Returns an array reference of the files in this storage location.
 
 =head3 showAll
 
-Whether or not to return all files, including ones with initial periods.
+Whether or not to return all files, including ones with initial periods and WebGUI created thumbnail
+files (files that start with "thumb-").
 
 =cut
 
 sub getFiles {
-    my $self = shift;
+    my $self    = shift;
     my $showAll = shift;
+    my $dir     = $self->getPathClassDir;
+    return [] if ! defined $dir;
+    my $dirStr  = $dir->stringify;
     my @list;
-    if ( opendir my $dir, $self->getPath ) {
-        @list = readdir $dir;
-        closedir $dir;
-        if (!$showAll) {
-            # if not showing all, filter out files beginning with a period
-            @list = grep { $_ !~ /^\./ } @list;
-            # filter out thumbnails
-            @list = grep { $_ !~ /^thumb-/ } @list;
+    $dir->recurse(
+        callback => sub {
+            my $obj = shift;
+            my $rel = $obj->relative($dir);
+            my $str = $rel->stringify;
+            if (! $showAll ) {
+                return if $str =~ /^thumb-/;
+                return if $str =~ /^\./;
+            }
+            push @list, $str;
         }
-    }
+    );
     return \@list;
 }
 
@@ -854,6 +1089,20 @@ sub getFileId {
 
 #-------------------------------------------------------------------
 
+=head2 getHexId ( )
+
+Returns the unique identifier of this storage location in hexadecimal format
+instead of base64.
+
+=cut
+
+sub getHexId {
+	my $self = shift;
+	return $self->{_hexId};
+}
+
+#-------------------------------------------------------------------
+
 =head2 getId ( )
 
 Returns the unique identifier of this storage location.
@@ -910,6 +1159,32 @@ sub getPath {
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 getPathClassDir (  )
+
+Returns a Path::Class::Dir object for this storage location.  If the uploadsPath is not
+configured, or the path parts don't exist, then it returns undef.
+
+=cut
+
+sub getPathClassDir {
+	my $self = shift;
+	my $file = shift;
+
+    unless ($self->session->config->get("uploadsPath") && $self->{_pathParts} && @{ $self->{_pathParts} }) {
+		$self->_addError("storage object malformed");
+		return undef;
+    }
+    my $dir = Path::Class::Dir->new($self->session->config->get("uploadsPath"), @{ $self->{_pathParts} });
+    if (! -e $dir->stringify) {
+        $self->_addError("directory for storage location ". $self->getId." does not exist");
+        return undef;
+    }
+    return $dir;
+}
+
+
 #-------------------------------------------------------------------
 
 =head2 getPathFrag (  )
@@ -995,18 +1270,32 @@ If specified, we'll return a URL to the file rather than the storage location.
 =cut
 
 sub getUrl {
-	my $self = shift;
-	my $file = shift;
-	my $url = $self->session->config->get("uploadsURL")
-            . '/'
-            . $self->getPathFrag;
-	if (defined $file) {
-		$url .= '/'.$file;
-	}
-	return $url;
-}
-
-
+    my $self   = shift;
+    my $file   = shift;
+    my $url    = $self->session->config->get("uploadsURL") . '/' . $self->getPathFrag;
+    my $cdnCfg = $self->session->config->get('cdn');
+    if (    $cdnCfg
+        and $cdnCfg->{'enabled'}
+        and $cdnCfg->{'url'}
+        and -e $self->getPath . '/.cdn' )
+    {
+        my $sep = '/';    # separator, if not already present trailing
+        if ($cdnCfg->{'sslAlt'} && $self->session->env->sslRequest) {
+            if ( $cdnCfg->{'sslUrl'} ) {
+                substr( $cdnCfg->{'sslUrl'}, -1 ) eq '/' and $sep = '';
+                $url = $cdnCfg->{'sslUrl'} . $sep . $self->getDirectoryId;
+            }             # else do NOT override $url with CDN URL  ($url = $sslUrl || $url)
+        }
+        else {
+            substr( $cdnCfg->{'url'}, -1 ) eq '/' and $sep = '';
+            $url = $cdnCfg->{'url'} . $sep . $self->getDirectoryId;
+        }
+    } ## end if ( $cdnCfg and $cdnCfg...
+    if ( defined $file ) {
+        $url .= '/' . $file;
+    }
+    return $url;
+} ## end sub getUrl
 
 #-------------------------------------------------------------------
 
@@ -1054,7 +1343,7 @@ sub renameFile {
 
 #-------------------------------------------------------------------
 
-=head2 resize ( filename [, width, height ] )
+=head2 crop ( filename [, width, height ] )
 
 Resizes the specified image by the specified height and width. If either is omitted the iamge will be scaleed proportionately to the non-omitted one.
 
@@ -1070,6 +1359,207 @@ The new width of the image in pixels.
 
 The new height of the image in pixels.
 
+=head3 x
+
+The top of the image in pixels.
+
+=head3 y
+
+The top of the image in pixels.
+
+=cut
+
+# TODO: Make this take a hash reference with width, height, and density keys.
+
+sub crop { 
+    my $self        = shift;
+    my $filename    = shift;
+    my $width       = shift;
+    my $height      = shift;
+    my $x           = shift;
+    my $y           = shift;
+    unless (defined $filename) {
+        $self->session->errorHandler->error("Can't resize when you haven't specified a file.");
+        return 0;
+    }
+    unless ($self->isImage($filename)) {
+        $self->session->errorHandler->error("Can't resize something that's not an image.");
+        return 0;
+    }
+    unless ($width || $height || $x || $y) {
+        $self->session->errorHandler->error("Can't resize with no resizing parameters.");
+        return 0;
+    }
+    my $image = Image::Magick->new;
+    my $error = $image->Read($self->getPath($filename));
+    if ($error) {
+        $self->session->errorHandler->error("Couldn't read image for resizing: ".$error);
+        return 0;
+    }
+
+    # Next, resize dimensions
+    if ( $width || $height || $x || $y ) {
+        $self->session->errorHandler->info( "Resizing $filename to w:$width h:$height x:$x y:$y" );
+        $image->Crop( height => $height, width => $width, x => $x, y => $y );
+    }
+
+    # Write our changes to disk
+    $error = $image->Write($self->getPath($filename));
+    if ($error) {
+        $self->session->errorHandler->error("Couldn't resize image: ".$error);
+        return 0;
+    }
+
+    return 1;
+}
+
+#-------------------------------------------------------------------
+
+=head2 annotate ( filename, $asset, $form )
+
+Adds annotation text to the image.
+
+=head3 filename
+
+The name of the file to annotate.
+
+=head3 asset
+
+The WebGUI Asset containing this image.
+
+=head3 $form
+
+A Session::Form object, used to fetch form data.
+
+=cut
+
+sub annotate { 
+    my $self        = shift;
+    my $filename    = shift;
+    my $asset       = shift;
+    my $form        = shift;
+    unless (defined $filename) {
+        $self->session->errorHandler->error("Can't rotate when you haven't specified a file.");
+        return 0;
+    }
+    unless ($self->isImage($filename)) {
+        $self->session->errorHandler->error("Can't rotate something that's not an image.");
+        return 0;
+    }
+    # unless ($annotate_text) {
+    # $self->session->errorHandler->error("Can't annotate with no text.");
+    # return 0;
+    # }
+    # unless ($annotate_top && $annotate_left && $annotate_width && $annotate_height) {
+    # $self->session->errorHandler->error("Can't annotate with no dimensions.");
+    # return 0;
+    # }
+
+    my $annotate = $asset->get('annotations');
+    my $save_annotate = "";
+	my @pieces = split(/\n/, $annotate);
+	for (my $i = 0; $i < $#pieces; $i += 3) {
+		my $top_left = $pieces[$i];
+		my $width_height = $pieces[$i + 1];
+		my $note = $pieces[$i + 2];
+
+        # warn("i: $i: ", $form->process("delAnnotate$i"));
+        next if $form->process("delAnnotate$i");
+
+        if ($save_annotate) {
+            $save_annotate .= "\n";
+        }
+        $save_annotate .= "$top_left\n$width_height\n$note";
+    }
+
+	my $annotate_text   = $form->process("annotate_text");
+	my $annotate_top    = $form->process("annotate_top");
+	my $annotate_left   = $form->process("annotate_left");
+	my $annotate_width  = $form->process("annotate_width");
+	my $annotate_height = $form->process("annotate_height");
+    # warn(qq(unless ($annotate_top && $annotate_left && $annotate_width && $annotate_height && $annotate_text !~ /^\s*$/)));
+    if (defined $annotate_top && defined $annotate_left && defined $annotate_width && defined $annotate_height && $annotate_text !~ /^\s*$/) {
+        if ($save_annotate) {
+            $save_annotate .= "\n";
+        }
+        # warn(qq($save_annotate .= "top: ${annotate_top}px; left: ${annotate_left}px;\nwidth: ${annotate_width}px; height: ${annotate_height}px;\n'$annotate_text'"));
+        $save_annotate .= "top: ${annotate_top}px; left: ${annotate_left}px;\nwidth: ${annotate_width}px; height: ${annotate_height}px;\n$annotate_text";
+    }
+    # warn($save_annotate);
+
+    $asset->update({ annotations => $save_annotate });
+    $save_annotate = $asset->get('annotations');
+    # warn($save_annotate);
+
+    return 1;
+}
+
+#-------------------------------------------------------------------
+
+=head2 rotate ( filename [ degrees ] )
+
+Rotates the image by the specified degrees.
+
+=head3 filename
+
+The name of the file to resize.
+
+=head3 width
+
+Number of degrees to rotate.
+
+=cut
+
+sub rotate { 
+    my $self        = shift;
+    my $filename    = shift;
+    my $degree      = shift || 0;
+    unless (defined $filename) {
+        $self->session->errorHandler->error("Can't rotate when you haven't specified a file.");
+        return 0;
+    }
+    unless ($self->isImage($filename)) {
+        $self->session->errorHandler->error("Can't rotate something that's not an image.");
+        return 0;
+    }
+    my $image = Image::Magick->new;
+    my $error = $image->Read($self->getPath($filename));
+    if ($error) {
+        $self->session->errorHandler->error("Couldn't read image for resizing: ".$error);
+        return 0;
+    }
+
+    $self->session->errorHandler->info( "Rotating $filename by $degree degrees" );
+    $image->Rotate( $degree );
+
+    # Write our changes to disk
+    $error = $image->Write($self->getPath($filename));
+    if ($error) {
+        $self->session->errorHandler->error("Couldn't rotate image: ".$error);
+        return 0;
+    }
+
+    return 1;
+}
+
+#-------------------------------------------------------------------
+
+=head2 resize ( filename [, width, height ] )
+
+Resizes the specified image by the specified height and width. If either is omitted the iamge will be scaled proportionately to the non-omitted one.
+
+=head3 filename
+
+The name of the file to resize.
+
+=head3 width
+
+The new width of the image in pixels.
+
+=head3 height
+
+The new height of the image in pixels.
+
 =head3 density
 
 The new image density in pixels per inch. 
@@ -1175,16 +1665,67 @@ sub setPrivileges {
 	my $owner = shift;
 	my $viewGroup = shift;
 	my $editGroup = shift;
-	
-    if ($owner eq '1' || $viewGroup eq '1' || $viewGroup eq '7' || $editGroup eq '1' || $editGroup eq '7') {
-        $self->deleteFile('.wgaccess');
-    }
-    else {
-        $self->addFileFromScalar(".wgaccess",$owner."\n".$viewGroup."\n".$editGroup);
-    }
+
+    my $dirObj = $self->getPathClassDir();
+    return undef if ! defined $dirObj;
+    $dirObj->recurse(
+        callback => sub {
+            my $obj = shift;
+            return unless $obj->is_dir;
+            my $rel = $obj->relative($dirObj);
+
+            if ($owner eq '1' || $viewGroup eq '1' || $viewGroup eq '7' || $editGroup eq '1' || $editGroup eq '7') {
+                $self->deleteFile($rel->file('.wgaccess')->stringify);
+            }
+            else {
+                $self->addFileFromScalar($rel->file('.wgaccess')->stringify,$owner."\n".$viewGroup."\n".$editGroup);
+            }
+        }
+    );
+
 }
 
 
+#-------------------------------------------------------------------
+
+=head2 syncToCdn ( )
+
+Run config/cdn/syncProgram to synchronize this location to Content Delivery Network.
+Replace %s with the path of the storage location.
+Also put an empty ".cdn" file in the storage location, and then delete
+the related file in config/cdn/queuePath.
+
+=cut
+
+sub syncToCdn {
+    my $self   = shift;
+    my $cdnCfg = $self->session->config->get('cdn');
+    if (    $cdnCfg
+        and $cdnCfg->{'enabled'}
+        and $cdnCfg->{'syncProgram'} )
+    {
+        my $originalDir = Cwd::cwd();
+        my $locDir = join '/', $self->session->config->get('uploadsPath'), @{ $self->{_pathParts} }[ 0 .. 1 ];
+        chdir $locDir or croak 'Unable to chdir to ' . $locDir . " : $!";
+        my $cmd = sprintf( $cdnCfg->{'syncProgram'}, $self->getDirectoryId );
+        system($cmd);
+        if ($?) {
+            $self->_addError("Error running CDN syncProgram: $?");
+        }
+        elsif ( $cdnCfg->{'queuePath'} ) {
+            unlink $cdnCfg->{'queuePath'} . '/' . $self->getDirectoryId;
+        }
+        chdir $originalDir;
+        my $dest;
+        my $cdnFile = $self->getPath . '/.cdn';
+        if ( open $dest, '>', $cdnFile ) {
+            close $dest;    # created empty file
+        }
+        else {
+            $self->_addError( "Couldn't open file $cdnFile for writing due to error: " . $! );
+        }
+    } ## end if ( $cdnCfg and $cdnCfg...
+} ## end sub syncToCdn
 
 #-------------------------------------------------------------------
 
@@ -1254,5 +1795,19 @@ sub untar {
     return $temp;
 }
 
+#-------------------------------------------------------------------
+
+=head2 getDirectoryId (  )
+
+Returns the id in base64 or hex depending on how it's stored in
+the uploads path.
+
+=cut
+
+sub getDirectoryId {
+    my $self = shift;
+    return $self->{_pathParts}[2];
+}
+
 1;
 
