@@ -21,6 +21,7 @@ use WebGUI::DatabaseLink;
 use WebGUI::Exception;
 use WebGUI::Utility;
 use WebGUI::Operation::Shared;
+use WebGUI::Workflow::Instance;
 use WebGUI::Shop::AddressBook;
 use JSON;
 use WebGUI::Exception;
@@ -42,20 +43,29 @@ This package provides an object-oriented way of managing WebGUI users as well as
  $u = WebGUI::User->newByEmail($session, $email);
  $u = WebGUI::User->newByUsername($session, $username);
 
- $authMethod =		$u->authMethod("WebGUI");
- $dateCreated = 	$u->dateCreated;
- $karma = 		    $u->karma;
- $lastUpdated = 	$u->lastUpdated;
- $languagePreference = 	$u->profileField("language",1);
- $referringAffiliate =	$u->referringAffiliate;
- $status =		$u->status("somestatus");
- $username =		$u->username("jonboy");
- $arrayRef =		$u->getGroups;
- $member =		$u->isInGroup($groupId);
+ # Get data
+ $authMethod            = $u->get("authMethod");
+ $dateCreated           = $u->get("dateCreated");
+ $karma                 = $u->karma;
+ $lastUpdated           = $u->get("lastUpdated");
+ $languagePreference    = $u->get("language");
+ $referringAffiliate    = $u->get("referringAffiliate");
+ $status                = $u->get("status");
+ $username              = $u->get("username");
+ $arrayRef              = $u->getGroups;
+ $isMember              = $u->isInGroup($groupId);
 
+ # Update data
+ $u->update({
+     username       => "m_bolton",
+     firstName      => "Mike",
+     likesSoftPop   => 1,
+ });
  $u->addToGroups(\@arr);
  $u->deleteFromGroups(\@arr);
  $u->delete;
+ $u->enable;
+ $u->disable;
 
  WebGUI::User->validUserId($session, $userId);
 
@@ -79,6 +89,7 @@ sub _create {
     my @fields     = @{WebGUI::ProfileField->getFields($session)};
     my $privacy  = {};
     foreach my $field (@fields) {
+        #$session->errorHandler->warn('getting privacy setting for field: '.$fieldName);
         my $privacySetting = $field->get('defaultPrivacySetting');
         next unless (WebGUI::Utility::isIn($privacySetting,qw(all none friends)));
         $privacy->{$field->get('fieldName')} = $privacySetting;
@@ -176,9 +187,9 @@ sub acceptsFriendsRequests {
     return 0 if($self->userId eq $user->userId);  #Can't be your own friend (why would you want to be?)
 
     my $me     = WebGUI::Friends->new($session,$self);
-    my $friend = WebGUI::Friends->new($session,$user);
-
     return 0 if ($me->isFriend($user->userId));  #Already a friend
+
+    my $friend = WebGUI::Friends->new($session,$user);
     return 0 if ($me->isInvited($user->userId) || $friend->isInvited($self->userId)); #Invitation sent by one or the other
 
     return $self->profileField('ableToBeFriend'); #Return profile setting
@@ -186,8 +197,38 @@ sub acceptsFriendsRequests {
 
 #-------------------------------------------------------------------
 
+=head2 authInstance
+
+Returns an instance of the authentication object for this user.
+
+=cut
+
+sub authInstance {
+    my $self    = shift;
+    my $session = $self->session;
+
+    my $authMethod;
+    if ($self->isVisitor) {
+        $authMethod = $session->setting->get("authMethod");
+    }
+    else {
+        $authMethod = $self->authMethod || $session->setting->get("authMethod");
+    }
+    if ( ! isIn($authMethod, @{ $session->config->get('authMethods') } ) ) {
+        $authMethod = $session->config->get('authMethods')->[0] || 'WebGUI';
+    }
+    my $authClass = 'WebGUI::Auth::' . $authMethod;
+    WebGUI::Pluggable::load($authClass);
+    my $auth = $authClass->new($session, $authMethod, $self->getId);
+    return $auth;
+}
+
+#-------------------------------------------------------------------
+
 =head2 authMethod ( [ value ] )
 
+DEPRECATED! Use get("authMethod") and update({ authMethod => "value })
+
 Returns the authentication method for this user.
 
 =head3 value
@@ -201,13 +242,9 @@ sub authMethod {
         $self = shift;
         $value = shift;
         if (defined $value) {
-		$self->uncache;
-                $self->{_user}{"authMethod"} = $value;
-                $self->{_user}{"lastUpdated"} =$self->session->datetime->time();
-                $self->session->db->write("update users set authMethod=".$self->session->db->quote($value).",
-			lastUpdated=".$self->session->datetime->time()." where userId=".$self->session->db->quote($self->{_userId}));
+            $self->update({ authMethod => $value });
         }
-        return $self->{_user}{"authMethod"};
+        return $self->get("authMethod");
 }
 
 #-------------------------------------------------------------------
@@ -313,12 +350,15 @@ sub canViewField {
 
 =head2 dateCreated ( )
 
+DEPRECATED! Use get("dateCreated") instead
+
 Returns the epoch for when this user was created.
 
 =cut
 
 sub dateCreated {
-        return $_[0]->{_user}{dateCreated};
+        my ( $self ) = @_;
+        return $self->get("dateCreated");
 }
 
 #-------------------------------------------------------------------
@@ -335,27 +375,45 @@ belong to this user.
 
 sub delete {
     my $self    = shift;
-    my $session = $self->session;
     my $userId  = $self->userId;
-	$self->uncache;
-    my $db = $self->session->db;
-	foreach my $groupId (@{$self->getGroups($userId)}) {
-		WebGUI::Group->new($session,$groupId)->deleteUsers([$userId]);
-	}
-    $self->friends->delete if ($self->{_user}{"friendsGroup"} ne "");
-	$db->write("delete from inbox where userId=? and (groupId is null or groupId='')",[$userId]);
-	require WebGUI::Operation::Auth;
-	my $authMethod = WebGUI::Operation::Auth::getInstance($self->session,$self->authMethod,$userId);
-	$authMethod->deleteParams($userId);
-	my $rs = $db->read("select sessionId from userSession where userId=?",[$userId]);
-	while (my ($id) = $rs->array) {
-        	$db->write("delete from userSessionScratch where sessionId=?",[$id]);
-	}
-    $db->write("delete from friendInvitations where inviterId=?",[$userId]);
-    $db->write("delete from friendInvitations where friendId=?",[$userId]);
+    my $session = $self->session;
+    my $db      = $session->db;
+    $self->uncache;
+
+    foreach my $groupId ( @{ $self->getGroups } ) {
+        WebGUI::Group->new($session, $groupId)->deleteUsers([$userId]);
+    }
+
+    my $auth = $self->authInstance;
+    $auth->deleteParams($userId);
+
+    $self->friends->delete
+        if ($self->{_user}{"friendsGroup"} ne "");
+
+    # clean up any user workflows
+    my $instances = WebGUI::Workflow::Instance->getInstancesForObject($session, {
+        className       => ref $self,
+        methodName      => 'new',
+        parameters      => $self->getId,
+        returnObjects   => 1,
+    });
+    for my $instance ( @{$instances} ) {
+        $instance->delete;
+    }
+
+    # remove sessions
+    $db->write(
+        "DELETE FROM userSessionScratch WHERE sessionId IN (SELECT sessionId FROM userSession WHERE userId=?)",
+        [$userId],
+    );
+    $db->write("DELETE FROM userSession WHERE userId=?",[$userId]);
+
+    # remove inbox entries
+    $db->write("DELETE FROM inbox_messageState WHERE userId=?",[$userId]);
+    $db->write("DELETE FROM inbox WHERE userId=? AND (groupId IS NULL OR groupId='')",[$userId]);
 
     # Shop cleanups
-    my $sth = $db->prepare('select addressBookId from addressBook where userId=?');
+    my $sth = $session->db->prepare('select addressBookId from addressBook where userId=?');
     $sth->execute([$userId]);
     BOOK: while (my $bookId = $sth->hashRef) {
         my $book;
@@ -363,9 +421,10 @@ sub delete {
         next BOOK if (my $e = Exception::Class->caught);
         $book->delete;
     }
-    $db->write("delete from userSession where userId=?",[$userId]);
-    $db->write("delete from userProfileData where userId=?",[$userId]);
-    $db->write("delete from users where userId=?",[$userId]);
+
+    # remove user itself
+    $db->write("DELETE FROM userProfileData WHERE userId=?",[$userId]);
+    $db->write("DELETE FROM users WHERE userId=?",[$userId]);
 }
 
 #-------------------------------------------------------------------
@@ -406,6 +465,48 @@ sub DESTROY {
         undef $self;
 }
 
+#----------------------------------------------------------------------------
+
+=head2 disable ( [options] )
+
+Disable the user. C<options> is an optional hashref with the following keys:
+
+ bySelf         - If true, the user is disabling themselves
+
+=cut
+
+sub disable {
+    my ( $self, $options ) = @_;
+    my $session     = $self->session;
+    my $db          = $session->db;
+
+    if ( $options->{bySelf} ) {
+        $self->update({ status => "Selfdestructed" });
+    }
+    else {
+        $self->update({ status => "Deactivated" });
+
+        # Remove sessions
+        my $rs = $db->read("select sessionId from userSession where userId=?",[$self->{_userId}]);
+        while (my ($id) = $rs->array) {
+            $db->write("delete from userSessionScratch where sessionId=?",[$id]);
+        }
+        $db->write("delete from userSession where userId=?",[$self->{_userId}]);
+    }
+}
+
+#----------------------------------------------------------------------------
+
+=head2 enable ( )
+
+Enable the user.
+
+=cut
+
+sub enable {
+    my ( $self ) = @_;
+    $self->update({ status => "Active" });
+}
 
 #-------------------------------------------------------------------
 
@@ -452,6 +553,64 @@ sub friends {
     return $myFriends;
 }
 
+#----------------------------------------------------------------------------
+
+=head2 get ( [field] )
+
+Get properties for this user. If C<field> is defined, will return the value
+of the field. Otherwise, returns a hash reference of all properties and profile
+fields.
+
+=cut
+
+sub get {
+    my ( $self, $field ) = @_;
+    my $session     = $self->session;
+
+    if ( $field ) {
+        if ( exists $self->{_user}->{$field} ) {
+            return $self->{_user}->{$field};
+        }
+        else {
+            # XXX Should the defaults be set in new() ...
+            if ( !exists $self->{_profile}->{$field} ) {
+                if ( !WebGUI::ProfileField->exists( $session, $field ) ) {
+                    $self->session->errorHandler->warn("No such profile field: $field");
+                }
+
+                my $default = $session->db->quickScalar("SELECT dataDefault FROM userProfileField WHERE fieldName=?", [$field]);
+                $self->{_profile}{$field}
+                    = WebGUI::Operation::Shared::secureEval($session, $default);
+                $self->cache; # XXX ... Because we cache them here!
+            }
+            if (ref $self->{_profile}{$field} eq 'ARRAY') {
+                ##Return a scalar, that is a string with all the defaults
+                return join ',', @{ $self->{_profile}{$field} };
+            }
+            return $self->{_profile}->{$field};
+        }
+    }
+
+    # Add any missing fields
+    my %default     = $session->db->buildHash(
+        "SELECT fieldName, dataDefault FROM userProfileField",
+    );
+    for my $key ( keys %default ) {
+        if ( !exists $self->{_profile}{$key} ) {
+            $self->{_profile}{$key}
+                = WebGUI::Operation::Shared::secureEval($session, $default{$key});
+        }
+    }
+
+    # Create a safe copy of everything to return
+    my %properties  = (
+        %{$self->{_user}},
+        %{$self->{_profile}},
+    );
+
+    return \%properties;
+}
+
 #-------------------------------------------------------------------
 
 =head2 getFirstName ( )
@@ -531,6 +690,74 @@ sub getGroupIdsRecursive {
     return [ keys %groupIds ];
 }
 
+#----------------------------------------------------------------------------
+
+=head2 getId ( )
+
+Get the user's ID
+
+=cut
+
+sub getId {
+    my ( $self ) = @_;
+    return $self->{_userId};
+}
+
+#-------------------------------------------------------------------
+
+=head2 getInboxNotificationAddresses ( )
+
+Return a string with addresses that the user wants to receive Inbox
+notifications.  If the user does not want Inbox notifications, then
+the string will be empty.
+
+This is called by L<WebGUI::Mail::Send>, and has the effect that if
+the site C<sendInboxNotificationsOnly> setting is on and the user
+has turned off C<receiveInboxEmailNotifications>, no email at all is
+sent.
+
+=cut
+
+sub getInboxNotificationAddresses {
+    my $self   = shift;
+    my $emails = '';
+    if ( $self->profileField('receiveInboxEmailNotifications')
+      && $self->profileField('email')) {
+        $emails = $self->profileField('email');
+    }
+    return $emails;
+}
+
+#-------------------------------------------------------------------
+
+=head2 getInboxSmsNotificationAddress ( )
+
+Return the email address that SMS notifications will be sent to for
+this user, constructed as:
+
+ cellPhone@smsGateway
+
+Non digits, such as area code separators, are removed from the cell phone information.
+
+=cut
+
+sub getInboxSmsNotificationAddress {
+    my $self   = shift;
+    
+    return unless $self->profileField('receiveInboxSmsNotifications');
+    
+    my $smsGateway = $self->session->setting->get('smsGateway');
+    return unless $smsGateway;
+    
+    my $cellPhone = $self->profileField('cellPhone');
+    return unless $cellPhone;
+    
+    # Remove non-numbers from cellPhone
+    $cellPhone =~ tr/0-9//dc;
+    
+    return join q{}, $cellPhone, '@', $smsGateway;
+}   
+
 #-------------------------------------------------------------------
 
 =head2 getProfileFieldPrivacySetting ( [field ])
@@ -587,7 +814,7 @@ sub getProfileUrl {
 
     my $identifier = $session->config->get("profileModuleIdentifier");
 
-    return qq{$page?op=account;module=$identifier;do=view;uid=}.$self->userId;
+    return $session->url->append($page,qq{op=account;module=$identifier;do=view;uid=}.$self->userId);
 
 }   
 
@@ -623,7 +850,13 @@ sub hasFriends {
 }
 
 #-------------------------------------------------------------------
-# This method is depricated and is provided only for reverse compatibility. See WebGUI::Auth instead.
+
+=head2 identifier
+
+This method is depricated and is provided only for reverse compatibility. See WebGUI::Auth instead.
+
+=cut
+
 sub identifier {
         my ($self, $value);
         $self = shift;
@@ -653,6 +886,19 @@ sub isAdmin {
 
 #-------------------------------------------------------------------
 
+=head2 isEnabled ()
+
+Returns 1 if the user is enabled.
+
+=cut
+
+sub isEnabled {
+    my $self = shift;
+    return $self->get('status') eq 'Active';
+}
+
+#-------------------------------------------------------------------
+
 =head2 isInGroup ( [ groupId ] )
 
 Returns a boolean (0|1) value signifying that the user has the required privileges. Always returns true for Admins.
@@ -664,8 +910,7 @@ The group that you wish to verify against the user. Defaults to group with Id 3
 =cut
 
 sub isInGroup {
-   my (@data, $groupId);
-   my ($self, $gid, $secondRun) = @_;
+   my ($self, $gid) = @_;
    $gid = 3 unless $gid;
    my $uid = $self->userId;
    ### The following several checks are to increase performance. If this section were removed, everything would continue to work as normal. 
@@ -760,6 +1005,8 @@ A description of why this user's karma was modified. For instance it could be "M
 
 =cut
 
+# NOTE: Should this be called "addKarma" instead?
+
 sub karma {
 	my $self = shift;
 	my $amount = shift;
@@ -778,12 +1025,15 @@ sub karma {
 
 =head2 lastUpdated ( )
 
+DEPRECATED! Use get("lastUpdated")
+
 Returns the epoch for when this user was last modified.
 
 =cut
 
 sub lastUpdated {
-        return $_[0]->{_user}{lastUpdated};
+        my ( $self ) = @_;
+        return $self->get('lastUpdated');
 }
 
 #-------------------------------------------------------------------
@@ -826,11 +1076,15 @@ sub new {
                 [$user{userId}]
             );
         delete $profile{userId};
+        delete $profile{wg_privacySettings};
 
-        # remove undefined fields so they will fall back on defaults when requested
+        # Fill in dataDefault
+        my $default = $session->db->buildHashRef(
+            "SELECT fieldName, dataDefault FROM userProfileField"
+        );
         for my $key (keys %profile) {
             if (!defined $profile{$key} || $profile{$key} eq '') {
-                delete $profile{$key};
+                delete $profile{$key}
             }
         }
 
@@ -908,6 +1162,8 @@ sub newByUsername {
 
 =head2 profileField ( fieldName [, value ] )
 
+DEPRECATED! Use get(fieldName) and update({ fieldName => "value" })
+
 Returns a profile field's value. If "value" is specified, it also sets the field to that value. 
 
 =head3 fieldName 
@@ -924,33 +1180,12 @@ sub profileField {
     my $self        = shift;
     my $fieldName   = shift;
     my $value       = shift;
-    my $db          = $self->session->db;
-    return "" if ($fieldName eq "wg_privacySettings");  # this is a special internal field, don't try to process it.
-    if (!exists $self->{_profile}{$fieldName} && !$self->session->db->quickScalar("SELECT COUNT(*) FROM userProfileField WHERE fieldName = ?", [$fieldName])) {
-        $self->session->errorHandler->warn("No such profile field: $fieldName");
-        return undef;
-    }
+
     if (defined $value) {
-        $self->uncache;
-        $self->{_profile}{$fieldName} = $value;
-        $db->write(
-            "UPDATE userProfileData SET ".$db->dbh->quote_identifier($fieldName)."=? WHERE userId=?",
-            [$value, $self->{_userId}]
-        );
-        my $time = $self->session->datetime->time;
-        $self->{_user}{"lastUpdated"} = $time;
-        $self->session->db->write("update users set lastUpdated=? where userId=?", [$time, $self->{_userId}]);
+        $self->update({ $fieldName => $value });
     }
-    elsif (!exists $self->{_profile}{$fieldName}) {
-        my $default = $self->session->db->quickScalar("SELECT dataDefault FROM userProfileField WHERE fieldName=?", [$fieldName]);
-        $self->{_profile}{$fieldName} = WebGUI::Operation::Shared::secureEval($self->session, $default);
-        $self->cache;
-    }
-    if (ref $self->{_profile}{$fieldName} eq 'ARRAY') {
-        ##Return a scalar, that is a string with all the defaults
-        return join ',', @{ $self->{_profile}{$fieldName} };
-    }
-	return $self->{_profile}{$fieldName};
+
+	return $self->get($fieldName);
 }
 
 #-------------------------------------------------------------------
@@ -972,6 +1207,7 @@ sub profileIsViewable {
     my $userId   = $user->userId;
 
     return 0 if ($self->isVisitor);  #Can't view visitor's profile
+    return 0 if ($user->isVisitor);  #User is not allowed to see anyone's profile, either
     return 1 if ($self->userId eq $userId);  #Users can always view their own profile
 
     my $profileSetting = $self->profileField('publicProfile');
@@ -987,6 +1223,8 @@ sub profileIsViewable {
 
 =head2 referringAffiliate ( [ value ] )
 
+DEPRECATED! Use get("referringAffiliate") and update({ referringAffiliate => "value" })
+
 Returns the unique identifier of the affiliate that referred this user to the site. 
 
 =head3 value
@@ -999,13 +1237,9 @@ sub referringAffiliate {
         my $self = shift;
         my $value = shift;
         if (defined $value) {
-		$self->uncache;
-                $self->{_user}{"referringAffiliate"} = $value;
-                $self->{_user}{"lastUpdated"} =$self->session->datetime->time();
-                $self->session->db->write("update users set referringAffiliate=".$self->session->db->quote($value).",
-                        lastUpdated=".$self->session->datetime->time()." where userId=".$self->session->db->quote($self->userId));
+            $self->update({ "referringAffiliate" => $value });
         }
-        return $self->{_user}{"referringAffiliate"};
+        return $self->get("referringAffiliate");
 }
 
 #-------------------------------------------------------------------
@@ -1021,16 +1255,16 @@ sub session {
 	return $self->{_session};
 }
 
-
 #-------------------------------------------------------------------
 
 =head2 setProfileFieldPrivacySetting ( settings ) 
 
-Sets the profile field privacy settings
+Sets the profile field privacy settings.  This updates the the db and
+the internally cached settings.  Valid settings are "all", "none" or "friends".
 
 =head3 settings
 
-hash ref containing the field and it's corresponding privacy setting
+Hash ref containing fields and their corresponding privacy settings
 
 =cut
 
@@ -1063,6 +1297,8 @@ sub setProfileFieldPrivacySetting {
 
 =head2 status ( [ value ] )
 
+DEPRECATED! Use get("status") and enable() and disable() instead
+
 Returns the status of the user. 
 
 =head3 value
@@ -1078,20 +1314,17 @@ sub status {
 	my $self = shift;
 	my $value = shift;
 	if (defined $value) {
-		$self->uncache;
-		$self->{_user}{"status"} = $value;
-		$self->{_user}{"lastUpdated"} =$self->session->datetime->time();
-		$self->session->db->write("update users set status=".$self->session->db->quote($value).",
-		lastUpdated=".$self->session->datetime->time()." where userId=".$self->session->db->quote($self->userId));
-		if ($value eq 'Deactivated') {
-			my $rs = $self->session->db->read("select sessionId from userSession where userId=?",[$self->{_userId}]);
-			while (my ($id) = $rs->array) {
-				$self->session->db->write("delete from userSessionScratch where sessionId=?",[$id]);
-			}
-			$self->session->db->write("delete from userSession where userId=?",[$self->{_userId}]);
-		}
+        if ( $value eq "Active" ) {
+            $self->enable;
+        }
+        elsif ( $value eq "Selfdestructed" ) {
+            $self->disable({ bySelf => 1 });
+        }
+        elsif ( $value eq "Deactivated" ) {
+            $self->disable;
+        }
 	}
-	return $self->{_user}{"status"};
+	return $self->get("status");
 }
 
 #-------------------------------------------------------------------
@@ -1108,10 +1341,92 @@ sub uncache {
 	$cache->delete;
 }
 
+#----------------------------------------------------------------------------
+
+=head2 update ( properties )
+
+Update properties for the user. C<properties> is a hash reference of user properties
+and/or profile fields.
+
+Valid user properties:
+
+ authMethod
+ dateCreated
+ friendsGroup
+ karma              - NOTE: To add karma, use the karma() method
+ lastUpdated
+ referringAffiliate
+ status             - One of "Activated", "Deactivated", or "Selfdestructed"
+ username
+
+Anything else is a profile field.
+
+=cut
+
+sub update {
+    my ( $self, $properties ) = @_;
+    my $session     = $self->session;
+    my $db          = $session->db;
+    
+    # Make a safe copy of properties, we'll be deleting from it
+    $properties = { %$properties };
+    $self->uncache;
+    $properties->{lastUpdated} ||= time;
+
+    # No userId, bad!
+    delete $properties->{userId};
+
+    # This is an internal field with its own api to set it
+    delete $properties->{wg_privacySettings};
+
+    # $self->{_user} contains all fields in `users` table
+    my @userFields  = ();
+    my @userValues  = ();
+    for my $key ( keys %{$self->{_user}} ) {
+        if ( exists $properties->{$key} ) {
+            # Delete the value because it's not a profile field
+            my $value   = delete $properties->{$key};
+            push @userFields, $db->dbh->quote_identifier( $key ) . " = ?";
+            push @userValues, $value;
+            $self->{_user}->{$key} = $value;
+        }
+    }
+    # No matter what we update properties
+    my $userFields  = join ", ", @userFields;
+    $db->write(
+        "UPDATE users SET $userFields WHERE userId=?",
+        [@userValues, $self->{_userId}]
+    );
+
+    # Everything else must be a profile field
+    my @profileFields   = ();
+    my @profileValues   = ();
+    for my $key ( keys %{$properties} ) {
+        if (!exists $self->{_profile}{$key} && !WebGUI::ProfileField->exists($session,$key)) {
+            $self->session->errorHandler->warn("No such profile field: $key");
+            next;
+        }
+        push @profileFields, $db->dbh->quote_identifier( $key ) . " = ?";
+        push @profileValues, $properties->{ $key };
+        $self->{_profile}->{$key} = $properties->{ $key };
+    }
+    if ( @profileFields ) {
+        my $profileFields  = join ", ", @profileFields;
+        $db->write(
+            "UPDATE userProfileData SET $profileFields WHERE userId=?",
+            [@profileValues, $self->{_userId}]
+        );
+    }
+
+    return;
+}
+
 #-------------------------------------------------------------------
 
 =head2 updateProfileFields ( profile )
 
+DEPRECATED! Use update(profile)
+
 Saves profile data to a user's profile.  Does not validate any of the data.
 
 =head3 profile
@@ -1124,15 +1439,14 @@ sub updateProfileFields {
     my $self    = shift;
     my $profile = shift;
 
-	foreach my $fieldName (keys %{$profile}) {
-		$self->profileField($fieldName,$profile->{$fieldName});
-	}
+    $self->update($profile);
 }
 
 #-------------------------------------------------------------------
 
 =head2 username ( [ value ] )
 
+DEPRECATED! Use get("username") and update({ username => "value" }) instead.
 Returns the username. 
 
 =head3 value
@@ -1145,25 +1459,23 @@ sub username {
     my $self = shift;
     my $value = shift;
     if (defined $value) {
-        $self->uncache;
-        $self->{_user}{"username"} = $value;
-        $self->{_user}{"lastUpdated"} = $self->session->datetime->time();
-        $self->session->db->write("update users set username=?, lastUpdated=? where userId=?",
-            [$value, $self->session->datetime->time(), $self->userId]);
+        $self->update({ username => $value });
     }
-    return $self->{_user}{"username"};
+    return $self->get("username");
 }
 
 #-------------------------------------------------------------------
 
 =head2 userId ( )
 
+DEPRECATED: Use getId() instead!
+
 Returns the userId for this user.
 
 =cut
 
 sub userId {
-        return $_[0]->{_userId};
+        return $_[0]->getId;
 }
 
 #-------------------------------------------------------------------
