@@ -79,9 +79,13 @@ sub autoCommitWorkingIfEnabled {
 
     # auto commit assets
     # save and commit button and site wide auto commit work the same
+    # Do not auto commit if tag is system wide tag or tag belongs to someone else
     if (
         $options->{override}
-        || $class->getVersionTagMode($session) eq q{autoCommit}
+      || ( $class->getVersionTagMode($session) eq q{autoCommit}
+        && ! $versionTag->get(q{isSiteWide})
+        && $versionTag->get(q{createdBy}) eq $session->user()->userId()
+         )
     ) {
         if ($session->setting->get("skipCommitComments") || !$options->{allowComments}) {
             $versionTag->requestCommit;
@@ -94,6 +98,7 @@ sub autoCommitWorkingIfEnabled {
             return 'redirect';
         }
     }
+    return undef;
 }
 
 #-------------------------------------------------------------------
@@ -247,7 +252,7 @@ sub get {
         return $self->{_data}{$name};
     }
     else {
-        return \%{ $self->{_data} },
+        return { %{ $self->{_data} } },
     }
 }
 
@@ -510,6 +515,20 @@ sub getWorking {
 
 #-------------------------------------------------------------------
 
+=head2 leaveTag ( )
+
+Make the user leave their current tag.
+
+=cut
+
+sub leaveTag {
+	my $self = shift;
+	$self->session->scratch->delete('versionTag');
+	$self->session->stow->delete("versionTag");
+}
+
+#-------------------------------------------------------------------
+
 =head2 lock ( )
 
 Sets this version tag up so no more revisions may be applied to it.
@@ -579,25 +598,39 @@ sub requestCommit {
 
 #-------------------------------------------------------------------
 
-=head2 rollback ( )
+=head2 rollback ( [ $options ] )
 
 Eliminates all revisions of all assets created under a specific version tag. Also removes the version tag.
 
+=head3 options
+
+A hashref of options for this method
+
+=head4 outputSub
+
+A subroutine for reporting the status of the rollback.  Typically used by WebGUI::ProgressBar
+
 =cut
 
 sub rollback {
-	my $self = shift;
-	my $tagId = $self->getId;
+	my $self      = shift;
+    my $session   = $self->session;
+    my $options   = shift || {};
+    my $outputSub = exists $options->{outputSub} ? $options->{outputSub} : sub {};
+	my $tagId     = $self->getId;
 	if ($tagId eq "pbversion0000000000001") {
-		$self->session->errorHandler->warn("You cannot rollback a tag that is required for the system to operate.");	
+		$session->errorHandler->warn("You cannot rollback a tag that is required for the system to operate.");	
 		return 0;
 	}
-	my $sth = $self->session->db->read("select asset.className, asset.assetId, assetData.revisionDate from assetData left join asset on asset.assetId=assetData.assetId where assetData.tagId = ? order by asset.lineage desc, assetData.revisionDate desc", [ $tagId ]);
-	while (my ($class, $id, $revisionDate) = $sth->array) {
-		my $revision = WebGUI::Asset->new($self->session,$id, $class, $revisionDate);
+	my $sth = $session->db->read("select asset.className, asset.assetId, assetData.revisionDate from assetData left join asset on asset.assetId=assetData.assetId where assetData.tagId = ? order by asset.lineage desc, assetData.revisionDate desc", [ $tagId ]);
+    my $i18n    = WebGUI::International->new($session, 'VersionTag');
+	REVISION: while (my ($class, $id, $revisionDate) = $sth->array) {
+		my $revision = WebGUI::Asset->new($session,$id, $class, $revisionDate);
+        next REVISION unless $revision;
+        $outputSub->(sprintf $i18n->get('Rolling back %s'), $revision->getTitle);
 		$revision->purgeRevision;
 	}
-	$self->session->db->write("delete from assetVersionTag where tagId=?", [$tagId]);
+	$session->db->write("delete from assetVersionTag where tagId=?", [$tagId]);
 	$self->clearWorking;
 	return 1;
 }
