@@ -3,6 +3,16 @@ use strict;
 
 our $I18N = {
 
+    'mark as read label' => {
+        message     => q{Mark as Read},
+        lastUpdated => 1241178876,
+    },
+
+    'mark as unread label' => {
+        message     => q{Mark as Unread},
+        lastUpdated => 1241178876,
+    },
+
     'title' => {
         message     => q{Inbox},
         lastUpdated => 1225724810,
@@ -265,11 +275,6 @@ our $I18N = {
         lastUpdated => 1119068809
     },
 
-    'delete label' => {
-        message     => q{Delete},
-        lastUpdated => 1119068809
-    },
-
     'cancel label' => {
         message     => q{Cancel},
         lastUpdated => 1119068809
@@ -286,9 +291,9 @@ our $I18N = {
     },
 
     'send label' => {
-		message => q{Send},
-		lastUpdated => 1181019679,
-	},
+        message => q{Send},
+        lastUpdated => 1181019679,
+    },
 
     'no access' => {
         message     => q{You do not have permission to view this message.},
@@ -326,9 +331,9 @@ our $I18N = {
     },
 
     'no self error' => {
-		message => q|You may not send messages to yourself.|,
-		lastUpdated => 1181019679,
-	},
+        message => q|You may not send messages to yourself.|,
+        lastUpdated => 1181019679,
+    },
 
     'no friends error' => {
         message => q{You do not have any active friends in the system.  If you wish to send a new private message to a user without having to locate their profile, you must first add them as a friend.},
@@ -362,27 +367,27 @@ our $I18N = {
 
     'blocked error' => {
         message => q|This user does not wish to receive private messages.|,
-		lastUpdated => 1181019679,
+        lastUpdated => 1181019679,
     },
 
     'remove text' => {
         message => q{remove},
-		lastUpdated => 1181019679,
+        lastUpdated => 1181019679,
     },
 
     'update label' => {
         message => q{Update},
-		lastUpdated => 1181019679,
+        lastUpdated => 1181019679,
     },
 
     'message sent label' => {
         message => q{Message Sent},
-		lastUpdated => 1181019679,
+        lastUpdated => 1181019679,
     },
 
     'message sent text' => {
         message => q{Your message has been sent successfully},
-		lastUpdated => 1181019679,
+        lastUpdated => 1181019679,
     },
 
     'error label' => {
@@ -466,9 +471,9 @@ our $I18N = {
     },
 
     'missing email' => {
-		message => q|The invitation cannot be sent because you did not enter an email address.|,
-		lastUpdated => 1181409056,
-	},
+        message => q|The invitation cannot be sent because you did not enter an email address.|,
+        lastUpdated => 1181409056,
+    },
 
     'invalid email' => {
 		message => q|Invalid email address entered: %s.|,
@@ -480,15 +485,15 @@ our $I18N = {
 		lastUpdated => 1181409432,
 	},
 
-	'missing subject' => {
-		message     => q|Your invitation must have a subject.|,
-		lastUpdated => 1181409433,
-	},
+    'missing subject' => {
+        message     => q|Your invitation must have a subject.|,
+        lastUpdated => 1181409433,
+    },
 
     'already a member' => {
-		message     => q|The email address you entered is already registered to <a href="%s">%s</a> on this site.|,
-		lastUpdated => 1227715272,
-	},
+        message     => q|The email address you entered is already registered to <a href="%s">%s</a> on this site.|,
+        lastUpdated => 1227715272,
+    },
 
     'currently invited' => {
 		message     => q|The email address you entered, %s, has an outstanding invitation on this site.|,
@@ -501,54 +506,54 @@ our $I18N = {
     },
 
     'invitation sent label' => {
-		message => q|Invitation Sent|,
-		lastUpdated => 1181410226,
-	},
+        message => q|Invitation Sent|,
+        lastUpdated => 1181410226,
+    },
 
     'invitation sent text' => {
-		message => q|Your invitation has been sent successfully.|,
-		lastUpdated => 1181410226,
-	},
+        message => q|Your invitation has been sent successfully.|,
+        lastUpdated => 1181410226,
+    },
 
     'common account variables' => {
-		message => q|Common Account, Inbox Plugin, Template Variables.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|Common Account, Inbox Plugin, Template Variables.|,
+        lastUpdated => 1230867391,
+    },
 
     'view_inbox_url' => {
-		message => q|URL to view the Inbox screen.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|URL to view the Inbox screen.|,
+        lastUpdated => 1230867391,
+    },
 
     'view_invitations_url' => {
-		message => q|URL to view the Invitations screen.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|URL to view the Invitations screen.|,
+        lastUpdated => 1230867391,
+    },
 
     'unread_message_count' => {
-		message => q|The number of unread messages in the Inbox.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|The number of unread messages in the Inbox.|,
+        lastUpdated => 1230867391,
+    },
 
     'invitation_count' => {
-		message => q|The number of Invitations this user has received.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|The number of Invitations this user has received.|,
+        lastUpdated => 1230867391,
+    },
 
     'invitations_enabled' => {
-		message => q|A boolean that is true if the user is willing to receive invitations to join Friends Networks on this site.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|A boolean that is true if the user is willing to receive invitations to join Friends Networks on this site.|,
+        lastUpdated => 1230867391,
+    },
 
     'user_invitations_enabled' => {
-		message => q|A boolean that is true if this site is configured to send invitations to users outside of the site.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|A boolean that is true if this site is configured to send invitations to users outside of the site.|,
+        lastUpdated => 1230867391,
+    },
 
     'invite_friend_url' => {
-		message => q|The URL to invite this user to be join your Friend Network.|,
-		lastUpdated => 1230867391,
-	},
+        message => q|The URL to invite this user to be join your Friend Network.|,
+        lastUpdated => 1230867391,
+    },
 
     'inbox rich editor label' => {
         message         => q{Inbox Rich Editor},
@@ -736,6 +741,83 @@ our $I18N = {
          context      => "template variable for view inbox template",
      },
 
+     'userFilter' => {
+         message         => q{A select box form element to filter the list of messages by who sent them.},
+         lastUpdated     => 1235421123,
+         context         => "Description of Inbox setting",
+     },
+
+     'All users' => {
+         message         => q{All users},
+         lastUpdated     => 1235421123,
+         context         => "Default setting for filtering in the Inbox.  Short for Show messages from all users.",
+     },
+
+    'inbox copy sender label' => {
+        message     => q|Copy Sender on Messages?|,
+        context     => q|Copy, as in receive a copy.  Label for Account/Inbox settings|,
+        lastUpdated => 0,
+    },
+
+    'inbox copy sender hoverHelp' => {
+        message     => q|If this setting is 'yes', users will receive a copy of messages that they send using the inbox.|,
+        lastUpdated => 0,
+    },
+ 
+    'send inbox notifications only' => {
+        message => q|Send only Inbox notifications|,
+        context => q|Site setting.  A notification is a short message that a message is in the Inbox.|,
+        lastUpdated => 1235685248,
+    },
+
+    'send inbox notifications only help' => {
+        message => q|Should WebGUI just send notifications about Inbox messages, instead of the message itself?|,
+        lastUpdated => 1235696295,
+    },
+    
+    'inbox notifications subject' => {
+        message => q|Inbox Notification Email Subject|,
+        context => q|Site setting.  A notification is a short message that a message is in the Inbox.|,
+        lastUpdated => 0,
+    },
+
+    'inbox notifications subject help' => {
+        message => q|The email subject to use for inbox notifications|,
+        lastUpdated => 0,
+    },
+
+    'inbox notification' => {
+        message => q|You have a new message in your Inbox.|,
+        lastUpdated => 1235708853,
+    },
+ 
+    'inbox notification template' => {
+        message => q|Inbox Notification Template|,
+        context => q|Site setting.  A notification is a short message that a message is in the Inbox.|,
+        lastUpdated => 1242274705,
+    },
+
+    'inbox notification template help' => {
+        message => q|Choose a template that will be used to display Inbox Notifications.|,
+        lastUpdated => 1242274703,
+    },
+    
+    'inbox sms notification' => {
+        message => q|You have a new message in your Inbox.|,
+        lastUpdated => 1235708853,
+    },
+ 
+    'inbox sms notification template' => {
+        message => q|Inbox SMS Notification Template|,
+        context => q|Site setting.  A notification is a short SMS message that a message is in the Inbox.|,
+        lastUpdated => 1242274705,
+    },
+
+    'inbox sms notification template help' => {
+        message => q|Choose a template that will be used to display Inbox SMS Notifications.|,
+        lastUpdated => 1242274703,
+    },
+
 };
 
 1;
