@@ -21,11 +21,17 @@ our $I18N = {
 	},
 
 	'keywords help' => {
-		message => q|Add some keywords here for this asset. They'll automatically be added to the search index, and as the keywords metatag.|,
-		lastUpdated => 0,
+		message => q|Keywords are automatically added to the keywords metatag for this asset, and are also put into the WebGUI search index.  Multiple keywords should be separated by commas.|,
+		lastUpdated => 1262117316,
 		context => q|help for the keywords property|
 	},
 
+	'keywords template var' => {
+		message => q|This will be a string with the keywords for this asset.  Individual keywords will be joined with spaces, unless the keyword contains spaces, in which case it will be quoted.|,
+		lastUpdated => 0,
+		context => q|help for the keywords template variable|
+	},
+
 	'add the missing page' => {
 		message => q|Add the missing page.|,
 		lastUpdated => 0,
@@ -211,13 +217,55 @@ our $I18N = {
 	'restore' => {
 		message => q|Restore|,
 		lastUpdated => 0,
-		context => q|Used in asset context menus.|
+		context => q|Used in asset context menus.|,
 	},
 
 	'promote' => {
 		message => q|Promote|,
 		lastUpdated => 0,
-		context => q|Used in asset context menus.|
+		context => q|Used in asset context menus.|,
+	},
+
+	'promote %s' => {
+		message => q|promote %s|,
+		lastUpdated => 0,
+		context => q|Used in asset context menus.|,
+	},
+
+	'Set Rank' => {
+		message => q|Set Rank|,
+		lastUpdated => 0,
+		context => q|Set, to assign.  Rank, meaning order.|,
+	},
+
+	'moving %s aside' => {
+		message => q|moving %s aside|,
+		lastUpdated => 0,
+		context => q||,
+	},
+
+	'moving %s back' => {
+		message => q|moving %s back|,
+		lastUpdated => 0,
+		context => q||,
+	},
+
+	'moving %s' => {
+		message => q|moving %s|,
+		lastUpdated => 0,
+		context => q||,
+	},
+
+	'swap first' => {
+		message => q|swap first|,
+		lastUpdated => 0,
+		context => q|swap, to exchange places|,
+	},
+
+	'swap second' => {
+		message => q|swap second|,
+		lastUpdated => 0,
+		context => q|swap, to exchange places|,
 	},
 
 	'promote help' => {
@@ -236,6 +284,12 @@ our $I18N = {
 		lastUpdated => 0,
 	},
 
+	'demote %s' => {
+		message => q|demote %s|,
+		lastUpdated => 0,
+		context => q|Used in asset context menus.|,
+	},
+
 	'cut' => {
 		message => q|Cut|,
 		lastUpdated => 0,
@@ -260,6 +314,53 @@ our $I18N = {
 		context => q|Same as Copy, but lower case.|
 	},
 
+	'Paste' => {
+		message => q|Paste|,
+		lastUpdated => 1245342798,
+		context => q|To remove an item from the clipboard, and put it on the current page.|
+	},
+
+	'Paste Assets' => {
+		message => q|Paste Assets|,
+		lastUpdated => 1245342798,
+	},
+
+	'pasting %s' => {
+		message => q|pasting %s|,
+		lastUpdated => 1245343280,
+		context => q|short for "I am pasting this asset onto the page."|
+	},
+
+	'indexing %s' => {
+		message => q|indexing %s|,
+		lastUpdated => 1245343280,
+		context => q|short for "I am adding this asset to the search index"|
+	},
+
+	'skipping %s' => {
+		message => q|skipping %s|,
+		lastUpdated => 1245343280,
+		context => q|To skip, to move over, to not process|
+	},
+
+	'Processing form data' => {
+		message => q|Processing form data|,
+		lastUpdated => 1245343280,
+		context => q|To edit or change|
+	},
+
+	'editing %s' => {
+		message => q|editing %s|,
+		lastUpdated => 1245343280,
+		context => q|To edit or change|
+	},
+
+	'Attempting to commit changes' => {
+		message => q|Attempting to commit changes|,
+		lastUpdated => 1245343280,
+		context => q||,
+	},
+
 	'this asset only' => {
 		message => q|This&nbsp;Asset&nbsp;Only|,
 		lastUpdated => 0,
@@ -748,8 +849,8 @@ case of the check box list, then enter one per line.  The total amount of data i
                    message => q|Hide from navigation?|
                  },
         '43' => {
-                  lastUpdated => 1149960436,
-                  message => q|Are you certain that you wish to delete this content? Note that this change is not versioned and will take place immediately.|
+                  message => q|Are you certain that you wish to delete this content, and all content below it? Note that this change is not versioned and will take place immediately.|,
+                  lastUpdated => 1250091423,
                 },
         '940' => {
                    lastUpdated => 1050438829,
@@ -999,6 +1100,16 @@ Couldn't open %-s because %-s <br />
 	    lastUpdated => 1100463645,
 	},
 
+	'assetId' => {
+		message => q|The unique identifier for this Asset, in base64 encoding.|,
+		lastUpdated => 1256327617,
+	},
+
+	'assetIdHex' => {
+		message => q|The unique identifier for this Asset, in hexadecimal format.|,
+		lastUpdated => 1256327617,
+	},
+
 	'title' => {
 		message => q|The title of the Asset|,
 		lastUpdated => 1160773957,
@@ -1191,6 +1302,102 @@ Couldn't open %-s because %-s <br />
         context     => q{Asset Manager label, as in "locked by admin"},
     },
 
+    'assetHistory' => {
+        message     => q{Asset History},
+        lastUpdated => 0,
+        context     => q{Admin Console label.  Shows the history of assets in this site.},
+    },
+
+    'usePackedHeadTags label' => {
+        message     => q{Use Packed Head Tags},
+        lastUpdated => 0,
+        context     => q{Label for asset property},
+    },
+
+    'usePackedHeadTags description' => {
+        message     => q{Use the packed version of the head tags for faster downloads},
+        lastUpdated => 0,
+        context     => q{Description of asset property},
+    },
+
+    'Error getting asset with assetId %s' => {
+        message     => q{Error getting asset with assetId %s},
+        lastUpdated => 0,
+        context     => q{Generic error when an asset cannot be looked up by assetId},
+    },
+
+    'You cannot edit the asset %s, skipping' => {
+        message     => q{You cannot edit the asset %s, skipping},
+        lastUpdated => 0,
+        context     => q{Generic error when an asset cannot be edited},
+    },
+
+    'Purging %s' => {
+        message     => q{Purging %s},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Trying to delete system page %s.  Aborting' => {
+        message     => q{Trying to delete system page %s.  Aborting},
+        lastUpdated => 1251071942,
+        context     => q{},
+    },
+
+    'Undefined child' => {
+        message     => q{Undefined child.},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Purging shortcuts' => {
+        message     => q{Purging shortcuts},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Deleting exported files' => {
+        message     => q{Deleting exported files},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Deleting keywords' => {
+        message     => q{Deleting keywords},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Clearing search index' => {
+        message     => q{Clearing search index},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Clearing cache' => {
+        message     => q{Clearing cache},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Clearing asset tables' => {
+        message     => q{Clearing asset tables},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Delete Assets' => {
+        message     => q{Delete Assets},
+        lastUpdated => 0,
+        context     => q{},
+    },
+
+    'Any Class' => {
+        message     => q{Any Class},
+        lastUpdated => 0,
+        context     => q{Class, as in name of class, or type of asset},
+    },
+
 };
 
 1;
