@@ -428,12 +428,12 @@ listing,|,
         },
 
     'maxgroup per description' => {
-        message => q|Specifies how many comparisons are allowed per group.|,
+        message => q|Specifies how many comparisons are allowed for the privileged group.|,
         lastUpdated => 0,
     },
 
     'maxgroup description' => {
-        message => q|Specifies which group is restricted.|,
+        message => q|Select a group for which a specific maximum comparisons can be selected.|,
         lastUpdated => 0,
     },
 
@@ -458,6 +458,16 @@ to increase performance. How long should we cache them?|,
         lastUpdated => 0,
     },
 
+    'listings cache timeout label' => {
+        message => q|Listings Cache Timeout|,
+        lastUpdated => 0,
+    },
+
+    'listings cache timeout description' => {
+        message => q|The complete list of Matrix listings can be cached for some time to increase performance. How long should we cache it?|,
+        lastUpdated => 0,
+    },
+
         'rating timeout description' => {
                 message => q|Set a timeout so that users are prevented from rating a given listing too often.|,
                 lastUpdated => 1135271460,
@@ -602,22 +612,17 @@ to increase performance. How long should we cache them?|,
         message => q|Compare Color: Yes|,
         lastUpdated => 0,
     },
-    
-    'maxgroup per label' => {
-		message => q|Maximum per Group|,
-		lastUpdated => 0,
-	},
-    
-    'maxgroup label' => {
-		message => q|Maximum Group|,
-		lastUpdated => 0,
-	},
-    
-    'maxgroup label' => {
-		message => q|Maximum Group|,
-		lastUpdated => 0,
-	},
 
+    'maxgroup per label' => {
+        message => q|Maximum for Privileged Group|,
+        lastUpdated => 0,
+    },
+
+    'maxgroup label' => {
+        message => q|Privileged Maximum Group|,
+        lastUpdated => 0,
+    },
+    
     'max comparisons label' => {
 		message => q|Maximum Comparisons|,
 		lastUpdated => 0,
@@ -853,16 +858,53 @@ selectBox.</p>|
     },
 
     'too many message' => {
-        lastUpdated => 0,
-        message => q|You tried to compare more than your maximum number of listings.|,
+        lastUpdated => 1260478380,
+        message => q|You can only compare up to %d items at a time. Please adjust your selections and try again.|,
         context => q|A message shown to the user when they have selected too many listings to compare.|,
     },
 
     'too few message' => {
-        lastUpdated => 0,
-        message => q|You tried to compare only one listing. If you want to view just one listing, click on its name.|,
+        lastUpdated => 1260478343,
+        message => q|To compare, at least two listings must be selected. If you want to view just one listing, click on its name.|,
         context => q|A message shown to the user when they have selected only one listing to compare.|,
-    }
+    },
+
+    'Sort by name' => {
+        lastUpdated => 1250146133,
+        message => q|Sort by name|,
+        context => q|To order a list of items by name|,
+    },
+
+    'Sort by views' => {
+        lastUpdated => 1250146133,
+        message => q|Sort by views|,
+        context => q|To order a list of items by the number of times it has been viewed|,
+    },
+
+    'Sort by compares' => {
+        lastUpdated => 1250146133,
+        message => q|Sort by compares|,
+        context => q|To order a list of items by the number of times it has been compared|,
+    },
+
+    'Sort by clicks' => {
+        lastUpdated => 1250146133,
+        message => q|Sort by clicks|,
+        context => q|To order a list of items by the number of times it has been clicked|,
+    },
+
+    'Sort by updated' => {
+        lastUpdated => 1250146133,
+        message => q|Sort by updated|,
+        context => q|To order a list of items by the number of times it was last updated|,
+    },
+
+    'Return to Matrix' => {
+        lastUpdated => 1250146133,
+        message => q|Return to Matrix|,
+        context => q|To go back to the Matrix main screen.|,
+    },
+
 };
 
 1;
