@@ -544,8 +544,8 @@ be useful, others may not.|,
 	},
 
 	'assetsToInclude' => {
-		message => q|An newline separated string containg the kids of assets to include in the Navigation, by relationship.  Ancestors, the asset itself, siblings, descendants and/or pedigree.|,
-		lastUpdated => 1173814820
+		message => q|A newline separated string containing the kids of assets to include in the Navigation, by relationship.  Ancestors, the asset itself, siblings, descendants and/or pedigree.|,
+		lastUpdated => 1254329336,
 	},
 
 	'startType' => {
