@@ -555,6 +555,12 @@ our $I18N = {
 		context => q|admin function label|
 	},
 
+    'vendor payouts' => {
+        message => q|Vendor payouts|,
+        lastUpdated => 0,
+        context => q|admin function label|
+    },
+
 	'update' => { 
 		message => q|Update|,
 		lastUpdated => 0,
@@ -615,6 +621,12 @@ our $I18N = {
 		context => q|field label|
 	},
 
+	'Status' => { 
+		message => q|Status|,
+		lastUpdated => 0,
+		context => q|Whether a transaction was successful, or not.|
+	},
+
 	'payment method' => { 
 		message => q|Payment Method|,
 		lastUpdated => 0,
@@ -657,6 +669,18 @@ our $I18N = {
 		context => q|commerce setting help|
 	},
 
+	'select gateway template' => { 
+		message => q|Select Gateway Template|,
+		lastUpdated => 0,
+		context => q|commerce setting|
+	},
+
+	'select gateway template help' => { 
+		message => q|This template is the template for the Select Payment Gateway step.|,
+		lastUpdated => 0,
+		context => q|commerce setting help|
+	},
+
 	'transactions' => { 
 		message => q|Transactions|,
 		lastUpdated => 0,
@@ -1522,6 +1546,191 @@ our $I18N = {
         context => q|template variable for Cart template|,
     },
 
+    'cart checkout minimum' => {
+        message => q|Minimum checkout amount|,
+        lastUpdated => 0,
+        context => q|shop setting label|,
+    },
+
+    'cart checkout minimum help' => {
+        message => q|Use this setting to require a minimum cart value to allow users to check out.|,
+        lastUpdated => 0,
+        context => q|shop setting hover help|,
+    },
+
+    'required minimum order amount' => {
+        message => q|Minimum order:|,
+        lastUpdated => 0,
+        context => q|message that is displayed in the cart view screen|,
+    },
+
+    'schedule all button' => {
+        message => q|Schedule all|,
+        lastUpdated => 0,
+        context => 'Label for the schedule all button in the vendor payouts manager',
+    },
+
+    'deschedule all button' => {
+        message => q|Deschedule all|,
+        lastUpdated => 0,
+        context => 'Label for the deschedule all button in the vendor payouts manager',
+    },
+
+    'submit scheduled payouts button' => {
+        message => q|Submit Scheduled Payouts|,
+        lastUpdated => 0,
+        context => 'Label for the submit scheduled payouts button in the vendor payouts manager',
+    },
+   
+    'vendor id'  => {
+        message => q|Vendor ID|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vendor name'  => {
+        message => q|Name|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'scheduled payout amount'  => {
+        message => q|Scheduled for payout|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'not scheduled payout amount'  => {
+        message => q|Not scheduled for payout|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vp select vendor' => {
+        message => q|Please select a vendor from the list above to manage individual payouts.|,
+        lastUpdated => 0,
+        context => q|Message in the vendor payouts manager when no vendor has been selected.|,
+    },
+
+    'vp vendors' => {
+        message => q|Vendors|,
+        lastUpdated => 0,
+        context => q|Label for the vendors section of the vendor payouts manager|,
+    },
+
+    'vp payouts' => {
+        message => q|Payouts|,
+        lastUpdated => 0,
+        context => q|Label for the vendors section of the vendor payouts manager|,
+    },
+   
+    'vp item id'  => {
+        message => q|Item ID|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vp item title'  => {
+        message => q|Item name|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vp item price'  => {
+        message => q|Price|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+    'vp item quantity'  => {
+        message => q|Qty|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vp item payout amount'  => {
+        message => q|Payout amount|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'vp item payout status'  => {
+        message => q|Payout status|,
+        lastUpdated => 0,
+        context => q|Table heading in the vendor payout manager.|,
+    },
+
+    'copy from home address' => {
+        message => q|Copy home address from profile|,
+        lastUpdated => 0,
+        context => q|Button label in the edit address screen.|,
+    },
+
+    'copy from work address' => {
+        message => q|Copy work address from profile|,
+        lastUpdated => 0,
+        context => q|Button label in the edit address screen.|,
+    },
+
+    'group label' => {
+        message => q|Label|,
+        lastUpdated => 0,
+        context => q|Label in the EU tax manager|,
+    },
+
+    'group rate' => {
+        message => q|Tax rate|,
+        lastUpdated => 0,
+        context => q|Label in the EU tax manager|,
+    },
+
+    'No shipping plugins configured' => {
+        message => q|No shipping plugins configured.  Please notify the site adminstrator.|,
+        lastUpdated => 0,
+        context => q|Error message in the cart|,
+    },
+
+    'No shippers' => {
+        message => q|No shipping drivers are configured.  Users will not be able to checkout until at least one is configured.|,
+        lastUpdated => 0,
+        context => q|Error message in the manage ship driver screen.|,
+    },
+
+	'receipt email template' => { 
+		message => q|Receipt Email Template|,
+		lastUpdated => 0,
+		context => q|commerce setting|
+	},
+
+	'receipt email template help' => { 
+		message => q|Which template should be used to generate an email that will be sent to the user to acknowledge their purchase?|,
+		lastUpdated => 0,
+		context => q|commerce setting help|
+	},
+
+	'sale notification group' => { 
+		message => q|Sale Notification Group|,
+		lastUpdated => 0,
+		context => q|commerce setting|
+	},
+
+	'sale notification group help' => { 
+		message => q|Who should be notified of new transactions?|,
+		lastUpdated => 0,
+		context => q|commerce setting help|
+	},
+
+	'Success' => { 
+		message => q|Success|,
+		lastUpdated => 0,
+		context => q|commerce setting help|
+	},
+
+	'Failed' => { 
+		message => q|Failure|,
+		lastUpdated => 0,
+		context => q|commerce setting help|
+	},
+
 };
 
 1;
