@@ -25,11 +25,12 @@ use Pod::Usage;
 use Cwd ();
 
 
-my ($os, $prereq, $dbi, $dbDrivers, $simpleReport, $help);
+my ($os, $prereq, $dbi, $dbDrivers, $simpleReport, $help, $noprompt);
 
 GetOptions(
+    'noprompt' => \$noprompt,
 	'simpleReport'=>\$simpleReport,
-	'help'=>\$help
+	'help'=>\$help,
 );
 
 pod2usage( verbose => 2 ) if $help;
@@ -59,68 +60,81 @@ if ($] >= 5.008) {
 ##Doing this as a global is not nice, but it works
 my $missingModule = 0;
 
-checkModule("LWP",                          5.80         );
+checkModule("LWP",                          5.833        );
 checkModule("HTTP::Request",                1.40         );
 checkModule("HTTP::Headers",                1.61         );
-checkModule("Test::More",                   0.61,      1 );
-checkModule("Test::MockObject",             1.02,      1 );
-checkModule("Test::Deep",                   0.095,     1 );
-checkModule("Pod::Coverage",                0.17,      2 );
-checkModule("Text::Balanced",               1.95,      1 );
-checkModule("Digest::MD5",                  2.20         );
-checkModule("DBI",                          1.40         );
-checkModule("DBD::mysql",                   3.0002       );
-checkModule("HTML::Parser",                 3.36         );
-checkModule("Archive::Tar",                 1.05         );
-checkModule("Archive::Zip",                 1.16         );
-checkModule("IO::Zlib",                     1.01         );
-checkModule("Compress::Zlib",               1.34         );
-checkModule("Net::SMTP",                    2.24         );
-checkModule("MIME::Tools",                  5.419        );
-checkModule("Net::POP3",                    2.28         );
+checkModule("Test::More",                   0.82,      2 );
+checkModule("Test::MockObject",             1.02,      2 );
+checkModule("Test::Deep",                   0.095,     2 );
+checkModule("Test::Exception",              0.27,      2 );
+checkModule("Test::Class",                  0.31,      2 );
+checkModule("Pod::Coverage",                0.19,      2 );
+checkModule("Text::Balanced",               2.00,      2 );
+checkModule("Digest::MD5",                  2.38         );
+checkModule("DBI",                          1.607        );
+checkModule("DBD::mysql",                   4.010        );
+checkModule("HTML::Parser",                 3.60         );
+checkModule("Archive::Tar",                 1.44         );
+checkModule("Archive::Zip",                 1.26         );
+checkModule("IO::Zlib",                     1.09         );
+checkModule("Compress::Zlib",               2.015        );
+checkModule("Net::SMTP",                    2.31         );
+checkModule("MIME::Tools",                  5.427        );
+checkModule("Net::POP3",                    2.29         );
 checkModule("Tie::IxHash",                  1.21         );
-checkModule("Tie::CPHash",                  1.001        );
-checkModule("XML::Simple",                  2.09         );
-checkModule("SOAP::Lite",                   0.60         );
-checkModule("DateTime",                     0.2901       );
-checkModule("Time::HiRes",                  1.38         );
-checkModule("DateTime::Format::Strptime",   1.0601       );
-checkModule("DateTime::Format::Mail",       0.2901       );
-checkModule("Image::Magick",                "6.0",     2 );
-checkModule("Log::Log4perl",                0.51         );
-checkModule("Net::LDAP",                    0.25         );
+checkModule("Tie::CPHash",                  1.04         );
+checkModule("XML::Simple",                  2.18         );
+checkModule("DateTime",                     0.4501       );
+checkModule("Time::HiRes",                  1.9719       );
+checkModule("DateTime::Format::Strptime",   1.0800       );
+checkModule("DateTime::Format::Mail",       0.3001       );
+checkModule("Image::Magick",                "6.0"        );
+checkModule("Log::Log4perl",                1.20         );
+checkModule("Net::LDAP",                    0.39         );
 checkModule("HTML::Highlight",              0.20         );
-checkModule("HTML::TagFilter",              0.07         );
+checkModule("HTML::TagFilter",              1.03         );
 checkModule("HTML::Template",               2.9          );
-checkModule("HTML::Template::Expr",         0.05,      2 );
-checkModule("XML::FeedPP",                  0.37         );
-checkModule("JSON",                         2.04         );
-checkModule("Config::JSON",                 "1.1.2"      );
-checkModule("Text::CSV_XS",                 "0.52"       );
+checkModule("HTML::Template::Expr",         0.07,      2 );
+checkModule("Template",                     2.20,      2 );
+checkModule("XML::FeedPP",                  0.40         );
+checkModule("JSON",                         2.12         );
+checkModule("Config::JSON",                 "1.3.1"      );
+checkModule("Text::CSV_XS",                 "0.64"       );
 checkModule("Net::Subnets",                 0.21         );
-checkModule("Finance::Quote",               1.08         );
-checkModule("POE",                          0.3202       );
-checkModule("POE::Component::IKC::Server",  0.18         );
-checkModule("POE::Component::Client::HTTP", 0.77         );
-checkModule("Data::Structure::Util",        0.11         );
-checkModule("Apache2::Request",             2.06         );
-checkModule("URI::Escape",                  "3.28"       );
+checkModule("Net::CIDR::Lite",              0.20,      2 );
+checkModule("Finance::Quote",               1.15         );
+checkModule("POE",                          1.005        );
+checkModule("POE::Component::IKC::Server",  0.2001       );
+checkModule("POE::Component::Client::HTTP", 0.88         );
+checkModule("Apache2::Request",             2.08         );
+checkModule("URI::Escape",                  "3.29"       );
 checkModule("POSIX"                                      );
 checkModule("List::Util"                                 );
 checkModule("Color::Calc"                                );
 checkModule("Text::Aspell",                 0.01,2       );
-checkModule("Weather::Com::Finder",         "0.5.1"      );
-checkModule("Class::InsideOut",             "1.06"       );
+checkModule("Weather::Com::Finder",         "0.5.3"      );
+checkModule("Class::InsideOut",             "1.09"       );
 checkModule("HTML::TagCloud",               "0.34"       );
-checkModule("Image::ExifTool",              "7.00"       );
-checkModule("Archive::Any",                 "0.093"      );
+checkModule("Image::ExifTool",              "7.67"       );
+checkModule("Archive::Any",                 "0.0932"     );
 checkModule("Path::Class",                  '0.16'       );
-checkModule("Exception::Class",             "1.23"       );
+checkModule("Exception::Class",             "1.26"       );
 checkModule("List::MoreUtils",              "0.22"       );
-checkModule("File::Path",                   "2.04"       );
+checkModule("File::Path",                   "2.07"       );
 checkModule("Module::Find",                 "0.06"       );
-checkModule("Class::C3",                    "0.19"       );
+checkModule("Class::C3",                    "0.21"       );
+checkModule("Params::Validate",             "0.91"       );
 checkModule("Clone",                        "0.31"       );
+checkModule('HTML::Packer',                 "0.4"        );
+checkModule('JavaScript::Packer',           '0.02'       );
+checkModule('CSS::Packer',                  '0.2'        );
+checkModule('Business::Tax::VAT::Validation', '0.20'     );
+checkModule('Crypt::SSLeay',                '0.57'       );
+checkModule('Scope::Guard',                 '0.03'       );
+checkModule('Digest::SHA',                  '5.47'       );
+checkModule("CSS::Minifier::XS",            "0.03"       );
+checkModule("JavaScript::Minifier::XS",     "0.05"       );
+checkModule("Readonly",                     "1.03"       );
 
 failAndExit("Required modules are missing, running no more checks.") if $missingModule;
 
@@ -240,8 +254,13 @@ sub checkModule {
 
             # if we're an admin let's offer to install it
             elsif (isRootRequirementMet()) {
-                my $installThisModule = prompt ("$currentVersion is installed, but we need at least "
-                    ."$version, do you want to upgrade it now?", "y", "y", "n");
+                
+                my $installThisModule = defined $noprompt ? "y" : "n";
+                if ( $installThisModule eq "n" ) {
+                    $installThisModule 
+                        = prompt ("$currentVersion is installed, but we need at least "
+                        . "$version, do you want to upgrade it now?", "y", "y", "n");
+                }
 
                 # does the user wish to install it
                 if ($installThisModule eq "y") {
@@ -278,7 +297,13 @@ sub checkModule {
 
         # if we're root lets try and install it
 		elsif (  isRootRequirementMet()) {
-            my $installThisModule = prompt ("Not installed, do you want to install it now?", "y", "y", "n");
+            my $installThisModule = defined $noprompt ? "y" : "n";
+            if ( $installThisModule eq "n" ) {
+                $installThisModule 
+                    = prompt ("Not installed, do you want to install it now?", 
+                        "y", "y", "n"
+                    );
+            }
 
             # user wishes to upgrade
             if ($installThisModule eq "y") {
